@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -33,7 +34,7 @@
 #include <core/layers.h>
 
 typedef struct {
-     volatile __u8 *mmio_base;
+     volatile u8 *mmio_base;
 } ATI128DriverData;
 
 typedef struct {
@@ -42,13 +43,13 @@ typedef struct {
      DFBSurfaceBlittingFlags blittingflags;
 
      /* store some ATI register values in native format */
-     __u32 ATI_dst_bpp;
-     __u32 ATI_color_compare;
-     __u32 ATI_blend_function;
+     u32 ATI_dst_bpp;
+     u32 ATI_color_compare;
+     u32 ATI_blend_function;
 
      /* used for the fake texture hack */
-     __u32 ATI_fake_texture_src;
-     __u32 fake_texture_color;
+     u32 ATI_fake_texture_src;
+     u32 fake_texture_color;
      unsigned int fake_texture_number;
 
      /* state validation */
