@@ -7,6 +7,8 @@
    version 2 of the License, or (at your option) any later version.
 */
 
+#include <config.h>
+
 #include <sys/io.h>
 
 #include "unichrome.h"
@@ -25,7 +27,7 @@
 
 void uc_ovl_setup_fifo(UcOverlayData* ucovl, int scrwidth)
 {
-     __u8* mclk_save = ucovl->mclk_save;
+     u8* mclk_save = ucovl->mclk_save;
 
      if (!iopl(3)) {
           if (scrwidth <= 1024) { // Disable
@@ -62,7 +64,7 @@ void uc_ovl_setup_fifo(UcOverlayData* ucovl, int scrwidth)
      ucovl->scrwidth = scrwidth;
 }
 
-void uc_ovl_vcmd_wait(volatile __u8* vio)
+void uc_ovl_vcmd_wait(volatile u8* vio)
 {
      while ((VIDEO_IN(vio, V_COMPOSE_MODE)
              & (V1_COMMAND_FIRE | V3_COMMAND_FIRE)));
@@ -92,13 +94,13 @@ DFBResult uc_ovl_update(UcDriverData* ucdrv,
      bool write_buffers = false;
      bool write_settings = false;
 
-     volatile __u8* vio = ucdrv->hwregs;
+     volatile u8* vio = ucdrv->hwregs;
 
-     __u32 win_start, win_end;   // Overlay register settings
-     __u32 zoom, mini;
-     __u32 dcount, falign, qwpitch;
-     __u32 y_start, u_start, v_start;
-     __u32 v_ctrl, fifo_ctrl;
+     u32 win_start, win_end;     // Overlay register settings
+     u32 zoom, mini;
+     u32 dcount, falign, qwpitch;
+     u32 y_start, u_start, v_start;
+     u32 v_ctrl, fifo_ctrl;
 
      int offset = surface->front_buffer->video.offset;
 
@@ -241,7 +243,7 @@ DFBResult uc_ovl_set_adjustment(CoreLayer            *layer,
     UcOverlayData* ucovl = (UcOverlayData*) layer_data;
     UcDriverData*  ucdrv = (UcDriverData*) driver_data;
     DFBColorAdjustment* ucadj;
-    __u32 a1, a2;
+    u32 a1, a2;
 
     ucadj = &ucovl->v1.adj;
 
