@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -64,7 +65,7 @@ DFB_GRAPHICS_DRIVER( cyber5k )
 
 
 /* HACK */
-volatile __u8 *cyber_mmio = NULL;
+volatile u8 *cyber_mmio = NULL;
 
 
 /* FIXME: support for destination color keying */
@@ -90,12 +91,14 @@ static bool cyber5kBlit( void *drv, void *dev,
 static bool cyber5kBlit24( void *drv, void *dev,
                            DFBRectangle *rect, int dx, int dy );
 
-static void cyber5kEngineSync( void *drv, void *dev )
+static DFBResult cyber5kEngineSync( void *drv, void *dev )
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
 
      cyber_waitidle( cdrv, cdev );
+
+     return DFB_OK;
 }
 
 static void cyber5kCheckState( void *drv, void *dev,
@@ -206,7 +209,7 @@ static inline void
 cyber5k_validate_color( CyberDriverData *cdrv,
                         CyberDeviceData *cdev, CardState *state )
 {
-     __u32 fill_color = 0;
+     u32 fill_color = 0;
 
      if (cdev->v_color)
           return;
@@ -325,7 +328,7 @@ static bool cyber5kFillRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
-     volatile __u8   *mmio = cdrv->mmio_base;
+     volatile u8     *mmio = cdrv->mmio_base;
 
      cyber_waitidle( cdrv, cdev );
 
@@ -343,7 +346,7 @@ static bool cyber5kFillRectangle24( void *drv, void *dev, DFBRectangle *rect )
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
-     volatile __u8   *mmio = cdrv->mmio_base;
+     volatile u8     *mmio = cdrv->mmio_base;
 
      cyber_waitidle( cdrv, cdev );
 
@@ -362,10 +365,10 @@ static bool cyber5kDrawRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
-     volatile __u8   *mmio = cdrv->mmio_base;
+     volatile u8     *mmio = cdrv->mmio_base;
 
-     __u32 dst = cdev->dst_pixeloffset +
-                 rect->y * cdev->dst_pixelpitch + rect->x;
+     u32 dst = cdev->dst_pixeloffset +
+               rect->y * cdev->dst_pixelpitch + rect->x;
 
      cyber_waitidle( cdrv, cdev );
      cyber_out32( mmio, DSTPTR, dst );
@@ -394,10 +397,10 @@ static bool cyber5kDrawRectangle24( void *drv, void *dev, DFBRectangle *rect )
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
-     volatile __u8   *mmio = cdrv->mmio_base;
+     volatile u8     *mmio = cdrv->mmio_base;
 
-     __u32 dst = cdev->dst_pixeloffset +
-                (rect->y * cdev->dst_pixelpitch + rect->x) * 3;
+     u32 dst = cdev->dst_pixeloffset +
+               (rect->y * cdev->dst_pixelpitch + rect->x) * 3;
 
      cyber_waitidle( cdrv, cdev );
      cyber_out8( mmio, DSTXROT, rect->x & 7 );
@@ -428,9 +431,9 @@ static bool cyber5kDrawLine( void *drv, void *dev, DFBRegion *line )
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
-     volatile __u8   *mmio = cdrv->mmio_base;
+     volatile u8     *mmio = cdrv->mmio_base;
 
-     __u32 cmd = COP_LINE_DRAW | PAT_FIXFGD;
+     u32 cmd = COP_LINE_DRAW | PAT_FIXFGD;
 
      int dx;
      int dy;
@@ -472,9 +475,9 @@ static bool cyber5kBlit( void *drv, void *dev, DFBRectangle *rect, int dx, int d
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
-     volatile __u8   *mmio = cdrv->mmio_base;
+     volatile u8     *mmio = cdrv->mmio_base;
 
-     __u32 cmd = cdev->blitting_cmd;
+     u32 cmd = cdev->blitting_cmd;
 
      cyber_waitidle( cdrv, cdev );
 
@@ -507,11 +510,11 @@ static bool cyber5kBlit24( void *drv, void *dev, DFBRectangle *rect, int dx, int
 {
      CyberDriverData *cdrv = (CyberDriverData*) drv;
      CyberDeviceData *cdev = (CyberDeviceData*) dev;
-     volatile __u8   *mmio = cdrv->mmio_base;
+     volatile u8     *mmio = cdrv->mmio_base;
 
-     __u32 cmd = cdev->blitting_cmd;
-     __u32 src = 0;
-     __u32 dst = 0;
+     u32 cmd = cdev->blitting_cmd;
+     u32 src = 0;
+     u32 dst = 0;
 
      cyber_waitidle( cdrv, cdev );
 
@@ -723,7 +726,7 @@ driver_init_driver( GraphicsDevice      *device,
      CyberDriverData *cdrv = (CyberDriverData*) driver_data;
 
      /* gain access to memory mapped registers */
-     cdrv->mmio_base = (volatile __u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
+     cdrv->mmio_base = (volatile u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
      if (!cdrv->mmio_base)
           return DFB_IO;
 
@@ -764,7 +767,7 @@ driver_init_device( GraphicsDevice     *device,
                     void               *device_data )
 {
      CyberDriverData *cdrv = (CyberDriverData*) driver_data;
-     volatile __u8   *mmio = cdrv->mmio_base;
+     volatile u8     *mmio = cdrv->mmio_base;
 
      /* fill device info */
      snprintf( device_info->name,
