@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -32,11 +33,11 @@
 #include <core/layers.h>
 
 /* HACK */
-extern volatile __u8 *cyber_mmio;
+extern volatile u8 *cyber_mmio;
 
 
 typedef struct {
-     volatile __u8 *mmio_base;
+     volatile u8 *mmio_base;
 } CyberDriverData;
 
 typedef struct {
@@ -48,12 +49,12 @@ typedef struct {
      int v_blitting_cmd;
 
      /* stored values */
-     __u32 dst_pixeloffset;
-     __u32 dst_pixelpitch;
-     __u32 src_pixeloffset;
-     __u32 src_pixelpitch;
+     u32 dst_pixeloffset;
+     u32 dst_pixelpitch;
+     u32 src_pixeloffset;
+     u32 src_pixelpitch;
 
-     __u32 blitting_cmd;
+     u32 blitting_cmd;
 } CyberDeviceData;
 
 
