@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -33,39 +34,39 @@
 #include "regs.h"
 
 static inline void
-cyber_out8(volatile __u8 *mmioaddr, __u32 reg, __u8 value)
+cyber_out8(volatile u8 *mmioaddr, u32 reg, u8 value)
 {
-     *((volatile __u8*)(mmioaddr+reg)) = value;
+     *((volatile u8*)(mmioaddr+reg)) = value;
 }
 
 static inline void
-cyber_out16(volatile __u8 *mmioaddr, __u32 reg, __u16 value)
+cyber_out16(volatile u8 *mmioaddr, u32 reg, u16 value)
 {
-     *((volatile __u16*)(mmioaddr+reg)) = value;
+     *((volatile u16*)(mmioaddr+reg)) = value;
 }
 
 static inline void
-cyber_out32(volatile __u8 *mmioaddr, __u32 reg, __u32 value)
+cyber_out32(volatile u8 *mmioaddr, u32 reg, u32 value)
 {
-     *((volatile __u32*)(mmioaddr+reg)) = value;
+     *((volatile u32*)(mmioaddr+reg)) = value;
 }
 
-static inline __u8
-cyber_in8(volatile __u8 *mmioaddr, __u32 reg)
+static inline u8
+cyber_in8(volatile u8 *mmioaddr, u32 reg)
 {
-     return *((volatile __u8*)(mmioaddr+reg));
+     return *((volatile u8*)(mmioaddr+reg));
 }
 
-static inline __u16
-cyber_in16(volatile __u8 *mmioaddr, __u32 reg)
+static inline u16
+cyber_in16(volatile u8 *mmioaddr, u32 reg)
 {
-     return *((volatile __u16*)(mmioaddr+reg));
+     return *((volatile u16*)(mmioaddr+reg));
 }
 
-static inline __u32
-cyber_in32(volatile __u8 *mmioaddr, __u32 reg)
+static inline u32
+cyber_in32(volatile u8 *mmioaddr, u32 reg)
 {
-     return *((volatile __u32*)(mmioaddr+reg));
+     return *((volatile u32*)(mmioaddr+reg));
 }
 
 /* Wait for idle accelerator */
