@@ -1,6 +1,6 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
@@ -21,6 +21,10 @@
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
+
+#include <config.h>
+
+#include <fbdev/fbdev.h>  /* FIXME: Needs to be included before dfb_types.h to work around a type clash with asm/types.h */
 #include <dfb_types.h>
 
 #include <stdio.h>
@@ -44,7 +48,6 @@
 #include <core/surfaces.h>
 
 /* need fb handle to get accel, MMIO programming in the i810 is useless */
-#include <fbdev/fbdev.h>
 #include <gfx/convert.h>
 #include <gfx/util.h>
 #include <misc/conf.h>
@@ -176,13 +179,15 @@ i810FlushTextureCache(void *drv, void *dev)
 	END_LRING(i810drv);
 }
 
-static void
+static DFBResult
 i810EngineSync(void *drv, void *dev)
 {
 	I810DriverData *i810drv = (I810DriverData *) drv;
 	I810DeviceData *i810dev = (I810DeviceData *) dev;
 
 	i810_wait_for_blit_idle(i810drv, i810dev);
+
+    return DFB_OK;
 }
 
 /*
@@ -546,7 +551,7 @@ i810Blit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy )
 	PUT_LRING(i810dev->srcpitch);
 	PUT_LRING(src);
 	PUT_LRING(i810dev->colorkey);
-	PUT_LRING((u32) i810drv->pattern_base);
+	PUT_LRING((u32)(unsigned long) i810drv->pattern_base);
 
 	END_LRING(i810drv);
 	
@@ -820,7 +825,7 @@ i810_agp_setup( GraphicsDevice *device,
 
 
      if (!idev->initialized) {
-          __u32 base;
+          u32 base;
 
           /* We'll attempt to bind at fb_base + fb_len + 1 MB,
           to be safe */
@@ -906,7 +911,7 @@ driver_init_driver( GraphicsDevice      *device,
 
      idrv->idev = device_data;
 
-     idrv->mmio_base = (volatile __u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
+     idrv->mmio_base = (volatile u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
      if (!idrv->mmio_base)
           return DFB_IO;
 
