@@ -2,8 +2,10 @@
    i810_overlay.c -- Video Overlay Support (based partly from
                      XFree86 i810_video.c)
 
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
+
+   All rights reserved.
 
    Written by Antonino Daplas <adaplas@pol.net>
 
@@ -23,10 +25,14 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <sys/ioctl.h>
 
 #include <string.h>
 
+#include <fbdev/fbdev.h>  /* FIXME: Needs to be included before dfb_types.h to work around a type clash with asm/types.h */
+
 #include <core/coredefs.h>
 #include <core/layers.h>
 #include <core/surfaces.h>
