@@ -23,7 +23,12 @@
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
-#include <asm/types.h>
+
+#include <config.h>
+
+#include <fbdev/fbdev.h>  /* FIXME: Needs to be included before dfb_types.h to work around a type clash with asm/types.h */
+
+#include "i830.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -45,8 +50,6 @@
 #include <core/gfxcard.h>
 #include <core/surfaces.h>
 
-#include <core/accelerators.h>
-
 /* need fb handle to get accel, MMIO programming in the i830 is useless */
 #include <fbdev/fbdev.h>
 #include <gfx/convert.h>
@@ -58,7 +61,6 @@
 
 DFB_GRAPHICS_DRIVER( i830 )
 
-#include "i830.h"
 
 /**************************************************************************************************/
 
@@ -77,9 +79,9 @@ DFB_GRAPHICS_DRIVER( i830 )
 /**************************************************************************************************/
 
 static void
-i830_lring_enable( I830DriverData *idrv, __u32 mode )
+i830_lring_enable( I830DriverData *idrv, u32 mode )
 {
-     __u32 tmp;
+     u32 tmp;
 
      D_DEBUG_AT( I830_Ring, "%s lp ring...\n", mode ? "Enabling" : "Disabling" );
 
@@ -94,8 +96,8 @@ static inline void
 i830_wait_for_blit_idle( I830DriverData *idrv,
                          I830DeviceData *idev )
 {
-     __u32 count = 0;
-     __u32 head , tail;
+     u32 count = 0;
+     u32 head , tail;
 
      if (idev != NULL)
           idev->idle_calls++;
@@ -120,7 +122,7 @@ static void
 i830_init_ringbuffer( I830DriverData *idrv,
                       I830DeviceData *idev )
 {
-     __u32 ring_enabled;
+     u32 ring_enabled;
 
      D_DEBUG_AT( I830_Ring, "Previous lp ring config: 0x%08x, 0x%08x, 0x%08x, 0x%08x\n",
                  i830_readl(idrv->mmio_base, LP_RING),
@@ -216,9 +218,9 @@ i830_wait_lp_ring( I830DriverData *idrv,
 static void
 i830FlushTextureCache( void *drv, void *dev )
 {
-     I830DriverData *idrv = drv;
-     I830DeviceData *idev = dev;
-     I830RingBlock   block;
+     I830DriverData *idrv  = drv;
+     I830DeviceData *idev  = dev;
+     I830RingBlock   block = {0};
 
      if (i830_begin_lp_ring( idrv, idev, 2, &block ))
           return;
@@ -229,13 +231,15 @@ i830FlushTextureCache( void *drv, void *dev )
      i830_advance_lp_ring( idrv, idev, &block );
 }
 
-static void
+static DFBResult
 i830EngineSync( void *drv, void *dev )
 {
      I830DriverData *idrv = drv;
      I830DeviceData *idev = dev;
 
      i830_wait_for_blit_idle( idrv, idev );
+
+     return DFB_OK;
 }
 
 /**************************************************************************************************/
@@ -395,7 +399,7 @@ i830_agp_setup( GraphicsDevice *device,
 
 
      if (!idev->initialized) {
-          __u32 base;
+          u32 base;
 
           /* We'll attempt to bind at fb_base + fb_len + 1 MB,
           to be safe */
@@ -471,7 +475,8 @@ static DFBResult
 driver_init_driver( GraphicsDevice      *device,
                     GraphicsDeviceFuncs *funcs,
                     void                *driver_data,
-                    void                *device_data )
+                    void                *device_data,
+                    CoreDFB             *core )
 {
      DFBResult       ret;
      I830DriverData *idrv = driver_data;
@@ -479,7 +484,7 @@ driver_init_driver( GraphicsDevice      *device,
 
      idrv->idev = device_data;
 
-     idrv->mmio_base = (volatile __u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
+     idrv->mmio_base = (volatile u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
      if (!idrv->mmio_base)
           return DFB_IO;
 
