@@ -43,6 +43,10 @@
  *        Matt Sottek <matthew.j.sottek@intel.com>
  */
 
+#include <config.h>
+
+#include <fbdev/fbdev.h>  /* FIXME: Needs to be included before dfb_types.h to work around a type clash with asm/types.h */
+
 #include "i830.h"
 
 #include <math.h>
@@ -62,7 +66,7 @@
 
 #include <gfx/convert.h>
 
-
+#include <misc/conf.h>
 
 #define I830_OVERLAY_SUPPORTED_OPTIONS (DLOP_DST_COLORKEY)
 
@@ -319,7 +323,7 @@ ovlSetColorAdjustment( CoreLayer          *layer,
 {
      I830DriverData *idrv = driver_data;
      I830DeviceData *idev = idrv->idev;
-     __u16 b, c, s;
+     u16 b, c, s;
 
      if (adj->flags & DCAF_BRIGHTNESS)
 	 b = ((adj->brightness >> 8) - 128) & 0xFF;
@@ -376,9 +380,9 @@ DisplayLayerFuncs i830OverlayFuncs = {
 
 
 typedef struct {
-     __u8 sign;
-     __u16 mantissa;
-     __u8 exponent;
+     u8 sign;
+     u16 mantissa;
+     u8 exponent;
 } coeffRec, *coeffPtr;
 
 static bool
@@ -523,8 +527,6 @@ ovl_calc_regs( I830DriverData        *idrv,
      int              width  = config->width;
      int              height = config->height;
 
-     DFBSurfacePixelFormat primary_format;
-
      int y_offset, u_offset = 0, v_offset = 0;
 
      unsigned int swidth;
@@ -634,7 +636,7 @@ ovl_calc_regs( I830DriverData        *idrv,
           int yscaleIntUV, yscaleFractUV;
           /* UV is half the size of Y -- YUV420 */
           int uvratio = 2;
-          __u32 newval;
+          u32 newval;
           coeffRec xcoeffY[N_HORIZ_Y_TAPS * N_PHASES];
           coeffRec xcoeffUV[N_HORIZ_UV_TAPS * N_PHASES];
           int i, j, pos;
@@ -765,17 +767,38 @@ ovl_calc_regs( I830DriverData        *idrv,
      /*
       * Destination color keying.
       */
+     regs->DCLRKV = PIXEL_RGB32 (config->dst_key.r, config->dst_key.g, config->dst_key.b );
 
-     primary_format = dfb_primary_layer_pixelformat();
+     switch (DFB_COLOR_BITS_PER_PIXEL( dfb_primary_layer_pixelformat() )) {
+          case 8:
+               regs->DCLRKM = 0xffffff;
+               break;
+          case 15:
+               regs->DCLRKM = 0x070707;
+               break;
+          case 16:
+               regs->DCLRKM = 0x070307;
+               break;
+          default:
+               regs->DCLRKM = 0;
+               break;
+     }
 
-     regs->DCLRKV = dfb_color_to_pixel( primary_format,
-                                        config->dst_key.r, config->dst_key.g, config->dst_key.b );
-
-     regs->DCLRKM = (1 << DFB_COLOR_BITS_PER_PIXEL( primary_format )) - 1;
+     if(dfb_config->i8xx_overlay_pipe_b)
+          regs->OCONFIG |= OVERLAY_PIPE_B;
 
      if (config->options & DLOP_DST_COLORKEY)
           regs->DCLRKM |= DEST_KEY_ENABLE;
 
+     /*
+      * Disable source color keying if not selected
+      */
+     if (!(config->options & DLOP_SRC_COLORKEY)) {
+          regs -> SCLRKVH = 0;
+          regs -> SCLRKVL = 0;
+          regs -> SCLRKEN = 0;
+     }
+
 
      //D_INFO("OCMD is 0x%08x\n", regs->OCMD);
 }
