@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <dfb_types.h>
 
 #include <stdio.h>
@@ -121,7 +124,7 @@ static void mach64EngineReset( void *drv, void *dev )
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) drv;
      Mach64DeviceData *mdev = (Mach64DeviceData*) dev;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      mach64_waitidle( mdrv, mdev );
 
@@ -155,19 +158,21 @@ static void mach64EngineReset( void *drv, void *dev )
           mach64_out32( mmio, HW_DEBUG, mdev->hw_debug );
 }
 
-static void mach64EngineSync( void *drv, void *dev )
+static DFBResult mach64EngineSync( void *drv, void *dev )
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) drv;
      Mach64DeviceData *mdev = (Mach64DeviceData*) dev;
 
      mach64_waitidle( mdrv, mdev );
+
+     return DFB_OK;
 }
 
 static void mach64FlushTextureCache( void *drv, void *dev )
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) drv;
      Mach64DeviceData *mdev = (Mach64DeviceData*) dev;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      if (mdev->chip >= CHIP_3D_RAGE_PRO) {
           mach64_waitfifo( mdrv, mdev, 1 );
@@ -259,6 +264,7 @@ static void mach64CheckState( void *drv, void *dev,
 {
      switch (state->destination->format) {
           case DSPF_RGB332:
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_RGB32:
@@ -295,6 +301,7 @@ static void mach64GTCheckState( void *drv, void *dev,
      Mach64DeviceData *mdev = (Mach64DeviceData*) dev;
 
      switch (state->destination->format) {
+          case DSPF_RGB444:
           case DSPF_ARGB4444:
                /* Not supported. */
                if (mdev->chip < CHIP_3D_RAGE_PRO)
@@ -305,6 +312,7 @@ static void mach64GTCheckState( void *drv, void *dev,
                    mach64_use_scaler_3d( mdev, state, accel ))
                     return;
           case DSPF_RGB332:
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_RGB32:
@@ -333,7 +341,9 @@ static void mach64GTCheckState( void *drv, void *dev,
 
           switch (source->format) {
                case DSPF_RGB332:
+               case DSPF_RGB555:
                case DSPF_ARGB1555:
+               case DSPF_RGB444:
                case DSPF_ARGB4444:
                case DSPF_RGB16:
                case DSPF_RGB32:
@@ -395,7 +405,7 @@ static void mach64SetState( void *drv, void *dev,
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) drv;
      Mach64DeviceData *mdev = (Mach64DeviceData*) dev;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      if (state->modified == SMF_ALL) {
           mdev->valid = 0;
@@ -481,7 +491,7 @@ static void mach64GTSetState( void *drv, void *dev,
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) drv;
      Mach64DeviceData *mdev = (Mach64DeviceData*) dev;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      bool use_scaler_3d;
 
@@ -652,7 +662,7 @@ static bool mach64FillRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) drv;
      Mach64DeviceData *mdev = (Mach64DeviceData*) dev;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      mach64_waitfifo( mdrv, mdev, 3 );
 
@@ -667,7 +677,7 @@ static bool mach64DrawRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) drv;
      Mach64DeviceData *mdev = (Mach64DeviceData*) dev;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      int x2 = rect->x + rect->w - 1;
      int y2 = rect->y + rect->h - 1;
@@ -693,9 +703,9 @@ static void mach64_draw_line( Mach64DriverData *mdrv,
                               int x2, int y2,
                               bool draw_3d )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
-     __u32 dst_cntl = 0;
+     u32 dst_cntl = 0;
      int   dx, dy;
 
      dx = x2 - x1;
@@ -782,9 +792,9 @@ static void mach64_fill_trapezoid( Mach64DriverData *mdrv,
                                    int X2l, int X2r,
                                    int Y, int dY )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
-     __u32 dst_cntl;
+     u32 dst_cntl;
      int   dXl, dXr;
 
      X1r++; X2r++;
@@ -868,9 +878,9 @@ static void mach64DoBlit2D( Mach64DriverData *mdrv,
                             DFBRectangle *srect,
                             DFBRectangle *drect )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
-     __u32 dst_cntl = 0;
+     u32 dst_cntl = 0;
 
      if (srect->x <= drect->x) {
           srect->x += srect->w - 1;
@@ -900,10 +910,10 @@ static void mach64DoBlitScaleOld( Mach64DriverData *mdrv,
                                   DFBRectangle     *drect,
                                   bool              filter )
 {
-     volatile __u8 *mmio   = mdrv->mmio_base;
+     volatile u8 *mmio     = mdrv->mmio_base;
      CoreSurface   *source = mdev->source;
 
-     __u32 scale_3d_cntl = SCALE_3D_FCN_SCALE | mdev->blit_blend;
+     u32 scale_3d_cntl = SCALE_3D_FCN_SCALE | mdev->blit_blend;
      int   hacc, vacc;
 
      if (!filter)
@@ -966,10 +976,10 @@ static void mach64DoBlitScale( Mach64DriverData *mdrv,
                                DFBRectangle     *drect,
                                bool              filter )
 {
-     volatile __u8 *mmio   = mdrv->mmio_base;
+     volatile u8 *mmio     = mdrv->mmio_base;
      CoreSurface   *source = mdev->source;
 
-     __u32 scale_3d_cntl = SCALE_3D_FCN_SCALE | mdev->blit_blend;
+     u32 scale_3d_cntl = SCALE_3D_FCN_SCALE | mdev->blit_blend;
      int   hacc, vacc;
 
      if (!filter)
@@ -989,7 +999,7 @@ static void mach64DoBlitScale( Mach64DriverData *mdrv,
      if (drect->y < mdev->clip.y1) {
           int sy, dy;
           dy = mdev->clip.y1 - drect->y;
-          sy = (__u64) srect->h * dy / drect->h;
+          sy = (u64) srect->h * dy / drect->h;
           srect->y += sy;
           srect->h -= sy;
           drect->y += dy;
@@ -1040,9 +1050,9 @@ static void mach64DoBlitTexOld( Mach64DriverData *mdrv,
                                 DFBRectangle     *drect,
                                 bool              filter )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
-     __u32 scale_3d_cntl = SCALE_3D_FCN_TEXTURE | MIP_MAP_DISABLE | mdev->blit_blend;
+     u32 scale_3d_cntl = SCALE_3D_FCN_TEXTURE | MIP_MAP_DISABLE | mdev->blit_blend;
 
      if (mdev->blit_deinterlace) {
           srect->y /= 2;
@@ -1101,9 +1111,9 @@ static void mach64DoBlitTex( Mach64DriverData *mdrv,
                              DFBRectangle     *drect,
                              bool              filter )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
-     __u32 scale_3d_cntl = SCALE_3D_FCN_TEXTURE | MIP_MAP_DISABLE | mdev->blit_blend;
+     u32 scale_3d_cntl = SCALE_3D_FCN_TEXTURE | MIP_MAP_DISABLE | mdev->blit_blend;
 
      if (mdev->blit_deinterlace) {
           srect->y /= 2;
@@ -1276,8 +1286,8 @@ static Mach64ChipType
 mach64_chip_type_vt( Mach64DriverData   *mdrv,
                      GraphicsDeviceInfo *device_info )
 {
-     __u32 config_chip_id = mach64_in32( mdrv->mmio_base, CONFIG_CHIP_ID );
-     __u32 cfg_chip_type = config_chip_id & CFG_CHIP_TYPE;
+     u32 config_chip_id = mach64_in32( mdrv->mmio_base, CONFIG_CHIP_ID );
+     u32 cfg_chip_type = config_chip_id & CFG_CHIP_TYPE;
 
      switch (cfg_chip_type) {
      case MACH64_CFG_CHIP_TYPE( 'V', 'T' ):
@@ -1318,8 +1328,8 @@ static Mach64ChipType
 mach64_chip_type_gt( Mach64DriverData   *mdrv,
                      GraphicsDeviceInfo *device_info )
 {
-     __u32 config_chip_id = mach64_in32( mdrv->mmio_base, CONFIG_CHIP_ID );
-     __u32 cfg_chip_type = config_chip_id & CFG_CHIP_TYPE;
+     u32 config_chip_id = mach64_in32( mdrv->mmio_base, CONFIG_CHIP_ID );
+     u32 cfg_chip_type = config_chip_id & CFG_CHIP_TYPE;
 
      switch (cfg_chip_type) {
      case MACH64_CFG_CHIP_TYPE( 'G', 'T' ):
@@ -1452,7 +1462,7 @@ driver_init_driver( GraphicsDevice      *device,
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) driver_data;
 
-     mdrv->mmio_base = (volatile __u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
+     mdrv->mmio_base = (volatile u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
      if (!mdrv->mmio_base)
           return DFB_IO;
 
@@ -1496,7 +1506,7 @@ driver_init_device( GraphicsDevice     *device,
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) driver_data;
      Mach64DeviceData *mdev = (Mach64DeviceData*) device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      /* fill device info */
      device_info->caps.flags = CCF_CLIPPING;
@@ -1573,7 +1583,7 @@ driver_close_device( GraphicsDevice *device,
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) driver_data;
      Mach64DeviceData *mdev = (Mach64DeviceData*) device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      D_DEBUG( "DirectFB/Mach64: FIFO Performance Monitoring:\n" );
      D_DEBUG( "DirectFB/Mach64:  %9d mach64_waitfifo calls\n",
