@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <directfb.h>
 
 #include <direct/messages.h>
@@ -50,7 +53,7 @@ void mach64_set_destination( Mach64DriverData *mdrv,
                              Mach64DeviceData *mdev,
                              CardState        *state )
 {
-     volatile __u8 *mmio        = mdrv->mmio_base;
+     volatile u8 *mmio          = mdrv->mmio_base;
      CoreSurface   *destination = state->destination;
      SurfaceBuffer *buffer      = destination->back_buffer;
      int            pitch       = buffer->video.pitch / DFB_BYTES_PER_PIXEL( destination->format );
@@ -60,6 +63,7 @@ void mach64_set_destination( Mach64DriverData *mdrv,
           case DSPF_RGB332:
                mdev->pix_width |= DST_PIX_WIDTH_8BPP;
                break;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                mdev->pix_width |= DST_PIX_WIDTH_15BPP;
                break;
@@ -83,7 +87,7 @@ void mach64gt_set_destination( Mach64DriverData *mdrv,
                                Mach64DeviceData *mdev,
                                CardState        *state )
 {
-     volatile __u8 *mmio        = mdrv->mmio_base;
+     volatile u8 *mmio          = mdrv->mmio_base;
      CoreSurface   *destination = state->destination;
      SurfaceBuffer *buffer      = destination->back_buffer;
      int            pitch       = buffer->video.pitch / DFB_BYTES_PER_PIXEL( destination->format );
@@ -93,9 +97,11 @@ void mach64gt_set_destination( Mach64DriverData *mdrv,
           case DSPF_RGB332:
                mdev->pix_width |= DST_PIX_WIDTH_RGB332;
                break;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                mdev->pix_width |= DST_PIX_WIDTH_ARGB1555;
                break;
+          case DSPF_RGB444:
           case DSPF_ARGB4444:
                mdev->pix_width |= DST_PIX_WIDTH_ARGB4444;
                break;
@@ -126,7 +132,7 @@ void mach64_set_source( Mach64DriverData *mdrv,
                         Mach64DeviceData *mdev,
                         CardState        *state )
 {
-     volatile __u8 *mmio   = mdrv->mmio_base;
+     volatile u8 *mmio     = mdrv->mmio_base;
      CoreSurface   *source = state->source;
      SurfaceBuffer *buffer = source->front_buffer;
      int            pitch  = buffer->video.pitch / DFB_BYTES_PER_PIXEL( source->format );
@@ -139,6 +145,7 @@ void mach64_set_source( Mach64DriverData *mdrv,
           case DSPF_RGB332:
                mdev->pix_width |= SRC_PIX_WIDTH_8BPP;
                break;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                mdev->pix_width |= SRC_PIX_WIDTH_15BPP;
                break;
@@ -164,7 +171,7 @@ void mach64gt_set_source( Mach64DriverData *mdrv,
                           Mach64DeviceData *mdev,
                           CardState        *state )
 {
-     volatile __u8 *mmio   = mdrv->mmio_base;
+     volatile u8 *mmio     = mdrv->mmio_base;
      CoreSurface   *source = state->source;
      SurfaceBuffer *buffer = source->front_buffer;
      int            pitch  = buffer->video.pitch / DFB_BYTES_PER_PIXEL( source->format );
@@ -177,9 +184,11 @@ void mach64gt_set_source( Mach64DriverData *mdrv,
           case DSPF_RGB332:
                mdev->pix_width |= SRC_PIX_WIDTH_RGB332;
                break;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                mdev->pix_width |= SRC_PIX_WIDTH_ARGB1555;
                break;
+          case DSPF_RGB444:
           case DSPF_ARGB4444:
                mdev->pix_width |= SRC_PIX_WIDTH_ARGB4444;
                break;
@@ -205,7 +214,7 @@ void mach64gt_set_source_scale( Mach64DriverData *mdrv,
                                 Mach64DeviceData *mdev,
                                 CardState        *state )
 {
-     volatile __u8 *mmio   = mdrv->mmio_base;
+     volatile u8 *mmio     = mdrv->mmio_base;
      CoreSurface   *source = state->source;
      SurfaceBuffer *buffer = source->front_buffer;
      int            offset = buffer->video.offset;
@@ -220,9 +229,11 @@ void mach64gt_set_source_scale( Mach64DriverData *mdrv,
           case DSPF_RGB332:
                mdev->pix_width |= SCALE_PIX_WIDTH_RGB332;
                break;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                mdev->pix_width |= SCALE_PIX_WIDTH_ARGB1555;
                break;
+          case DSPF_RGB444:
           case DSPF_ARGB4444:
                mdev->pix_width |= SCALE_PIX_WIDTH_ARGB4444;
                break;
@@ -282,7 +293,7 @@ void mach64_set_clip( Mach64DriverData *mdrv,
                       Mach64DeviceData *mdev,
                       CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      mach64_waitfifo( mdrv, mdev, 2 );
      mach64_out32( mmio, SC_LEFT_RIGHT, (S13( state->clip.x2 ) << 16) | S13( state->clip.x1 ) );
@@ -293,9 +304,9 @@ void mach64_set_color( Mach64DriverData *mdrv,
                        Mach64DeviceData *mdev,
                        CardState        *state )
 {
-     volatile __u8 *mmio  = mdrv->mmio_base;
+     volatile u8 *mmio    = mdrv->mmio_base;
      DFBColor       color = state->color;
-     __u32          clr;
+     u32            clr;
 
      if (MACH64_IS_VALID( m_color ))
           return;
@@ -312,12 +323,22 @@ void mach64_set_color( Mach64DriverData *mdrv,
                                    color.g,
                                    color.b );
                break;
+          case DSPF_RGB555:
+               clr = PIXEL_RGB555( color.r,
+                                   color.g,
+                                   color.b );
+               break;
           case DSPF_ARGB1555:
                clr = PIXEL_ARGB1555( color.a,
                                      color.r,
                                      color.g,
                                      color.b );
                break;
+          case DSPF_RGB444:
+               clr = PIXEL_RGB444( color.r,
+                                   color.g,
+                                   color.b );
+               break;
           case DSPF_ARGB4444:
                clr = PIXEL_ARGB4444( color.a,
                                      color.r,
@@ -355,7 +376,7 @@ void mach64_set_color_3d( Mach64DriverData *mdrv,
                           Mach64DeviceData *mdev,
                           CardState        *state )
 {
-     volatile __u8 *mmio  = mdrv->mmio_base;
+     volatile u8 *mmio    = mdrv->mmio_base;
      DFBColor       color = state->color;
 
      if (MACH64_IS_VALID( m_color_3d ))
@@ -385,7 +406,7 @@ void mach64_set_color_tex( Mach64DriverData *mdrv,
                            Mach64DeviceData *mdev,
                            CardState        *state )
 {
-     volatile __u8 *mmio  = mdrv->mmio_base;
+     volatile u8 *mmio    = mdrv->mmio_base;
      DFBColor       color = state->color;
 
      if (MACH64_IS_VALID( m_color_tex ))
@@ -419,7 +440,7 @@ void mach64_set_src_colorkey( Mach64DriverData *mdrv,
                               Mach64DeviceData *mdev,
                               CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      if (MACH64_IS_VALID( m_srckey ))
           return;
@@ -438,8 +459,8 @@ void mach64_set_src_colorkey_scale( Mach64DriverData *mdrv,
                                     Mach64DeviceData *mdev,
                                     CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
-     __u32          clr, msk;
+     volatile u8 *mmio = mdrv->mmio_base;
+     u32          clr, msk;
 
      if (MACH64_IS_VALID( m_srckey_scale ))
           return;
@@ -452,12 +473,14 @@ void mach64_set_src_colorkey_scale( Mach64DriverData *mdrv,
                           ((state->src_colorkey & 0x03) <<  6);
                     msk = 0xE0E0C0;
                     break;
+               case DSPF_RGB444:
                case DSPF_ARGB4444:
                     clr = ((state->src_colorkey & 0x0F00) << 12) |
                           ((state->src_colorkey & 0x00F0) <<  8) |
                           ((state->src_colorkey & 0x000F) <<  4);
                     msk = 0xF0F0F0;
                     break;
+               case DSPF_RGB555:
                case DSPF_ARGB1555:
                     clr = ((state->src_colorkey & 0x7C00) << 9) |
                           ((state->src_colorkey & 0x03E0) << 6) |
@@ -497,7 +520,7 @@ void mach64_set_dst_colorkey( Mach64DriverData *mdrv,
                               Mach64DeviceData *mdev,
                               CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      if (MACH64_IS_VALID( m_dstkey ))
           return;
@@ -515,7 +538,7 @@ void mach64_set_dst_colorkey( Mach64DriverData *mdrv,
 void mach64_disable_colorkey( Mach64DriverData *mdrv,
                               Mach64DeviceData *mdev )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      if (MACH64_IS_VALID( m_disable_key ))
           return;
@@ -527,7 +550,7 @@ void mach64_disable_colorkey( Mach64DriverData *mdrv,
      MACH64_INVALIDATE( m_srckey | m_srckey_scale | m_dstkey );
 }
 
-static __u32 mach64SourceBlend[] = {
+static u32 mach64SourceBlend[] = {
      ALPHA_BLND_SRC_ZERO,
      ALPHA_BLND_SRC_ONE,
      0,
@@ -541,7 +564,7 @@ static __u32 mach64SourceBlend[] = {
      ALPHA_BLND_SAT
 };
 
-static __u32 mach64DestBlend[] = {
+static u32 mach64DestBlend[] = {
      ALPHA_BLND_DST_ZERO,
      ALPHA_BLND_DST_ONE,
      ALPHA_BLND_DST_SRCCOLOR,
@@ -559,7 +582,7 @@ void mach64_set_draw_blend( Mach64DriverData *mdrv,
                             Mach64DeviceData *mdev,
                             CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      if (MACH64_IS_VALID( m_draw_blend ))
           return;
@@ -582,7 +605,7 @@ void mach64_set_blit_blend( Mach64DriverData *mdrv,
                             Mach64DeviceData *mdev,
                             CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      if (MACH64_IS_VALID( m_blit_blend ))
           return;
@@ -590,6 +613,12 @@ void mach64_set_blit_blend( Mach64DriverData *mdrv,
      mdev->blit_blend &= SCALE_PIX_EXPAND | DITHER_EN;
 
      if (state->blittingflags & (DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA)) {
+          /* Disable dithering because it is applied even when
+           * the source pixels are completely transparent.
+           */
+          if (DFB_PIXELFORMAT_HAS_ALPHA( state->source->format ))
+               mdev->blit_blend &= ~DITHER_EN;
+
           mdev->blit_blend |= ALPHA_FOG_EN_ALPHA |
                               mach64SourceBlend[state->src_blend - 1] |
                               mach64DestBlend  [state->dst_blend - 1];
