@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -58,7 +59,10 @@ typedef enum {
      m_srckey       = 0x0200,
 
      m_drawBlend    = 0x1000,
-     m_blitBlend    = 0x2000
+     m_blitBlend    = 0x2000,
+
+     m_destination  = 0x4000,
+     m_clip         = 0x8000,
 } MatroxStateBits;
 
 #define MGA_VALIDATE(b)       (mdev->valid |= (b))
@@ -82,30 +86,31 @@ typedef struct {
      unsigned int fifo_cache_hits;
 
      /* ATYPE_BLK or ATYPE_RSTR, depending on SGRAM setting */
-     __u32 atype_blk_rstr;
+     u32 atype_blk_rstr;
 
      /* State handling */
      MatroxStateBits valid;
 
      /* Stored values */
      int dst_pitch;
-     int dst_offset[3];
+     int dst_offset[2][3];
      int src_pitch;
-     int src_offset[3];
+     int src_offset[2][3];
      int w, h, w2, h2;
-     __u32 color[3];
+     u32 color[3];
 
      bool draw_blend;
      bool blit_src_colorkey;
 
      bool blit_deinterlace;
+     bool blit_fields;
      int field;
 
      bool depth_buffer;
 
-     __u32 texctl;
+     u32 texctl;
 
-     __u32 idle_status;
+     u32 idle_status;
 
      DFBRegion clip;
 
@@ -115,12 +120,14 @@ typedef struct {
      } fb;
      unsigned int tlut_offset;
      CorePalette *rgb332_palette;
+
+     bool crtc2_separated;
 } MatroxDeviceData;
 
 typedef struct {
      int            accelerator;
      int            maven_fd;
-     volatile __u8 *mmio_base;
+     volatile u8   *mmio_base;
 
      CoreScreen    *primary;
      CoreScreen    *secondary;
