@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,33 +26,21 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
+#include <config.h>
 
-#include <sys/mman.h>
-#include <fcntl.h>
-#include <sys/ioctl.h>
+#include <stdio.h>
 
 #include <directfb.h>
 
-#include <fusion/shmalloc.h>
+#include <direct/messages.h>
 
 #include <core/coredefs.h>
 #include <core/coretypes.h>
-#include <core/gfxcard.h>
 #include <core/layers.h>
 #include <core/screen.h>
 #include <core/surfaces.h>
-#include <core/windows.h>
-
-#include <direct/mem.h>
-#include <direct/messages.h>
 
 #include <misc/conf.h>
-#include <misc/util.h>
-
-#include <fbdev/fbdev.h>
 
 #include "regs.h"
 #include "mmio.h"
@@ -66,20 +55,20 @@ typedef struct {
      /* Stored registers */
      struct {
           /* CRTC2 */
-          __u32 c2CTL;
-          __u32 c2DATACTL;
-          __u32 c2MISC;
-          __u32 c2OFFSET;
+          u32 c2CTL;
+          u32 c2DATACTL;
+          u32 c2MISC;
+          u32 c2OFFSET;
 
-          __u32 c2HPARAM;
-          __u32 c2VPARAM;
+          u32 c2HPARAM;
+          u32 c2VPARAM;
 
-          __u32 c2STARTADD0;
-          __u32 c2STARTADD1;
-          __u32 c2PL2STARTADD0;
-          __u32 c2PL2STARTADD1;
-          __u32 c2PL3STARTADD0;
-          __u32 c2PL3STARTADD1;
+          u32 c2STARTADD0;
+          u32 c2STARTADD1;
+          u32 c2PL2STARTADD0;
+          u32 c2PL2STARTADD1;
+          u32 c2PL3STARTADD0;
+          u32 c2PL3STARTADD1;
      } regs;
 
      MatroxMavenData mav;
@@ -117,6 +106,47 @@ crtc2LayerDataSize()
      return sizeof(MatroxCrtc2LayerData);
 }
 
+static const DFBColorAdjustment adjustments[2][2] = {
+     /* G400 */
+     {
+          /* PAL / PAL-60 */
+          {
+               flags:      DCAF_BRIGHTNESS | DCAF_CONTRAST | DCAF_HUE | DCAF_SATURATION,
+               brightness: 0xA800,
+               saturation: 0x9500,
+               contrast:   0xFF00,
+               hue:        0x0000,
+          },
+          /* NTSC */
+          {
+               flags:      DCAF_BRIGHTNESS | DCAF_CONTRAST | DCAF_HUE | DCAF_SATURATION,
+               brightness: 0xB500,
+               saturation: 0x8E00,
+               contrast:   0xEA00,
+               hue:        0x0000,
+          }
+     },
+     /* G450 / G550 */
+     {
+          /* PAL / PAL-60 */
+          {
+               flags:      DCAF_BRIGHTNESS | DCAF_CONTRAST | DCAF_HUE | DCAF_SATURATION,
+               brightness: 0x9E00,
+               saturation: 0xBB00,
+               contrast:   0xFF00,
+               hue:        0x0000,
+          },
+          /* NTSC */
+          {
+               flags:      DCAF_BRIGHTNESS | DCAF_CONTRAST | DCAF_HUE | DCAF_SATURATION,
+               brightness: 0xAA00,
+               saturation: 0xAE00,
+               contrast:   0xEA00,
+               hue:        0x0000,
+          }
+     }
+};
+
 static DFBResult
 crtc2InitLayer( CoreLayer                  *layer,
                 void                       *driver_data,
@@ -129,7 +159,6 @@ crtc2InitLayer( CoreLayer                  *layer,
      MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
      MatroxDeviceData     *mdev   = mdrv->device_data;
      MatroxMavenData      *mav    = &mcrtc2->mav;
-     bool                  ntsc   = dfb_config->matrox_ntsc;
      DFBResult             res;
 
      if ((res = maven_init( mav, mdrv )) != DFB_OK)
@@ -150,7 +179,7 @@ crtc2InitLayer( CoreLayer                  *layer,
                             DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE |
                             DLCONF_OPTIONS | DLCONF_SURFACE_CAPS;
      config->width        = 720;
-     config->height       = ntsc ? 480 : 576;
+     config->height       = (dfb_config->matrox_tv_std != DSETV_PAL) ? 480 : 576;
      config->pixelformat  = DSPF_YUY2;
      config->buffermode   = DLBM_FRONTONLY;
      config->options      = DLOP_NONE;
@@ -158,17 +187,7 @@ crtc2InitLayer( CoreLayer                  *layer,
 
      /* fill out default color adjustment,
         only fields set in flags will be accepted from applications */
-     adjustment->flags = DCAF_BRIGHTNESS | DCAF_CONTRAST |
-                         DCAF_HUE | DCAF_SATURATION;
-     if (mdev->g450_matrox) {
-          adjustment->brightness = ntsc ? 0xAA00 : 0x9E00;
-          adjustment->saturation = ntsc ? 0xAE00 : 0xBB00;
-     } else {
-          adjustment->brightness = ntsc ? 0xB500 : 0xA800;
-          adjustment->saturation = ntsc ? 0x8E00 : 0x9500;
-     }
-     adjustment->contrast = ntsc ? 0xEA00 : 0xFF00;
-     adjustment->hue      = 0x0000;
+     *adjustment = adjustments[mdev->g450_matrox][dfb_config->matrox_tv_std == DSETV_NTSC];
 
      /* remember color adjustment */
      mcrtc2->adj = *adjustment;
@@ -188,12 +207,13 @@ crtc2TestRegion( CoreLayer                  *layer,
      if (config->options & ~CRTC2_SUPPORTED_OPTIONS)
           fail |= CLRCF_OPTIONS;
 
-     if (config->surface_caps & ~DSCAPS_INTERLACED)
+     if (config->surface_caps & ~(DSCAPS_INTERLACED | DSCAPS_SEPARATED))
           fail |= CLRCF_SURFACE_CAPS;
 
      switch (config->format) {
           case DSPF_ARGB:
           case DSPF_RGB32:
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_YUY2:
@@ -208,7 +228,7 @@ crtc2TestRegion( CoreLayer                  *layer,
      if (config->width != 720)
           fail |= CLRCF_WIDTH;
 
-     if (config->height != (dfb_config->matrox_ntsc ? 480 : 576))
+     if (config->height != ((dfb_config->matrox_tv_std != DSETV_PAL) ? 480 : 576))
           fail |= CLRCF_HEIGHT;
 
      if (failed)
@@ -243,6 +263,7 @@ crtc2SetRegion( CoreLayer                  *layer,
      DFBResult             ret;
      MatroxDriverData     *mdrv   = (MatroxDriverData*) driver_data;
      MatroxCrtc2LayerData *mcrtc2 = (MatroxCrtc2LayerData*) layer_data;
+     MatroxDeviceData     *mdev   = mdrv->device_data;
 
      /* remember configuration */
      mcrtc2->config = *config;
@@ -258,6 +279,8 @@ crtc2SetRegion( CoreLayer                  *layer,
           ret = crtc2_enable_output( mdrv, mcrtc2 );
           if (ret)
                return ret;
+
+          mdev->crtc2_separated = !!(surface->caps & DSCAPS_SEPARATED);
      }
 
      return DFB_OK;
@@ -287,7 +310,7 @@ crtc2FlipRegion( CoreLayer           *layer,
 {
      MatroxDriverData     *mdrv    = (MatroxDriverData*) driver_data;
      MatroxCrtc2LayerData *mcrtc2  = (MatroxCrtc2LayerData*) layer_data;
-     volatile __u8        *mmio    = mdrv->mmio_base;
+     volatile u8          *mmio    = mdrv->mmio_base;
 
      crtc2_calc_buffer( mdrv, mcrtc2, surface, false );
 
@@ -379,7 +402,7 @@ DisplayLayerFuncs matroxCrtc2Funcs = {
 static void crtc2_set_regs( MatroxDriverData     *mdrv,
                             MatroxCrtc2LayerData *mcrtc2 )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      mga_out32( mmio, mcrtc2->regs.c2CTL,     C2CTL );
      mga_out32( mmio, mcrtc2->regs.c2DATACTL, C2DATACTL );
@@ -421,6 +444,7 @@ static void crtc2_calc_regs( MatroxDriverData      *mdrv,
      mcrtc2->regs.c2CTL |= 1 << 8;
 
      switch (surface->format) {
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_RGB32:
@@ -431,7 +455,7 @@ static void crtc2_calc_regs( MatroxDriverData      *mdrv,
                break;
      }
 
-     if (dfb_config->matrox_ntsc)
+     if (dfb_config->matrox_tv_std != DSETV_PAL)
           mcrtc2->regs.c2DATACTL |= C2NTSCEN;
 
      /* pixel format settings */
@@ -449,6 +473,7 @@ static void crtc2_calc_regs( MatroxDriverData      *mdrv,
                mcrtc2->regs.c2CTL |= C2DEPTH_YCBCR422;
                break;
 
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                mcrtc2->regs.c2CTL |= C2DEPTH_15BPP;
                break;
@@ -470,13 +495,14 @@ static void crtc2_calc_regs( MatroxDriverData      *mdrv,
      if (!(surface->caps & DSCAPS_INTERLACED))
           mcrtc2->regs.c2CTL |= C2VCBCRSINGLE;
 
-     /* interleaved fields */
-     mcrtc2->regs.c2OFFSET = surface->front_buffer->video.pitch * 2;
+     mcrtc2->regs.c2OFFSET = surface->front_buffer->video.pitch;
+     if (!(surface->caps & DSCAPS_SEPARATED))
+          mcrtc2->regs.c2OFFSET *= 2;
 
      {
           int hdisplay, htotal, vdisplay, vtotal;
 
-          if (dfb_config->matrox_ntsc) {
+          if (dfb_config->matrox_tv_std != DSETV_PAL) {
                hdisplay = 720;
                htotal = 858;
                vdisplay = 480 / 2;
@@ -511,14 +537,20 @@ static void crtc2_calc_buffer( MatroxDriverData     *mdrv,
      SurfaceBuffer *buffer       = front ? surface->front_buffer : surface->back_buffer;
      int            field_offset = buffer->video.pitch;
 
+     if (surface->caps & DSCAPS_SEPARATED)
+          field_offset *= surface->height / 2;
+
      mcrtc2->regs.c2STARTADD1 = buffer->video.offset;
      mcrtc2->regs.c2STARTADD0 = mcrtc2->regs.c2STARTADD1 + field_offset;
 
      if (surface->caps & DSCAPS_INTERLACED)
-          field_offset /= 2;
+          field_offset = buffer->video.pitch / 2;
      else
           field_offset = 0;
 
+     if (surface->caps & DSCAPS_SEPARATED)
+          field_offset *= surface->height / 4;
+
      switch (surface->format) {
           case DSPF_I420:
                mcrtc2->regs.c2PL2STARTADD1 = mcrtc2->regs.c2STARTADD1 + surface->height * buffer->video.pitch;
@@ -542,7 +574,7 @@ static void crtc2_calc_buffer( MatroxDriverData     *mdrv,
 static void crtc2_set_buffer( MatroxDriverData     *mdrv,
                               MatroxCrtc2LayerData *mcrtc2 )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      mga_out32( mmio, mcrtc2->regs.c2STARTADD0, C2STARTADD0 );
      mga_out32( mmio, mcrtc2->regs.c2STARTADD1, C2STARTADD1 );
@@ -557,7 +589,7 @@ crtc2OnOff( MatroxDriverData     *mdrv,
             MatroxCrtc2LayerData *mcrtc2,
             int                   on )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      if (on)
           mcrtc2->regs.c2CTL |= C2EN;
@@ -580,8 +612,8 @@ crtc2OnOff( MatroxDriverData     *mdrv,
 static void crtc2_set_mafc( MatroxDriverData     *mdrv,
                             int                   on )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
-     __u8           val;
+     volatile u8   *mmio = mdrv->mmio_base;
+     u8             val;
 
      val = mga_in_dac( mmio, XMISCCTRL );
      if (on) {
@@ -613,8 +645,8 @@ crtc2_disable_output( MatroxDriverData     *mdrv,
      maven_close( mav, mdrv );
 
      if (mdev->g450_matrox) {
-          volatile __u8 *mmio = mdrv->mmio_base;
-          __u8           val;
+          volatile u8   *mmio = mdrv->mmio_base;
+          u8             val;
 
           /* Set Rset to 0.7 V */
           val = mga_in_dac( mmio, XGENIOCTRL );
@@ -643,15 +675,15 @@ crtc2_enable_output( MatroxDriverData      *mdrv,
 {
      MatroxDeviceData *mdev = mdrv->device_data;
      MatroxMavenData  *mav  = &mcrtc2->mav;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
      DFBResult         res;
 
      if ((res = maven_open( mav, mdrv )) != DFB_OK)
           return res;
 
      if (mdev->g450_matrox) {
-          volatile __u8 *mmio = mdrv->mmio_base;
-          __u8           val;
+          volatile u8   *mmio = mdrv->mmio_base;
+          u8             val;
 
           /* Set Rset to 1.0 V */
           val = mga_in_dac( mmio, XGENIOCTRL );
