@@ -21,6 +21,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -29,64 +31,117 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
+#include <errno.h>
+#include <dirent.h>
+
+#include <linux/i2c.h>
 #include <linux/i2c-dev.h>
 
-#include <misc/memcpy.h>
-#include <misc/util.h>
+#include <directfb.h>
+
+#include <direct/memcpy.h>
+#include <direct/messages.h>
+#include <direct/util.h>
+
+#include <misc/conf.h>
 
 #include "matrox.h"
 #include "regs.h"
 #include "mmio.h"
 #include "matrox_maven.h"
 
-#ifndef min
-#define min(a,b) (a < b ? a : b)
-#endif
-#ifndef max
-#define max(a,b) (a > b ? a : b)
-#endif
-
-#define MAVEN_I2CID	(0x1B)
+#define SYS_CLASS_I2C_DEV "/sys/class/i2c-dev"
 
 static void
 maven_write_byte( MatroxMavenData  *mav,
                   MatroxDriverData *mdrv,
-                  __u8              reg,
-                  __u8              val )
+                  u8                reg,
+                  u8                val )
 {
-     if (mav->g450) {
-          volatile __u8 *mmio = mdrv->mmio_base;
+     MatroxDeviceData *mdev = mdrv->device_data;
+
+     if (mdev->g450_matrox) {
+          volatile u8 *mmio = mdrv->mmio_base;
 
           mga_out_dac( mmio, 0x87, reg );
           mga_out_dac( mmio, 0x88, val );
-     } else
-          i2c_smbus_write_byte_data( mdrv->maven_fd, reg, val );
+     } else {
+          union  i2c_smbus_data       data;
+          struct i2c_smbus_ioctl_data args;
+
+          data.byte = val;
+
+          args.read_write = I2C_SMBUS_WRITE;
+          args.command    = reg;
+          args.size       = I2C_SMBUS_BYTE_DATA;
+          args.data       = &data;
+
+          ioctl( mdrv->maven_fd, I2C_SMBUS, &args );
+     }
 }
 
 static void
 maven_write_word( MatroxMavenData  *mav,
                   MatroxDriverData *mdrv,
-                  __u8              reg,
-                  __u16             val )
+                  u8                reg,
+                  u16               val )
 {
-     if (mav->g450) {
-          volatile __u8 *mmio = mdrv->mmio_base;
+     MatroxDeviceData *mdev = mdrv->device_data;
+
+     if (mdev->g450_matrox) {
+          volatile u8 *mmio = mdrv->mmio_base;
 
           mga_out_dac( mmio, 0x87, reg );
           mga_out_dac( mmio, 0x88, val );
           mga_out_dac( mmio, 0x87, reg + 1 );
-          mga_out_dac( mmio, 0x88, val >> 8);
-     } else
-          i2c_smbus_write_word_data( mdrv->maven_fd, reg, val );
+          mga_out_dac( mmio, 0x88, val >> 8 );
+     } else {
+          union  i2c_smbus_data       data;
+          struct i2c_smbus_ioctl_data args;
+
+          data.word = val;
+
+          args.read_write = I2C_SMBUS_WRITE;
+          args.command    = reg;
+          args.size       = I2C_SMBUS_WORD_DATA;
+          args.data       = &data;
+
+          ioctl( mdrv->maven_fd, I2C_SMBUS, &args );
+     }
 }
 
+#if 0
+/* i2c_smbus_read_byte_data() doesn't work with maven. */
+static int
+i2c_read_byte( int fd, u8 addr, u8 reg )
+{
+     int ret;
+     u8 val;
+     struct i2c_msg msgs[] = {
+          { addr, I2C_M_REV_DIR_ADDR, sizeof(reg), &reg },
+          { addr, I2C_M_RD | I2C_M_NOSTART, sizeof(val), &val }
+     };
+     struct i2c_rdwr_ioctl_data data = {
+          msgs, 2
+     };
+
+     ret = ioctl( fd, I2C_RDWR, &data );
+     if (ret < 0)
+          return ret;
+
+     return val;
+}
+#endif
+
 void
 maven_disable( MatroxMavenData  *mav,
                MatroxDriverData *mdrv )
 {
+     MatroxDeviceData *mdev = mdrv->device_data;
+
      maven_write_byte( mav, mdrv, 0x3E, 0x01 );
 
-     if (mav->g450) {
+     if (mdev->g450_matrox) {
           maven_write_byte( mav, mdrv, 0x80, 0x00 );
           return;
      }
@@ -102,15 +157,16 @@ void
 maven_enable( MatroxMavenData  *mav,
               MatroxDriverData *mdrv )
 {
-     if (mav->g450) {
+     MatroxDeviceData *mdev = mdrv->device_data;
+     bool              ntsc = dfb_config->matrox_tv_std != DSETV_PAL;
+
+     if (mdev->g450_matrox) {
           if (dfb_config->matrox_cable == 1)
                /* SCART RGB */
-               maven_write_byte( mav, mdrv, 0x80,
-                                 dfb_config->matrox_ntsc ? 0x43 : 0x41 );
+               maven_write_byte( mav, mdrv, 0x80, ntsc ? 0x43 : 0x41 );
           else
                /* Composite / S-Video */
-               maven_write_byte( mav, mdrv, 0x80,
-                                 dfb_config->matrox_ntsc ? 0x03 : 0x01 );
+               maven_write_byte( mav, mdrv, 0x80, ntsc ? 0x03 : 0x01 );
      }
      else
           maven_write_byte( mav, mdrv, 0x82, 0x20 );
@@ -122,7 +178,9 @@ void
 maven_sync( MatroxMavenData  *mav,
             MatroxDriverData *mdrv )
 {
-     if (mav->g450)
+     MatroxDeviceData *mdev = mdrv->device_data;
+
+     if (mdev->g450_matrox)
           return;
 
      maven_write_byte( mav, mdrv, 0xD4, 0x01 );
@@ -135,9 +193,12 @@ maven_sync( MatroxMavenData  *mav,
 void
 maven_set_regs( MatroxMavenData       *mav,
                 MatroxDriverData      *mdrv,
-                DFBDisplayLayerConfig *config,
+                CoreLayerRegionConfig *config,
                 DFBColorAdjustment    *adj )
 {
+     MatroxDeviceData *mdev = mdrv->device_data;
+     bool              ntsc = dfb_config->matrox_tv_std != DSETV_PAL;
+
      LR(0x00);
      LR(0x01);
      LR(0x02);
@@ -189,9 +250,8 @@ maven_set_regs( MatroxMavenData       *mav,
      LR(0x37);
      LR(0x38);
 
-     if (mav->g450) {
-          maven_write_word( mav, mdrv, 0x82,
-                            dfb_config->matrox_ntsc ? 0x0014 : 0x0017 );
+     if (mdev->g450_matrox) {
+          maven_write_word( mav, mdrv, 0x82, ntsc ? 0x0014 : 0x0017 );
           maven_write_word( mav, mdrv, 0x84, 0x0001 );
      } else {
           maven_write_byte( mav, mdrv, 0xB3, 0x01 );
@@ -199,13 +259,18 @@ maven_set_regs( MatroxMavenData       *mav,
           maven_write_byte( mav, mdrv, 0xD3, 0x01 );
           maven_write_byte( mav, mdrv, 0x8C, 0x10 );
           maven_write_byte( mav, mdrv, 0x94, 0xA2 );
-          maven_write_byte( mav, mdrv, 0x8D, 0x03);
-          maven_write_byte( mav, mdrv, 0xB9, 0x78);
-          maven_write_byte( mav, mdrv, 0xBF, 0x02);
+          maven_write_byte( mav, mdrv, 0x8D, 0x03 );
+          maven_write_byte( mav, mdrv, 0xB9, 0x78 );
+          maven_write_byte( mav, mdrv, 0xBF, 0x02 );
 
-          /*  deflicker: 0x00, 0xB1, 0xA2 */
-          maven_write_byte( mav, mdrv, 0x93,
-                            config->flags & DLOP_FLICKER_FILTERING ? 0xA2 : 0x00 );
+          /*
+           * Deflicker: 0x00, 0xB1, 0xA2
+           * Doesn't work due to:
+           * - ITU-R BT.656 mode?
+           * - scaler is not used?
+           * - something else?
+           */
+          maven_write_byte( mav, mdrv, 0x93, 0x00 );
      }
 
      maven_set_saturation( mav, mdrv, adj->saturation >> 8 );
@@ -213,7 +278,7 @@ maven_set_regs( MatroxMavenData       *mav,
      maven_set_bwlevel( mav, mdrv, adj->brightness >> 8,
                         adj->contrast >> 8 );
 
-     if (!mav->g450) {
+     if (!mdev->g450_matrox) {
           LR(0x83);
           LR(0x84);
           LR(0x85);
@@ -244,7 +309,7 @@ maven_set_regs( MatroxMavenData       *mav,
 void
 maven_set_hue( MatroxMavenData  *mav,
                MatroxDriverData *mdrv,
-               __u8              hue )
+               u8                hue )
 {
      maven_write_byte( mav, mdrv, 0x25, hue );
 }
@@ -252,7 +317,7 @@ maven_set_hue( MatroxMavenData  *mav,
 void
 maven_set_saturation( MatroxMavenData  *mav,
                       MatroxDriverData *mdrv,
-                      __u8              saturation )
+                      u8                saturation )
 {
      maven_write_byte( mav, mdrv, 0x20, saturation );
      maven_write_byte( mav, mdrv, 0x22, saturation );
@@ -261,15 +326,16 @@ maven_set_saturation( MatroxMavenData  *mav,
 void
 maven_set_bwlevel( MatroxMavenData  *mav,
                    MatroxDriverData *mdrv,
-                   __u8              brightness,
-                   __u8              contrast )
+                   u8                brightness,
+                   u8                contrast )
 {
+     MatroxDeviceData *mdev = mdrv->device_data;
      int b, c, bl, wl, wlmax, blmin, range;
-     bool ntsc = dfb_config->matrox_ntsc;
+     bool ntsc = dfb_config->matrox_tv_std == DSETV_NTSC;
 
-     if (mav->g450) {
-          wlmax = ntsc ? 869 : 881;
-          blmin = ntsc ? 200 : 224;
+     if (mdev->g450_matrox) {
+          wlmax = ntsc ? 936 : 938;
+          blmin = ntsc ? 267 : 281;
      } else {
           wlmax = 786;
           blmin = ntsc ? 242 : 255;
@@ -279,8 +345,8 @@ maven_set_bwlevel( MatroxMavenData  *mav,
      b = brightness * range / 255 + blmin;
      c = contrast * range / 2 / 255 + 64;
 
-     bl = max( b - c, blmin );
-     wl = min( b + c, wlmax );
+     bl = MAX( b - c, blmin );
+     wl = MIN( b + c, wlmax );
 
      blmin = ((blmin << 8) & 0x0300) | ((blmin >> 2) & 0x00FF);
      bl    = ((bl    << 8) & 0x0300) | ((bl    >> 2) & 0x00FF);
@@ -295,25 +361,27 @@ DFBResult
 maven_open( MatroxMavenData  *mav,
             MatroxDriverData *mdrv )
 {
-     if (mav->g450)
+     MatroxDeviceData *mdev = mdrv->device_data;
+
+     if (mdev->g450_matrox)
           return DFB_OK;
 
      if (mdrv->maven_fd != -1)
-          BUG( "DirectFB/Matrox/Maven: Device already open!\n" );
+          D_BUG( "DirectFB/Matrox/Maven: Device already open!\n" );
 
      if ((mdrv->maven_fd = open( mav->dev, O_RDWR )) < 0) {
-          PERRORMSG( "DirectFB/Matrox/Maven: Error opening `%s'!\n",
+          D_PERROR( "DirectFB/Matrox/Maven: Error opening `%s'!\n",
                      mav->dev );
           mdrv->maven_fd = -1;
-          return errno2dfb( errno );
+          return errno2result( errno );
      }
 
-     if (ioctl( mdrv->maven_fd, I2C_SLAVE, MAVEN_I2CID ) < 0) {
-          PERRORMSG( "DirectFB/Matrox/Maven: Error controlling `%s'!\n",
+     if (ioctl( mdrv->maven_fd, I2C_SLAVE, mav->address ) < 0) {
+          D_PERROR( "DirectFB/Matrox/Maven: Error controlling `%s'!\n",
                      mav->dev );
           close( mdrv->maven_fd );
           mdrv->maven_fd = -1;
-          return errno2dfb( errno );
+          return errno2result( errno );
      }
 
      return DFB_OK;
@@ -323,11 +391,13 @@ void
 maven_close( MatroxMavenData  *mav,
              MatroxDriverData *mdrv )
 {
-     if (mav->g450)
+     MatroxDeviceData *mdev = mdrv->device_data;
+
+     if (mdev->g450_matrox)
           return;
 
      if (mdrv->maven_fd == -1)
-          BUG( "DirectFB/Matrox/Maven: Device not open!\n" );
+          D_BUG( "DirectFB/Matrox/Maven: Device not open!\n" );
 
      close( mdrv->maven_fd );
      mdrv->maven_fd = -1;
@@ -336,89 +406,65 @@ maven_close( MatroxMavenData  *mav,
 DFBResult maven_init( MatroxMavenData  *mav,
                       MatroxDriverData *mdrv )
 {
+     MatroxDeviceData *mdev = mdrv->device_data;
      char  line[512];
-     int   fd, found;
+     int   fd;
      FILE *file;
-
-     /* Determine chip type (G400 or G450/G550) */
-     {
-          unsigned int devfn, devid, rev;
-
-          found = 0;
-
-          file = fopen( "/proc/bus/pci/devices", "r" );
-          if (!file) {
-               PERRORMSG( "DirectFB/Matrox/Maven: "
-                          "Error opening `/proc/bus/pci/devices'!\n" );
-               return errno2dfb( errno );
-          }
-          while (fgets( line, 512, file )) {
-               sscanf( line,
-                       "%x %x %*x %*x %*x %*x"
-                       "%*x %*x %*x %*x %*x"
-                       "%*x %*x %*x %*x %*x"
-                       "%*x",
-                       &devfn, &devid );
-
-               /* vendor */
-               if ((devid >> 16) != 0x102B)
-                    continue;
-
-               /* device */
-               if ((devid & 0xFFFF) == 0x2527) {
-                    /* G550 */
-                    mav->g450 = 1;
-                    found = 1;
-                    break;
-               } else if ((devid & 0xFFFF) == 0x0525) {
-                    /* G400 or G450 -> check revision */
-                    snprintf( line, 512,
-                              "/proc/bus/pci/%02x/%02x.%x",
-                              devfn >> 8,
-                              (devfn >> 3) & 0x1F,
-                              devfn & 0x07 );
-
-                    fd = open( line, O_RDONLY );
-                    if (fd < 0)
-                         continue;
-
-                    if (lseek( fd, 0x08, SEEK_SET ) != 0x08) {
-                         close( fd );
-                         continue;
-                    }
-                    if (read( fd, &rev, 1 ) != 1) {
-                         close( fd );
-                         continue;
-                    }
-                    close( fd );
-
-                    if ((rev & 0xFF) >= 0x80)
-                         mav->g450 = 1;
-                    else
-                         mav->g450 = 0;
-
-                    found = 1;
-                    break;
-               }
-          }
-          fclose( file );
-
-          if (!found) {
-               ERRORMSG( "DirectFB/Matrox/Maven: "
-                         "Can't determine if chip is G400 or G450/G550!\n" );
-               return DFB_UNSUPPORTED;
-          }
-     }
+     bool  found = false;
+     DIR  *dir;
 
      /* Locate G400 maven /dev/i2c file */
-     if (!mav->g450) {
-          found = 0;
 
+     /* Try sysfs */
+     if (!mdev->g450_matrox && (dir = opendir( SYS_CLASS_I2C_DEV )) != NULL) {
+          char path[PATH_MAX];
+          struct dirent *ent;
+
+          while ((ent = readdir( dir )) != NULL) {
+               if (!strcmp( ent->d_name, "." ))
+                    continue;
+               if (!strcmp( ent->d_name, ".." ))
+                    continue;
+
+               snprintf( path, sizeof(path), "%s/%s/name", SYS_CLASS_I2C_DEV, ent->d_name );
+
+               FILE *fp = fopen( path, "r" );
+               if (!fp) {
+                    D_PERROR( "DirectFB/Matrox/Maven: Error opening `%s'!\n", path );
+                    continue;
+               }
+
+               memset( line, 0, 6 );
+
+               fread( line, 1, 5, fp );
+               if (ferror( fp )) {
+                    D_PERROR( "DirectFB/Matrox/Maven: Error reading `%s'!\n", path );
+                    fclose( fp );
+                    continue;
+               }
+
+               fclose( fp );
+
+               if (strcmp( line, "MAVEN" ))
+                    continue;
+
+               snprintf( mav->dev, sizeof(mav->dev), "/dev/%s", ent->d_name );
+               found = true;
+               break;
+          }
+          if (!ent && errno)
+               D_PERROR( "DirectFB/Matrox/Maven: Error reading `%s'!\n", SYS_CLASS_I2C_DEV );
+
+          closedir( dir );
+     }
+
+     /* Try /proc/bus/i2c */
+     if (!mdev->g450_matrox && !found) {
           file = fopen( "/proc/bus/i2c", "r" );
           if (!file) {
-               PERRORMSG( "DirectFB/Matrox/Maven: "
+               D_PERROR( "DirectFB/Matrox/Maven: "
                           "Error opening `/proc/bus/i2c'!\n" );
-               return errno2dfb( errno );
+               return errno2result( errno );
           }
           while (fgets( line, 512, file )) {
                if (strstr( line, "MAVEN" )) {
@@ -426,39 +472,53 @@ DFBResult maven_init( MatroxMavenData  *mav,
                     while (!isspace( *p ))
                          p++;
                     *p = '\0';
-                    strncpy( mav->dev, "/dev/", 6 );
+                    direct_snputs( mav->dev, "/dev/", 6 );
                     strncat( mav->dev, line, 250 );
-                    found = 1;
+                    found = true;
                     break;
                }
           }
           fclose( file );
+     }
 
+     if (!mdev->g450_matrox) {
           if (!found) {
-               ERRORMSG( "DirectFB/Matrox/Maven: "
-                         "Can't find MAVEN in /proc/bus/i2c!\n" );
+               D_ERROR( "DirectFB/Matrox/Maven: "
+                         "Can't find MAVEN i2c device file!\n" );
                return DFB_UNSUPPORTED;
           }
 
           /* Try to use it */
           if ((fd = open( mav->dev, O_RDWR )) < 0) {
-               PERRORMSG( "DirectFB/Matrox/Maven: Error opening `%s'!\n",
+               D_PERROR( "DirectFB/Matrox/Maven: Error opening `%s'!\n",
                           mav->dev );
-               return errno2dfb( errno );
+               return errno2result( errno );
           }
 
-          if (ioctl( fd, I2C_SLAVE, MAVEN_I2CID ) < 0) {
-               PERRORMSG( "DirectFB/Matrox/Maven: Error controlling `%s'!\n",
-                          mav->dev );
-               close( fd );
-               return errno2dfb( errno );
-          }
+#if 0
+          /* FIXME: This fails for some people */
+          /* Check if maven is at address 0x1B (DH board) or 0x1A (DH add-on) */
+          if (i2c_read_byte( fd, 0x1B, 0xB2 ) < 0) {
+               if (i2c_read_byte( fd, 0x1A, 0xB2 ) < 0) {
+                    D_ERROR( "DirectFB/Matrox/Maven: Error reading from maven chip!\n" );
+                    close( fd );
+                    return errno2result( errno );
+               } else
+                    mav->address = 0x1A;
+          } else
+               mav->address = 0x1B;
+#else
+          mav->address = 0x1B;
+#endif
+
           close( fd );
      }
 
      /* Maven registers */
      {
-          const __u8 ntscregs[64] = {
+          static const u8 ntscregs[2][0x40] = {
+          /* G400 */
+          {
                0x21, 0xF0, 0x7C, 0x1F, /* 00-03 */
                0x00, /* 04 */
                0x00,
@@ -511,8 +571,66 @@ DFBResult maven_init( MatroxMavenData  *mav,
                0x3C, 0x00, /* 3C-3D */
                0x00, /* 3E */
                0x00
+          },
+          /* G450 / G550 */
+          {
+               0x21, 0xF0, 0x7C, 0x1F, /* 00-03 */
+               0x00, /* 04 */
+               0x00,
+               0x00,
+               0x00,
+               0x7E, /* 08 */
+               0x44, /* 09 */
+               0x76, /* 0A */
+               0x49, /* 0B */
+               0x00, /* 0C */
+               0x00,
+               0x4E, 0x03, /* 0E-0F */
+               0x42, 0x03, /* 10-11 */
+               0x17, /* 12 */
+               0x21, /* 13 */
+               0x1B, /* 14 */
+               0x1B, /* 15 */
+               0x24, /* 16 */
+               0x83, 0x01, /* 17-18 */
+               0x00, /* 19 */
+               0x0F, /* 1A */
+               0x0F, /* 1B */
+               0x60, /* 1C */
+               0x05, /* 1D */
+               0xEA, 0x00, /* 1E-1F */
+               0xAE, /* 20 */
+               0x04, /* 21 */
+               0xAE, /* 22 */
+               0x01, /* 23 */
+               0x02, /* 24 */
+               0x00, /* 25 */
+               0x0A, /* 26 */
+               0x05, /* 27 */
+               0x00, /* 28 */
+               0x11, /* 29 */
+               0xFF, 0x03, /* 2A-2B */
+               0x20, /* 2C */
+               0x0F, 0x78, /* 2D-2E */
+               0x00, 0x00, /* 2F-30 */
+               0xB4, 0x00, /* 31-32 */
+               0x14, /* 33 */
+               0x02, /* 34 */
+               0x00, /* 35 */
+               0x00,
+               0xBD, /* 37 */
+               0xDA, /* 38 */
+               0x15, /* 39 */
+               0x05, /* 3A */
+               0x15, /* 3B */
+               0x42, 0x03, /* 3C-3D */
+               0x00, /* 3E */
+               0x00
+          }
           };
-          const __u8 palregs[64] = {
+          static const u8 palregs[2][0x40] = {
+          /* G400 */
+          {
                0x2A, 0x09, 0x8A, 0xCB, /* 00-03 */
                0x00, /* 04 */
                0x00,
@@ -565,58 +683,89 @@ DFBResult maven_init( MatroxMavenData  *mav,
                0x3F, 0x03, /* 3C-3D */
                0x00, /* 3E */
                0x00,
+          },
+          /* G450 / G550 */
+          {
+               0x2A, 0x09, 0x8A, 0xCB, /* 00-03 */
+               0x00, /* 04 */
+               0x00,
+               0x00,
+               0x00,
+               0x7E, /* 08 */
+               0x3A, /* 09 */
+               0x8A, /* 0A */
+               0x38, /* 0B */
+               0x28, /* 0C */
+               0x00,
+               0x46, 0x01, /* 0E-0F */
+               0x46, 0x01, /* 10-11 */
+               0x1A, /* 12 */
+               0x2A, /* 13 */
+               0x1C, /* 14 */
+               0x3D, /* 15 */
+               0x14, /* 16 */
+               0x9C, 0x01, /* 17-18 */
+               0x00, /* 19 */
+               0xFE, /* 1A */
+               0x7E, /* 1B */
+               0x60, /* 1C */
+               0x05, /* 1D */
+               0xEA, 0x00, /* 1E-1F */
+               0xBB, /* 20 */
+               0x07, /* 21 */
+               0xBB, /* 22 */
+               0x00, /* 23 */
+               0x00, /* 24 */
+               0x00, /* 25 */
+               0x08, /* 26 */
+               0x04, /* 27 */
+               0x00, /* 28 */
+               0x1A, /* 29 */
+               0x55, 0x01, /* 2A-2B */
+               0x18, /* 2C */
+               0x07, 0x7E, /* 2D-2E */
+               0x02, 0x54, /* 2F-30 */
+               0xB4, 0x00, /* 31-32 */
+               0x16, /* 33 */
+               0x49, /* 34 */
+               0x00, /* 35 */
+               0x00,
+               0xB9, /* 37 */
+               0xDD, /* 38 */
+               0x22, /* 39 */
+               0x02, /* 3A */
+               0x22, /* 3B */
+               0x46, 0x00, /* 3C-3D */
+               0x00, /* 3E */
+               0x00,
+          }
           };
 
-          if (dfb_config->matrox_ntsc)
-               dfb_memcpy( mav->regs, ntscregs, 64 );
+          if (dfb_config->matrox_tv_std != DSETV_PAL)
+               direct_memcpy( mav->regs, ntscregs[mdev->g450_matrox], 64 );
           else
-               dfb_memcpy( mav->regs, palregs, 64 );
+               direct_memcpy( mav->regs, palregs[mdev->g450_matrox], 64 );
 
-          if (mav->g450) {
-               if (dfb_config->matrox_ntsc) {
-                    mav->regs[0x09] = 0x44;
-                    mav->regs[0x0A] = 0x76;
-                    mav->regs[0x0B] = 0x49;
-                    mav->regs[0x0C] = 0x00;
-                    mav->regs[0x0E] = 0x3E;
-                    mav->regs[0x0F] = 0x00;
-                    mav->regs[0x10] = 0x32;
-                    mav->regs[0x11] = 0x00;
-                    mav->regs[0x1E] = 0xD9;
-                    mav->regs[0x1F] = 0x01;
-                    mav->regs[0x20] = 0xAE;
-                    mav->regs[0x22] = 0xAE;
-                    mav->regs[0x29] = 0x11;
-                    mav->regs[0x2C] = 0x20;
-                    mav->regs[0x33] = 0x14;
-                    mav->regs[0x35] = 0x00;
-                    mav->regs[0x37] = 0xBD;
-                    mav->regs[0x38] = 0xDA;
-                    mav->regs[0x3C] = 0x42;
-                    mav->regs[0x3D] = 0x03;
-               } else {
-                    mav->regs[0x09] = 0x3A;
-                    mav->regs[0x0A] = 0x8A;
-                    mav->regs[0x0B] = 0x38;
-                    mav->regs[0x0C] = 0x28;
-                    mav->regs[0x0E] = 0x38;
-                    mav->regs[0x0F] = 0x00;
-                    mav->regs[0x10] = 0x38;
-                    mav->regs[0x11] = 0x00;
-                    mav->regs[0x1E] = 0xDB;
-                    mav->regs[0x1F] = 0x03;
-                    mav->regs[0x20] = 0xBB;
-                    mav->regs[0x22] = 0xBB;
-                    mav->regs[0x29] = 0x1A;
-                    mav->regs[0x2C] = 0x18;
-                    mav->regs[0x33] = 0x16;
-                    mav->regs[0x35] = 0x00;
-                    mav->regs[0x37] = 0xB9;
-                    mav->regs[0x38] = 0xDD;
-                    mav->regs[0x3C] = 0x46;
-                    mav->regs[0x3D] = 0x00;
-               }
-          } else {
+          if (dfb_config->matrox_tv_std == DSETV_PAL_60) {
+               mav->regs[0x00] = palregs[mdev->g450_matrox][0x00];
+               mav->regs[0x01] = palregs[mdev->g450_matrox][0x01];
+               mav->regs[0x02] = palregs[mdev->g450_matrox][0x02];
+               mav->regs[0x03] = palregs[mdev->g450_matrox][0x03];
+               mav->regs[0x0B] = palregs[mdev->g450_matrox][0x0B];
+               mav->regs[0x0C] = palregs[mdev->g450_matrox][0x0C];
+               mav->regs[0x0E] = palregs[mdev->g450_matrox][0x0E];
+               mav->regs[0x0F] = palregs[mdev->g450_matrox][0x0F];
+               mav->regs[0x10] = palregs[mdev->g450_matrox][0x10];
+               mav->regs[0x11] = palregs[mdev->g450_matrox][0x11];
+               mav->regs[0x1E] = palregs[mdev->g450_matrox][0x1E];
+               mav->regs[0x1F] = palregs[mdev->g450_matrox][0x1F];
+               mav->regs[0x20] = palregs[mdev->g450_matrox][0x20];
+               mav->regs[0x22] = palregs[mdev->g450_matrox][0x22];
+               mav->regs[0x25] = palregs[mdev->g450_matrox][0x25];
+               mav->regs[0x34] = palregs[mdev->g450_matrox][0x34];
+          }
+
+          if (!mdev->g450_matrox) {
                /* gamma */
                mav->regs[0x83] = 0x00;
                mav->regs[0x84] = 0x00;
