@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,10 +26,11 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <directfb.h>
 
 #include <direct/messages.h>
-#include <direct/util.h>
 
 #include <core/coredefs.h>
 #include <core/coretypes.h>
@@ -39,7 +41,6 @@
 #include <core/palette.h>
 
 #include <gfx/convert.h>
-#include <misc/util.h>
 
 #include "regs.h"
 #include "mmio.h"
@@ -49,44 +50,119 @@
 
 #define MGA_KEYMASK(format)   ((1 << DFB_COLOR_BITS_PER_PIXEL(format)) - 1)
 
-void matrox_set_destination( MatroxDriverData *mdrv,
-                             MatroxDeviceData *mdev,
-                             CoreSurface      *destination )
+static void matrox_calc_offsets( MatroxDeviceData *mdev,
+                                 CoreSurface      *surface,
+                                 SurfaceBuffer    *buffer,
+                                 bool              unit_pixel,
+                                 int               offset[2][3] )
 {
-     volatile __u8 *mmio            = mdrv->mmio_base;
+     int bytes_per_pixel = DFB_BYTES_PER_PIXEL( surface->format );
+     int pitch;
+
+     if (unit_pixel) {
+          offset[0][0] = buffer->video.offset / bytes_per_pixel;
+          pitch        = buffer->video.pitch / bytes_per_pixel;
+     } else {
+          offset[0][0] = mdev->fb.offset + buffer->video.offset;
+          pitch        = buffer->video.pitch;
+     }
+
+     switch (surface->format) {
+     case DSPF_NV12:
+     case DSPF_NV21:
+          offset[0][1] = offset[0][0] + surface->height * pitch;
+          offset[0][2] = 0;
+          break;
+     case DSPF_I420:
+          offset[0][1] = offset[0][0] + surface->height * pitch;
+          offset[0][2] = offset[0][1] + surface->height/2 * pitch/2;
+          break;
+     case DSPF_YV12:
+          offset[0][2] = offset[0][0] + surface->height * pitch;
+          offset[0][1] = offset[0][2] + surface->height/2 * pitch/2;
+          break;
+     default:
+          offset[0][1] = 0;
+          offset[0][2] = 0;
+     }
+
+     D_ASSERT( offset[0][0] % 64 == 0 );
+     D_ASSERT( offset[0][1] % 64 == 0 );
+     D_ASSERT( offset[0][2] % 64 == 0 );
+
+     if (mdev->blit_fields || mdev->blit_deinterlace) {
+          if (surface->caps & DSCAPS_SEPARATED) {
+               offset[1][0] = offset[0][0] + surface->height/2 * pitch;
+               switch (surface->format) {
+               case DSPF_NV12:
+               case DSPF_NV21:
+                    offset[1][1] = offset[0][1] + surface->height/4 * pitch;
+                    offset[1][2] = 0;
+                    break;
+               case DSPF_I420:
+               case DSPF_YV12:
+                    offset[1][1] = offset[0][1] + surface->height/4 * pitch/2;
+                    offset[1][2] = offset[0][2] + surface->height/4 * pitch/2;
+                    break;
+               default:
+                    offset[1][1] = 0;
+                    offset[1][2] = 0;
+               }
+          } else {
+               offset[1][0] = offset[0][0] + pitch;
+               switch (surface->format) {
+               case DSPF_NV12:
+               case DSPF_NV21:
+                    offset[1][1] = offset[0][1] + pitch;
+                    offset[1][2] = 0;
+                    break;
+               case DSPF_I420:
+               case DSPF_YV12:
+                    offset[1][1] = offset[0][1] + pitch/2;
+                    offset[1][2] = offset[0][2] + pitch/2;
+                    break;
+               default:
+                    offset[1][1] = 0;
+                    offset[1][2] = 0;
+               }
+          }
+
+          D_ASSERT( offset[1][0] % 64 == 0 );
+          D_ASSERT( offset[1][1] % 64 == 0 );
+          D_ASSERT( offset[1][2] % 64 == 0 );
+     }
+}
+
+void matrox_validate_destination( MatroxDriverData *mdrv,
+                                  MatroxDeviceData *mdev,
+                                  CardState        *state )
+{
+     volatile u8   *mmio            = mdrv->mmio_base;
+     CoreSurface   *destination     = state->destination;
      SurfaceBuffer *buffer          = destination->back_buffer;
      SurfaceBuffer *depth_buffer    = destination->depth_buffer;
      int            bytes_per_pixel = DFB_BYTES_PER_PIXEL(buffer->format);
 
+     if (MGA_IS_VALID( m_destination ))
+          return;
+
      mdev->dst_pitch = buffer->video.pitch / bytes_per_pixel;
 
-     D_ASSERT( mdev->dst_pitch % 32 == 0 );
-
-     D_ASSERT( buffer->video.offset % 64 == 0 );
-
      mdev->depth_buffer = depth_buffer != NULL;
 
      if (destination->format == DSPF_YUY2 || destination->format == DSPF_UYVY)
           mdev->dst_pitch /= 2;
 
-     if (mdev->old_matrox) {
-          mdev->dst_offset[0] = buffer->video.offset / bytes_per_pixel;
-     } else {
-          mdev->dst_offset[0] = mdev->fb.offset + buffer->video.offset;
-          if (destination->format == DSPF_I420) {
-               mdev->dst_offset[1] = mdev->dst_offset[0] + destination->height * buffer->video.pitch;
-               mdev->dst_offset[2] = mdev->dst_offset[1] + destination->height * buffer->video.pitch / 4;
-          } else if (destination->format == DSPF_YV12) {
-               mdev->dst_offset[2] = mdev->dst_offset[0] + destination->height * buffer->video.pitch;
-               mdev->dst_offset[1] = mdev->dst_offset[2] + destination->height * buffer->video.pitch / 4;
-          } else if (destination->format == DSPF_NV12 || destination->format == DSPF_NV21) {
-               mdev->dst_offset[1] = mdev->dst_offset[0] + destination->height * buffer->video.pitch;
-          }
-     }
+     if (mdev->blit_fields && !(destination->caps & DSCAPS_SEPARATED))
+          mdev->dst_pitch *= 2;
+
+     D_ASSERT( mdev->dst_pitch % 32 == 0 );
+
+     matrox_calc_offsets( mdev, destination, buffer, mdev->old_matrox, mdev->dst_offset );
 
      mga_waitfifo( mdrv, mdev, depth_buffer ? 4 : 3 );
 
-     mga_out32( mmio, mdev->dst_offset[0], mdev->old_matrox ? YDSTORG : DSTORG );
+     mga_out32( mmio, mdev->dst_offset[0][0], mdev->old_matrox ? YDSTORG : DSTORG );
      mga_out32( mmio, mdev->dst_pitch, PITCH );
 
      if (depth_buffer)
@@ -99,9 +175,11 @@ void matrox_set_destination( MatroxDriverData *mdrv,
           case DSPF_RGB332:
                mga_out32( mmio, PW8, MACCESS );
                break;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                mga_out32( mmio, PW16 | DIT555, MACCESS );
                break;
+          case DSPF_RGB444:
           case DSPF_ARGB4444:
           case DSPF_RGB16:
                mga_out32( mmio, PW16, MACCESS );
@@ -127,20 +205,22 @@ void matrox_set_destination( MatroxDriverData *mdrv,
                D_BUG( "unexpected pixelformat!" );
                break;
      }
+
+     MGA_VALIDATE( m_destination );
 }
 
 void matrox_set_clip( MatroxDriverData *mdrv,
                       MatroxDeviceData *mdev,
                       DFBRegion        *clip )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
      mga_waitfifo( mdrv, mdev, 3 );
 
      if (mdev->old_matrox) {
-          mga_out32( mmio, (mdev->dst_offset[0] +
+          mga_out32( mmio, (mdev->dst_offset[0][0] +
                             mdev->dst_pitch * clip->y1) & 0xFFFFFF, YTOP );
-          mga_out32( mmio, (mdev->dst_offset[0] +
+          mga_out32( mmio, (mdev->dst_offset[0][0] +
                             mdev->dst_pitch * clip->y2) & 0xFFFFFF, YBOT );
      }
      else {
@@ -156,7 +236,7 @@ void matrox_validate_drawColor( MatroxDriverData *mdrv,
                                 CardState        *state )
 {
      DFBColor       color = state->color;
-     volatile __u8 *mmio  = mdrv->mmio_base;
+     volatile u8   *mmio  = mdrv->mmio_base;
 
      if (MGA_IS_VALID( m_drawColor ))
           return;
@@ -184,7 +264,7 @@ void matrox_validate_blitColor( MatroxDriverData *mdrv,
                                 CardState        *state )
 {
      DFBColor       color = state->color;
-     volatile __u8 *mmio  = mdrv->mmio_base;
+     volatile u8   *mmio  = mdrv->mmio_base;
 
      if (MGA_IS_VALID( m_blitColor ))
           return;
@@ -220,10 +300,10 @@ void matrox_validate_color( MatroxDriverData *mdrv,
                             CardState        *state )
 {
      DFBColor       color = state->color;
-     volatile __u8 *mmio  = mdrv->mmio_base;
+     volatile u8   *mmio  = mdrv->mmio_base;
 
-     __u32 fcol;
-     __u8  cb, cr;
+     u32 fcol;
+     u8  cb, cr;
 
      if (MGA_IS_VALID( m_color ))
           return;
@@ -252,6 +332,12 @@ void matrox_validate_color( MatroxDriverData *mdrv,
                fcol |= fcol << 8;
                fcol |= fcol << 16;
                break;
+          case DSPF_RGB444:
+               fcol = PIXEL_RGB444( color.r,
+                                    color.g,
+                                    color.b );
+               fcol |= fcol << 16;
+               break;
           case DSPF_ARGB4444:
                fcol = PIXEL_ARGB4444( color.a,
                                       color.r,
@@ -259,6 +345,12 @@ void matrox_validate_color( MatroxDriverData *mdrv,
                                       color.b );
                fcol |= fcol << 16;
                break;
+          case DSPF_RGB555:
+               fcol = PIXEL_RGB555( color.r,
+                                    color.g,
+                                    color.b );
+               fcol |= fcol << 16;
+               break;
           case DSPF_ARGB1555:
                fcol = PIXEL_ARGB1555( color.a,
                                       color.r,
@@ -352,7 +444,7 @@ void matrox_validate_color( MatroxDriverData *mdrv,
      MGA_INVALIDATE( m_srckey );
 }
 
-static __u32 matroxSourceBlend[] = {
+static u32 matroxSourceBlend[] = {
      SRC_ZERO,                /* DSBF_ZERO         */
      SRC_ONE,                 /* DSBF_ONE          */
      0,                       /* DSBF_SRCCOLOR     */
@@ -366,7 +458,7 @@ static __u32 matroxSourceBlend[] = {
      SRC_SRC_ALPHA_SATURATE   /* DSBF_SRCALPHASAT  */
 };
 
-static __u32 matroxDestBlend[] = {
+static u32 matroxDestBlend[] = {
      DST_ZERO,                /* DSBF_ZERO         */
      DST_ONE,                 /* DSBF_ONE          */
      DST_SRC_COLOR,           /* DSBF_SRCCOLOR     */
@@ -384,9 +476,9 @@ void matrox_validate_drawBlend( MatroxDriverData *mdrv,
                                 MatroxDeviceData *mdev,
                                 CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
-     __u32 alphactrl;
+     u32 alphactrl;
 
      if (MGA_IS_VALID( m_drawBlend ))
           return;
@@ -402,7 +494,7 @@ void matrox_validate_drawBlend( MatroxDriverData *mdrv,
      MGA_INVALIDATE( m_blitBlend );
 }
 
-static __u32 matroxAlphaSelect[] = {
+static u32 matroxAlphaSelect[] = {
      0,
      0,
      DIFFUSEDALPHA,
@@ -413,9 +505,9 @@ void matrox_validate_blitBlend( MatroxDriverData *mdrv,
                                 MatroxDeviceData *mdev,
                                 CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
-     __u32 alphactrl;
+     u32 alphactrl;
 
      if (MGA_IS_VALID( m_blitBlend ))
           return;
@@ -466,8 +558,8 @@ static void matrox_tlutload( MatroxDriverData *mdrv,
                              MatroxDeviceData *mdev,
                              CorePalette      *palette )
 {
-     volatile __u8  *mmio = mdrv->mmio_base;
-     volatile __u16 *dst  = dfb_gfxcard_memory_virtual( NULL, mdev->tlut_offset );
+     volatile u8  *mmio = mdrv->mmio_base;
+     volatile u16 *dst  = dfb_gfxcard_memory_virtual( NULL, mdev->tlut_offset );
      int             i;
 
      for (i = 0; i < palette->num_entries; i++)
@@ -475,7 +567,7 @@ static void matrox_tlutload( MatroxDriverData *mdrv,
                                 palette->entries[i].g,
                                 palette->entries[i].b );
 
-     mga_waitfifo( mdrv, mdev, 9 );
+     mga_waitfifo( mdrv, mdev, mdev->old_matrox ? 8 : 9 );
      mga_out32( mmio, BLTMOD_BU32RGB | BOP_COPY | SHFTZERO |
                 SGNZERO | LINEAR | ATYPE_RSTR | OP_BITBLT, DWGCTL );
      mga_out32( mmio, 1024, PITCH );
@@ -495,72 +587,49 @@ static void matrox_tlutload( MatroxDriverData *mdrv,
      mga_out32( mmio, 0, FXBNDRY );
      mga_out32( mmio, PW16 | TLUTLOAD, MACCESS );
      mga_out32( mmio, palette->num_entries, YDSTLEN | EXECUTE );
+
+     MGA_INVALIDATE( m_destination );
 }
 
 void matrox_validate_Source( MatroxDriverData *mdrv,
                              MatroxDeviceData *mdev,
                              CardState        *state )
 {
-     volatile __u8 *mmio            = mdrv->mmio_base;
+     volatile u8   *mmio            = mdrv->mmio_base;
      CoreSurface   *surface         = state->source;
      SurfaceBuffer *buffer          = surface->front_buffer;
      int            bytes_per_pixel = DFB_BYTES_PER_PIXEL(surface->format);
-     __u32          texctl = 0, texctl2 = 0;
+     u32            texctl = 0, texctl2 = 0;
 
      if (MGA_IS_VALID( m_Source ))
           return;
 
      mdev->src_pitch = buffer->video.pitch / bytes_per_pixel;
      mdev->field     = surface->field;
-
-     D_ASSERT( mdev->src_pitch % 32 == 0 );
-
-     D_ASSERT( buffer->video.offset % 32 == 0 );
-
-     mdev->src_offset[0] = mdev->fb.offset + buffer->video.offset;
-     if (surface->format == DSPF_I420) {
-          mdev->src_offset[1] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
-          mdev->src_offset[2] = mdev->src_offset[1] + surface->height * buffer->video.pitch / 4;
-     } else if (surface->format == DSPF_YV12) {
-          mdev->src_offset[2] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
-          mdev->src_offset[1] = mdev->src_offset[2] + surface->height * buffer->video.pitch / 4;
-     } else if (surface->format == DSPF_NV12 || surface->format == DSPF_NV21) {
-          mdev->src_offset[1] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
-     }
-
-     mdev->w = surface->width;
-     mdev->h = surface->height;
+     mdev->w         = surface->width;
+     mdev->h         = surface->height;
 
      if (state->destination->format == DSPF_YUY2 || state->destination->format == DSPF_UYVY) {
           mdev->w /= 2;
           mdev->src_pitch /= 2;
      }
 
-     if (mdev->blit_deinterlace) {
+     if (mdev->blit_deinterlace || mdev->blit_fields) {
           mdev->h /= 2;
-          if (surface->caps & DSCAPS_SEPARATED) {
-               if (mdev->field) {
-                    mdev->src_offset[0] += mdev->h * buffer->video.pitch;
-                    if (surface->format == DSPF_NV12 || surface->format == DSPF_NV21) {
-                         mdev->src_offset[1] += mdev->h * buffer->video.pitch / 2;
-                    } else {
-                         mdev->src_offset[1] += mdev->h * buffer->video.pitch / 4;
-                         mdev->src_offset[2] += mdev->h * buffer->video.pitch / 4;
-                    }
-               }
-          } else {
+          if (!(surface->caps & DSCAPS_SEPARATED))
                mdev->src_pitch *= 2;
-               if (mdev->field) {
-                    mdev->src_offset[0] += buffer->video.pitch;
-                    if (surface->format == DSPF_NV12 || surface->format == DSPF_NV21) {
-                         mdev->src_offset[1] += buffer->video.pitch;
-                    } else {
-                         mdev->src_offset[1] += buffer->video.pitch / 2;
-                         mdev->src_offset[2] += buffer->video.pitch / 2;
-                    }
-               }
-          }
      }
+
+     D_ASSERT( mdev->src_pitch % 32 == 0 );
+
+     matrox_calc_offsets( mdev, surface, buffer, false, mdev->src_offset );
+
+     if (mdev->blit_deinterlace && mdev->field) {
+          mdev->src_offset[0][0] = mdev->src_offset[1][0];
+          mdev->src_offset[0][1] = mdev->src_offset[1][1];
+          mdev->src_offset[0][2] = mdev->src_offset[1][1];
+     }
+
      mdev->w2 = mga_log2( mdev->w );
      mdev->h2 = mga_log2( mdev->h );
 
@@ -583,9 +652,11 @@ void matrox_validate_Source( MatroxDriverData *mdrv,
           case DSPF_A8:
                texctl |= TW8A;
                break;
+          case DSPF_RGB444:
           case DSPF_ARGB4444:
                texctl |= TW12;
                break;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                texctl |= TW15;
                break;
@@ -599,12 +670,10 @@ void matrox_validate_Source( MatroxDriverData *mdrv,
           case DSPF_LUT8:
                matrox_tlutload( mdrv, mdev, surface->palette );
                texctl |= TW8;
-               state->modified |= SMF_DESTINATION;
                break;
           case DSPF_RGB332:
                matrox_tlutload( mdrv, mdev, mdev->rgb332_palette );
                texctl |= TW8;
-               state->modified |= SMF_DESTINATION;
                break;
           default:
                D_BUG( "unexpected pixelformat!" );
@@ -635,15 +704,15 @@ void matrox_validate_Source( MatroxDriverData *mdrv,
      mga_out32( mmio, texctl,  TEXCTL );
      mga_out32( mmio, texctl2, TEXCTL2 );
 
-     mga_out32( mmio, ( (((__u32)(mdev->w - 1) & 0x7ff) << 18) |
-                        (((__u32)(4 - mdev->w2) & 0x3f) <<  9) |
-                        (((__u32)(mdev->w2 + 4) & 0x3f)      )  ), TEXWIDTH );
+     mga_out32( mmio, ( (((u32)(mdev->w - 1) & 0x7ff) << 18) |
+                        (((u32)(4 - mdev->w2) & 0x3f) <<  9) |
+                        (((u32)(mdev->w2 + 4) & 0x3f)      )  ), TEXWIDTH );
 
-     mga_out32( mmio, ( (((__u32)(mdev->h - 1) & 0x7ff) << 18) |
-                        (((__u32)(4 - mdev->h2) & 0x3f) <<  9) |
-                        (((__u32)(mdev->h2 + 4) & 0x3f)      )  ), TEXHEIGHT );
+     mga_out32( mmio, ( (((u32)(mdev->h - 1) & 0x7ff) << 18) |
+                        (((u32)(4 - mdev->h2) & 0x3f) <<  9) |
+                        (((u32)(mdev->h2 + 4) & 0x3f)      )  ), TEXHEIGHT );
 
-     mga_out32( mmio, mdev->src_offset[0], TEXORG );
+     mga_out32( mmio, mdev->src_offset[0][0], TEXORG );
 
      MGA_VALIDATE( m_Source );
 }
@@ -652,7 +721,7 @@ void matrox_validate_source( MatroxDriverData *mdrv,
                              MatroxDeviceData *mdev,
                              CardState        *state )
 {
-     volatile __u8 *mmio            = mdrv->mmio_base;
+     volatile u8   *mmio            = mdrv->mmio_base;
      CoreSurface   *surface         = state->source;
      SurfaceBuffer *buffer          = surface->front_buffer;
      int            bytes_per_pixel = DFB_BYTES_PER_PIXEL(surface->format);
@@ -662,28 +731,19 @@ void matrox_validate_source( MatroxDriverData *mdrv,
 
      mdev->src_pitch = buffer->video.pitch / bytes_per_pixel;
 
-     D_ASSERT( mdev->src_pitch % 32 == 0 );
-
-     D_ASSERT( buffer->video.offset % 64 == 0 );
-
      if (state->destination->format == DSPF_YUY2 || state->destination->format == DSPF_UYVY)
           mdev->src_pitch /= 2;
 
-     if (mdev->old_matrox) {
-          mdev->src_offset[0] = buffer->video.offset / bytes_per_pixel;
-     } else {
-          mdev->src_offset[0] = mdev->fb.offset + buffer->video.offset;
-          if (surface->format == DSPF_I420) {
-               mdev->src_offset[1] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
-               mdev->src_offset[2] = mdev->src_offset[1] + surface->height * buffer->video.pitch / 4;
-          } else if (surface->format == DSPF_YV12) {
-               mdev->src_offset[2] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
-               mdev->src_offset[1] = mdev->src_offset[2] + surface->height * buffer->video.pitch / 4;
-          } else if (surface->format == DSPF_NV12 || surface->format == DSPF_NV21) {
-               mdev->src_offset[1] = mdev->src_offset[0] + surface->height * buffer->video.pitch;
-          }
+     if (mdev->blit_fields && !(surface->caps & DSCAPS_SEPARATED))
+          mdev->src_pitch *= 2;
+
+     D_ASSERT( mdev->src_pitch % 32 == 0 );
+
+     matrox_calc_offsets( mdev, surface, buffer, mdev->old_matrox, mdev->src_offset );
+
+     if (!mdev->old_matrox) {
           mga_waitfifo( mdrv, mdev, 1 );
-          mga_out32( mmio, mdev->src_offset[0], SRCORG );
+          mga_out32( mmio, mdev->src_offset[0][0], SRCORG );
      }
 
      MGA_VALIDATE( m_source );
@@ -693,10 +753,10 @@ void matrox_validate_SrcKey( MatroxDriverData *mdrv,
                              MatroxDeviceData *mdev,
                              CardState        *state )
 {
-     volatile __u8 *mmio    = mdrv->mmio_base;
+     volatile u8   *mmio    = mdrv->mmio_base;
      CoreSurface   *surface = state->source;
-     __u32          key;
-     __u32          mask;
+     u32            key;
+     u32            mask;
 
      if (MGA_IS_VALID( m_SrcKey ))
           return;
@@ -721,10 +781,10 @@ void matrox_validate_srckey( MatroxDriverData *mdrv,
                              MatroxDeviceData *mdev,
                              CardState        *state )
 {
-     volatile __u8 *mmio    = mdrv->mmio_base;
+     volatile u8   *mmio    = mdrv->mmio_base;
      CoreSurface   *surface = state->source;
-     __u32          key;
-     __u32          mask;
+     u32            key;
+     u32            mask;
 
      if (MGA_IS_VALID( m_srckey ))
           return;
