@@ -1,39 +1,12 @@
-/*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
-
-   All rights reserved.
-
-   Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-*/
-
 #ifndef __MATROX__REGS_H__
 #define __MATROX__REGS_H__
 
-#define U8_TO_F0915(x)          (((__u32) ((x+1) << 15)) & 0x00FFFFFF)
+#define U8_TO_F0915(x)          (((u32) ((x+1) << 15)) & 0x00FFFFFF)
 
-#define RS16(val)               ( (__u16)((__s16)(val)))
-#define RS18(val)               (((__u32)((__s32)(val)))&0x003ffff)
-#define RS24(val)               (((__u32)((__s32)(val)))&0x0ffffff)
-#define RS27(val)               (((__u32)((__s32)(val)))&0x7ffffff)
+#define RS16(val)               ( (u16)((s16)(val)))
+#define RS18(val)               (((u32)((s32)(val)))&0x003ffff)
+#define RS24(val)               (((u32)((s32)(val)))&0x0ffffff)
+#define RS27(val)               (((u32)((s32)(val)))&0x7ffffff)
 
 #define DWGSYNC          0x2C4C
 #define SYNC_DMA_BUSY    0x8325340              /* just a random number */
@@ -41,6 +14,8 @@
 #define RST              0x1E40
 #define OPMODE           0x1E54
 
+#define CACHEFLUSH       0x1FFF
+
 /* CRTC2 registers */
 #define C2CTL            0x3C10
 #    define C2EN                0x00000001
