@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <dfb_types.h>
 
 #include <stdio.h>
@@ -53,27 +56,27 @@
 #include "neomagic.h"
 
 typedef volatile struct {
-  __u32 bltStat;
-  __u32 bltCntl;
-  __u32 xpColor;
-  __u32 fgColor;
-  __u32 bgColor;
-  __u32 pitch;
-  __u32 clipLT;
-  __u32 clipRB;
-  __u32 srcBitOffset;
-  __u32 srcStart;
-  __u32 reserved0;
-  __u32 dstStart;
-  __u32 xyExt;
+  u32 bltStat;
+  u32 bltCntl;
+  u32 xpColor;
+  u32 fgColor;
+  u32 bgColor;
+  u32 pitch;
+  u32 clipLT;
+  u32 clipRB;
+  u32 srcBitOffset;
+  u32 srcStart;
+  u32 reserved0;
+  u32 dstStart;
+  u32 xyExt;
 
-  __u32 reserved1[19];
+  u32 reserved1[19];
 
-  __u32 pageCntl;
-  __u32 pageBase;
-  __u32 postBase;
-  __u32 postPtr;
-  __u32 dataPtr;
+  u32 pageCntl;
+  u32 pageBase;
+  u32 postBase;
+  u32 postPtr;
+  u32 dataPtr;
 } Neo2200;
 
 typedef struct {
@@ -87,7 +90,7 @@ typedef struct {
      int srcPitch;
      int srcPixelWidth;
 
-     __u32 bltCntl;
+     u32 bltCntl;
 
      bool src_dst_equal;
 
@@ -262,12 +265,14 @@ static inline void neo2200_validate_xpColor( Neo2200DriverData *ndrv,
 
 /* required implementations */
 
-static void neo2200EngineSync( void *drv, void *dev )
+static DFBResult neo2200EngineSync( void *drv, void *dev )
 {
      Neo2200DriverData *ndrv = (Neo2200DriverData*) drv;
      Neo2200DeviceData *ndev = (Neo2200DeviceData*) dev;
 
      neo2200_waitidle( ndrv, ndev );
+
+     return DFB_OK;
 }
 
 #define NEO_SUPPORTED_DRAWINGFLAGS \
@@ -401,9 +406,9 @@ static bool neo2200DrawRectangle( void *drv, void *dev, DFBRectangle *rect )
      Neo2200DeviceData *ndev    = (Neo2200DeviceData*) dev;
      Neo2200           *neo2200 = ndrv->neo2200;
 
-     __u32 dst = ndev->dstOrg +
-                (rect->y * ndev->dstPitch) +
-                (rect->x * ndev->dstPixelWidth);
+     u32 dst = ndev->dstOrg +
+              (rect->y * ndev->dstPitch) +
+              (rect->x * ndev->dstPixelWidth);
 
      neo2200_waitfifo( ndrv, ndev, 3 );
 
@@ -443,8 +448,8 @@ static bool neo2200Blit( void *drv, void *dev,
      Neo2200DeviceData *ndev    = (Neo2200DeviceData*) dev;
      Neo2200           *neo2200 = ndrv->neo2200;
 
-     __u32 src_start, dst_start;
-     __u32 bltCntl = ndev->bltCntl;
+     u32 src_start, dst_start;
+     u32 bltCntl = ndev->bltCntl;
 
 //     fprintf(stderr, "blit: %d, %d (%dx%d) -> %d, %d\n",
 //             rect->x, rect->y, rect->w, rect->h, dx, dy);
