@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <sys/io.h>
 
@@ -39,15 +42,15 @@ typedef struct {
 
      /* overlay registers */
      struct {
-          __u32 OFFSET;
-          __u16 PITCH;
-          __u16 X1;
-          __u16 X2;
-          __u16 Y1;
-          __u16 Y2;
-          __u16 HSCALE;
-          __u16 VSCALE;
-          __u8  CONTROL;
+          u32 OFFSET;
+          u16 PITCH;
+          u16 X1;
+          u16 X2;
+          u16 Y1;
+          u16 Y2;
+          u16 HSCALE;
+          u16 VSCALE;
+          u8  CONTROL;
      } regs;
 } NeoOverlayLayerData;
 
