@@ -1,13 +1,13 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2005  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
               Sven Neumann <neo@directfb.org>,
-              Ville Syrjl <syrjala@sci.fi> and
+              Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
@@ -35,14 +35,12 @@
 #include <core/screens.h>
 #include <core/layers.h>
 
-#include "nvidia_regs.h"
-
 
 /*
- * Objects' Identifiers
+ * Object's identifier
  */
 enum {
-     OBJ_DMA            = 0x00800000,
+     OBJ_DMA_IN         = 0x00800000,
      OBJ_SURFACES2D     = 0x00800001,
      OBJ_SURFACES3D     = 0x00800002,
      OBJ_CLIP           = 0x00800003,
@@ -55,14 +53,15 @@ enum {
      OBJ_IMAGEBLT       = 0x00800014,
      OBJ_SCALEDIMAGE    = 0x00800015,
      OBJ_STRETCHEDIMAGE = 0x00800016,
-     OBJ_TEXTRIANGLE    = 0x00800017
+     OBJ_TEXTRIANGLE    = 0x00800017,
+     OBJ_DMA_OUT        = 0x00800018
 };
 
 /*
- * Objects' offsets into context table [PRAMIN + (address)*16]
+ * Object's offset into context table [PRAMIN + (address)*16]
  */
 enum {
-     ADDR_DMA            = 0x1160,
+     ADDR_DMA_IN         = 0x1160,
      ADDR_SURFACES2D     = 0x1162,
      ADDR_SURFACES3D     = 0x1163,
      ADDR_CLIP           = 0x1164,
@@ -75,9 +74,30 @@ enum {
      ADDR_IMAGEBLT       = 0x116B,
      ADDR_SCALEDIMAGE    = 0x116C,
      ADDR_STRETCHEDIMAGE = 0x116D,
-     ADDR_TEXTRIANGLE    = 0x116E
+     ADDR_TEXTRIANGLE    = 0x116E,
+     ADDR_DMA_OUT        = 0x116F
 };
 
+/*
+ * Object's subchannel
+ */
+enum {
+     SUBC_SURFACES2D     = 0,
+     SUBC_SURFACES3D     = 0,
+     SUBC_BETA1          = 0,
+     SUBC_BETA4          = 0,
+     SUBC_CLIP           = 1,
+     SUBC_RECTANGLE      = 2,
+     SUBC_TRIANGLE       = 3,
+     SUBC_LINE           = 4,
+     SUBC_SCREENBLT      = 5,
+     SUBC_IMAGEBLT       = 5,
+     SUBC_SCALEDIMAGE    = 6,
+     SUBC_STRETCHEDIMAGE = 6,
+     SUBC_TEXTRIANGLE    = 7
+};
+     
+
 #define SMF_DRAWING_COLOR  (SMF_COLOR << 16)
 #define SMF_BLITTING_COLOR (SMF_COLOR << 17)
 #define SMF_SOURCE_TEXTURE (SMF_SOURCE << 1) 
@@ -85,81 +105,100 @@ enum {
 typedef struct {
      StateModificationFlags  set;
      
-     __u32                   fb_offset;
-     __u32                   fb_size;
-     __u32                   fb_mask;
+     u32                     fb_offset;
+     u32                     fb_size;
+     u32                     agp_offset;
      
      DFBSurfacePixelFormat   dst_format;
-     __u32                   dst_offset;
-     __u32                   dst_pitch;
+     u32                     dst_offset;
+     u32                     dst_pitch;
      bool                    dst_422;
 
      DFBSurfacePixelFormat   src_format;
-     __u32                   src_offset;
-     __u8                   *src_address;
-     __u32                   src_pitch;
-     __u32                   src_width;
-     __u32                   src_height;
+     u32                     src_offset;
+     u8                     *src_address;
+     u32                     src_pitch;
+     u32                     src_width;
+     u32                     src_height;
      bool                    src_system;
      bool                    src_interlaced;
 
      DFBRectangle            clip;
 
-     __u32                   color2d;
-     __u32                   color3d;
+     u32                     color2d;
+     u32                     color3d;
      
      DFBSurfaceDrawingFlags  drawingflags;
      DFBSurfaceBlittingFlags blittingflags;
      
      /* NVRectangle/NVTriangle/NVLine registers */
-     __u32                   drawing_operation; // SetOperation
+     u32                     drawing_operation; // SetOperation
      
      /* NVScaledImage registers */
-     __u32                   scaler_operation;  // SetOperation
-     __u32                   scaler_format;     // SetColorFormat
-     __u32                   scaler_filter;     // SetImageInFormat
+     u32                     scaler_operation;  // SetOperation
+     u32                     scaler_format;     // SetColorFormat
+     u32                     scaler_filter;     // SetImageInFormat
      
      /* NVImageBlt/NVStretchedImage registers */
-     __u32                   system_operation;  // SetOperation
-     __u32                   system_format;     // SetColorFormat
+     u32                     system_operation;  // SetOperation
+     u32                     system_format;     // SetColorFormat
 
      /* Remember value of NVBeta1 & NVBeta4 */
      bool                    beta1_set;
-     __u32                   beta1_val;
+     u32                     beta1_val;
      bool                    beta4_set;
-     __u32                   beta4_val;
+     u32                     beta4_val;
 
      /* 3D stuff */
      bool                    enabled_3d;       // 3d engine enabled
-     __u32                   buf_offset[2];    // reserved buffers
+     u32                     buf_offset[2];    // reserved buffers
      SurfaceBuffer          *src_texture;      // current source for TextureTriangles
-     __u32                   max_texture_size;
+     u32                     max_texture_size;
      
      struct {
           bool               modified;
-          __u32              colorkey;
-          __u32              offset;
-          __u32              format;
-          __u32              filter;
-          __u32              blend;
-          __u32              control;
-          __u32              fog;
+          u32                colorkey;
+          u32                offset;
+          u32                format;
+          u32                filter;
+          u32                blend;
+          u32                control;
+          u32                fog;
      } state3d[2]; // 0 => drawing | 1 => blitting
 
      /* Remember subchannels configuration */
-     __u32                   subchannel_object[8];
+     u32                     subchannel_object[8];
      
      /* Chipsets informations */
-     __u32                   chip;
-     __u32                   arch;
+     u32                     chip;
+     u32                     arch;
+     
+     /* AGP control */
+     bool                    use_agp;
+     int                     agp_key;
+     unsigned int            agp_aper_base;
+     unsigned int            agp_aper_size;
+     
+     /* DMA control */
+     bool                    use_dma;
+     unsigned int            dma_size;
+     unsigned int            dma_offset;
+     unsigned int            dma_max;
+     unsigned int            dma_cur;
+     unsigned int            dma_free;
+     unsigned int            dma_put;
+     unsigned int            dma_get;
+     volatile u32           *cmd_ptr;
 
-     /* for fifo/performance monitoring */
-     unsigned int            fifo_space;
-     unsigned int            waitfifo_sum;
-     unsigned int            waitfifo_calls;
-     unsigned int            fifo_waitcycles;
+     /* FIFO control */
+     unsigned int            fifo_free;
+
+     /* for performance monitoring */
+     unsigned int            waitfree_sum;
+     unsigned int            waitfree_calls;
+     unsigned int            free_waitcycles;
      unsigned int            idle_waitcycles;
-     unsigned int            fifo_cache_hits;
+     unsigned int            cache_hits;
 } NVidiaDeviceData;
 
 
@@ -175,22 +214,10 @@ typedef struct {
      GraphicsDevice         *device;
      NVidiaDeviceData       *device_data;
 
-     volatile __u8          *mmio_base;
-     NVFifoChannel          *Fifo;
-     
-     NVSurfaces2D           *Surfaces2D;
-     NVSurfaces3D           *Surfaces3D;
-     NVClip                 *Clip;
-     NVBeta1                *Beta1;
-     NVBeta4                *Beta4;
-     NVRectangle            *Rectangle;
-     NVTriangle             *Triangle;
-     NVLine                 *Line;
-     NVScreenBlt            *ScreenBlt;
-     NVImageBlt             *ImageBlt;
-     NVScaledImage          *ScaledImage;
-     NVStretchedImage       *StretchedImage;
-     NVTexturedTriangleDx5  *TexTriangle;
+     volatile void          *fb_base;
+     volatile void          *agp_base;
+     volatile void          *mmio_base; 
+     volatile void          *dma_base;
 } NVidiaDriverData;
 
 
