@@ -1,4 +1,9 @@
 /*
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
+
+   All rights reserved.
+
    Written by Oliver Schwartz <Oliver.Schwartz@gmx.de> and
               Claudio Ciccani <klan@users.sf.net>.
 
@@ -55,32 +60,34 @@ typedef struct {
      int                   field;
 
      struct {
-          __u32 BUFFER;
-          __u32 STOP;
-          __u32 BASE_0;
-          __u32 BASE_1;
-          __u32 SIZE_IN_0;
-          __u32 SIZE_IN_1;
-          __u32 POINT_IN_0;
-          __u32 POINT_IN_1;
-          __u32 DS_DX_0;
-          __u32 DS_DX_1;
-          __u32 DT_DY_0;
-          __u32 DT_DY_1;
-          __u32 POINT_OUT_0;
-          __u32 POINT_OUT_1;
-          __u32 SIZE_OUT_0;
-          __u32 SIZE_OUT_1;
-          __u32 FORMAT_0;
-          __u32 FORMAT_1;
+          u32 BUFFER;
+          u32 STOP;
+          u32 BASE_0;
+          u32 BASE_1;
+          u32 SIZE_IN_0;
+          u32 SIZE_IN_1;
+          u32 POINT_IN_0;
+          u32 POINT_IN_1;
+          u32 DS_DX_0;
+          u32 DS_DX_1;
+          u32 DT_DY_0;
+          u32 DT_DY_1;
+          u32 POINT_OUT_0;
+          u32 POINT_OUT_1;
+          u32 SIZE_OUT_0;
+          u32 SIZE_OUT_1;
+          u32 FORMAT_0;
+          u32 FORMAT_1;
      } regs;
 } NVidiaOverlayLayerData;
 
-static void ov0_set_regs    ( NVidiaDriverData       *nvdrv,
-                              NVidiaOverlayLayerData *nvov0 );
-static void ov0_calc_regs   ( NVidiaDriverData       *nvdrv,
-                              NVidiaOverlayLayerData *nvov0,
-                              CoreLayerRegionConfig  *config );
+static void ov0_set_regs    ( NVidiaDriverData           *nvdrv,
+                              NVidiaOverlayLayerData     *nvov0,
+                              CoreLayerRegionConfigFlags  flags );
+static void ov0_calc_regs   ( NVidiaDriverData           *nvdrv,
+                              NVidiaOverlayLayerData     *nvov0,
+                              CoreLayerRegionConfig      *config,
+                              CoreLayerRegionConfigFlags  flags );
 static void ov0_set_colorkey( NVidiaDriverData       *nvdrv,
                               NVidiaOverlayLayerData *nvov0,
                               CoreLayerRegionConfig  *config );
@@ -109,7 +116,6 @@ ov0InitLayer( CoreLayer                  *layer,
               DFBColorAdjustment         *adjustment )
 {
      NVidiaDriverData       *nvdrv = (NVidiaDriverData*) driver_data;
-     NVidiaDeviceData       *nvdev = nvdrv->device_data;
      NVidiaOverlayLayerData *nvov0 = (NVidiaOverlayLayerData*) layer_data;
      
      /* set capabilities and type */
@@ -141,14 +147,6 @@ ov0InitLayer( CoreLayer                  *layer,
      adjustment->contrast   = 0x8000;
      adjustment->saturation = 0x8000;
      adjustment->hue        = 0x8000;
-
-     /* set video buffers start and limit */
-     if (nvdev->arch >= NV_ARCH_10) {
-          nv_out32( nvdrv->mmio_base, PVIDEO_OFFSET_0, 0 );
-          nv_out32( nvdrv->mmio_base, PVIDEO_OFFSET_1, 0 );
-          nv_out32( nvdrv->mmio_base, PVIDEO_LIMIT_0, nvdev->fb_size - 1 );
-          nv_out32( nvdrv->mmio_base, PVIDEO_LIMIT_1, nvdev->fb_size - 1 );
-     }
  
      /* reset overlay */
      nvov0->brightness = 0;
@@ -167,7 +165,7 @@ ov0Remove( CoreLayer *layer,
            void      *region_data )
 {
      NVidiaDriverData *nvdrv = (NVidiaDriverData*) driver_data;
-     volatile __u8    *mmio  = nvdrv->mmio_base;
+     volatile u8      *mmio  = nvdrv->mmio_base;
      
      /* disable overlay */
      nv_out32( mmio, PVIDEO_STOP, PVIDEO_STOP_OVERLAY_ACTIVE | 
@@ -247,8 +245,8 @@ ov0SetRegion( CoreLayer                  *layer,
      if (updated & (CLRCF_WIDTH  | CLRCF_HEIGHT | CLRCF_FORMAT  |
                     CLRCF_SOURCE | CLRCF_DEST   | CLRCF_OPTIONS | CLRCF_OPACITY))
      {
-          ov0_calc_regs( nvdrv, nvov0, config );
-          ov0_set_regs( nvdrv, nvov0 );
+          ov0_calc_regs( nvdrv, nvov0, config, updated );
+          ov0_set_regs( nvdrv, nvov0, updated );
      }
 
      /* set destination colorkey */
@@ -441,10 +439,10 @@ ov0DeallocateSurface( CoreLayer   *layer,
 static void
 ov0CopyData420
 (
-     __u8 *src1,
-     __u8 *src2,
-     __u8 *src3,
-     __u8 *dst1,
+     u8 *src1,
+     u8 *src2,
+     u8 *src3,
+     u8 *dst1,
      int   srcPitch,
      int   srcPitch2,
      int   dstPitch,
@@ -452,14 +450,14 @@ ov0CopyData420
      int   w
 )
 {
-     __u32 *dst;
-     __u8 *s1, *s2, *s3;
+     u32 *dst;
+     u8 *s1, *s2, *s3;
      int i, j;
 
      w >>= 1;
 
      for(j = 0; j < h; j++) {
-          dst = (__u32 *)dst1;
+          dst = (u32 *)dst1;
           s1 = src1;  s2 = src2;  s3 = src3;
           i = w;
           while(i > 4) {
@@ -513,11 +511,11 @@ ov0FlipRegion ( CoreLayer           *layer,
      if (DFB_PLANAR_PIXELFORMAT( surface->format )) {  
           SurfaceBuffer *data_buffer;
           SurfaceBuffer *video_buffer;
-          __u32 srcPitch, srcPitch2, dstPitch, s2offset, s3offset, tmp;
-          __u32 width  = surface->width;
-          __u32 height = surface->height;
-          __u8 *dstStart;
-          __u8 *buf;
+          u32 srcPitch, srcPitch2, dstPitch, s2offset, s3offset, tmp;
+          u32 width  = surface->width;
+          u32 height = surface->height;
+          u8 *dstStart;
+          u8 *buf;
           
           data_buffer  = surface->front_buffer;
           video_buffer = nvov0->videoSurface->front_buffer;
@@ -543,8 +541,8 @@ ov0FlipRegion ( CoreLayer           *layer,
                           dstPitch, height, width );
      }
 
-     ov0_calc_regs( nvdrv, nvov0, &nvov0->config );
-     ov0_set_regs( nvdrv, nvov0 );
+     ov0_calc_regs( nvdrv, nvov0, &nvov0->config, CLRCF_SURFACE );
+     ov0_set_regs( nvdrv, nvov0, CLRCF_SURFACE );
 
      if (flags & DSFLIP_WAIT)
           dfb_layer_wait_vsync( layer );
@@ -566,11 +564,11 @@ ov0UpdateRegion ( CoreLayer           *layer,
      if (DFB_PLANAR_PIXELFORMAT( surface->format )) {  
           SurfaceBuffer *data_buffer;
           SurfaceBuffer *video_buffer;
-          __u32 srcPitch, srcPitch2, dstPitch, s2offset, s3offset, tmp;
-          __u32 width  = surface->width;
-          __u32 height = surface->height;
-          __u8 *dstStart;
-          __u8 *buf;
+          u32 srcPitch, srcPitch2, dstPitch, s2offset, s3offset, tmp;
+          u32 width  = surface->width;
+          u32 height = surface->height;
+          u8 *dstStart;
+          u8 *buf;
           
           data_buffer  = surface->front_buffer;
           video_buffer = nvov0->videoSurface->front_buffer;
@@ -646,7 +644,7 @@ ov0SetInputField( CoreLayer *layer,
      nvov0->field = field;
      nvov0->regs.BUFFER = 1 << (field << 2);
      
-     ov0_set_regs( nvdrv, nvov0 );
+     nv_out32( nvdrv->mmio_base, PVIDEO_BUFFER, nvov0->regs.BUFFER );
      
      return DFB_OK;
 }
@@ -669,108 +667,129 @@ DisplayLayerFuncs nvidiaOverlayFuncs = {
 
 /* internal */
 
-static void ov0_set_regs( NVidiaDriverData *nvdrv, NVidiaOverlayLayerData *nvov0 )
+static void ov0_set_regs( NVidiaDriverData           *nvdrv, 
+                          NVidiaOverlayLayerData     *nvov0,
+                          CoreLayerRegionConfigFlags  flags )
 {
-     volatile __u8 *mmio = nvdrv->mmio_base;
+     volatile u8 *mmio = nvdrv->mmio_base;
      
-     nv_out32( mmio, PVIDEO_BASE_0,      nvov0->regs.BASE_0 );
-     nv_out32( mmio, PVIDEO_BASE_1,      nvov0->regs.BASE_1 );
-     nv_out32( mmio, PVIDEO_SIZE_IN_0,   nvov0->regs.SIZE_IN_0 );
-     nv_out32( mmio, PVIDEO_SIZE_IN_1,   nvov0->regs.SIZE_IN_1 );
-     nv_out32( mmio, PVIDEO_POINT_IN_0,  nvov0->regs.POINT_IN_0 );
-     nv_out32( mmio, PVIDEO_POINT_IN_1,  nvov0->regs.POINT_IN_1 );
-     nv_out32( mmio, PVIDEO_DS_DX_0,     nvov0->regs.DS_DX_0 );
-     nv_out32( mmio, PVIDEO_DS_DX_1,     nvov0->regs.DS_DX_1 );
-     nv_out32( mmio, PVIDEO_DT_DY_0,     nvov0->regs.DT_DY_0 );
-     nv_out32( mmio, PVIDEO_DT_DY_1,     nvov0->regs.DT_DY_1 );
-     nv_out32( mmio, PVIDEO_POINT_OUT_0, nvov0->regs.POINT_OUT_0 );
-     nv_out32( mmio, PVIDEO_POINT_OUT_1, nvov0->regs.POINT_OUT_1 );
-     nv_out32( mmio, PVIDEO_SIZE_OUT_0,  nvov0->regs.SIZE_OUT_0 );
-     nv_out32( mmio, PVIDEO_SIZE_OUT_1,  nvov0->regs.SIZE_OUT_1 );
-     nv_out32( mmio, PVIDEO_FORMAT_0,    nvov0->regs.FORMAT_0 );
-     nv_out32( mmio, PVIDEO_FORMAT_1,    nvov0->regs.FORMAT_1 );
-
+     if (flags & CLRCF_SURFACE) {
+          nv_out32( mmio, PVIDEO_BASE_0, nvov0->regs.BASE_0 );
+          nv_out32( mmio, PVIDEO_BASE_1, nvov0->regs.BASE_1 );
+     }
+     if (flags & (CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_OPTIONS)) {
+          nv_out32( mmio, PVIDEO_SIZE_IN_0, nvov0->regs.SIZE_IN_0 );
+          nv_out32( mmio, PVIDEO_SIZE_IN_1, nvov0->regs.SIZE_IN_1 );
+     }
+     if (flags & (CLRCF_SOURCE | CLRCF_DEST | CLRCF_OPTIONS)) {
+          nv_out32( mmio, PVIDEO_POINT_IN_0, nvov0->regs.POINT_IN_0 );
+          nv_out32( mmio, PVIDEO_POINT_IN_1, nvov0->regs.POINT_IN_1 );
+          nv_out32( mmio, PVIDEO_DS_DX_0,    nvov0->regs.DS_DX_0 );
+          nv_out32( mmio, PVIDEO_DS_DX_1,    nvov0->regs.DS_DX_1 );
+          nv_out32( mmio, PVIDEO_DT_DY_0,    nvov0->regs.DT_DY_0 );
+          nv_out32( mmio, PVIDEO_DT_DY_1,    nvov0->regs.DT_DY_1 );
+     }
+     if (flags & CLRCF_DEST) {
+          nv_out32( mmio, PVIDEO_POINT_OUT_0, nvov0->regs.POINT_OUT_0 );
+          nv_out32( mmio, PVIDEO_POINT_OUT_1, nvov0->regs.POINT_OUT_1 );
+          nv_out32( mmio, PVIDEO_SIZE_OUT_0,  nvov0->regs.SIZE_OUT_0 );
+          nv_out32( mmio, PVIDEO_SIZE_OUT_1,  nvov0->regs.SIZE_OUT_1 );
+     }
+     if (flags & (CLRCF_FORMAT | CLRCF_SURFACE | CLRCF_OPTIONS)) {
+          nv_out32( mmio, PVIDEO_FORMAT_0, nvov0->regs.FORMAT_0 );
+          nv_out32( mmio, PVIDEO_FORMAT_1, nvov0->regs.FORMAT_1 );
+     }
      nv_out32( mmio, PVIDEO_BUFFER, nvov0->regs.BUFFER );
      nv_out32( mmio, PVIDEO_STOP,   nvov0->regs.STOP );
 }
 
 static void
-ov0_calc_regs( NVidiaDriverData       *nvdrv,
-               NVidiaOverlayLayerData *nvov0,
-               CoreLayerRegionConfig  *config )
+ov0_calc_regs( NVidiaDriverData           *nvdrv,
+               NVidiaOverlayLayerData     *nvov0,
+               CoreLayerRegionConfig      *config,
+               CoreLayerRegionConfigFlags  flags )
 {
-     NVidiaDeviceData *nvdev   = nvdrv->device_data;
-     DFBRectangle      source  = config->source;
-     DFBRectangle      dest    = config->dest;
-     SurfaceBuffer    *buffer  = nvov0->videoSurface->front_buffer;
-     __u32             offset  = buffer->video.offset;
-     __u32             pitch   = buffer->video.pitch;
-     int               width   = config->width;
-     int               height  = config->height;
-     __u32             format;
-
-     source.x <<= 4;
-     source.y <<= 4;
+     NVidiaDeviceData *nvdev = nvdrv->device_data;
+ 
+     if (flags & (CLRCF_WIDTH  | CLRCF_HEIGHT | 
+                  CLRCF_SOURCE | CLRCF_DEST   | CLRCF_OPTIONS)) {
+          int          width  = config->width;
+          int          height = config->height;
+          DFBRectangle source = config->source;
+          DFBRectangle dest   = config->dest;
      
-     if (dest.x < 0) {
-          source.x -= (dest.x * source.w << 4) / dest.w;
-          source.w += dest.x * source.w / dest.w;
-          dest.w   += dest.x;
-          dest.x    = 0;
-     }
-
-     if (dest.y < 0) {
-          source.y -= (dest.y * source.h << 4) / dest.h;
-          source.h += dest.y * source.h / dest.h;
-          dest.h   += dest.y;
-          dest.y    = 0;
-     }
-
-     if (source.w < 1 || source.h < 1 || dest.w < 1 || dest.h < 1) {
-          nvov0->regs.BUFFER = 0;
-          nvov0->regs.STOP   = PVIDEO_STOP_OVERLAY_ACTIVE |
-                               PVIDEO_STOP_METHOD_NORMALLY;
-          return;
-     }
-
-     if (config->options & DLOP_DEINTERLACING) {
-          height   /= 2;
-          source.y /= 2;
-          source.h /= 2;
-          pitch    *= 2;
-     }
+          source.x <<= 4;
+          source.y <<= 4;
      
-     if (config->format == DSPF_UYVY)
-          format = PVIDEO_FORMAT_COLOR_YB8CR8YA8CB8;
-     else
-          format = PVIDEO_FORMAT_COLOR_CR8YB8CB8YA8;
+          if (dest.x < 0) {
+               source.x -= (dest.x * source.w << 4) / dest.w;
+               source.w += dest.x * source.w / dest.w;
+               dest.w   += dest.x;
+               dest.x    = 0;
+          }
 
-     if (config->options & DLOP_DST_COLORKEY)
-          format |= PVIDEO_FORMAT_DISPLAY_COLOR_KEY_EQUAL;
+          if (dest.y < 0) {
+               source.y -= (dest.y * source.h << 4) / dest.h;
+               source.h += dest.y * source.h / dest.h;
+               dest.h   += dest.y;
+               dest.y    = 0;
+          }
+
+          if (config->options & DLOP_DEINTERLACING) {
+               height   /= 2;
+               source.y /= 2;
+               source.h /= 2;
+          }
+
+          if (source.w < 1 || source.h < 1 || dest.w < 1 || dest.h < 1) {
+               nvov0->regs.STOP = PVIDEO_STOP_OVERLAY_ACTIVE |
+                                  PVIDEO_STOP_METHOD_NORMALLY;
+               return;
+          }
           
-     /* Use Buffer 0 for Odd field */
-     nvov0->regs.BASE_0      = (nvdev->fb_offset + offset) & PVIDEO_BASE_MSK;
-     nvov0->regs.SIZE_IN_0   = ((height << 16) & PVIDEO_SIZE_IN_HEIGHT_MSK) |
-                               ( width         & PVIDEO_SIZE_IN_WIDTH_MSK);
-     nvov0->regs.POINT_IN_0  = ((source.y << 16) & PVIDEO_POINT_IN_T_MSK) |
-                               ( source.x        & PVIDEO_POINT_IN_S_MSK);
-     nvov0->regs.DS_DX_0     = (source.w << 20) / dest.w;
-     nvov0->regs.DT_DY_0     = (source.h << 20) / dest.h;
-     nvov0->regs.POINT_OUT_0 = ((dest.y << 16) & PVIDEO_POINT_OUT_Y_MSK) |
-                               ( dest.x        & PVIDEO_POINT_OUT_X_MSK);
-     nvov0->regs.SIZE_OUT_0  = ((dest.h << 16) & PVIDEO_SIZE_OUT_HEIGHT_MSK) |
-                               ( dest.w        & PVIDEO_SIZE_OUT_WIDTH_MSK);   
-     nvov0->regs.FORMAT_0    = (pitch & PVIDEO_FORMAT_PITCH_MSK) | format; 
+          nvov0->regs.SIZE_IN_0   = 
+          nvov0->regs.SIZE_IN_1   = ((height << 16) & PVIDEO_SIZE_IN_HEIGHT_MSK) |
+                                    ( width         & PVIDEO_SIZE_IN_WIDTH_MSK);
+          nvov0->regs.POINT_IN_0  =
+          nvov0->regs.POINT_IN_1  = ((source.y << 16) & PVIDEO_POINT_IN_T_MSK) |
+                                    ( source.x        & PVIDEO_POINT_IN_S_MSK);
+          nvov0->regs.DS_DX_0     =
+          nvov0->regs.DS_DX_1     = (source.w << 20) / dest.w;
+          nvov0->regs.DT_DY_0     =
+          nvov0->regs.DT_DY_1     = (source.h << 20) / dest.h;
+          nvov0->regs.POINT_OUT_0 =
+          nvov0->regs.POINT_OUT_1 = ((dest.y << 16) & PVIDEO_POINT_OUT_Y_MSK) |
+                                    ( dest.x        & PVIDEO_POINT_OUT_X_MSK);
+          nvov0->regs.SIZE_OUT_0  =
+          nvov0->regs.SIZE_OUT_1  = ((dest.h << 16) & PVIDEO_SIZE_OUT_HEIGHT_MSK) |
+                                    ( dest.w        & PVIDEO_SIZE_OUT_WIDTH_MSK);
+     }
      
-     /* Use Buffer 1 for Even field */    
-     nvov0->regs.BASE_1      = nvov0->regs.BASE_0 + buffer->video.pitch;
-     nvov0->regs.SIZE_IN_1   = nvov0->regs.SIZE_IN_0;
-     nvov0->regs.POINT_IN_1  = nvov0->regs.POINT_IN_0;
-     nvov0->regs.DS_DX_1     = nvov0->regs.DS_DX_0;
-     nvov0->regs.DT_DY_1     = nvov0->regs.DT_DY_0;
-     nvov0->regs.POINT_OUT_1 = nvov0->regs.POINT_OUT_0;
-     nvov0->regs.SIZE_OUT_1  = nvov0->regs.SIZE_OUT_0;
-     nvov0->regs.FORMAT_1    = nvov0->regs.FORMAT_0;
+     if (flags & (CLRCF_SURFACE | CLRCF_FORMAT | CLRCF_OPTIONS)) {
+          SurfaceBuffer *buffer = nvov0->videoSurface->front_buffer;
+          u32            format;
+          
+          if (config->options & DLOP_DEINTERLACING)
+               format = (buffer->video.pitch*2) & PVIDEO_FORMAT_PITCH_MSK;
+          else
+               format =  buffer->video.pitch    & PVIDEO_FORMAT_PITCH_MSK;
+     
+          if (config->format == DSPF_UYVY)
+               format |= PVIDEO_FORMAT_COLOR_YB8CR8YA8CB8;
+          else
+               format |= PVIDEO_FORMAT_COLOR_CR8YB8CB8YA8;
+
+          if (config->options & DLOP_DST_COLORKEY)
+               format |= PVIDEO_FORMAT_DISPLAY_COLOR_KEY_EQUAL;
+               
+          /* Use Buffer 0 for Odd field */
+          nvov0->regs.BASE_0   = (nvdev->fb_offset +
+                                  buffer->video.offset) & PVIDEO_BASE_MSK;
+          /* Use Buffer 1 for Even field */
+          nvov0->regs.BASE_1   = nvov0->regs.BASE_0 + buffer->video.pitch;
+          nvov0->regs.FORMAT_0 =
+          nvov0->regs.FORMAT_1 = format;
+     }
  
      nvov0->regs.BUFFER = 1 << (nvov0->field << 2);
      nvov0->regs.STOP   = (config->opacity)
@@ -784,7 +803,7 @@ ov0_set_colorkey( NVidiaDriverData       *nvdrv,
                   NVidiaOverlayLayerData *nvov0,
                   CoreLayerRegionConfig  *config )
 {
-     __u32 key;
+     u32 key;
      
      key = dfb_color_to_pixel( dfb_primary_layer_pixelformat(),
                                config->dst_key.r,
@@ -798,10 +817,10 @@ static void
 ov0_set_csc( NVidiaDriverData       *nvdrv,
              NVidiaOverlayLayerData *nvov0 )
 {
-     volatile __u8 *mmio = nvdrv->mmio_base;
-     __s32          satSine;
-     __s32          satCosine;
-     double         angle;
+     volatile u8 *mmio = nvdrv->mmio_base;
+     s32          satSine;
+     s32          satCosine;
+     double       angle;
 
      angle = (double) nvov0->hue * M_PI / 180.0;
      satSine = nvov0->saturation * sin(angle);
