@@ -1,13 +1,13 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2006  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
               Sven Neumann <neo@directfb.org>,
-              Ville Syrjl <syrjala@sci.fi> and
+              Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
@@ -26,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -65,12 +67,12 @@ void nv_set_destination( NVidiaDriverData *nvdrv,
 {
      CoreSurface   *surface     = state->destination;
      SurfaceBuffer *buffer      = surface->back_buffer;
-     volatile __u8 *mmio        = nvdrv->mmio_base;
-     __u32          dst_offset;
-     __u32          dst_pitch;
-     __u32          src_pitch;
-     __u32          depth_offset;
-     __u32          depth_pitch;
+     volatile u8   *mmio        = nvdrv->mmio_base;
+     u32            dst_offset;
+     u32            dst_pitch;
+     u32            src_pitch;
+     u32            depth_offset;
+     u32            depth_pitch;
      
      if (NVIDIA_IS_SET( DESTINATION ))
           return;
@@ -89,18 +91,24 @@ void nv_set_destination( NVidiaDriverData *nvdrv,
      }
 
      if (nvdev->dst_format != buffer->format) {
-          __u32 sformat2D = 0;
-          __u32 sformat3D = 0;
-          __u32 cformat   = 0;
+          u32   sformat2D = 0;
+          u32   sformat3D = 0;
+          u32   cformat   = 0;
           bool  dst_422 = false;
 
           switch (buffer->format) {
                case DSPF_A8:
                case DSPF_LUT8:
+               case DSPF_ALUT44:
                case DSPF_RGB332:
                     sformat2D = SURFACES2D_FORMAT_Y8;
                     cformat   = RECT_COLOR_FORMAT_Y32;
                     break;
+               case DSPF_RGB555:
+                    sformat2D = SURFACES2D_FORMAT_X1R5G5B5;
+                    sformat3D = SURFACES3D_FORMAT_COLOR_X1R5G5B5;
+                    cformat   = RECT_COLOR_FORMAT_A1Y15;
+                    break;
                case DSPF_ARGB1555:
                     sformat2D = SURFACES2D_FORMAT_A1R5G5B5;
                     sformat3D = SURFACES3D_FORMAT_COLOR_A1R5G5B5;
@@ -170,15 +178,13 @@ void nv_set_destination( NVidiaDriverData *nvdrv,
           nv_begin( SUBC_SURFACES2D, SURFACES2D_DST_OFFSET, 1 );
           nv_outr( dst_offset );
 
-          if (nvdev->enabled_3d && sformat3D) { 
-               sformat3D |= SURFACES3D_FORMAT_TYPE_PITCH;
-               
+          if (nvdev->enabled_3d && sformat3D) {
                nv_assign_object( nvdrv, nvdev, 
                                  SUBC_SURFACES3D, OBJ_SURFACES3D, false );
                
                if (surface->caps & DSCAPS_DEPTH) {
                     nv_begin( SUBC_SURFACES3D, SURFACES3D_FORMAT, 1 );
-                    nv_outr( sformat3D );
+                    nv_outr( sformat3D | SURFACES3D_FORMAT_TYPE_PITCH );
                     nv_begin( SUBC_SURFACES3D, SURFACES3D_PITCH, 3 );
                     nv_outr( (depth_pitch << 16) | (dst_pitch & 0xFFFF) );
                     nv_outr( dst_offset );
@@ -188,7 +194,7 @@ void nv_set_destination( NVidiaDriverData *nvdrv,
                } 
                else {
                     nv_begin( SUBC_SURFACES3D, SURFACES3D_FORMAT, 1 );
-                    nv_outr( sformat3D );
+                    nv_outr( sformat3D | SURFACES3D_FORMAT_TYPE_PITCH );
                     nv_begin( SUBC_SURFACES3D, SURFACES3D_PITCH, 2 );
                     nv_outr( (depth_pitch << 16) | (dst_pitch & 0xFFFF) );
                     nv_outr( dst_offset );
@@ -204,9 +210,10 @@ void nv_set_destination( NVidiaDriverData *nvdrv,
           }
           
           NVIDIA_UNSET( COLOR );
-          NVIDIA_UNSET( SRC_BLEND );
+          NVIDIA_UNSET( DST_BLEND );
      }
-     else {
+     else if (nvdev->dst_offset != dst_offset ||
+              nvdev->dst_pitch  != dst_pitch) {
           nv_assign_object( nvdrv, nvdev,
                             SUBC_SURFACES2D, OBJ_SURFACES2D, false );
 
@@ -218,8 +225,8 @@ void nv_set_destination( NVidiaDriverData *nvdrv,
           if (nvdev->enabled_3d) {
                nv_assign_object( nvdrv, nvdev,
                                  SUBC_SURFACES3D, OBJ_SURFACES3D, false );
-
-               if (surface->caps & DSCAPS_DEPTH) {
+               
+               if (surface->caps & DSCAPS_DEPTH) {     
                     nv_begin( SUBC_SURFACES3D, SURFACES3D_PITCH, 3 );
                     nv_outr( (depth_pitch << 16) | (dst_pitch & 0xFFFF) );
                     nv_outr( dst_offset );
@@ -272,15 +279,18 @@ void nv_set_source( NVidiaDriverData *nvdrv,
           nvdev->src_system  = true;
      }
      else {
-          __u32 src_offset = (buffer->video.offset + nvdev->fb_offset) & ~63;
-          __u32 src_pitch  = buffer->video.pitch & ~31;
+          u32 src_offset = (buffer->video.offset + nvdev->fb_offset) & ~63;
+          u32 src_pitch  = buffer->video.pitch & ~31;
 
-          nv_assign_object( nvdrv, nvdev,
-                            SUBC_SURFACES2D, OBJ_SURFACES2D, false );
+          if (nvdev->src_offset != src_offset ||
+              nvdev->src_pitch  != src_pitch) {
+               nv_assign_object( nvdrv, nvdev,
+                                 SUBC_SURFACES2D, OBJ_SURFACES2D, false );
           
-          nv_begin( SUBC_SURFACES2D, SURFACES2D_PITCH, 2 );
-          nv_outr( (nvdev->dst_pitch << 16) | (src_pitch & 0xFFFF) );
-          nv_outr( src_offset );
+               nv_begin( SUBC_SURFACES2D, SURFACES2D_PITCH, 2 );
+               nv_outr( (nvdev->dst_pitch << 16) | (src_pitch & 0xFFFF) );
+               nv_outr( src_offset );
+          }
 
           if (nvdev->src_system) {
                nv_assign_object( nvdrv, nvdev,
@@ -318,16 +328,18 @@ void nv_set_source( NVidiaDriverData *nvdrv,
           nvdev->src_interlaced = false;
      
      if (nvdev->enabled_3d) {
-          __u32 size_u = direct_log2( surface->width  ) & 0xF;
-          __u32 size_v = direct_log2( surface->height ) & 0xF;
+          u32 size_u = direct_log2( surface->width  ) & 0xF;
+          u32 size_v = direct_log2( surface->height ) & 0xF;
 
           nvdev->state3d[1].offset  = nvdev->fb_offset + nvdev->buf_offset[1];
           nvdev->state3d[1].format &= 0xFF00FFFF;
           nvdev->state3d[1].format |= (size_u << 16) | (size_v << 20);
      }
 
-     if (nvdev->src_format != buffer->format)
+     if (nvdev->src_format != buffer->format) {
+          NVIDIA_UNSET( SRC_BLEND );
           NVIDIA_UNSET( BLITTING_FLAGS );
+     }
      nvdev->src_format = buffer->format;
      
      NVIDIA_SET( SOURCE );
@@ -376,11 +388,16 @@ void nv_set_drawing_color( NVidiaDriverData *nvdrv,
           case DSPF_LUT8:
                nvdev->color2d = state->color_index;
                break;
+          case DSPF_ALUT44:
+               nvdev->color2d = (state->color_index & 0x0F) |
+                                (state->color.a     & 0xF0);
+               break;
           case DSPF_RGB332:
                nvdev->color2d = PIXEL_RGB332( color.r,
                                               color.g,
                                               color.b );
                break;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                nvdev->color2d = PIXEL_ARGB1555( color.a,
                                                 color.r,
@@ -461,27 +478,27 @@ void nv_set_blitting_color( NVidiaDriverData *nvdrv,
      if (NVIDIA_IS_SET( BLITTING_COLOR ) && NVIDIA_IS_SET( BLITTING_FLAGS ))
           return;
                     
-     if (state->blittingflags & (DSBLIT_COLORIZE | DSBLIT_SRC_PREMULTCOLOR)) {          
+     if (state->blittingflags & (DSBLIT_COLORIZE | DSBLIT_SRC_PREMULTCOLOR) ||
+        (state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL && state->src_blend == DSBF_ONE)) {       
           nvdev->color3d = (state->blittingflags & DSBLIT_BLEND_COLORALPHA)
                            ? (color.a << 24) : 0xFF000000;
           
           if (state->blittingflags & DSBLIT_COLORIZE &&
-              state->blittingflags & DSBLIT_SRC_PREMULTCOLOR) {
+              state->blittingflags & (DSBLIT_SRC_PREMULTCOLOR | DSBLIT_BLEND_COLORALPHA)) {
                nvdev->color3d |= PIXEL_RGB32( color.r * color.a / 0xFF,
                                               color.g * color.a / 0xFF,
                                               color.b * color.a / 0xFF );
           }
           else if (state->blittingflags & DSBLIT_COLORIZE) {
-               nvdev->color3d |= PIXEL_RGB32( color.r,
-                                              color.g,
-                                              color.b );
+               nvdev->color3d |= PIXEL_RGB32( color.r, color.g, color.b );
           }
-          else if (state->blittingflags & DSBLIT_SRC_PREMULTCOLOR) {
-               nvdev->color3d |= PIXEL_RGB32( color.a,
-                                              color.a,
-                                              color.a );
-          }        
-               
+          else if (state->blittingflags & (DSBLIT_SRC_PREMULTCOLOR | DSBLIT_BLEND_COLORALPHA)) {
+               nvdev->color3d |= PIXEL_RGB32( color.a, color.a, color.a );
+          }
+          else {
+               nvdev->color3d |= 0x00FFFFFF;
+          }
+
           if (!nvdev->beta4_set || nvdev->beta4_val != nvdev->color3d) {
                nv_assign_object( nvdrv, nvdev,
                                  SUBC_BETA4, OBJ_BETA4, false );
@@ -495,7 +512,7 @@ void nv_set_blitting_color( NVidiaDriverData *nvdrv,
      }
      else if (state->blittingflags & (DSBLIT_BLEND_COLORALPHA |
                                       DSBLIT_BLEND_ALPHACHANNEL)) {
-          __u32 beta1;
+          u32 beta1;
           
           if (state->blittingflags & DSBLIT_BLEND_COLORALPHA) {
                nvdev->color3d = (color.a << 24) | 0x00FFFFFF;
@@ -525,32 +542,46 @@ void nv_set_blend_function( NVidiaDriverData *nvdrv,
                             NVidiaDeviceData *nvdev,
                             CardState        *state )
 {
-     __u32 blend;
+     DFBSurfaceBlendFunction sblend, dblend;
      
      if (NVIDIA_IS_SET( SRC_BLEND ) && NVIDIA_IS_SET( DST_BLEND ))
           return;
-          
-     blend = state->dst_blend << 28;
+       
+     sblend = state->src_blend;
+     dblend = state->dst_blend;
      
-     if (!DFB_PIXELFORMAT_HAS_ALPHA( nvdev->dst_format )) {
-          switch (state->src_blend) {
-               case DSBF_DESTALPHA:
-                    blend |= TXTRI_BLEND_SRCBLEND_ONE;
-                    break;
-               case DSBF_INVDESTALPHA:
-                    blend |= TXTRI_BLEND_SRCBLEND_ZERO;
-                    break;
-               default:
-                    blend |= state->src_blend << 24;
-          }
-     } else
-          blend |= state->src_blend << 24;
+#if 0
+     if (!DFB_PIXELFORMAT_HAS_ALPHA(nvdev->src_format)) {
+          if (sblend == DSBF_SRCALPHA)
+               sblend = DSBF_ONE;
+          else if (sblend == DSBF_INVSRCALPHA)
+               sblend = DSBF_ZERO;
+               
+          if (dblend == DSBF_SRCALPHA)
+               dblend = DSBF_ONE;
+          else if (dblend == DSBF_INVSRCALPHA)
+               dblend = DSBF_ZERO;
+     }
+#endif
+     if (!DFB_PIXELFORMAT_HAS_ALPHA(nvdev->dst_format)) {
+          if (sblend == DSBF_DESTALPHA)
+               sblend = DSBF_ONE;
+          else if (sblend == DSBF_INVDESTALPHA)
+               sblend = DSBF_ZERO;
+               
+          if (dblend == DSBF_DESTALPHA)
+               dblend = DSBF_ONE;
+          else if (dblend == DSBF_INVDESTALPHA)
+               dblend = DSBF_ZERO;
+     }
           
      nvdev->state3d[0].blend &= 0x00FFFFFF;
-     nvdev->state3d[0].blend |= blend;
+     nvdev->state3d[0].blend |= (sblend << 24) | (dblend << 28);
      nvdev->state3d[1].blend &= 0x00FFFFFF;
-     nvdev->state3d[1].blend |= blend;   
-          
+     nvdev->state3d[1].blend |= (sblend << 24) | (dblend << 28);
+         
+     if (!NVIDIA_IS_SET( SRC_BLEND ))
+          NVIDIA_UNSET( BLITTING_FLAGS );
      NVIDIA_SET( SRC_BLEND );
      NVIDIA_SET( DST_BLEND );
 }
@@ -563,7 +594,7 @@ void nv_set_drawingflags( NVidiaDriverData *nvdrv,
           return;
           
      if (!nvdev->enabled_3d) {
-          __u32 operation;
+          u32 operation;
           
           if (state->drawingflags & DSDRAW_BLEND)
                operation = OPERATION_BLEND;
@@ -593,29 +624,35 @@ void nv_set_blittingflags( NVidiaDriverData *nvdrv,
                            NVidiaDeviceData *nvdev,
                            CardState        *state )
 {
-     __u32 operation;
-     bool  src_alpha;
+     u32  operation;
+     bool src_alpha;
      
      if (NVIDIA_IS_SET( BLITTING_FLAGS ))
           return;
           
-     operation = (nvdev->arch > NV_ARCH_04)
-                 ? OPERATION_SRCCOPY : OPERATION_COPY;
-     src_alpha = !(  state->blittingflags & DSBLIT_BLEND_COLORALPHA  && 
-                   !(state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL) );
+     operation = (nvdev->arch > NV_ARCH_04) ? OPERATION_SRCCOPY : OPERATION_COPY;
+     src_alpha = true;
      
      if (state->blittingflags & (DSBLIT_BLEND_COLORALPHA |
                                  DSBLIT_BLEND_ALPHACHANNEL)) {
-          if (state->blittingflags & (DSBLIT_COLORIZE | DSBLIT_SRC_PREMULTCOLOR))
+          if (state->blittingflags & (DSBLIT_COLORIZE | DSBLIT_SRC_PREMULTCOLOR) ||
+             (state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL && state->src_blend == DSBF_ONE)) {
                operation = OPERATION_BLEND_PREMULTIPLIED;
-          else 
+          }
+          else {
                operation = OPERATION_BLEND;
+               src_alpha = !!(state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL);
+          }
      }
-     else if (state->blittingflags & (DSBLIT_COLORIZE | DSBLIT_SRC_PREMULTCOLOR))
+     else if (state->blittingflags & (DSBLIT_COLORIZE | DSBLIT_SRC_PREMULTCOLOR)) {
           operation = OPERATION_COLOR_MULTIPLY;
+     }
 
      if (nvdev->src_system) {
           switch (nvdev->src_format) {
+               case DSPF_RGB555:
+                    nvdev->system_format = IBLIT_COLOR_FORMAT_X1R5G5B5;
+                    break;
                case DSPF_ARGB1555:
                     nvdev->system_format = src_alpha
                                            ? IBLIT_COLOR_FORMAT_A1R5G5B5 
@@ -653,9 +690,13 @@ void nv_set_blittingflags( NVidiaDriverData *nvdrv,
                     nvdev->scaler_format = SCALER_COLOR_FORMAT_AY8;
                     break;
                case DSPF_LUT8:
+               case DSPF_ALUT44:
                case DSPF_RGB332:
                     nvdev->scaler_format = SCALER_COLOR_FORMAT_Y8;
                     break;
+               case DSPF_RGB555:
+                    nvdev->scaler_format = SCALER_COLOR_FORMAT_X1R5G5B5;
+                    break;
                case DSPF_ARGB1555:
                     nvdev->scaler_format = src_alpha
                                            ? SCALER_COLOR_FORMAT_A1R5G5B5
@@ -700,6 +741,9 @@ void nv_set_blittingflags( NVidiaDriverData *nvdrv,
           nvdev->state3d[1].blend  &= 0xFF00FFF0;
 
           switch (nvdev->src_format) {
+               case DSPF_RGB555:
+                    nvdev->state3d[1].format |= TXTRI_FORMAT_COLOR_X1R5G5B5;
+                    break;
                case DSPF_ARGB1555:
                     nvdev->state3d[1].format |= TXTRI_FORMAT_COLOR_A1R5G5B5;
                     break;
@@ -712,7 +756,8 @@ void nv_set_blittingflags( NVidiaDriverData *nvdrv,
                     break;
           }
           
-          if (state->blittingflags) {                         
+          if (state->blittingflags & (DSBLIT_BLEND_COLORALPHA | DSBLIT_COLORIZE | 
+                                      DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_SRC_PREMULTCOLOR)) {                         
                if (state->blittingflags & DSBLIT_BLEND_COLORALPHA)
                     nvdev->state3d[1].blend |= 
                          TXTRI_BLEND_TEXTUREMAPBLEND_MODULATEALPHA;
