@@ -46,55 +46,57 @@
 
 
 static __inline__ void
-out_vertex2d0( volatile __u8 *mmio, float x, float y )
+out_vertex2d0( volatile u8 *mmio, float x, float y )
 {
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = x;
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = y;
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(x) );
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(y) );
 }
 
 static __inline__ void
-out_vertex2d2( volatile __u8 *mmio, float x, float y, float s, float t )
+out_vertex2d2( volatile u8 *mmio, float x, float y, float s, float t )
 {
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = x;
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = y;
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = s;
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = t;
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(x) );
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(y) );
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(s) );
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(t) );
 }
 
 static __inline__ void
-out_vertex3d( volatile __u8 *mmio,
+out_vertex3d( volatile u8 *mmio,
               float x, float y, float z, float w, float s, float t )
 {
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = x;
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = y;
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = z;
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = w;
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = s;
-     *((volatile float*)(mmio+SE_PORT_DATA0)) = t;
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(x) );
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(y) );
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(z) );
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(w) );
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(s) );
+     radeon_out32( mmio, SE_PORT_DATA0, f2d(t) );
 }
 
 
 static void
-r200DoFillRectangle3D( RadeonDriverData *rdrv,
+r100DoFillRectangle3D( RadeonDriverData *rdrv,
                        RadeonDeviceData *rdev,
                        DFBRectangle     *rect )
 {
-     volatile __u8 *mmio = rdrv->mmio_base;
+     volatile u8 *mmio = rdrv->mmio_base;
 
      if (rect->w == 1 && rect->h == 1) {
           radeon_waitfifo( rdrv, rdev, 3 );
           
           radeon_out32( mmio, SE_VF_CNTL, VF_PRIM_TYPE_POINT_LIST |
-                                          VF_PRIM_WALK_DATA      |
+                                          VF_PRIM_WALK_DATA       |
+                                          VF_RADEON_MODE          |
                                           (1 << VF_NUM_VERTICES_SHIFT) );
 
-          out_vertex2d0( mmio, rect->x, rect->y );
+          out_vertex2d0( mmio, rect->x+1, rect->y+1 );
      }
      else {
           radeon_waitfifo( rdrv, rdev, 7 );
      
           radeon_out32( mmio, SE_VF_CNTL, VF_PRIM_TYPE_RECTANGLE_LIST |
                                           VF_PRIM_WALK_DATA           |
+                                          VF_RADEON_MODE              |
                                           (3 << VF_NUM_VERTICES_SHIFT) );
 
           out_vertex2d0( mmio, rect->x        , rect->y         );
@@ -103,25 +105,25 @@ r200DoFillRectangle3D( RadeonDriverData *rdrv,
      }
 }
 
-bool r200FillRectangle3D( void *drv, void *dev, DFBRectangle *rect )
+bool r100FillRectangle3D( void *drv, void *dev, DFBRectangle *rect )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
 
-     r200DoFillRectangle3D( rdrv, rdev, rect );
+     r100DoFillRectangle3D( rdrv, rdev, rect );
 
      return true;
 }
 
-bool r200FillRectangle3D_420( void *drv, void *dev, DFBRectangle *rect )
+bool r100FillRectangle3D_420( void *drv, void *dev, DFBRectangle *rect )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
-     DFBRegion      *clip = &rdev->clip;
-     volatile __u8  *mmio = rdrv->mmio_base;
+     DFBRegion        *clip = &rdev->clip;
+     volatile u8      *mmio = rdrv->mmio_base;
 
      /* Fill Luma plane */
-     r200DoFillRectangle3D( rdrv, rdev, rect );
+     r100DoFillRectangle3D( rdrv, rdev, rect );
           
      /* Scale coordinates */
      rect->x /= 2;
@@ -137,17 +139,17 @@ bool r200FillRectangle3D_420( void *drv, void *dev, DFBRectangle *rect )
                                       (clip->x1/2 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2/2 << 16) |
                                           (clip->x2/2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->cb_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->cb_cop );
 
      /* Fill Cb plane */
-     r200DoFillRectangle3D( rdrv, rdev, rect );
+     r100DoFillRectangle3D( rdrv, rdev, rect );
 
      radeon_waitfifo( rdrv, rdev, 2 );
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset_cr );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->cr_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->cr_cop );
 
      /* Fill Cr plane */
-     r200DoFillRectangle3D( rdrv, rdev, rect );
+     r100DoFillRectangle3D( rdrv, rdev, rect );
 
      /* Reset */
      radeon_waitfifo( rdrv, rdev, 5 );
@@ -157,22 +159,23 @@ bool r200FillRectangle3D_420( void *drv, void *dev, DFBRectangle *rect )
                                       (clip->x1 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2 << 16) |
                                           (clip->x2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->y_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->y_cop );
 
      return true;
 }
 
 static void
-r200DoFillTriangle( RadeonDriverData *rdrv,
+r100DoFillTriangle( RadeonDriverData *rdrv,
                     RadeonDeviceData *rdev,
                     DFBTriangle      *tri )
 {
-     volatile __u8 *mmio = rdrv->mmio_base;
+     volatile u8 *mmio = rdrv->mmio_base;
 
      radeon_waitfifo( rdrv, rdev, 7 );
      
      radeon_out32( mmio, SE_VF_CNTL, VF_PRIM_TYPE_TRIANGLE_LIST |
                                      VF_PRIM_WALK_DATA          |
+                                     VF_RADEON_MODE             |
                                      (3 << VF_NUM_VERTICES_SHIFT) );
      
      out_vertex2d0( mmio, tri->x1, tri->y1 );
@@ -180,25 +183,25 @@ r200DoFillTriangle( RadeonDriverData *rdrv,
      out_vertex2d0( mmio, tri->x3, tri->y3 );
 }
 
-bool r200FillTriangle( void *drv, void *dev, DFBTriangle *tri )
+bool r100FillTriangle( void *drv, void *dev, DFBTriangle *tri )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
 
-     r200DoFillTriangle( rdrv, rdev, tri );
+     r100DoFillTriangle( rdrv, rdev, tri );
      
      return true;
 }
 
-bool r200FillTriangle_420( void *drv, void *dev, DFBTriangle *tri )
+bool r100FillTriangle_420( void *drv, void *dev, DFBTriangle *tri )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
      DFBRegion        *clip = &rdev->clip;
-     volatile __u8    *mmio = rdrv->mmio_base;
+     volatile u8      *mmio = rdrv->mmio_base;
 
      /* Fill Luma plane */
-     r200DoFillTriangle( rdrv, rdev, tri );
+     r100DoFillTriangle( rdrv, rdev, tri );
           
      /* Scale coordinates */
      tri->x1 /= 2;
@@ -216,17 +219,17 @@ bool r200FillTriangle_420( void *drv, void *dev, DFBTriangle *tri )
                                       (clip->x1/2 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2/2 << 16) |
                                           (clip->x2/2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->cb_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->cb_cop );
 
      /* Fill Cb plane */
-     r200DoFillTriangle( rdrv, rdev, tri );
+     r100DoFillTriangle( rdrv, rdev, tri );
 
      radeon_waitfifo( rdrv, rdev, 2 );
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset_cr );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->cr_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->cr_cop );
 
      /* Fill Cr plane */
-     r200DoFillTriangle( rdrv, rdev, tri );
+     r100DoFillTriangle( rdrv, rdev, tri );
 
      /* Reset */
      radeon_waitfifo( rdrv, rdev, 5 );
@@ -235,18 +238,18 @@ bool r200FillTriangle_420( void *drv, void *dev, DFBTriangle *tri )
      radeon_out32( mmio, RE_TOP_LEFT, (clip->y1 << 16) |
                                       (clip->x1 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2 << 16) |
-                                        (clip->x2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->y_cop );
+                                          (clip->x2 & 0xffff) );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->y_cop );
      
      return true;
 }
 
 static void
-r200DoDrawRectangle3D( RadeonDriverData *rdrv,
+r100DoDrawRectangle3D( RadeonDriverData *rdrv,
                        RadeonDeviceData *rdev,
                        DFBRectangle     *rect )
 {
-     volatile __u8 *mmio = rdrv->mmio_base;
+     volatile u8 *mmio = rdrv->mmio_base;
 
      radeon_waitfifo( rdrv, rdev, 25 );
           
@@ -272,25 +275,25 @@ r200DoDrawRectangle3D( RadeonDriverData *rdrv,
      out_vertex2d0( mmio, rect->x+1        , rect->y+rect->h-1 );
 }
 
-bool r200DrawRectangle3D( void *drv, void *dev, DFBRectangle *rect )
+bool r100DrawRectangle3D( void *drv, void *dev, DFBRectangle *rect )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
 
-     r200DoDrawRectangle3D( rdrv, rdev, rect );
-     
+     r100DoDrawRectangle3D( rdrv, rdev, rect );
+
      return true;
 }
 
-bool r200DrawRectangle3D_420( void *drv, void *dev, DFBRectangle *rect )
+bool r100DrawRectangle3D_420( void *drv, void *dev, DFBRectangle *rect )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
      DFBRegion        *clip = &rdev->clip;
-     volatile __u8    *mmio = rdrv->mmio_base;
-
+     volatile u8      *mmio = rdrv->mmio_base;
+     
      /* Fill Luma plane */
-     r200DoDrawRectangle3D( rdrv, rdev, rect );
+     r100DoDrawRectangle3D( rdrv, rdev, rect );
           
      /* Scale coordinates */
      rect->x  /= 2;
@@ -306,17 +309,18 @@ bool r200DrawRectangle3D_420( void *drv, void *dev, DFBRectangle *rect )
                                       (clip->x1/2 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2/2 << 16) |
                                           (clip->x2/2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->cb_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->cb_cop );
 
      /* Fill Cb plane */
-     r200DoDrawRectangle3D( rdrv, rdev, rect );
+     r100DoDrawRectangle3D( rdrv, rdev, rect );
 
+     /* Prepare Cr plane */
      radeon_waitfifo( rdrv, rdev, 2 );
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset_cr );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->cr_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->cr_cop );
 
      /* Fill Cr plane */
-     r200DoDrawRectangle3D( rdrv, rdev, rect );
+     r100DoDrawRectangle3D( rdrv, rdev, rect );
 
      /* Reset */
      radeon_waitfifo( rdrv, rdev, 5 );
@@ -326,52 +330,53 @@ bool r200DrawRectangle3D_420( void *drv, void *dev, DFBRectangle *rect )
                                       (clip->x1 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2 << 16) |
                                           (clip->x2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->y_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->y_cop );
 
      return true;
 }
 
 static void
-r200DoDrawLine3D( RadeonDriverData *rdrv,
+r100DoDrawLine3D( RadeonDriverData *rdrv,
                   RadeonDeviceData *rdev,
                   DFBRegion        *line )
 {
-     volatile __u8 *mmio = rdrv->mmio_base;
+     volatile u8 *mmio = rdrv->mmio_base;
      
      radeon_waitfifo( rdrv, rdev, 5 );
      
      radeon_out32( mmio, SE_VF_CNTL, VF_PRIM_TYPE_LINE_LIST |
                                      VF_PRIM_WALK_DATA      |
+                                     VF_RADEON_MODE         |
                                      (2 << VF_NUM_VERTICES_SHIFT) );
 
      out_vertex2d0( mmio, line->x1, line->y1 );
      out_vertex2d0( mmio, line->x2, line->y2 );
 }
 
-bool r200DrawLine3D( void *drv, void *dev, DFBRegion *line )
+bool r100DrawLine3D( void *drv, void *dev, DFBRegion *line )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
 
-     r200DoDrawLine3D( rdrv, rdev, line );
+     r100DoDrawLine3D( rdrv, rdev, line );
 
      return true;
 }
 
-bool r200DrawLine3D_420( void *drv, void *dev, DFBRegion *line )
+bool r100DrawLine3D_420( void *drv, void *dev, DFBRegion *line )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
-     DFBRegion      *clip = &rdev->clip;
-     volatile __u8  *mmio = rdrv->mmio_base;
+     DFBRegion        *clip = &rdev->clip;
+     volatile u8      *mmio = rdrv->mmio_base;
      
      line->x1 &= ~1;
      line->y1 &= ~1;
      line->x2 &= ~1;
      line->y2 &= ~1;
-        
+          
      /* Fill Luma plane */
-     r200DoDrawLine3D( rdrv, rdev, line );
+     r100DoDrawLine3D( rdrv, rdev, line );
           
      /* Scale coordinates */
      line->x1 /= 2;
@@ -387,17 +392,17 @@ bool r200DrawLine3D_420( void *drv, void *dev, DFBRegion *line )
                                       (clip->x1/2 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2/2 << 16) |
                                           (clip->x2/2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->cb_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->cb_cop );
 
      /* Fill Cb plane */
-     r200DoDrawLine3D( rdrv, rdev, line );
+     r100DoDrawLine3D( rdrv, rdev, line );
 
      radeon_waitfifo( rdrv, rdev, 2 );
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset_cr );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->cr_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->cr_cop );
 
      /* Fill Cr plane */
-     r200DoDrawLine3D( rdrv, rdev, line );
+     r100DoDrawLine3D( rdrv, rdev, line );
 
      /* Reset */
      radeon_waitfifo( rdrv, rdev, 5 );
@@ -407,21 +412,22 @@ bool r200DrawLine3D_420( void *drv, void *dev, DFBRegion *line )
                                       (clip->x1 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2 << 16) |
                                           (clip->x2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_1, rdev->y_cop );
+     radeon_out32( mmio, PP_TFACTOR_1, rdev->y_cop );
 
      return true;
 }
 
 static void
-r200DoBlit3D( RadeonDriverData *rdrv, RadeonDeviceData *rdev,
+r100DoBlit3D( RadeonDriverData *rdrv, RadeonDeviceData *rdev,
               DFBRectangle     *sr,   DFBRectangle     *dr )
 {
-     volatile __u8 *mmio = rdrv->mmio_base;
+     volatile u8 *mmio = rdrv->mmio_base;
      
      radeon_waitfifo( rdrv, rdev, 13 );
 
      radeon_out32( mmio, SE_VF_CNTL, VF_PRIM_TYPE_RECTANGLE_LIST |
                                      VF_PRIM_WALK_DATA           |
+                                     VF_RADEON_MODE              |
                                      (3 << VF_NUM_VERTICES_SHIFT) );
      
      out_vertex2d2( mmio, dr->x      , dr->y      , sr->x      , sr->y       );
@@ -429,7 +435,7 @@ r200DoBlit3D( RadeonDriverData *rdrv, RadeonDeviceData *rdev,
      out_vertex2d2( mmio, dr->x+dr->w, dr->y+dr->h, sr->x+sr->w, sr->y+sr->h );
 }
 
-bool r200Blit3D( void *drv, void *dev, DFBRectangle *sr, int dx, int dy )
+bool r100Blit3D( void *drv, void *dev, DFBRectangle *sr, int dx, int dy )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
@@ -439,20 +445,20 @@ bool r200Blit3D( void *drv, void *dev, DFBRectangle *sr, int dx, int dy )
           sr->y /= 2;
           sr->h /= 2;
      }
+          
+     r100DoBlit3D( rdrv, rdev, sr, &dr );
 
-     r200DoBlit3D( rdrv, rdev, sr, &dr );
-     
      return true;
 }
 
-bool r200Blit3D_420( void *drv, void *dev, DFBRectangle *sr, int dx, int dy )
+bool r100Blit3D_420( void *drv, void *dev, DFBRectangle *sr, int dx, int dy )
 {
      DFBRectangle dr = { dx, dy, sr->w, sr->h };
-
-     return r200StretchBlit_420( drv, dev, sr, &dr );
+          
+     return r100StretchBlit_420( drv, dev, sr, &dr );
 }
 
-bool r200StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *dr )
+bool r100StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *dr )
 {
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
@@ -462,17 +468,17 @@ bool r200StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *dr )
           sr->h /= 2;
      }
      
-     r200DoBlit3D( rdrv, rdev, sr, dr );
+     r100DoBlit3D( rdrv, rdev, sr, dr );
      
      return true;
 }
 
-bool r200StretchBlit_420( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *dr )
+bool r100StretchBlit_420( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *dr )
 {
      RadeonDriverData *rdrv    = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev    = (RadeonDeviceData*) dev; 
      DFBRegion        *clip    = &rdev->clip;
-     volatile __u8    *mmio    = rdrv->mmio_base;
+     volatile u8      *mmio    = rdrv->mmio_base;
      bool              src_420 = DFB_PLANAR_PIXELFORMAT( rdev->src_format );
      
      if (rdev->blittingflags & DSBLIT_DEINTERLACE) {
@@ -481,7 +487,7 @@ bool r200StretchBlit_420( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *
      }
 
      /* Blit Luma plane */
-     r200DoBlit3D( rdrv, rdev, sr, dr );
+     r100DoBlit3D( rdrv, rdev, sr, dr );
 
      /* Scale coordinates */
      if (src_420) {
@@ -500,59 +506,61 @@ bool r200StretchBlit_420( void *drv, void *dev, DFBRectangle *sr, DFBRectangle *
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset_cb );
      radeon_out32( mmio, RB3D_COLORPITCH, rdev->dst_pitch/2 );
      if (src_420) {
-          radeon_out32( mmio, R200_PP_TXSIZE_0, ((rdev->src_height/2-1) << 16) |
-                                                ((rdev->src_width/2-1) & 0xffff) );
-          radeon_out32( mmio, R200_PP_TXPITCH_0, rdev->src_pitch/2 - 32 );
-          radeon_out32( mmio, R200_PP_TXOFFSET_0, rdev->src_offset_cb );
+          radeon_out32( mmio, PP_TEX_SIZE_0, ((rdev->src_height/2-1) << 16) |
+                                             ((rdev->src_width/2-1) & 0xffff) );
+          radeon_out32( mmio, PP_TEX_PITCH_0, rdev->src_pitch/2 - 32 );
+          radeon_out32( mmio, PP_TXOFFSET_0, rdev->src_offset_cb );
      }
      radeon_out32( mmio, RE_TOP_LEFT, (clip->y1/2 << 16) |
                                       (clip->x1/2 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2/2 << 16) | 
                                           (clip->x2/2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_0, rdev->cb_cop );
+     radeon_out32( mmio, PP_TFACTOR_0, rdev->cb_cop );
 
      /* Blit Cb plane */
-     r200DoBlit3D( rdrv, rdev, sr, dr );
+     r100DoBlit3D( rdrv, rdev, sr, dr );
 
      /* Prepare Cr plane */
      radeon_waitfifo( rdrv, rdev, src_420 ? 3 : 2 );
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset_cr );
      if (src_420)
-          radeon_out32( mmio, R200_PP_TXOFFSET_0, rdev->src_offset_cr );
-     radeon_out32( mmio, R200_PP_TFACTOR_0, rdev->cr_cop );
+          radeon_out32( mmio, PP_TXOFFSET_0, rdev->src_offset_cr );
+     radeon_out32( mmio, PP_TFACTOR_0, rdev->cr_cop );
 
      /* Blit Cr plane */
-     r200DoBlit3D( rdrv, rdev, sr, dr );
+     r100DoBlit3D( rdrv, rdev, sr, dr );
           
      /* Reset */
      radeon_waitfifo( rdrv, rdev, src_420 ? 8 : 5 );
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset );
      radeon_out32( mmio, RB3D_COLORPITCH, rdev->dst_pitch );
      if (src_420) {
-          radeon_out32( mmio, R200_PP_TXSIZE_0, ((rdev->src_height-1) << 16) |
-                                                ((rdev->src_width-1) & 0xffff) );
-          radeon_out32( mmio, R200_PP_TXPITCH_0, rdev->src_pitch - 32 );
-          radeon_out32( mmio, R200_PP_TXOFFSET_0, rdev->src_offset );
+          radeon_out32( mmio, PP_TEX_SIZE_0, ((rdev->src_height-1) << 16) |
+                                             ((rdev->src_width-1) & 0xffff) );
+          radeon_out32( mmio, PP_TEX_PITCH_0, rdev->src_pitch - 32 );
+          radeon_out32( mmio, PP_TXOFFSET_0, rdev->src_offset );
      }
      radeon_out32( mmio, RE_TOP_LEFT, (clip->y1 << 16) |
                                       (clip->x1 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2 << 16) |
                                           (clip->x2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_0, rdev->y_cop );
+     radeon_out32( mmio, PP_TFACTOR_0, rdev->y_cop );
      
      return true;
 }
 
 static void
-r200DoTextureTriangles( RadeonDriverData *rdrv, RadeonDeviceData *rdev,
-                        DFBVertex *ve, int num, __u32 primitive )
+r100DoTextureTriangles( RadeonDriverData *rdrv, RadeonDeviceData *rdev,
+                        DFBVertex *ve, int num, u32 primitive )
 {
-     volatile __u8 *mmio = rdrv->mmio_base;
+     volatile u8   *mmio = rdrv->mmio_base;
      int            i;
  
      radeon_waitfifo( rdrv, rdev, 1 ); 
      
-     radeon_out32( mmio, SE_VF_CNTL, primitive | VF_PRIM_WALK_DATA |
+     radeon_out32( mmio, SE_VF_CNTL, primitive         | 
+                                     VF_PRIM_WALK_DATA |
+                                     VF_RADEON_MODE    |
                                      (num << VF_NUM_VERTICES_SHIFT) );
 
      for (; num >= 10; num -= 10) {
@@ -571,15 +579,15 @@ r200DoTextureTriangles( RadeonDriverData *rdrv, RadeonDeviceData *rdev,
      }
 }
 
-bool r200TextureTriangles( void *drv, void *dev, DFBVertex *ve,
+bool r100TextureTriangles( void *drv, void *dev, DFBVertex *ve,
                            int num, DFBTriangleFormation formation )
 { 
      RadeonDriverData *rdrv = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev = (RadeonDeviceData*) dev;
-     __u32             prim = 0;
+     u32               prim = 0;
 
      if (num > 65535) {
-          D_WARN( "R200 supports maximum 65535 vertices" );
+          D_WARN( "R100 supports maximum 65535 vertices" );
           return false;
      }
 
@@ -598,24 +606,24 @@ bool r200TextureTriangles( void *drv, void *dev, DFBVertex *ve,
                return false;
      }
      
-     r200DoTextureTriangles( rdrv, rdev, ve, num, prim );
+     r100DoTextureTriangles( rdrv, rdev, ve, num, prim );
 
      return true;
 }
 
-bool r200TextureTriangles_420( void *drv, void *dev, DFBVertex *ve,
+bool r100TextureTriangles_420( void *drv, void *dev, DFBVertex *ve,
                                int num, DFBTriangleFormation formation )
 { 
      RadeonDriverData *rdrv    = (RadeonDriverData*) drv;
      RadeonDeviceData *rdev    = (RadeonDeviceData*) dev;
      DFBRegion        *clip    = &rdev->clip;
-     volatile __u8    *mmio    = rdrv->mmio_base;
+     volatile u8      *mmio    = rdrv->mmio_base;
      bool              src_420 = DFB_PLANAR_PIXELFORMAT( rdev->src_format );
-     __u32             prim    = 0;
+     u32               prim    = 0;
      int               i;
 
      if (num > 65535) {
-          D_WARN( "R200 supports maximum 65535 vertices" );
+          D_WARN( "R100 supports maximum 65535 vertices" );
           return false;
      }
 
@@ -635,8 +643,8 @@ bool r200TextureTriangles_420( void *drv, void *dev, DFBVertex *ve,
      }
 
      /* Map Luma plane */
-     r200DoTextureTriangles( rdrv, rdev, ve, num, prim );
-     
+     r100DoTextureTriangles( rdrv, rdev, ve, num, prim );
+
      /* Scale coordinates */
      for (i = 0; i < num; i++) {
           ve[i].x *= 0.5;
@@ -648,45 +656,45 @@ bool r200TextureTriangles_420( void *drv, void *dev, DFBVertex *ve,
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset_cb );
      radeon_out32( mmio, RB3D_COLORPITCH, rdev->dst_pitch/2 );
      if (src_420) {
-          radeon_out32( mmio, R200_PP_TXSIZE_0, ((rdev->src_height/2-1) << 16) |
-                                                ((rdev->src_width/2-1) & 0xffff) );
-          radeon_out32( mmio, R200_PP_TXPITCH_0, rdev->src_pitch/2 - 32 );
-          radeon_out32( mmio, R200_PP_TXOFFSET_0, rdev->src_offset_cb );
+          radeon_out32( mmio, PP_TEX_SIZE_0, ((rdev->src_height/2-1) << 16) |
+                                             ((rdev->src_width/2-1) & 0xffff) );
+          radeon_out32( mmio, PP_TEX_PITCH_0, rdev->src_pitch/2 - 32 );
+          radeon_out32( mmio, PP_TXOFFSET_0, rdev->src_offset_cb );
      }
      radeon_out32( mmio, RE_TOP_LEFT, (clip->y1/2 << 16) |
                                       (clip->x1/2 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2/2 << 16) | 
                                           (clip->x2/2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_0, rdev->cb_cop );
+     radeon_out32( mmio, PP_TFACTOR_0, rdev->cb_cop );
      
      /* Map Cb plane */
-     r200DoTextureTriangles( rdrv, rdev, ve, num, prim );
+     r100DoTextureTriangles( rdrv, rdev, ve, num, prim );
      
      /* Prepare Cr plane */
      radeon_waitfifo( rdrv, rdev, src_420 ? 3 : 2 );
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset_cr );
      if (src_420)
-          radeon_out32( mmio, R200_PP_TXOFFSET_0, rdev->src_offset_cr );
-     radeon_out32( mmio, R200_PP_TFACTOR_0, rdev->cr_cop );
+          radeon_out32( mmio, PP_TXOFFSET_0, rdev->src_offset_cr );
+     radeon_out32( mmio, PP_TFACTOR_0, rdev->cr_cop );
 
      /* Map Cr plane */
-     r200DoTextureTriangles( rdrv, rdev, ve, num, prim );
+     r100DoTextureTriangles( rdrv, rdev, ve, num, prim );
      
      /* Reset */
      radeon_waitfifo( rdrv, rdev, src_420 ? 8 : 5 );
      radeon_out32( mmio, RB3D_COLOROFFSET, rdev->dst_offset );
      radeon_out32( mmio, RB3D_COLORPITCH, rdev->dst_pitch );
      if (src_420) {
-          radeon_out32( mmio, R200_PP_TXSIZE_0, ((rdev->src_height-1) << 16) |
-                                                ((rdev->src_width-1) & 0xffff) );
-          radeon_out32( mmio, R200_PP_TXPITCH_0, rdev->src_pitch - 32 );
-          radeon_out32( mmio, R200_PP_TXOFFSET_0, rdev->src_offset );
+          radeon_out32( mmio, PP_TEX_SIZE_0, ((rdev->src_height-1) << 16) |
+                                             ((rdev->src_width-1) & 0xffff) );
+          radeon_out32( mmio, PP_TEX_PITCH_0, rdev->src_pitch - 32 );
+          radeon_out32( mmio, PP_TXOFFSET_0, rdev->src_offset );
      }
      radeon_out32( mmio, RE_TOP_LEFT, (clip->y1 << 16) |
                                       (clip->x1 & 0xffff) );
      radeon_out32( mmio, RE_BOTTOM_RIGHT, (clip->y2 << 16) |
                                           (clip->x2 & 0xffff) );
-     radeon_out32( mmio, R200_PP_TFACTOR_0, rdev->y_cop );
+     radeon_out32( mmio, PP_TFACTOR_0, rdev->y_cop );
      
      return true;
 }
