@@ -44,7 +44,7 @@
 #include "radeon_state.h"
 
 
-static __u32 r200SrcBlend[] = {
+static const u32 r200SrcBlend[] = {
      SRC_BLEND_GL_ZERO,                 // DSBF_ZERO
      SRC_BLEND_GL_ONE,                  // DSBF_ONE
      SRC_BLEND_GL_SRC_COLOR,            // DSBF_SRCCOLOR
@@ -58,7 +58,7 @@ static __u32 r200SrcBlend[] = {
      SRC_BLEND_GL_SRC_ALPHA_SATURATE    // DSBF_SRCALPHASAT
 };
 
-static __u32 r200DstBlend[] = {
+static const u32 r200DstBlend[] = {
      DST_BLEND_GL_ZERO,                 // DSBF_ZERO
      DST_BLEND_GL_ONE,                  // DSBF_ONE
      DST_BLEND_GL_SRC_COLOR,            // DSBF_SRCCOLOR
@@ -73,15 +73,55 @@ static __u32 r200DstBlend[] = {
 };
 
 
+void r200_restore( RadeonDriverData *rdrv, RadeonDeviceData *rdev )
+{
+     volatile u8 *mmio = rdrv->mmio_base;
+     
+     radeon_waitfifo( rdrv, rdev, 19 );
+     /* enable caches */
+     radeon_out32( mmio, RB2D_DSTCACHE_MODE, RB2D_DC_2D_CACHE_AUTOFLUSH     |
+                                             RB2D_DC_3D_CACHE_AUTOFLUSH     |
+                                             R200_RB2D_DC_2D_CACHE_AUTOFREE |
+                                             R200_RB2D_DC_3D_CACHE_AUTOFREE );
+     radeon_out32( mmio, RB3D_DSTCACHE_MODE, RB3D_DC_2D_CACHE_AUTOFLUSH     |
+                                             RB3D_DC_3D_CACHE_AUTOFLUSH     |
+                                             R200_RB3D_DC_2D_CACHE_AUTOFREE |
+                                             R200_RB3D_DC_3D_CACHE_AUTOFREE );          
+     /* restore 3d engine state */
+     radeon_out32( mmio, SE_LINE_WIDTH, 0x10 );
+     radeon_out32( mmio, RE_POINTSIZE, 0x10 );
+     radeon_out32( mmio, PP_MISC, ALPHA_TEST_PASS ); 
+     radeon_out32( mmio, R200_PP_CNTL_X, 0 );
+     radeon_out32( mmio, R200_PP_TXMULTI_CTL_0, 0 ); 
+     radeon_out32( mmio, R200_RE_CNTL, R200_SCISSOR_ENABLE );
+     radeon_out32( mmio, R200_SE_VTX_STATE_CNTL, 0 );
+     radeon_out32( mmio, R200_SE_VAP_CNTL, R200_VAP_VF_MAX_VTX_NUM |
+                                           R200_VAP_FORCE_W_TO_ONE );
+#ifdef WORDS_BIGENDIAN
+     radeon_out32( mmio, R200_SE_VAP_CNTL_STATUS, R200_TCL_BYPASS | R200_VC_32BIT_SWAP );
+#else
+     radeon_out32( mmio, R200_SE_VAP_CNTL_STATUS, R200_TCL_BYPASS );
+#endif
+     radeon_out32( mmio, RB3D_ZSTENCILCNTL, Z_TEST_ALWAYS );
+     radeon_out32( mmio, RB3D_ROPCNTL, ROP_XOR );
+     radeon_out32( mmio, R200_PP_TXFORMAT_X_0, 0 );
+     radeon_out32( mmio, R200_PP_BORDER_COLOR_0, 0 );
+     /* set YUV422 color buffer */
+     radeon_out32( mmio, R200_PP_TXFILTER_1, 0 );
+     radeon_out32( mmio, R200_PP_TXFORMAT_1, R200_TXFORMAT_VYUY422 );
+     radeon_out32( mmio, R200_PP_TXFORMAT_X_1, 0 );
+     radeon_out32( mmio, R200_PP_BORDER_COLOR_1, 0 );
+}
+
 void r200_set_destination( RadeonDriverData *rdrv,
                            RadeonDeviceData *rdev,
                            CardState        *state )
 {
      CoreSurface   *surface = state->destination;
      SurfaceBuffer *buffer  = surface->back_buffer;
-     volatile __u8 *mmio    = rdrv->mmio_base;
-     __u32          offset;
-     __u32          pitch;
+     volatile u8   *mmio    = rdrv->mmio_base;
+     u32            offset;
+     u32            pitch;
     
      if (RADEON_IS_SET( DESTINATION ))
           return;
@@ -113,10 +153,12 @@ void r200_set_destination( RadeonDriverData *rdrv,
                     rdev->gui_master_cntl = GMC_DST_16BPP;
                     rdev->rb3d_cntl = COLOR_FORMAT_RGB565;
                     break;
+               case DSPF_RGB444:
                case DSPF_ARGB4444:
                     rdev->gui_master_cntl = GMC_DST_16BPP;
                     rdev->rb3d_cntl = COLOR_FORMAT_ARGB4444 | DITHER_ENABLE;
                     break;
+               case DSPF_RGB555:
                case DSPF_ARGB1555:          
                     rdev->gui_master_cntl = GMC_DST_15BPP;
                     rdev->rb3d_cntl = COLOR_FORMAT_ARGB1555 | DITHER_ENABLE;
@@ -167,30 +209,13 @@ void r200_set_destination( RadeonDriverData *rdrv,
                                    GMC_DST_PITCH_OFFSET_CNTL |
                                    GMC_DST_CLIPPING;
           
-          radeon_waitfifo( rdrv, rdev, 2 ); 
+          radeon_waitfifo( rdrv, rdev, 4 ); 
           radeon_out32( mmio, DST_OFFSET, offset );
-          radeon_out32( mmio, DST_PITCH,  pitch );
-          
-          radeon_waitfifo( rdrv, rdev, 2 );
+          radeon_out32( mmio, DST_PITCH, pitch );
           radeon_out32( mmio, RB3D_COLOROFFSET, offset );
           radeon_out32( mmio, RB3D_COLORPITCH,  
                               pitch / DFB_BYTES_PER_PIXEL(buffer->format) );
           
-          if (surface->caps & DSCAPS_DEPTH) {
-               SurfaceBuffer *depth = surface->depth_buffer;
-               
-               offset = radeon_buffer_offset( rdev, depth );
-               pitch  = depth->video.pitch >> 1;
-               
-               radeon_waitfifo( rdrv, rdev, 3 );
-               radeon_out32( mmio, RB3D_DEPTHOFFSET, offset );
-               radeon_out32( mmio, RB3D_DEPTHPITCH,  pitch );
-               radeon_out32( mmio, RB3D_ZSTENCILCNTL, DEPTH_FORMAT_16BIT_INT_Z |
-                                                      Z_TEST_ALWAYS );
-          
-               rdev->rb3d_cntl |= Z_ENABLE;
-          }
-          
           if (rdev->dst_format != buffer->format) {
                if (dst_422 && !rdev->dst_422) {
                     RADEON_UNSET( SOURCE );
@@ -198,7 +223,7 @@ void r200_set_destination( RadeonDriverData *rdrv,
                }
                
                RADEON_UNSET( COLOR );
-               RADEON_UNSET( SRC_BLEND );
+               RADEON_UNSET( DST_BLEND );
           }
           
           rdev->dst_format = buffer->format;
@@ -216,9 +241,9 @@ void r200_set_source( RadeonDriverData *rdrv,
 {
      CoreSurface   *surface  = state->source;
      SurfaceBuffer *buffer   = surface->front_buffer;
-     volatile __u8 *mmio     = rdrv->mmio_base;
-     __u32          txformat = R200_TXFORMAT_NON_POWER2;
-     __u32          txfilter = R200_MAG_FILTER_LINEAR  |
+     volatile u8   *mmio     = rdrv->mmio_base;
+     u32            txformat = R200_TXFORMAT_NON_POWER2;
+     u32            txfilter = R200_MAG_FILTER_LINEAR  |
                                R200_MIN_FILTER_LINEAR  |
                                R200_CLAMP_S_CLAMP_LAST |
                                R200_CLAMP_T_CLAMP_LAST;
@@ -265,11 +290,19 @@ void r200_set_source( RadeonDriverData *rdrv,
                              R200_MIN_FILTER_LINEAR);
                rdev->src_mask = 0x00003fff;
                break;
+          case DSPF_RGB444:
+               txformat |= R200_TXFORMAT_ARGB4444;
+               rdev->src_mask = 0x00000fff;
+               break;     
           case DSPF_ARGB4444:
                txformat |= R200_TXFORMAT_ARGB4444 |
                            R200_TXFORMAT_ALPHA_IN_MAP;
                rdev->src_mask = 0x00000fff;
                break;
+          case DSPF_RGB555:
+               txformat |= R200_TXFORMAT_ARGB1555;
+               rdev->src_mask = 0x00007fff;
+               break; 
           case DSPF_ARGB1555:
                txformat |= R200_TXFORMAT_ARGB1555 |
                            R200_TXFORMAT_ALPHA_IN_MAP;
@@ -341,14 +374,11 @@ void r200_set_source( RadeonDriverData *rdrv,
           }
      }
 
-     radeon_waitfifo( rdrv, rdev, 2 );
+     radeon_waitfifo( rdrv, rdev, 7 );
      radeon_out32( mmio, SRC_OFFSET, rdev->src_offset );
-     radeon_out32( mmio, SRC_PITCH,  rdev->src_pitch );
-           
-     radeon_waitfifo( rdrv, rdev, 6 );
+     radeon_out32( mmio, SRC_PITCH, rdev->src_pitch );
      radeon_out32( mmio, R200_PP_TXFILTER_0, txfilter );
      radeon_out32( mmio, R200_PP_TXFORMAT_0, txformat );
-     radeon_out32( mmio, R200_PP_TXFORMAT_X_0, 0 );
      radeon_out32( mmio, R200_PP_TXSIZE_0, ((rdev->src_height-1) << 16) | 
                                            ((rdev->src_width-1) & 0xffff) );
      radeon_out32( mmio, R200_PP_TXPITCH_0, rdev->src_pitch - 32 );
@@ -365,8 +395,8 @@ void r200_set_clip( RadeonDriverData *rdrv,
                     RadeonDeviceData *rdev,
                     CardState        *state )
 {
-     DFBRegion     *clip = &state->clip;
-     volatile __u8 *mmio = rdrv->mmio_base;
+     DFBRegion   *clip = &state->clip;
+     volatile u8 *mmio = rdrv->mmio_base;
      
      if (RADEON_IS_SET( CLIP ))
           return;
@@ -412,13 +442,19 @@ void r200_set_drawing_color( RadeonDriverData *rdrv,
 {
      DFBColor color   = state->color;
      int      index   = state->color_index;
-     __u32    color2d;
-     __u32    color3d;
+     u32      color2d;
+     u32      color3d;
      int      y, u, v;
 
      if (RADEON_IS_SET( COLOR ) && RADEON_IS_SET( DRAWING_FLAGS ))
           return;
 
+     if (state->drawingflags & DSDRAW_SRC_PREMULTIPLY) {
+          color.r = ((long) color.r * color.a / 255L);
+          color.g = ((long) color.g * color.a / 255L);
+          color.b = ((long) color.b * color.a / 255L);
+     }
+
      color3d = PIXEL_ARGB( color.a, color.r,
                            color.g, color.b );
  
@@ -440,10 +476,12 @@ void r200_set_drawing_color( RadeonDriverData *rdrv,
                color2d = PIXEL_ARGB2554( color.a, color.r,
                                          color.g, color.b );
                break;
+          case DSPF_RGB444:
           case DSPF_ARGB4444:
                color2d = PIXEL_ARGB4444( color.a, color.r,
                                          color.g, color.b );
                break;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                color2d = PIXEL_ARGB1555( color.a, color.r,
                                          color.g, color.b );
@@ -502,7 +540,7 @@ void r200_set_blitting_color( RadeonDriverData *rdrv,
                               CardState        *state )
 {
      DFBColor color   = state->color;
-     __u32    color3d;
+     u32      color3d;
      int      y, u, v;
      
      if (RADEON_IS_SET( COLOR ) && RADEON_IS_SET( BLITTING_FLAGS ))
@@ -551,7 +589,7 @@ void r200_set_src_colorkey( RadeonDriverData *rdrv,
                             RadeonDeviceData *rdev,
                             CardState        *state )
 {
-     volatile __u8 *mmio = rdrv->mmio_base;
+     volatile u8 *mmio = rdrv->mmio_base;
      
      if (RADEON_IS_SET( SRC_COLORKEY ))
           return;
@@ -563,14 +601,13 @@ void r200_set_src_colorkey( RadeonDriverData *rdrv,
      RADEON_SET( SRC_COLORKEY );
 }
 
-void
-r200_set_blend_function( RadeonDriverData *rdrv,
-                         RadeonDeviceData *rdev,
-                         CardState        *state )
+void r200_set_blend_function( RadeonDriverData *rdrv,
+                              RadeonDeviceData *rdev,
+                              CardState        *state )
 {
-     volatile __u8 *mmio   = rdrv->mmio_base;
-     __u32          sblend;
-     __u32          dblend;
+     volatile u8   *mmio   = rdrv->mmio_base;
+     u32            sblend;
+     u32            dblend;
      
      if (RADEON_IS_SET( SRC_BLEND ) && RADEON_IS_SET( DST_BLEND ))
           return;
@@ -578,17 +615,16 @@ r200_set_blend_function( RadeonDriverData *rdrv,
      sblend = r200SrcBlend[state->src_blend-1];
      dblend = r200DstBlend[state->dst_blend-1];
 
-     if (!DFB_PIXELFORMAT_HAS_ALPHA( rdev->dst_format )) {
-          switch (state->src_blend) {
-               case DSBF_DESTALPHA:
-                    sblend = SRC_BLEND_GL_ONE;
-                    break;
-               case DSBF_INVDESTALPHA:
-                    sblend = SRC_BLEND_GL_ZERO;
-                    break;
-               default:
-                    break;
-          }
+     if (!DFB_PIXELFORMAT_HAS_ALPHA(rdev->dst_format)) {
+          if (sblend == SRC_BLEND_GL_DST_ALPHA)
+               sblend = SRC_BLEND_GL_ONE;
+          else if (sblend == SRC_BLEND_GL_ONE_MINUS_DST_ALPHA)
+               sblend = SRC_BLEND_GL_ZERO;
+                    
+          if (dblend == DST_BLEND_GL_DST_ALPHA)
+               dblend = DST_BLEND_GL_ONE;
+          else if (dblend == DST_BLEND_GL_ONE_MINUS_DST_ALPHA)
+               dblend = DST_BLEND_GL_ZERO;
      }
 
      radeon_waitfifo( rdrv, rdev, 1 ); 
@@ -608,14 +644,14 @@ void r200_set_drawingflags( RadeonDriverData *rdrv,
                             RadeonDeviceData *rdev,
                             CardState        *state )
 {
-     volatile __u8 *mmio        = rdrv->mmio_base;
-     __u32          master_cntl = rdev->gui_master_cntl       |
+     volatile u8   *mmio        = rdrv->mmio_base;
+     u32            master_cntl = rdev->gui_master_cntl       |
                                   GMC_SRC_DATATYPE_MONO_FG_LA | 
                                   GMC_BRUSH_SOLID_COLOR       |
                                   GMC_CLR_CMP_CNTL_DIS;
-     __u32          rb3d_cntl   = rdev->rb3d_cntl & ~DITHER_ENABLE;
-     __u32          pp_cntl     = TEX_BLEND_1_ENABLE;
-     __u32          cblend      = R200_TXC_ARG_C_TFACTOR_COLOR;
+     u32            rb3d_cntl   = rdev->rb3d_cntl & ~DITHER_ENABLE;
+     u32            pp_cntl     = TEX_BLEND_1_ENABLE;
+     u32            cblend      = R200_TXC_ARG_C_TFACTOR_COLOR;
      
      if (RADEON_IS_SET( DRAWING_FLAGS ))
           return;
@@ -638,11 +674,9 @@ void r200_set_drawingflags( RadeonDriverData *rdrv,
      } else
           master_cntl |= GMC_ROP3_PATCOPY;
      
-     radeon_waitfifo( rdrv, rdev, 2 );
+     radeon_waitfifo( rdrv, rdev, 11 );
      radeon_out32( mmio, DP_GUI_MASTER_CNTL, master_cntl );
      radeon_out32( mmio, DP_CNTL, DST_X_LEFT_TO_RIGHT | DST_Y_TOP_TO_BOTTOM );
-     
-     radeon_waitfifo( rdrv, rdev, 9 );
      radeon_out32( mmio, RB3D_CNTL, rb3d_cntl );
      radeon_out32( mmio, SE_CNTL, DIFFUSE_SHADE_FLAT  |
                                   ALPHA_SHADE_FLAT    |
@@ -673,21 +707,21 @@ void r200_set_blittingflags( RadeonDriverData *rdrv,
                              RadeonDeviceData *rdev,
                              CardState        *state )
 {
-     volatile __u8 *mmio        = rdrv->mmio_base;
-     __u32          master_cntl = rdev->gui_master_cntl |
+     volatile u8   *mmio        = rdrv->mmio_base;
+     u32            master_cntl = rdev->gui_master_cntl |
                                   GMC_BRUSH_NONE        |
                                   GMC_SRC_DATATYPE_COLOR;
-     __u32          cmp_cntl    = 0;
-     __u32          rb3d_cntl   = rdev->rb3d_cntl;
-     __u32          se_cntl     = BFACE_SOLID        |
+     u32            cmp_cntl    = 0;
+     u32            rb3d_cntl   = rdev->rb3d_cntl;
+     u32            se_cntl     = BFACE_SOLID        |
                                   FFACE_SOLID        |
                                   VTX_PIX_CENTER_OGL |
                                   ROUND_MODE_ROUND;
-     __u32          pp_cntl     = TEX_0_ENABLE;
-     __u32          cblend      = R200_TXC_ARG_C_R0_COLOR;
-     __u32          ablend      = R200_TXA_ARG_C_R0_ALPHA;
-     __u32          vtx_fmt     = R200_VTX_XY;
-     __u32          vte_cntl;
+     u32            pp_cntl     = TEX_0_ENABLE;
+     u32            cblend      = R200_TXC_ARG_C_R0_COLOR;
+     u32            ablend      = R200_TXA_ARG_C_R0_ALPHA;
+     u32            vtx_fmt     = R200_VTX_XY;
+     u32            vte_cntl;
      
      if (RADEON_IS_SET( BLITTING_FLAGS ))
           return;
@@ -768,11 +802,9 @@ void r200_set_blittingflags( RadeonDriverData *rdrv,
      } else
           master_cntl |= GMC_ROP3_SRCCOPY;
 
-     radeon_waitfifo( rdrv, rdev, 2 );
+     radeon_waitfifo( rdrv, rdev, 12 );
      radeon_out32( mmio, CLR_CMP_CNTL, cmp_cntl );
      radeon_out32( mmio, DP_GUI_MASTER_CNTL, master_cntl );
-     
-     radeon_waitfifo( rdrv, rdev, 10 );
      radeon_out32( mmio, RB3D_CNTL, rb3d_cntl );
      radeon_out32( mmio, SE_CNTL, se_cntl );
      radeon_out32( mmio, PP_CNTL, pp_cntl );
