@@ -60,6 +60,7 @@ typedef struct {
      float                  saturation;
      float                  hue;
      int                    field;
+     int                    level;
    
      CoreScreen            *screen;
      int                    crtc2;
@@ -68,36 +69,36 @@ typedef struct {
      
      /* overlay registers */
      struct {
-          __u32 H_INC;
-          __u32 STEP_BY;
-          __u32 Y_X_START;
-          __u32 Y_X_END;
-          __u32 V_INC;
-          __u32 P1_BLANK_LINES_AT_TOP;
-          __u32 P23_BLANK_LINES_AT_TOP;
-          __u32 VID_BUF_PITCH0_VALUE;
-          __u32 VID_BUF_PITCH1_VALUE;
-          __u32 P1_X_START_END;
-          __u32 P2_X_START_END;
-          __u32 P3_X_START_END;
-          __u32 BASE_ADDR;
-          __u32 VID_BUF0_BASE_ADRS;
-          __u32 VID_BUF1_BASE_ADRS;
-          __u32 VID_BUF2_BASE_ADRS;
-          __u32 VID_BUF3_BASE_ADRS;
-          __u32 VID_BUF4_BASE_ADRS;
-          __u32 VID_BUF5_BASE_ADRS;
-          __u32 P1_V_ACCUM_INIT;
-          __u32 P23_V_ACCUM_INIT;
-          __u32 P1_H_ACCUM_INIT;
-          __u32 P23_H_ACCUM_INIT;
-          __u32 VID_KEY_CLR_LOW;
-          __u32 VID_KEY_CLR_HIGH;
-          __u32 GRPH_KEY_CLR_LOW;
-          __u32 GRPH_KEY_CLR_HIGH;
-          __u32 KEY_CNTL;
-          __u32 MERGE_CNTL;
-          __u32 SCALE_CNTL;
+          u32 H_INC;
+          u32 STEP_BY;
+          u32 Y_X_START;
+          u32 Y_X_END;
+          u32 V_INC;
+          u32 P1_BLANK_LINES_AT_TOP;
+          u32 P23_BLANK_LINES_AT_TOP;
+          u32 VID_BUF_PITCH0_VALUE;
+          u32 VID_BUF_PITCH1_VALUE;
+          u32 P1_X_START_END;
+          u32 P2_X_START_END;
+          u32 P3_X_START_END;
+          u32 BASE_ADDR;
+          u32 VID_BUF0_BASE_ADRS;
+          u32 VID_BUF1_BASE_ADRS;
+          u32 VID_BUF2_BASE_ADRS;
+          u32 VID_BUF3_BASE_ADRS;
+          u32 VID_BUF4_BASE_ADRS;
+          u32 VID_BUF5_BASE_ADRS;
+          u32 P1_V_ACCUM_INIT;
+          u32 P23_V_ACCUM_INIT;
+          u32 P1_H_ACCUM_INIT;
+          u32 P23_H_ACCUM_INIT;
+          u32 VID_KEY_CLR_LOW;
+          u32 VID_KEY_CLR_HIGH;
+          u32 GRPH_KEY_CLR_LOW;
+          u32 GRPH_KEY_CLR_HIGH;
+          u32 KEY_CNTL;
+          u32 MERGE_CNTL;
+          u32 SCALE_CNTL;
      } regs;
 } RadeonOverlayLayerData;
 
@@ -123,7 +124,7 @@ static void ovl_set_adjustment( RadeonDriverData        *rdrv,
                                 float                    saturation,
                                 float                    hue );
                 
-#define OV0_SUPPORTED_OPTIONS \
+#define OVL_SUPPORTED_OPTIONS \
      ( DLOP_DST_COLORKEY | DLOP_OPACITY | DLOP_DEINTERLACING )
 
 /**********************/
@@ -144,12 +145,14 @@ ovlInitLayer( CoreLayer                  *layer,
 {
      RadeonDriverData       *rdrv = (RadeonDriverData*) driver_data;
      RadeonOverlayLayerData *rovl = (RadeonOverlayLayerData*) layer_data;
-     volatile __u8          *mmio = rdrv->mmio_base;
+     volatile u8            *mmio = rdrv->mmio_base;
      DFBScreenDescription    dsc;
      
      dfb_screen_get_info( layer->screen, NULL, &dsc );
      if (strstr( dsc.name, "CRTC2" ))
           rovl->crtc2 = 1;
+          
+     rovl->level = 1;
      
      /* fill layer description */
      description->type = DLTF_GRAPHICS | DLTF_VIDEO | DLTF_STILL_PICTURE;
@@ -157,7 +160,7 @@ ovlInitLayer( CoreLayer                  *layer,
                          DLCAPS_BRIGHTNESS    | DLCAPS_CONTRAST        |
                          DLCAPS_SATURATION    | DLCAPS_HUE             |
                          DLCAPS_DST_COLORKEY  | DLCAPS_OPACITY         |
-                         DLCAPS_DEINTERLACING;
+                         DLCAPS_DEINTERLACING | DLCAPS_LEVELS;
 
      snprintf( description->name,
                DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, 
@@ -182,13 +185,12 @@ ovlInitLayer( CoreLayer                  *layer,
      adjustment->hue        = 0x8000;
 
      /* reset overlay */
-     radeon_waitfifo( rdrv, rdrv->device_data, 6 );
      radeon_out32( mmio, OV0_SCALE_CNTL, SCALER_SOFT_RESET ); 
      radeon_out32( mmio, OV0_AUTO_FLIP_CNTL, 0 );
      radeon_out32( mmio, OV0_DEINTERLACE_PATTERN, 0 );
      radeon_out32( mmio, OV0_EXCLUSIVE_HORZ, 0 ); 
      radeon_out32( mmio, OV0_FILTER_CNTL, FILTER_HARDCODED_COEF );
-     radeon_out32( mmio, OV0_TEST, 0 ); 
+     radeon_out32( mmio, OV0_TEST, 0 );
      
      /* reset color adjustments */
      ovl_set_adjustment( rdrv, rovl, 0, 0, 0, 0 );
@@ -203,15 +205,22 @@ ovlTestRegion( CoreLayer                  *layer,
                CoreLayerRegionConfig      *config,
                CoreLayerRegionConfigFlags *failed )
 {
-     CoreLayerRegionConfigFlags fail = 0;
+     RadeonOverlayLayerData     *rovl = (RadeonOverlayLayerData*) layer_data;
+     CoreLayerRegionConfigFlags  fail = 0;
 
      /* check for unsupported options */
-     if (config->options & ~OV0_SUPPORTED_OPTIONS)
+     if (config->options & ~OVL_SUPPORTED_OPTIONS)
           fail |= CLRCF_OPTIONS;
 
-     if (config->options & DLOP_OPACITY &&
-         config->options & (DLOP_SRC_COLORKEY | DLOP_DST_COLORKEY))
-          fail |= CLRCF_OPTIONS;
+     if (rovl->level == -1) {
+          if (config->options & ~DLOP_DEINTERLACING)
+               fail |= CLRCF_OPTIONS;
+     }
+     else {
+          if (config->options &  DLOP_OPACITY &&
+              config->options & (DLOP_SRC_COLORKEY | DLOP_DST_COLORKEY))
+               fail |= CLRCF_OPTIONS;
+     }
 
      /* check buffermode */
      switch (config->buffermode) {
@@ -228,6 +237,7 @@ ovlTestRegion( CoreLayer                  *layer,
      
      /* check pixel format */
      switch (config->format) {
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_RGB32:
@@ -382,6 +392,45 @@ ovlSetInputField( CoreLayer *layer,
      return DFB_OK;
 }
 
+static DFBResult
+ovlGetLevel( CoreLayer *layer,
+             void      *driver_data,
+             void      *layer_data,
+             int       *level )
+{
+     RadeonOverlayLayerData *rovl = (RadeonOverlayLayerData*) layer_data;
+     
+     *level = rovl->level;
+     
+     return DFB_OK;
+}
+
+static DFBResult
+ovlSetLevel( CoreLayer *layer,
+             void      *driver_data,
+             void      *layer_data,
+             int        level )
+{
+     RadeonDriverData       *rdrv = (RadeonDriverData*) driver_data;
+     RadeonOverlayLayerData *rovl = (RadeonOverlayLayerData*) layer_data;
+     
+     if (!rovl->surface)
+          return DFB_UNSUPPORTED;
+     
+     switch (level) {
+          case -1:
+          case  1:
+               rovl->level = level;
+               ovl_calc_regs( rdrv, rovl, rovl->surface, &rovl->config );
+               ovl_set_regs( rdrv, rovl );
+               break;
+          default:
+               return DFB_UNSUPPORTED;
+     }
+     
+     return DFB_OK;
+}
+
 static DFBResult
 ovlRemoveRegion( CoreLayer *layer,
                  void      *driver_data,
@@ -408,7 +457,9 @@ DisplayLayerFuncs RadeonOverlayFuncs = {
      .RemoveRegion       = ovlRemoveRegion,
      .FlipRegion         = ovlFlipRegion,
      .SetColorAdjustment = ovlSetColorAdjustment,
-     .SetInputField      = ovlSetInputField
+     .SetInputField      = ovlSetInputField,
+     .GetLevel           = ovlGetLevel,
+     .SetLevel           = ovlSetLevel
 };
 
 
@@ -423,12 +474,12 @@ ovl_calc_coordinates( RadeonDriverData       *rdrv,
      RadeonDeviceData *rdev    = rdrv->device_data;
      DFBRectangle      source  = config->source;
      DFBRectangle      dest    = config->dest; 
-     __u32             ecp_div = 0;
-     __u32             h_inc;
-     __u32             h_inc2;
-     __u32             v_inc;
-     __u32             step_by;
-     __u32             tmp;
+     u32               ecp_div = 0;
+     u32               h_inc;
+     u32               h_inc2;
+     u32               v_inc;
+     u32               step_by;
+     u32               tmp;
      int               xres;
      int               yres;
 
@@ -497,11 +548,12 @@ ovl_calc_coordinates( RadeonDriverData       *rdrv,
           h_inc >>= 1;
      }
 
-     switch (surface->format) { 
-          case DSPF_RGB32:
-          case DSPF_ARGB:
+     switch (surface->format) {
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
           case DSPF_RGB16:
+          case DSPF_RGB32:
+          case DSPF_ARGB:
                h_inc2 = h_inc;
                break;
           default:
@@ -569,8 +621,8 @@ ovl_calc_buffers( RadeonDriverData       *rdrv,
      RadeonDeviceData *rdev       = rdrv->device_data;
      SurfaceBuffer    *buffer     = surface->front_buffer;
      DFBRectangle      source     = config->source;
-     __u32             offsets[3] = { 0, 0, 0 };
-     __u32             pitch      = buffer->video.pitch;
+     u32               offsets[3] = { 0, 0, 0 };
+     u32               pitch      = buffer->video.pitch;
      int               even       = 0;
      int               cropleft;
      int               croptop;
@@ -609,7 +661,7 @@ ovl_calc_buffers( RadeonDriverData       *rdrv,
           }
 
           if (surface->format == DSPF_YV12) {
-               __u32 tmp  = offsets[1];
+               u32 tmp    = offsets[1];
                offsets[1] = offsets[2];
                offsets[2] = tmp;
           }
@@ -665,8 +717,16 @@ ovl_calc_regs( RadeonDriverData       *rdrv,
      /* Configure buffers */                                   
      ovl_calc_buffers( rdrv, rovl, surface, config );
      
-     /* Configure options */
-     if (config->options & DLOP_OPACITY) {
+     /* Configure scaler */
+     if (rovl->level == -1) {
+          rovl->regs.KEY_CNTL   = GRAPHIC_KEY_FN_FALSE |
+                                  VIDEO_KEY_FN_FALSE   |
+                                  CMP_MIX_AND;
+          rovl->regs.MERGE_CNTL = DISP_ALPHA_MODE_PER_PIXEL |
+                                  0x00ff0000 | /* graphic alpha */
+                                  0xff000000; /* overlay alpha */
+     }
+     else if (config->options & DLOP_OPACITY) {
           rovl->regs.KEY_CNTL   = GRAPHIC_KEY_FN_TRUE |
                                   VIDEO_KEY_FN_TRUE   |
                                   CMP_MIX_AND;
@@ -690,7 +750,6 @@ ovl_calc_regs( RadeonDriverData       *rdrv,
           rovl->regs.MERGE_CNTL = 0xffff0000;
      }
      
-     /* Enable scaler */
      if (config->opacity) {
           rovl->regs.SCALE_CNTL = SCALER_SMART_SWITCH   |
                                   SCALER_DOUBLE_BUFFER  |
@@ -703,6 +762,7 @@ ovl_calc_regs( RadeonDriverData       *rdrv,
                rovl->regs.SCALE_CNTL |= SCALER_VERT_PICK_NEAREST;
  
           switch (surface->format) {
+               case DSPF_RGB555:
                case DSPF_ARGB1555:
                     rovl->regs.SCALE_CNTL |= SCALER_SOURCE_15BPP |
                                              SCALER_PRG_LOAD_START;
@@ -741,7 +801,7 @@ ovl_set_regs( RadeonDriverData       *rdrv,
               RadeonOverlayLayerData *rovl )
 {
      RadeonDeviceData *rdev = rdrv->device_data;
-     volatile __u8    *mmio = rdrv->mmio_base;
+     volatile u8      *mmio = rdrv->mmio_base;
      
      radeon_waitfifo( rdrv, rdev, 1 );
      radeon_out32( mmio, OV0_REG_LOAD_CNTL, REG_LD_CTL_LOCK );
@@ -791,7 +851,7 @@ ovl_set_buffers( RadeonDriverData       *rdrv,
                  RadeonOverlayLayerData *rovl )
 {
      RadeonDeviceData *rdev = rdrv->device_data;
-     volatile __u8    *mmio = rdrv->mmio_base;
+     volatile u8      *mmio = rdrv->mmio_base;
       
      radeon_waitfifo( rdrv, rdev, 1 );
      radeon_out32( mmio, OV0_REG_LOAD_CNTL, REG_LD_CTL_LOCK );
@@ -814,10 +874,10 @@ ovl_set_colorkey( RadeonDriverData       *rdrv,
                   RadeonOverlayLayerData *rovl,
                   CoreLayerRegionConfig  *config )              
 {
-     volatile __u8 *mmio = rdrv->mmio_base; 
-     __u32          SkeyLow, SkeyHigh;
-     __u32          DkeyLow, DkeyHigh;
-     __u32          tmp;
+     volatile u8 *mmio = rdrv->mmio_base;
+     u32          SkeyLow, SkeyHigh;
+     u32          DkeyLow, DkeyHigh;
+     u32          tmp;
      
      SkeyLow  = PIXEL_RGB32( config->src_key.r,
                              config->src_key.g,
@@ -868,17 +928,17 @@ ovl_set_adjustment( RadeonDriverData       *rdrv,
                     float                   saturation,
                     float                   hue )
 {
-     volatile __u8 *mmio = rdrv->mmio_base;
+     volatile u8   *mmio = rdrv->mmio_base;
      float          HueSin, HueCos; 
      float          Luma;
      float          RCb, RCr;
      float          GCb, GCr;
      float          BCb, BCr;
      float          AdjOff, ROff, GOff, BOff;
-     __u32          dwLuma, dwROff, dwGOff, dwBOff;
-     __u32          dwRCb, dwRCr;
-     __u32          dwGCb, dwGCr;
-     __u32          dwBCb, dwBCr;
+     u32            dwLuma, dwROff, dwGOff, dwBOff;
+     u32            dwRCb, dwRCr;
+     u32            dwGCb, dwGCr;
+     u32            dwBCb, dwBCr;
      
      HueSin = sin( hue );
      HueCos = cos( hue );
@@ -898,17 +958,17 @@ ovl_set_adjustment( RadeonDriverData       *rdrv,
      ROff   = CLAMP( ROff, -2048.0, 2047.5 );
      GOff   = CLAMP( GOff, -2048.0, 2047.5 );
      BOff   = CLAMP( BOff, -2048.0, 2047.5 );
-     dwROff = ((__u32)(ROff * 2.0)) & 0x1fff;
-     dwGOff = ((__u32)(GOff * 2.0)) & 0x1fff;
-     dwBOff = ((__u32)(BOff * 2.0)) & 0x1fff;
+     dwROff = ((u32)(ROff * 2.0)) & 0x1fff;
+     dwGOff = ((u32)(GOff * 2.0)) & 0x1fff;
+     dwBOff = ((u32)(BOff * 2.0)) & 0x1fff;
  
-	dwLuma = (((__u32)(Luma * 256.0)) & 0xfff) << 20;
-	dwRCb  = (((__u32)(RCb  * 256.0)) & 0xfff) <<  4;
-	dwRCr  = (((__u32)(RCr  * 256.0)) & 0xfff) << 20;
-	dwGCb  = (((__u32)(GCb  * 256.0)) & 0xfff) <<  4;
-	dwGCr  = (((__u32)(GCr  * 256.0)) & 0xfff) << 20;
-	dwBCb  = (((__u32)(BCb  * 256.0)) & 0xfff) <<  4;
-	dwBCr  = (((__u32)(BCr  * 256.0)) & 0xfff) << 20;
+     dwLuma = (((u32)(Luma * 256.0)) & 0xfff) << 20;
+     dwRCb  = (((u32)(RCb  * 256.0)) & 0xfff) <<  4;
+     dwRCr  = (((u32)(RCr  * 256.0)) & 0xfff) << 20;
+     dwGCb  = (((u32)(GCb  * 256.0)) & 0xfff) <<  4;
+     dwGCr  = (((u32)(GCr  * 256.0)) & 0xfff) << 20;
+     dwBCb  = (((u32)(BCb  * 256.0)) & 0xfff) <<  4;
+     dwBCr  = (((u32)(BCr  * 256.0)) & 0xfff) << 20;
  
      radeon_waitfifo( rdrv, rdrv->device_data, 6 );
      radeon_out32( mmio, OV0_LIN_TRANS_A, dwRCb  | dwLuma );
