@@ -23,7 +23,11 @@
 #ifndef __RADEON_STATE_H__
 #define __RADEON_STATE_H__
 
+#include <core/surfaces.h>
+
 /* R100 state funcs */
+void r100_restore           ( RadeonDriverData *rdrv,
+                              RadeonDeviceData *rdev  );
 void r100_set_destination   ( RadeonDriverData *rdrv,
                               RadeonDeviceData *rdev,
                               CardState        *state );
@@ -53,6 +57,8 @@ void r100_set_blittingflags ( RadeonDriverData *rdrv,
                               CardState        *state );
                               
 /* R200 state funcs */
+void r200_restore           ( RadeonDriverData *rdrv,
+                              RadeonDeviceData *rdev  );
 void r200_set_destination   ( RadeonDriverData *rdrv,
                               RadeonDeviceData *rdev,
                               CardState        *state );
@@ -82,12 +88,17 @@ void r200_set_blittingflags ( RadeonDriverData *rdrv,
                               CardState        *state );
                               
 /* R300 state funcs */
+void r300_restore           ( RadeonDriverData *rdrv,
+                              RadeonDeviceData *rdev  );
 void r300_set_destination   ( RadeonDriverData *rdrv,
                               RadeonDeviceData *rdev,
                               CardState        *state );
 void r300_set_source        ( RadeonDriverData *rdrv,
                               RadeonDeviceData *rdev,
                               CardState        *state );
+void r300_set_clip3d        ( RadeonDriverData *rdrv,
+                              RadeonDeviceData *rdev,
+                              const DFBRegion  *clip  );
 void r300_set_clip          ( RadeonDriverData *rdrv,
                               RadeonDeviceData *rdev,
                               CardState        *state );
@@ -121,7 +132,7 @@ void r300_set_blittingflags ( RadeonDriverData *rdrv,
      rdev->set &= ~(SMF_##flag)
      
      
-static inline __u32
+static inline u32
 radeon_buffer_offset( RadeonDeviceData *rdev, SurfaceBuffer *buffer )
 {
      switch (buffer->storage) {
