@@ -1,12 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <dfb_types.h>
 
 #include <stdio.h>
@@ -62,12 +66,14 @@
 
 /* required implementations */
 
-static void savage3DEngineSync( void *drv, void *dev )
+static DFBResult savage3DEngineSync( void *drv, void *dev )
 {
      Savage3DDriverData *sdrv = (Savage3DDriverData*) drv;
      Savage3DDeviceData *sdev = (Savage3DDeviceData*) dev;
 
      savage3D_waitidle( sdrv, sdev );
+
+     return DFB_OK;
 }
 
 #define SAVAGE3D_DRAWING_FLAGS \
@@ -87,7 +93,7 @@ static inline void savage3D_validate_gbd( Savage3DDriverData *sdrv,
                                           Savage3DDeviceData *sdev,
                                           CardState          *state )
 {
-     __u32 BitmapDescriptor;
+     u32 BitmapDescriptor;
      CoreSurface *destination;
      SurfaceBuffer *buffer;
      int bpp;
@@ -122,7 +128,7 @@ static inline void savage3D_validate_pbd( Savage3DDriverData *sdrv,
                                           Savage3DDeviceData *sdev,
                                           CardState          *state )
 {
-     __u32 BitmapDescriptor;
+     u32 BitmapDescriptor;
      CoreSurface *source;
      SurfaceBuffer *buffer;
      int bpp;
@@ -410,9 +416,9 @@ static bool savage3DBlit( void *drv, void *dev,
      Savage3DDriverData *sdrv = (Savage3DDriverData*) drv;
      Savage3DDeviceData *sdev = (Savage3DDeviceData*) dev;
 
-     __u32 cmd = ( BCI_CMD_RECT | sdev->Cmd_Src_Transparent |
-                   BCI_CMD_CLIP_CURRENT | BCI_CMD_DEST_GBD | 
-                   BCI_CMD_SRC_PBD_COLOR | (0xcc << 16) );
+     u32 cmd = ( BCI_CMD_RECT | sdev->Cmd_Src_Transparent |
+                 BCI_CMD_CLIP_CURRENT | BCI_CMD_DEST_GBD |
+                 BCI_CMD_SRC_PBD_COLOR | (0xcc << 16) );
      
      if (dx < rect->x) {
           cmd |= BCI_CMD_RECT_XP;
@@ -496,7 +502,7 @@ savage3d_init_device( GraphicsDevice     *device,
                       void               *device_data )
 {
      SavageDriverData *sdrv = (SavageDriverData*) driver_data;
-     volatile __u8    *mmio = sdrv->mmio_base;
+     volatile u8      *mmio = sdrv->mmio_base;
 
      unsigned long	cobIndex;	/* size index */
      unsigned long	cobSize;	/* size in bytes */
