@@ -1,12 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <dfb_types.h>
 
 #include <stdio.h>
@@ -71,13 +75,15 @@
 
 /* required implementations */
 
-static void savage4EngineSync( void *drv, void *dev )
+static DFBResult savage4EngineSync( void *drv, void *dev )
 {
      Savage4DriverData *sdrv = (Savage4DriverData*) drv;
      Savage4DeviceData *sdev = (Savage4DeviceData*) dev;
      
      SVGDBG("savage4enginesync\n");
      savage4_waitidle( sdrv, sdev );
+
+     return DFB_OK;
 }
 
 #define SAVAGE4_DRAWING_FLAGS \
@@ -97,7 +103,7 @@ static inline void savage4_validate_gbd( Savage4DriverData *sdrv,
                                          Savage4DeviceData *sdev,
                                          CardState         *state )
 {
-     __u32 BitmapDescriptor;
+     u32 BitmapDescriptor;
      CoreSurface *destination;
      SurfaceBuffer *buffer;
      int bpp;
@@ -129,7 +135,7 @@ static inline void savage4_validate_pbd( Savage4DriverData *sdrv,
                                          Savage4DeviceData *sdev,
                                          CardState         *state )
 {
-     __u32 BitmapDescriptor;
+     u32 BitmapDescriptor;
      CoreSurface *source;
      SurfaceBuffer *buffer;
      int bpp;
@@ -428,9 +434,9 @@ static bool savage4Blit( void *drv, void *dev,
      Savage4DriverData *sdrv = (Savage4DriverData*) drv;
      Savage4DeviceData *sdev = (Savage4DeviceData*) dev;
      
-     __u32 cmd = ( BCI_CMD_RECT | sdev->Cmd_Src_Transparent |
-                   BCI_CMD_CLIP_CURRENT | BCI_CMD_DEST_GBD | 
-                   BCI_CMD_SRC_PBD_COLOR | (0xcc << 16) );
+     u32 cmd = ( BCI_CMD_RECT | sdev->Cmd_Src_Transparent |
+                 BCI_CMD_CLIP_CURRENT | BCI_CMD_DEST_GBD |
+                 BCI_CMD_SRC_PBD_COLOR | (0xcc << 16) );
  
      SVGDBG("savage4Blit x:%i y:%i w:%i h:%i dx:%i dy:%i\n",
             rect->x, rect->y, rect->w, rect->h, dx, dy);
@@ -528,7 +534,7 @@ savage4_init_device( GraphicsDevice     *device,
                      void               *device_data )
 {
      SavageDriverData *sdrv = (SavageDriverData*) driver_data;
-     volatile __u8    *mmio = sdrv->mmio_base;
+     volatile u8      *mmio = sdrv->mmio_base;
 
      SVGDBG("savage4initdevice\n");
 
