@@ -7,6 +7,8 @@
    version 2 of the License, or (at your option) any later version.
 */
 
+#include <config.h>
+
 #include <direct/messages.h>
 #include <gfx/convert.h>
 
@@ -31,9 +33,9 @@
  *       See ddover.c, DDOVER_HQVCalcZoomHeight()
  */
 
-bool uc_ovl_map_vzoom(int sh, int dh, __u32* zoom, __u32* mini)
+bool uc_ovl_map_vzoom(int sh, int dh, u32* zoom, u32* mini)
 {
-    __u32 sh1, tmp, d;
+    u32 sh1, tmp, d;
     bool zoom_ok = true;
 
     if (sh == dh) { // No zoom
@@ -93,10 +95,10 @@ bool uc_ovl_map_vzoom(int sh, int dh, __u32* zoom, __u32* mini)
  * @note Derived from VIA's V4L driver.
  *       See ddover.c, DDOVER_HQVCalcZoomWidth() and DDOver_GetDisplayCount()
  */
-bool uc_ovl_map_hzoom(int sw, int dw,  __u32* zoom, __u32* mini,
-                      __u32* falign, __u32* dcount)
+bool uc_ovl_map_hzoom(int sw, int dw,  u32* zoom, u32* mini,
+                      u32* falign, u32* dcount)
 {
-    __u32 tmp, sw1, d;
+    u32 tmp, sw1, d;
     int md; // Minify-divider
     bool zoom_ok = true;
 
@@ -152,35 +154,35 @@ bool uc_ovl_map_hzoom(int sw, int dw,  __u32* zoom, __u32* mini,
 /**
  * @param falign    fetch alignment
  * @param format    overlay pixel format
- * @param sw        source width
+ * @param pfetch    source pixels per line
  *
- * @returns qword pitch register setting
+ * @returns qword fetch register setting
  *
  * @note Derived from VIA's V4L driver. See ddover.c, DDOver_GetFetch()
  * @note Only call after uc_ovl_map_hzoom()
  */
-__u32 uc_ovl_map_qwpitch(int falign, DFBSurfacePixelFormat format, int sw)
+u32 uc_ovl_map_qwfetch(int falign, DFBSurfacePixelFormat format, int pfetch)
 {
     int fetch = 0;
 
     switch (format) {
     case DSPF_YV12:
-        fetch = ALIGN_TO(sw, 32) >> 4;
+        fetch = ALIGN_TO(pfetch, 32) >> 4;
         break;
     case DSPF_I420:
-        fetch = (ALIGN_TO(sw, 16) >> 4) + 1;
+        fetch = (ALIGN_TO(pfetch, 16) >> 4) + 1;
         break;
     case DSPF_UYVY:
     case DSPF_YUY2:
-        fetch = (ALIGN_TO(sw << 1, 16) >> 4) + 1;
+        fetch = (ALIGN_TO(pfetch << 1, 16) >> 4) + 1;
         break;
     case DSPF_ARGB1555:
     case DSPF_RGB16:
-        fetch = (ALIGN_TO(sw << 1, 16) >> 4) + 1;
+        fetch = (ALIGN_TO(pfetch << 1, 16) >> 4) + 1;
         break;
     case DSPF_RGB32:
     case DSPF_ARGB:
-        fetch = (ALIGN_TO(sw << 2, 16) >> 4) + 1;
+        fetch = (ALIGN_TO(pfetch << 2, 16) >> 4) + 1;
         break;
     default:
         D_BUG("Unexpected pixelformat!");
@@ -200,7 +202,7 @@ __u32 uc_ovl_map_qwpitch(int falign, DFBSurfacePixelFormat format, int sw)
  *
  * @note Derived from VIA's V4L driver. See ddover.c, DDOver_GetV1Format()
  */
-__u32 uc_ovl_map_format(DFBSurfacePixelFormat format)
+u32 uc_ovl_map_format(DFBSurfacePixelFormat format)
 {
     switch (format) {
     case DSPF_YV12:
@@ -237,9 +239,11 @@ __u32 uc_ovl_map_format(DFBSurfacePixelFormat format)
  *
  * @parm ox         will hold new leftmost coordinate in source surface
  * @parm oy         will hold new topmost coordinate in source surface
+ * @parm pfetch     will hold number of required source pixels per line
  */
 void uc_ovl_map_window(int scrw, int scrh, DFBRectangle* win, int sw, int sh,
-                       __u32* win_start, __u32* win_end, int* ox, int* oy)
+                       u32* win_start, u32* win_end,
+                       int* ox, int* oy, int *pfetch)
 {
     int x1, y1, x2, y2;
     int x,y,dw,dh;      // These help making the code readable...
@@ -270,9 +274,10 @@ void uc_ovl_map_window(int scrw, int scrh, DFBRectangle* win, int sw, int sh,
     }
     else if ((y < 0) && (y+dh < scrh)) {
         // Top clip
-        y1 = 0;
         y2 = y+dh-1;
         *oy = (int) (((float) (sh * -y)) / ((float) dh) + 0.5);
+        y1 = ((4-*oy)&3)*dh/sh;
+        *oy = (*oy + 3) & ~3;
     }
     else if ((y >= 0) && (y+dh >= scrh)) {
         // Bottom clip
@@ -281,9 +286,10 @@ void uc_ovl_map_window(int scrw, int scrh, DFBRectangle* win, int sw, int sh,
     }
     else { // if (y < 0) && (y+dh >= scrh)
         // Top and bottom clip
-        y1 = 0;
         y2 = scrh-1;
         *oy = (int) (((float) (sh * -y)) / ((float) dh) + 0.5);
+        y1 = ((4-*oy)&3)*dh/sh;
+        *oy = (*oy + 3) & ~3;
     }
 
     // Horizontal clipping
@@ -292,25 +298,34 @@ void uc_ovl_map_window(int scrw, int scrh, DFBRectangle* win, int sw, int sh,
         // No clipping
         x1 = x;
         x2 = x+dw-1;
+        *pfetch = sw;
     }
     else if ((x < 0) && (x+dw < scrw)) {
         // Left clip
-        x1 = 0;
         x2 = x+dw-1;
         *ox = (int) (((float) (sw * -x)) / ((float) dw) + 0.5);
+        x1 = ((32-*ox)&31)*dw/sw;
+        *ox = (*ox + 31) & ~31;
+        *pfetch = sw - *ox;
     }
     else if ((x >= 0) && (x+dw >= scrw)) {
         // Right clip
         x1 = x;
         x2 = scrw-1;
+        *pfetch = sw - (x+dw-scrw)*sw/dw;
     }
     else { // if (x < 0) && (x+dw >= scrw)
         // Left and right clip
-        x1 = 0;
         x2 = scrw-1;
         *ox = (int) (((float) (sw * -x)) / ((float) dw) + 0.5);
+        x1 = ((32-*ox)&31)*dw/sw;
+        *ox = (*ox + 31) & ~31;
+        *pfetch = sw - *ox - (x+dw-scrw)*sw/dw;
     }
 
+    if (pfetch < 0)
+        pfetch = 0;
+
     *win_start = (x1 << 16) | y1;
     *win_end = (x2 << 16) | y2;
 
@@ -338,14 +353,14 @@ void uc_ovl_map_window(int scrw, int scrh, DFBRectangle* win, int sw, int sh,
  * @note Derived from VIA's V4L driver. See ddover.c,
  *       DDOver_GetSrcStartAddress() and DDOVer_GetYCbCrStartAddress()
  */
-void uc_ovl_map_buffer(DFBSurfacePixelFormat format, __u32 buf,
+void uc_ovl_map_buffer(DFBSurfacePixelFormat format, u32 buf,
                        int ox, int oy, int sw, int sh, int sp, int field,
-                       __u32* y_start, __u32* u_start, __u32* v_start)
+                       u32* y_start, u32* u_start, u32* v_start)
 {
     int swap_cb_cr = 0;
 
-    __u32 tmp;
-    __u32 y_offset, uv_offset = 0;
+    u32 tmp;
+    u32 y_offset, uv_offset = 0;
 
     switch (format) {
 
@@ -418,9 +433,9 @@ void uc_ovl_map_buffer(DFBSurfacePixelFormat format, __u32 buf,
  * @note: Derived from ddmpeg.c, VIAAlphaWin()
  */
 
-__u32 uc_ovl_map_alpha(int opacity)
+u32 uc_ovl_map_alpha(int opacity)
 {
-    __u32 ctrl = 0x00080000;    // Not sure what this number is, supposedly
+    u32 ctrl = 0x00080000;      // Not sure what this number is, supposedly
                                 // it is the "expire number divided by 4".
 
     if (opacity > 255) opacity = 255;
@@ -447,7 +462,7 @@ __u32 uc_ovl_map_alpha(int opacity)
  */
 void uc_ovl_map_v1_control(DFBSurfacePixelFormat format, int sw,
                            int hwrev, bool extfifo_on,
-                           __u32* control, __u32* fifo)
+                           u32* control, u32* fifo)
 {
     *control = V1_BOB_ENABLE | V1_ENABLE | uc_ovl_map_format(format);
 
@@ -505,11 +520,11 @@ static float clamp(float x, float lo, float hi)
  * @param mask  Bitmask
  * @param shift Position in hardware register.
  */
-static int fmt(float x, int ndec, int sbit, __u32 mask, int shift)
+static int fmt(float x, int ndec, int sbit, u32 mask, int shift)
 {
     int y = (x * (1 << ndec));
     if (sbit && (y < 0)) y = -y | (1 << sbit);
-    return (((__u32) y) & mask) << shift;
+    return (((u32) y) & mask) << shift;
 }
 
 /**
@@ -519,7 +534,7 @@ static int fmt(float x, int ndec, int sbit, __u32 mask, int shift)
  * @param a1    Will hold value for V1_ColorSpaceReg_1
  * @param a2    Will hold value for V1_ColorSpaceReg_2
  */
-void uc_ovl_map_adjustment(DFBColorAdjustment* adj, __u32* a1, __u32* a2)
+void uc_ovl_map_adjustment(DFBColorAdjustment* adj, u32* a1, u32* a2)
 {
     float con, sat, bri, hue;
     float c, s;
@@ -558,9 +573,9 @@ void uc_ovl_map_adjustment(DFBColorAdjustment* adj, __u32* a1, __u32* a2)
         fmt(B3, 2, 0, 0xf, 10) | fmt(C3, 2, 3, 0xf,  2);
 }
 
-__u32 uc_ovl_map_colorkey(DFBColor* c)
+u32 uc_ovl_map_colorkey(DFBColor* c)
 {
-    __u32 color;
+    u32 color;
     DFBSurfacePixelFormat fmt;
 
     color = 0;
