@@ -9,6 +9,10 @@
 
 // DirectFB headers
 
+#include <config.h>
+
+#include <fbdev/fbdev.h>
+
 #include <directfb.h>
 
 #include <direct/messages.h>
@@ -43,6 +47,7 @@
 #include "uc_state.h"
 #include "uc_accel.h"
 #include "uc_fifo.h"
+#include "uc_ioctl.h"
 #include "mmio.h"
 #include "uc_probe.h"
 
@@ -64,7 +69,7 @@ DFB_GRAPHICS_DRIVER(unichrome)
 static int pci_config_in8( unsigned int bus,
                            unsigned int slot,
                            unsigned int func,
-                           __u8 reg )
+                           u8 reg )
 {
     char  filename[512];
     int   fd;
@@ -121,6 +126,7 @@ DFBResult uc_probe_pci( UcDriverData *ucdrv )
         for (i = 0; uc_via_devices[i].id != 0; i++) {
             if (device == uc_via_devices[i].id) {
                 // Found a Unichrome device.
+                ucdrv->hwid = device;
                 ucdrv->name = uc_via_devices[i].name;
                 // Read its revision number from the host bridge.
                 ucdrv->hwrev = pci_config_in8(0, 0, 0, 0xf6);
@@ -153,7 +159,7 @@ DFBResult uc_probe_pci( UcDriverData *ucdrv )
 static void uc_dump_vq(UcDeviceData *ucdev)
 {
      int i;
-     __u8* vq;
+     u8* vq;
 
      if (!ucdev->vq_start) return;
      vq = dfb_system_video_memory_virtual(ucdev->vq_start);
@@ -196,7 +202,7 @@ static DFBResult uc_alloc_vq(GraphicsDevice *device, UcDeviceData *ucdev)
 DFBResult uc_init_2d_engine(GraphicsDevice *device, UcDeviceData *ucdev, UcDriverData *ucdrv, bool enable)
 {
      DFBResult result = DFB_OK;
-     volatile __u8* hwregs = ucdrv->hwregs;
+     volatile u8* hwregs = ucdrv->hwregs;
      int i;
 
      // Init 2D engine registers to reset 2D engine
@@ -264,9 +270,9 @@ DFBResult uc_init_2d_engine(GraphicsDevice *device, UcDeviceData *ucdev, UcDrive
      return result;
 }
 
-void uc_init_3d_engine(volatile __u8* hwregs, int hwrev, bool init_all)
+void uc_init_3d_engine(volatile u8* hwregs, int hwrev, bool init_all)
 {
-     __u32 i;
+     u32 i;
 
      if (init_all) {
 
@@ -352,11 +358,13 @@ static void uc_after_set_var(void* drv, void* dev)
      VGA_OUT8(ucdrv->hwregs, 0x3c5, VGA_IN8(ucdrv->hwregs, 0x3c5) & 0xbf);
      // Set bit 2, it might make a difference.
      VGA_OUT8(ucdrv->hwregs, 0x3c5, VGA_IN8(ucdrv->hwregs, 0x3c5) | 0x4);
+
+     VIA_OUT(ucdrv->hwregs, VIA_REG_CURSOR_MODE, VIA_IN(ucdrv->hwregs, VIA_REG_CURSOR_MODE) & 0xFFFFFFFE);
 }
 
 /** Wait until the engine is idle. */
 
-static void uc_engine_sync(void* drv, void* dev)
+static DFBResult uc_engine_sync(void* drv, void* dev)
 {
      UcDriverData* ucdrv = (UcDriverData*) drv;
      UcDeviceData* ucdev = (UcDeviceData*) dev;
@@ -371,6 +379,8 @@ static void uc_engine_sync(void* drv, void* dev)
           if (++loop > MAXLOOP) {
                D_ERROR("DirectFB/Unichrome: Timeout waiting for idle engine!\n");
                break;
+
+               /* FIXME: return DFB_TIMEOUT and implement EngineReset! */
           }
      }
 
@@ -381,6 +391,8 @@ static void uc_engine_sync(void* drv, void* dev)
 
      ucdev->idle_waitcycles += loop;
      ucdev->must_wait = 0;
+
+     return DFB_OK;
 }
 
 
@@ -426,6 +438,16 @@ static void driver_get_info(GraphicsDevice* device,
      info->device_data_size = sizeof (UcDeviceData);
 }
 
+static void uc_probe_fbdev(UcDriverData *ucdrv)
+{
+     struct fb_flip flip;
+     FBDev *dfb_fbdev = dfb_system_data();
+     flip.device = VIAFB_FLIP_NOP;
+     if (ioctl(dfb_fbdev->fd, FBIO_FLIPONVSYNC, &flip) == 0)
+          ucdrv->canfliponvsync = true;
+     else
+          ucdrv->canfliponvsync = false;
+}
 
 static DFBResult driver_init_driver(GraphicsDevice* device,
                                     GraphicsDeviceFuncs* funcs,
@@ -454,13 +476,16 @@ static DFBResult driver_init_driver(GraphicsDevice* device,
           ucdrv->file = fd;
 
           ucdrv->hwregs = mmap(NULL, 0x1000000, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
-          if ((int) ucdrv->hwregs == -1)
+          if (ucdrv->hwregs == MAP_FAILED)
                return DFB_IO;
      }
 
      // Get hardware id and revision.
      uc_probe_pci(ucdrv);
 
+     // Check framebuffer device capabilities
+     uc_probe_fbdev(ucdrv);
+
      /* FIXME: this belongs to device_data! */
      ucdrv->fifo = uc_fifo_create(ucdrv->pool, UC_FIFO_SIZE);
      if (!ucdrv->fifo)
