@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -51,7 +52,7 @@ typedef enum {
 
 #define DFB_KEY(type,index)        ((DIKT_##type) | (index))
 
-#define DFB_KEY_TYPE(symbol)       ((symbol) & 0xFF00)
+#define DFB_KEY_TYPE(symbol)       ((((symbol) & ~0xFFF) == 0xF000) ? (symbol) & 0xFF00 : DIKT_UNICODE)
 #define DFB_KEY_INDEX(symbol)      ((symbol) & 0x00FF)
 
 #define DFB_KEY_IS_ASCII(symbol)   ((symbol) < 128)
@@ -512,7 +513,7 @@ typedef enum {
      /*
       * Unicode private area - DirectFB Custom keys
       *
-      * More function keys are available via DFB_CUSTOM_KEY(n).
+      * More custom keys are available via DFB_CUSTOM_KEY(n).
       */
      DIKS_CUSTOM0                  = DFB_CUSTOM_KEY( 0 ),
      DIKS_CUSTOM1                  = DFB_CUSTOM_KEY( 1 ),
@@ -523,7 +524,97 @@ typedef enum {
      DIKS_CUSTOM6                  = DFB_CUSTOM_KEY( 6 ),
      DIKS_CUSTOM7                  = DFB_CUSTOM_KEY( 7 ),
      DIKS_CUSTOM8                  = DFB_CUSTOM_KEY( 8 ),
-     DIKS_CUSTOM9                  = DFB_CUSTOM_KEY( 9 )
+     DIKS_CUSTOM9                  = DFB_CUSTOM_KEY( 9 ),
+     DIKS_CUSTOM10                 = DFB_CUSTOM_KEY( 10 ),
+     DIKS_CUSTOM11                 = DFB_CUSTOM_KEY( 11 ),
+     DIKS_CUSTOM12                 = DFB_CUSTOM_KEY( 12 ),
+     DIKS_CUSTOM13                 = DFB_CUSTOM_KEY( 13 ),
+     DIKS_CUSTOM14                 = DFB_CUSTOM_KEY( 14 ),
+     DIKS_CUSTOM15                 = DFB_CUSTOM_KEY( 15 ),
+     DIKS_CUSTOM16                 = DFB_CUSTOM_KEY( 16 ),
+     DIKS_CUSTOM17                 = DFB_CUSTOM_KEY( 17 ),
+     DIKS_CUSTOM18                 = DFB_CUSTOM_KEY( 18 ),
+     DIKS_CUSTOM19                 = DFB_CUSTOM_KEY( 19 ),
+     DIKS_CUSTOM20                 = DFB_CUSTOM_KEY( 20 ),
+     DIKS_CUSTOM21                 = DFB_CUSTOM_KEY( 21 ),
+     DIKS_CUSTOM22                 = DFB_CUSTOM_KEY( 22 ),
+     DIKS_CUSTOM23                 = DFB_CUSTOM_KEY( 23 ),
+     DIKS_CUSTOM24                 = DFB_CUSTOM_KEY( 24 ),
+     DIKS_CUSTOM25                 = DFB_CUSTOM_KEY( 25 ),
+     DIKS_CUSTOM26                 = DFB_CUSTOM_KEY( 26 ),
+     DIKS_CUSTOM27                 = DFB_CUSTOM_KEY( 27 ),
+     DIKS_CUSTOM28                 = DFB_CUSTOM_KEY( 28 ),
+     DIKS_CUSTOM29                 = DFB_CUSTOM_KEY( 29 ),
+     DIKS_CUSTOM30                 = DFB_CUSTOM_KEY( 30 ),
+     DIKS_CUSTOM31                 = DFB_CUSTOM_KEY( 31 ),
+     DIKS_CUSTOM32                 = DFB_CUSTOM_KEY( 32 ),
+     DIKS_CUSTOM33                 = DFB_CUSTOM_KEY( 33 ),
+     DIKS_CUSTOM34                 = DFB_CUSTOM_KEY( 34 ),
+     DIKS_CUSTOM35                 = DFB_CUSTOM_KEY( 35 ),
+     DIKS_CUSTOM36                 = DFB_CUSTOM_KEY( 36 ),
+     DIKS_CUSTOM37                 = DFB_CUSTOM_KEY( 37 ),
+     DIKS_CUSTOM38                 = DFB_CUSTOM_KEY( 38 ),
+     DIKS_CUSTOM39                 = DFB_CUSTOM_KEY( 39 ),
+     DIKS_CUSTOM40                 = DFB_CUSTOM_KEY( 40 ),
+     DIKS_CUSTOM41                 = DFB_CUSTOM_KEY( 41 ),
+     DIKS_CUSTOM42                 = DFB_CUSTOM_KEY( 42 ),
+     DIKS_CUSTOM43                 = DFB_CUSTOM_KEY( 43 ),
+     DIKS_CUSTOM44                 = DFB_CUSTOM_KEY( 44 ),
+     DIKS_CUSTOM45                 = DFB_CUSTOM_KEY( 45 ),
+     DIKS_CUSTOM46                 = DFB_CUSTOM_KEY( 46 ),
+     DIKS_CUSTOM47                 = DFB_CUSTOM_KEY( 47 ),
+     DIKS_CUSTOM48                 = DFB_CUSTOM_KEY( 48 ),
+     DIKS_CUSTOM49                 = DFB_CUSTOM_KEY( 49 ),
+     DIKS_CUSTOM50                 = DFB_CUSTOM_KEY( 50 ),
+     DIKS_CUSTOM51                 = DFB_CUSTOM_KEY( 51 ),
+     DIKS_CUSTOM52                 = DFB_CUSTOM_KEY( 52 ),
+     DIKS_CUSTOM53                 = DFB_CUSTOM_KEY( 53 ),
+     DIKS_CUSTOM54                 = DFB_CUSTOM_KEY( 54 ),
+     DIKS_CUSTOM55                 = DFB_CUSTOM_KEY( 55 ),
+     DIKS_CUSTOM56                 = DFB_CUSTOM_KEY( 56 ),
+     DIKS_CUSTOM57                 = DFB_CUSTOM_KEY( 57 ),
+     DIKS_CUSTOM58                 = DFB_CUSTOM_KEY( 58 ),
+     DIKS_CUSTOM59                 = DFB_CUSTOM_KEY( 59 ),
+     DIKS_CUSTOM60                 = DFB_CUSTOM_KEY( 60 ),
+     DIKS_CUSTOM61                 = DFB_CUSTOM_KEY( 61 ),
+     DIKS_CUSTOM62                 = DFB_CUSTOM_KEY( 62 ),
+     DIKS_CUSTOM63                 = DFB_CUSTOM_KEY( 63 ),
+     DIKS_CUSTOM64                 = DFB_CUSTOM_KEY( 64 ),
+     DIKS_CUSTOM65                 = DFB_CUSTOM_KEY( 65 ),
+     DIKS_CUSTOM66                 = DFB_CUSTOM_KEY( 66 ),
+     DIKS_CUSTOM67                 = DFB_CUSTOM_KEY( 67 ),
+     DIKS_CUSTOM68                 = DFB_CUSTOM_KEY( 68 ),
+     DIKS_CUSTOM69                 = DFB_CUSTOM_KEY( 69 ),
+     DIKS_CUSTOM70                 = DFB_CUSTOM_KEY( 70 ),
+     DIKS_CUSTOM71                 = DFB_CUSTOM_KEY( 71 ),
+     DIKS_CUSTOM72                 = DFB_CUSTOM_KEY( 72 ),
+     DIKS_CUSTOM73                 = DFB_CUSTOM_KEY( 73 ),
+     DIKS_CUSTOM74                 = DFB_CUSTOM_KEY( 74 ),
+     DIKS_CUSTOM75                 = DFB_CUSTOM_KEY( 75 ),
+     DIKS_CUSTOM76                 = DFB_CUSTOM_KEY( 76 ),
+     DIKS_CUSTOM77                 = DFB_CUSTOM_KEY( 77 ),
+     DIKS_CUSTOM78                 = DFB_CUSTOM_KEY( 78 ),
+     DIKS_CUSTOM79                 = DFB_CUSTOM_KEY( 79 ),
+     DIKS_CUSTOM80                 = DFB_CUSTOM_KEY( 80 ),
+     DIKS_CUSTOM81                 = DFB_CUSTOM_KEY( 81 ),
+     DIKS_CUSTOM82                 = DFB_CUSTOM_KEY( 82 ),
+     DIKS_CUSTOM83                 = DFB_CUSTOM_KEY( 83 ),
+     DIKS_CUSTOM84                 = DFB_CUSTOM_KEY( 84 ),
+     DIKS_CUSTOM85                 = DFB_CUSTOM_KEY( 85 ),
+     DIKS_CUSTOM86                 = DFB_CUSTOM_KEY( 86 ),
+     DIKS_CUSTOM87                 = DFB_CUSTOM_KEY( 87 ),
+     DIKS_CUSTOM88                 = DFB_CUSTOM_KEY( 88 ),
+     DIKS_CUSTOM89                 = DFB_CUSTOM_KEY( 89 ),
+     DIKS_CUSTOM90                 = DFB_CUSTOM_KEY( 90 ),
+     DIKS_CUSTOM91                 = DFB_CUSTOM_KEY( 91 ),
+     DIKS_CUSTOM92                 = DFB_CUSTOM_KEY( 92 ),
+     DIKS_CUSTOM93                 = DFB_CUSTOM_KEY( 93 ),
+     DIKS_CUSTOM94                 = DFB_CUSTOM_KEY( 94 ),
+     DIKS_CUSTOM95                 = DFB_CUSTOM_KEY( 95 ),
+     DIKS_CUSTOM96                 = DFB_CUSTOM_KEY( 96 ),
+     DIKS_CUSTOM97                 = DFB_CUSTOM_KEY( 97 ),
+     DIKS_CUSTOM98                 = DFB_CUSTOM_KEY( 98 ),
+     DIKS_CUSTOM99                 = DFB_CUSTOM_KEY( 99 )
 } DFBInputDeviceKeySymbol;
 
 /*
