@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -57,7 +58,7 @@ DFB_INPUT_DRIVER( dbox2remote )
 
 typedef struct {
      DFBInputDeviceKeySymbol  key;
-     __u16                    rccode;
+     u16                      rccode;
 } KeyCode;
 
 /* REMOTE_NEW is the one with _fewer_ buttons */
@@ -153,7 +154,7 @@ typedef struct {
  * helper function for translating rccode
  */
 static DFBInputDeviceKeySymbol
-dbox2remote_parse_rccode( __u16 rccode )
+dbox2remote_parse_rccode( u16 rccode )
 {
      KeyCode *keycode;
 
@@ -184,7 +185,7 @@ dbox2remoteEventThread( DirectThread *thread, void *driver_data )
 {
      Dbox2remoteData *data = (Dbox2remoteData*) driver_data;
      int              readlen;
-     __u16            rccode;
+     u16              rccode;
      DFBInputEvent    evt;
 
      while ((readlen = read( data->fd, &rccode, 2 )) == 2) {
