@@ -5,16 +5,18 @@
 
    Written by Marcel Siegert <mws@directfb.org>
 
-   Mainly based on dbox2remote
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   Mainly based on dbox2remote:
+
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -63,7 +65,7 @@ DFB_INPUT_DRIVER( dreamboxremote )
 
 typedef struct {
      DFBInputDeviceKeySymbol  key;
-     __u16                    rccode;
+     u16                      rccode;
 } KeyCode;
 
 static KeyCode keycodes_remote[] = {
@@ -130,10 +132,10 @@ typedef struct {
  * helper function for translating rccode
  */
 static DFBInputDeviceKeySymbol
-dreamboxremote_parse_rccode( __u16 rccode )
+dreamboxremote_parse_rccode( u16 rccode )
 {
 	KeyCode *keycode;
-	static  __u16 rccodeOld = 0;
+	static  u16 rccodeOld = 0;
 
 	keycode = keycodes_remote;
 	/* 0x00ff indicates key was released
@@ -173,7 +175,7 @@ dreamboxremoteEventThread( DirectThread *thread, void *driver_data )
 {
      DreamboxremoteData *data = (DreamboxremoteData*) driver_data;
      int              readlen;
-     __u16            rccode;
+     u16              rccode;
      DFBInputEvent    evt;
 
      while ((readlen = read( data->fd, &rccode, 2 )) == 2) {
