@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -100,6 +103,30 @@ keyboard_get_symbol( int                             code,
                          return index;
                }
                break;
+          case KT_DEAD:
+               switch (value) {
+                    case K_DGRAVE:
+                         return DIKS_DEAD_GRAVE;
+
+                    case K_DACUTE:
+                         return DIKS_DEAD_ACUTE;
+
+                    case K_DCIRCM:
+                         return DIKS_DEAD_CIRCUMFLEX;
+
+                    case K_DTILDE:
+                         return DIKS_DEAD_TILDE;
+
+                    case K_DDIERE:
+                         return DIKS_DEAD_DIAERESIS;
+
+                    case K_DCEDIL:
+                         return DIKS_DEAD_CEDILLA;
+
+                    default:
+                         break;
+               }
+               break;
           case KT_PAD:
                if (index <= 9 && level != DIKSI_BASE)
                     return DIKS_0 + index;
@@ -164,6 +191,9 @@ keyboard_get_symbol( int                             code,
      }
 
      /* special keys not in the map, hack? */
+     if (code == 124)         /* keypad equal key */
+          return DIKS_EQUALS_SIGN;
+
      if (code == 125)         /* left windows key */
           return DIKS_META;
 
@@ -217,6 +247,9 @@ keyboard_get_identifier( int code, unsigned short value )
      }
 
      /* special keys not in the map, hack? */
+     if (code == 124)         /* keypad equal key */
+          return DIKI_KP_EQUAL;
+
      if (code == 125)         /* left windows key */
           return DIKI_META_L;
 
@@ -306,8 +339,7 @@ driver_get_info( InputDriverInfo *info )
                DFB_INPUT_DRIVER_INFO_NAME_LENGTH, "Keyboard Driver" );
 
      snprintf( info->vendor,
-               DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH,
-               "convergence integrated media GmbH" );
+               DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH, "directfb.org" );
 
      info->version.major = 0;
      info->version.minor = 9;
