@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,11 +26,31 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
 
+
+#include <linux/version.h>
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+typedef unsigned long kernel_ulong_t;
+#define BITS_PER_LONG    (sizeof(long)*8)
+#endif
+
+/* compat defines for older kernel like 2.4.x */
+#ifndef EV_SYN
+#define EV_SYN			0x00
+#define SYN_REPORT              0
+#define SYN_CONFIG              1
+#define ABS_TOOL_WIDTH		0x1c
+#define BTN_TOOL_DOUBLETAP	0x14d
+#define BTN_TOOL_TRIPLETAP	0x14e
+#endif
+
 #include <linux/input.h>
 #ifndef KEY_OK
 /* Linux kernel 2.5.42+ defines additional keys in linux/input.h */
@@ -50,6 +71,8 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/kd.h>
+#include <stdlib.h>
+
 
 #include <linux/keyboard.h>
 
@@ -70,21 +93,37 @@
 #include <misc/conf.h>
 #include <misc/util.h>
 
+#ifdef LINUX_INPUT_USE_FBDEV
 #include <fbdev/fbdev.h>
+#endif
 
 #include <core/input_driver.h>
 
 
 DFB_INPUT_DRIVER( linux_input )
 
-
+#ifndef BITS_PER_LONG
 #define BITS_PER_LONG        (sizeof(long) * 8)
+#endif
 #define NBITS(x)             ((((x)-1)/BITS_PER_LONG)+1)
 #define OFF(x)               ((x)%BITS_PER_LONG)
 #define BIT(x)               (1UL<<OFF(x))
 #define LONG(x)              ((x)/BITS_PER_LONG)
+#undef test_bit
 #define test_bit(bit, array) ((array[LONG(bit)] >> OFF(bit)) & 1)
 
+/* compat for 2.4.x kernel - just a compile fix */
+#ifndef HAVE_INPUT_ABSINFO
+struct input_absinfo {
+        __s32 value;
+        __s32 minimum;
+        __s32 maximum;
+        __s32 fuzz;
+        __s32 flat;
+};
+#endif
+
+
 /*
  * declaration of private data
  */
@@ -98,7 +137,14 @@ typedef struct {
      unsigned long            led_state[NBITS(LED_MAX)];
      DFBInputDeviceLockState  locks;
 
+#ifdef LINUX_INPUT_USE_FBDEV
      VirtualTerminal         *vt;
+#endif
+
+     int                      dx;
+     int                      dy;
+
+     bool                     touchpad;
 } LinuxInputData;
 
 
@@ -140,7 +186,7 @@ int basic_keycodes [] = {
      DFB_FUNCTION_KEY(16), DFB_FUNCTION_KEY(17), DFB_FUNCTION_KEY(18),
      DFB_FUNCTION_KEY(19), DFB_FUNCTION_KEY(20),
 
-     DIKI_KP_ENTER, DIKI_CONTROL_R, DIKI_KP_DIV, DIKI_PRINT, DIKI_ALTGR,
+     DIKI_KP_ENTER, DIKI_CONTROL_R, DIKI_KP_DIV, DIKI_PRINT, DIKS_ALTGR,
 
      /*KEY_LINEFEED*/ DIKI_UNKNOWN,
 
@@ -195,7 +241,7 @@ int basic_keycodes [] = {
 
      /*KEY_ISO,*/ DIKI_UNKNOWN,
      /*KEY_CONFIG,*/ DIKS_SETUP,
-     /*KEY_HOMEPAGE, KEY_REFRESH,*/ DIKI_UNKNOWN, DIKI_UNKNOWN,
+     /*KEY_HOMEPAGE, KEY_REFRESH,*/ DIKI_UNKNOWN, DIKS_SHUFFLE,
 
      DIKS_EXIT, /*KEY_MOVE,*/ DIKI_UNKNOWN, DIKS_EDITOR,
 
@@ -225,16 +271,16 @@ int basic_keycodes [] = {
 
      /* KEY_PLAY */
      DIKS_PLAY,
-     
+
      /* KEY_FASTFORWARD */
      DIKS_FASTFORWARD,
-     
+
      /* KEY_BASSBOOST */
      DIKI_UNKNOWN,
-     
+
      /* KEY_PRINT */
      DIKS_PRINT,
-     
+
      /* KEY_HP             */  DIKI_UNKNOWN,
      /* KEY_CAMERA         */  DIKI_UNKNOWN,
      /* KEY_SOUND          */  DIKS_AUDIO,
@@ -265,26 +311,60 @@ int ext_keycodes [] = {
      DIKS_MP3, DIKS_AUDIO, DIKS_VIDEO, DIKS_DIRECTORY, DIKS_LIST, DIKS_MEMO,
      DIKS_CALENDAR, DIKS_RED, DIKS_GREEN, DIKS_YELLOW, DIKS_BLUE,
      DIKS_CHANNEL_UP, DIKS_CHANNEL_DOWN, DIKS_FIRST, DIKS_LAST, DIKS_AB,
-     DIKS_PLAY, DIKS_RESTART, DIKS_SLOW, DIKS_SHUFFLE, DIKS_FASTFORWARD,
+     DIKS_NEXT, DIKS_RESTART, DIKS_SLOW, DIKS_SHUFFLE, DIKS_FASTFORWARD,
      DIKS_PREVIOUS, DIKS_NEXT, DIKS_DIGITS, DIKS_TEEN, DIKS_TWEN, DIKS_BREAK
 };
 
+/*
+ * Touchpads related stuff
+ */
+enum {
+     TOUCHPAD_FSM_START,
+     TOUCHPAD_FSM_MAIN,
+     TOUCHPAD_FSM_DRAG_START,
+     TOUCHPAD_FSM_DRAG_MAIN,
+};
+struct touchpad_axis {
+     int old, min, max;
+};
+struct touchpad_fsm_state {
+     int fsm_state;
+     struct touchpad_axis x;
+     struct touchpad_axis y;
+     struct timeval timeout;
+};
+
+static void
+touchpad_fsm_init( struct touchpad_fsm_state *state );
+static int
+touchpad_fsm( struct touchpad_fsm_state *state,
+              struct input_event        *levt,
+              DFBInputEvent             *devt );
+
+static int
+timeout_passed( struct timeval *timeout, struct timeval *current );
+static int
+timeout_is_set( struct timeval *timeout );
+static void
+timeout_sub( struct timeval *timeout, struct timeval *sub );
+
 /*
  * Translates a Linux input keycode into a DirectFB keycode.
  */
 static int
 translate_key( unsigned short code )
 {
-     if (code < sizeof(basic_keycodes)/sizeof(basic_keycodes[0]))
+     if (code < D_ARRAY_SIZE( basic_keycodes ))
           return basic_keycodes[code];
 
      if (code >= KEY_OK)
-          if (code - KEY_OK < sizeof(ext_keycodes)/sizeof(ext_keycodes[0]))
+          if (code - KEY_OK < D_ARRAY_SIZE( ext_keycodes ))
                return ext_keycodes[code-KEY_OK];
 
      return DIKI_UNKNOWN;
 }
 
+#ifdef LINUX_INPUT_USE_FBDEV
 static DFBInputDeviceKeySymbol
 keyboard_get_symbol( int                             code,
                      unsigned short                  value,
@@ -312,6 +392,30 @@ keyboard_get_symbol( int                             code,
                          return index;
                }
                break;
+          case KT_DEAD:
+               switch (value) {
+                    case K_DGRAVE:
+                         return DIKS_DEAD_GRAVE;
+
+                    case K_DACUTE:
+                         return DIKS_DEAD_ACUTE;
+
+                    case K_DCIRCM:
+                         return DIKS_DEAD_CIRCUMFLEX;
+
+                    case K_DTILDE:
+                         return DIKS_DEAD_TILDE;
+
+                    case K_DDIERE:
+                         return DIKS_DEAD_DIAERESIS;
+
+                    case K_DCEDIL:
+                         return DIKS_DEAD_CEDILLA;
+
+                    default:
+                         break;
+               }
+               break;
           case KT_PAD:
                if (index <= 9 && level != DIKSI_BASE)
                     return DIKS_0 + index;
@@ -379,6 +483,9 @@ keyboard_get_symbol( int                             code,
      if (code == 99)
           return DIKS_PRINT;
 
+     if (code == 124)         /* keypad equal key */
+          return DIKS_EQUALS_SIGN;
+
      if (code == 125)         /* left windows key */
           return DIKS_META;
 
@@ -427,11 +534,15 @@ keyboard_get_identifier( int code, unsigned short value )
           case 53: return DIKI_SLASH;
           case 54: return DIKI_SHIFT_R;
           case 97: return DIKI_CONTROL_R;
+          case 100: return DIKI_ALT_R;
           default:
                ;
      }
 
      /* special keys not in the map, hack? */
+     if (code == 124)         /* keypad equal key */
+          return DIKI_KP_EQUAL;
+
      if (code == 125)         /* left windows key */
           return DIKI_META_L;
 
@@ -462,6 +573,7 @@ keyboard_read_value( LinuxInputData *data,
 
      return entry.kb_value;
 }
+#endif /* LINUX_INPUT_USE_FBDEV */
 
 /*
  * Translates key and button events.
@@ -474,7 +586,11 @@ key_event( struct input_event *levt,
      if (levt->code == BTN_TOUCH || levt->code == BTN_TOOL_FINGER)
           levt->code = BTN_MOUSE;
 
-     if (levt->code >= BTN_MOUSE && levt->code < BTN_JOYSTICK) {
+     if ((levt->code >= BTN_MOUSE && levt->code < BTN_JOYSTICK) || levt->code == BTN_TOUCH) {
+          /* ignore repeat events for buttons */
+          if (levt->value == 2)
+               return 0;
+
           devt->type   = levt->value ? DIET_BUTTONPRESS : DIET_BUTTONRELEASE;
           /* don't set DIEF_BUTTONS, it will be set by the input core */
           devt->button = DIBI_FIRST + levt->code - BTN_MOUSE;
@@ -616,6 +732,38 @@ set_led( LinuxInputData *data, int led, int state )
      write( data->fd, &levt, sizeof(levt) );
 }
 
+static void
+flush_xy( LinuxInputData *data )
+{
+     DFBInputEvent evt;
+
+     if (data->dx) {
+          evt.type    = DIET_AXISMOTION;
+          evt.flags   = DIEF_AXISREL;
+          evt.axis    = DIAI_X;
+          evt.axisrel = data->dx;
+
+          /* Signal immediately following event. */
+          if (data->dy)
+               evt.flags |= DIEF_FOLLOW;
+
+          dfb_input_dispatch( data->device, &evt );
+
+          data->dx = 0;
+     }
+
+     if (data->dy) {
+          evt.type    = DIET_AXISMOTION;
+          evt.flags   = DIEF_AXISREL;
+          evt.axis    = DIAI_Y;
+          evt.axisrel = data->dy;
+
+          dfb_input_dispatch( data->device, &evt );
+
+          data->dy = 0;
+     }
+}
+
 /*
  * Input thread reading from device.
  * Generates events on incoming data.
@@ -624,19 +772,111 @@ static void*
 linux_input_EventThread( DirectThread *thread, void *driver_data )
 {
      LinuxInputData    *data = (LinuxInputData*) driver_data;
-     int                readlen;
-     struct input_event levt;
-     DFBInputEvent      devt;
+     int                readlen, status, i;
+     struct input_event levt[64];
+     fd_set             set;
+     struct touchpad_fsm_state fsm_state;
+
+     /* Query min/max coordinates. */
+     if (data->touchpad) {
+          touchpad_fsm_init( &fsm_state );
+
+          struct input_absinfo absinfo;
+
+          ioctl( data->fd, EVIOCGABS(ABS_X), &absinfo );
+          fsm_state.x.min = absinfo.minimum;
+          fsm_state.x.max = absinfo.maximum;
+
+          ioctl( data->fd, EVIOCGABS(ABS_Y), &absinfo );
+          fsm_state.y.min = absinfo.minimum;
+          fsm_state.y.max = absinfo.maximum;
+     }
+
+     while (1) {
+          FD_ZERO( &set );
+          FD_SET( data->fd, &set );
+
+          if (data->touchpad && timeout_is_set( &fsm_state.timeout )) {
+               struct timeval time;
+               gettimeofday( &time, NULL );
+
+               if (!timeout_passed( &fsm_state.timeout, &time )) {
+                    struct timeval timeout = fsm_state.timeout;
+                    timeout_sub( &timeout, &time );
+                    status = select( data->fd + 1, &set, NULL, NULL, &timeout );
+               } else {
+                    status = 0;
+               }
+          }
+          else {
+               status = select( data->fd + 1, &set, NULL, NULL, NULL );
+          }
+
+          if (status < 0 && errno != EINTR)
+               break;
+
+          direct_thread_testcancel( thread );
+
+          if (status < 0)
+               continue;
+
+          /* timeout? */
+          if (status == 0) {
+               DFBInputEvent devt;
+
+               if (data->touchpad && touchpad_fsm( &fsm_state, NULL, &devt ) > 0)
+                    dfb_input_dispatch( data->device, &devt );
+
+               continue;
+          }
+
+          readlen = read( data->fd, levt, sizeof(levt) ) / sizeof(levt[0]);
+
+          if (readlen < 0 && errno != EINTR)
+               break;
 
-     while ((readlen = read( data->fd, &levt, sizeof(levt) )) == sizeof(levt)
-            || (readlen < 0 && errno == EINTR))
-     {
           direct_thread_testcancel( thread );
 
           if (readlen <= 0)
                continue;
 
-          if (translate_event( &levt, &devt )) {
+          for (i=0; i<readlen; i++) {
+               DFBInputEvent devt;
+
+               if (data->touchpad) {
+                    status = touchpad_fsm( &fsm_state, &levt[i], &devt );
+                    if (status < 0) {
+                         /* Not handled. Try the direct approach. */
+                         if (!translate_event( &levt[i], &devt ))
+                              continue;
+                    }
+                    else if (status == 0) {
+                         /* Handled but no further processing is necessary. */
+                         continue;
+                    }
+               }
+               else {
+                    if (!translate_event( &levt[i], &devt ))
+                         continue;
+               }
+
+               if (devt.type == DIET_AXISMOTION && (devt.flags & DIEF_AXISREL)) {
+                    switch (devt.axis) {
+                         case DIAI_X:
+                              data->dx += devt.axisrel;
+                              continue;
+
+                         case DIAI_Y:
+                              data->dy += devt.axisrel;
+                              continue;
+
+                         default:
+                              break;
+                    }
+               }
+
+               flush_xy( data );
+
                dfb_input_dispatch( data->device, &devt );
 
                if (data->has_leds && (devt.locks != data->locks)) {
@@ -646,9 +886,11 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
                     data->locks = devt.locks;
                }
           }
+
+          flush_xy( data );
      }
 
-     if (readlen <= 0)
+     if (status <= 0)
           D_PERROR ("linux_input thread died\n");
 
      return NULL;
@@ -660,7 +902,8 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
  */
 static void
 get_device_info( int              fd,
-                 InputDeviceInfo *info )
+                 InputDeviceInfo *info,
+                 bool            *touchpad )
 {
      unsigned int  num_keys     = 0;
      unsigned int  num_ext_keys = 0;
@@ -674,20 +917,20 @@ get_device_info( int              fd,
      unsigned long absbit[NBITS(ABS_MAX)];
 
      /* get device name */
-     ioctl( fd, EVIOCGNAME(DFB_INPUT_DEVICE_DESC_NAME_LENGTH), info->desc.name );
+     ioctl( fd, EVIOCGNAME(DFB_INPUT_DEVICE_DESC_NAME_LENGTH - 1), info->desc.name );
 
      /* set device vendor */
      snprintf( info->desc.vendor,
                DFB_INPUT_DEVICE_DESC_VENDOR_LENGTH, "Linux" );
 
      /* get event type bits */
-     ioctl( fd, EVIOCGBIT(0, EV_MAX), evbit );
+     ioctl( fd, EVIOCGBIT(0, sizeof(evbit)), evbit );
 
      if (test_bit( EV_KEY, evbit )) {
           int i;
 
           /* get keyboard bits */
-          ioctl( fd, EVIOCGBIT(EV_KEY, KEY_MAX), keybit );
+          ioctl( fd, EVIOCGBIT(EV_KEY, sizeof(keybit)), keybit );
 
 	  /**  count typical keyboard keys only */
           for (i=KEY_Q; i<KEY_M; i++)
@@ -707,7 +950,7 @@ get_device_info( int              fd,
           int i;
 
           /* get bits for relative axes */
-          ioctl( fd, EVIOCGBIT(EV_REL, REL_MAX), relbit );
+          ioctl( fd, EVIOCGBIT(EV_REL, sizeof(relbit)), relbit );
 
           for (i=0; i<REL_MAX; i++)
                if (test_bit( i, relbit ))
@@ -718,13 +961,26 @@ get_device_info( int              fd,
           int i;
 
           /* get bits for absolute axes */
-          ioctl( fd, EVIOCGBIT(EV_ABS, ABS_MAX), absbit );
+          ioctl( fd, EVIOCGBIT(EV_ABS, sizeof(absbit)), absbit );
 
           for (i=0; i<ABS_PRESSURE; i++)
                if (test_bit( i, absbit ))
                     num_abs++;
      }
 
+     /* Touchpad? */
+     /* FIXME: can we rely on BTN_TOUCH? xorg synaptics driver doesn't use it. */
+     if (test_bit( EV_KEY, evbit ) &&
+         test_bit( BTN_TOUCH, keybit ) &&
+         test_bit( BTN_TOOL_FINGER, keybit) &&
+         test_bit( EV_ABS, evbit ) &&
+         test_bit( ABS_X, absbit ) &&
+         test_bit( ABS_Y, absbit ) &&
+         test_bit( ABS_PRESSURE, absbit ))
+          *touchpad = true;
+     else
+          *touchpad = false;
+
      /* Mouse, Touchscreen or Smartpad ? */
      if ((test_bit( EV_KEY, evbit ) &&
           (test_bit( BTN_TOUCH, keybit ) || test_bit( BTN_TOOL_FINGER, keybit ))) ||
@@ -784,8 +1040,10 @@ driver_get_available()
 {
      int i;
 
+#ifdef LINUX_INPUT_USE_FBDEV
      if (dfb_system_type() != CORE_FBDEV)
           return 0;
+#endif
 
      for (i=0; i<MAX_LINUX_INPUT_DEVICES; i++) {
           int  fd;
@@ -801,6 +1059,7 @@ driver_get_available()
           }
           else {
                InputDeviceInfo info;
+               bool touchpad;
 
                /* try to grab the device */
                /* 2.4 kernels don't have EVIOCGRAB so ignore EINVAL */
@@ -811,7 +1070,7 @@ driver_get_available()
 
                memset( &info, 0, sizeof(InputDeviceInfo) );
 
-               get_device_info( fd, &info );
+               get_device_info( fd, &info, &touchpad );
 
                ioctl( fd, EVIOCGRAB, 0 );
                close( fd );
@@ -836,8 +1095,7 @@ driver_get_info( InputDriverInfo *info )
      snprintf ( info->name,
                 DFB_INPUT_DRIVER_INFO_NAME_LENGTH, "Linux Input Driver" );
      snprintf ( info->vendor,
-                DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH,
-                "convergence integrated media GmbH" );
+                DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH, "directfb.org" );
 
      info->version.major = 0;
      info->version.minor = 1;
@@ -858,7 +1116,10 @@ driver_open_device( CoreInputDevice  *device,
      char             buf[32];
      unsigned long    ledbit[NBITS(LED_MAX)];
      LinuxInputData  *data;
+#ifdef LINUX_INPUT_USE_FBDEV
      FBDev           *dfb_fbdev = dfb_system_data();
+#endif
+     bool             touchpad;
 
      snprintf( buf, 32, "/dev/input/event%d", device_nums[number] );
 
@@ -879,17 +1140,20 @@ driver_open_device( CoreInputDevice  *device,
      }
 
      /* fill device info structure */
-     get_device_info( fd, info );
+     get_device_info( fd, info, &touchpad );
 
      /* allocate and fill private data */
      data = D_CALLOC( 1, sizeof(LinuxInputData) );
 
      data->fd     = fd;
      data->device = device;
+#ifdef LINUX_INPUT_USE_FBDEV
      data->vt     = dfb_fbdev->vt;
+#endif
+     data->touchpad = touchpad;
 
      /* check if the device has LEDs */
-     ret = ioctl( fd, EVIOCGBIT(EV_LED, LED_MAX), ledbit );
+     ret = ioctl( fd, EVIOCGBIT(EV_LED, sizeof(ledbit)), ledbit );
      if (ret < 0)
           D_PERROR( "DirectFB/linux_input: could not get LED bits" );
      else
@@ -899,7 +1163,7 @@ driver_open_device( CoreInputDevice  *device,
 
      if (data->has_leds) {
           /* get LED state */
-          ret = ioctl( fd, EVIOCGLED(LED_MAX), data->led_state );
+          ret = ioctl( fd, EVIOCGLED(sizeof(data->led_state)), data->led_state );
           if (ret < 0) {
                D_PERROR( "DirectFB/linux_input: could not get LED state" );
                ioctl( fd, EVIOCGRAB, 0 );
@@ -932,6 +1196,7 @@ driver_get_keymap_entry( CoreInputDevice           *device,
                          void                      *driver_data,
                          DFBInputDeviceKeymapEntry *entry )
 {
+#ifdef LINUX_INPUT_USE_FBDEV
      LinuxInputData             *data = (LinuxInputData*) driver_data;
      int                         code = entry->code;
      unsigned short              value;
@@ -984,6 +1249,9 @@ driver_get_keymap_entry( CoreInputDevice           *device,
                                                             DIKSI_ALT_SHIFT );
 
      return DFB_OK;
+#else
+     return DFB_UNSUPPORTED;
+#endif
 }
 
 /*
@@ -1015,3 +1283,262 @@ driver_close_device( void *driver_data )
      /* free private data */
      D_FREE( data );
 }
+
+static int
+timeout_is_set( struct timeval *timeout )
+{
+     return timeout->tv_sec || timeout->tv_usec;
+}
+
+static int
+timeout_passed( struct timeval *timeout, struct timeval *current )
+{
+     if (!timeout_is_set( timeout ))
+          return 1;
+
+     return current->tv_sec > timeout->tv_sec ||
+          (current->tv_sec == timeout->tv_sec && current->tv_usec > timeout->tv_usec);
+}
+
+static void
+timeout_clear( struct timeval *timeout )
+{
+     timeout->tv_sec  = 0;
+     timeout->tv_usec = 0;
+}
+
+static void
+timeout_add( struct timeval *timeout, struct timeval *add )
+{
+     timeout->tv_sec += add->tv_sec;
+     timeout->tv_usec += add->tv_usec;
+     while (timeout->tv_usec >= 1000000) {
+          timeout->tv_sec++;
+          timeout->tv_usec -= 1000000;
+     }
+}
+
+static void
+timeout_sub( struct timeval *timeout, struct timeval *sub )
+{
+     timeout->tv_sec -= sub->tv_sec;
+     timeout->tv_usec -= sub->tv_usec;
+     while (timeout->tv_usec < 0) {
+          timeout->tv_sec--;
+          timeout->tv_usec += 1000000;
+     }
+}
+
+static void
+touchpad_fsm_init( struct touchpad_fsm_state *state )
+{
+     state->x.old = -1;
+     state->y.old = -1;
+     state->fsm_state = TOUCHPAD_FSM_START;
+     timeout_clear( &state->timeout );
+}
+
+static int
+touchpad_normalize( struct touchpad_axis *axis, int value )
+{
+     return ((value - axis->min) << 9) / (axis->max - axis->min);
+}
+
+static int
+touchpad_translate( struct touchpad_fsm_state *state,
+                    struct input_event        *levt,
+                    DFBInputEvent             *devt )
+{
+     struct touchpad_axis *axis = NULL;
+     int abs, rel;
+
+     devt->flags     = DIEF_TIMESTAMP | DIEF_AXISREL;
+     devt->timestamp = levt->time;
+     devt->type      = DIET_AXISMOTION;
+
+     switch (levt->code) {
+     case ABS_X:
+          axis       = &state->x;
+          devt->axis = DIAI_X;
+          break;
+     case ABS_Y:
+          axis       = &state->y;
+          devt->axis = DIAI_Y;
+          break;
+     default:
+          return 0;
+     }
+
+     abs = touchpad_normalize( axis, levt->value );
+     if (axis->old == -1)
+          axis->old = abs;
+     rel = abs - axis->old;
+
+#define ACCEL_THRESHOLD 25
+#define ACCEL_NUM       3
+#define ACCEL_DENOM     1
+
+     if (rel > ACCEL_THRESHOLD)
+          rel += (rel - ACCEL_THRESHOLD) * ACCEL_NUM / ACCEL_DENOM;
+     else if (rel < -ACCEL_THRESHOLD)
+          rel += (rel + ACCEL_THRESHOLD) * ACCEL_NUM / ACCEL_DENOM;
+
+     axis->old     = abs;
+     devt->axisrel = rel;
+
+     return 1;
+}
+
+static int
+touchpad_finger_landing( const struct input_event *levt )
+{
+     return levt->type == EV_KEY && levt->code == BTN_TOUCH && levt->value == 1;
+}
+
+static int
+touchpad_finger_leaving( const struct input_event *levt )
+{
+     return levt->type == EV_KEY && levt->code == BTN_TOUCH && levt->value == 0;
+}
+
+static int
+touchpad_finger_moving( const struct input_event *levt )
+{
+     return levt->type == EV_ABS && (levt->code == ABS_X || levt->code == ABS_Y);
+}
+
+/*
+ * This FSM takes into accout finger landing on touchpad and leaving and
+ * translates absolute DFBInputEvent into a relative one
+ */
+static int
+touchpad_fsm( struct touchpad_fsm_state *state,
+              struct input_event        *levt,
+              DFBInputEvent             *devt )
+{
+     struct timeval timeout = { 0, 125000 };
+
+     /* select() timeout? */
+     if (!levt) {
+          /* Check if button release is due. */
+          if (state->fsm_state == TOUCHPAD_FSM_DRAG_START) {
+               devt->flags     = DIEF_TIMESTAMP;
+               devt->timestamp = state->timeout; /* timeout of current time? */
+               devt->type      = DIET_BUTTONRELEASE;
+               devt->button    = DIBI_FIRST;
+
+               touchpad_fsm_init( state );
+               return 1;
+          }
+
+          /* Already passed, clear it so select() won't return until there is something to do. */
+          timeout_clear( &state->timeout );
+          return 0;
+     }
+
+     /* More or less ignore these events for now */
+     if ((levt->type == EV_SYN && levt->code == SYN_REPORT) ||
+         (levt->type == EV_ABS && levt->code == ABS_PRESSURE) ||
+         (levt->type == EV_ABS && levt->code == ABS_TOOL_WIDTH) ||
+         (levt->type == EV_KEY && levt->code == BTN_TOOL_FINGER) ||
+         (levt->type == EV_KEY && levt->code == BTN_TOOL_DOUBLETAP) ||
+         (levt->type == EV_KEY && levt->code == BTN_TOOL_TRIPLETAP)) {
+
+          /* Check if button release is due. */
+          if (state->fsm_state == TOUCHPAD_FSM_DRAG_START &&
+              timeout_passed( &state->timeout, &levt->time )) {
+               devt->flags     = DIEF_TIMESTAMP;
+               devt->timestamp = state->timeout; /* timeout of levt->time? */
+               devt->type      = DIET_BUTTONRELEASE;
+               devt->button    = DIBI_FIRST;
+
+               touchpad_fsm_init( state );
+               return 1;
+          }
+
+          return 0;
+     }
+
+     /* Use translate_event() for other events. */
+     if (!(levt->type == EV_KEY && levt->code == BTN_TOUCH) &&
+         !(levt->type == EV_ABS && (levt->code == ABS_X || levt->code == ABS_Y)))
+          return -1;
+
+     switch (state->fsm_state) {
+     case TOUCHPAD_FSM_START:
+          if (touchpad_finger_landing( levt )) {
+               state->fsm_state = TOUCHPAD_FSM_MAIN;
+               state->timeout = levt->time;
+               timeout_add( &state->timeout, &timeout );
+          }
+          return 0;
+
+     case TOUCHPAD_FSM_MAIN:
+          if (touchpad_finger_moving( levt )) {
+               if (1){//timeout_passed( &state->timeout, &levt->time )) {
+                    //timeout_clear( &state->timeout );
+                    return touchpad_translate( state, levt, devt );
+               }
+          }
+          else if (touchpad_finger_leaving( levt )) {
+               if (!timeout_passed( &state->timeout, &levt->time )) {
+                    devt->flags     = DIEF_TIMESTAMP;
+                    devt->timestamp = levt->time;
+                    devt->type      = DIET_BUTTONPRESS;
+                    devt->button    = DIBI_FIRST;
+
+                    touchpad_fsm_init( state );
+                    state->fsm_state = TOUCHPAD_FSM_DRAG_START;
+                    state->timeout = levt->time;
+                    timeout_add( &state->timeout, &timeout );
+                    return 1;
+               }
+               else {
+                    touchpad_fsm_init( state );
+               }
+          }
+          return 0;
+
+     case TOUCHPAD_FSM_DRAG_START:
+          if (timeout_passed( &state->timeout, &levt->time )){
+               devt->flags     = DIEF_TIMESTAMP;
+               devt->timestamp = state->timeout; /* timeout of levt->time? */
+               devt->type      = DIET_BUTTONRELEASE;
+               devt->button    = DIBI_FIRST;
+
+               touchpad_fsm_init(state);
+               return 1;
+          }
+          else {
+               if (touchpad_finger_landing( levt )) {
+                    state->fsm_state = TOUCHPAD_FSM_DRAG_MAIN;
+                    state->timeout = levt->time;
+                    timeout_add( &state->timeout, &timeout );
+               }
+          }
+          return 0;
+
+     case TOUCHPAD_FSM_DRAG_MAIN:
+          if (touchpad_finger_moving( levt )) {
+               if (1){//timeout_passed( &state->timeout, &levt->time )) {
+                   //timeout_clear( &state->timeout );
+                    return touchpad_translate( state, levt, devt );
+               }
+          }
+          else if (touchpad_finger_leaving( levt )) {
+               devt->flags     = DIEF_TIMESTAMP;
+               devt->timestamp = levt->time;
+               devt->type      = DIET_BUTTONRELEASE;
+               devt->button    = DIBI_FIRST;
+
+               touchpad_fsm_init( state );
+               return 1;
+          }
+          return 0;
+
+     default:
+          return 0;
+     }
+
+     return 0;
+}
