@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -48,6 +51,7 @@
 #include <direct/mem.h>
 #include <direct/messages.h>
 #include <direct/thread.h>
+#include <direct/util.h>
 
 #include <core/input_driver.h>
 
@@ -87,7 +91,7 @@ static DFBInputDeviceKeySymbol lirc_parse_line(const char *line)
 
      if (!keynames_sorted) {
           qsort ( keynames,
-                  sizeof(keynames) / sizeof(keynames[0]),
+                  D_ARRAY_SIZE( keynames ),
                   sizeof(keynames[0]),
                   (__compar_fn_t) keynames_sort_compare );
           keynames_sorted = true;
@@ -114,7 +118,7 @@ static DFBInputDeviceKeySymbol lirc_parse_line(const char *line)
                return (DFBInputDeviceKeySymbol) name[0];
           default:
                symbol_name = bsearch( name, keynames,
-                                      sizeof(keynames)/sizeof(keynames[0]),
+                                      D_ARRAY_SIZE( keynames ),
                                       sizeof(keynames[0]),
                                       (__compar_fn_t) keynames_compare );
                if (symbol_name)
@@ -231,7 +235,7 @@ driver_get_available()
      struct sockaddr_un addr;
 
      addr.sun_family = AF_UNIX;
-     strncpy( addr.sun_path, "/dev/lircd", sizeof(addr.sun_path) );
+     direct_snputs( addr.sun_path, "/dev/lircd", sizeof(addr.sun_path) );
 
      fd = socket( PF_UNIX, SOCK_STREAM, 0 );
      if (fd < 0)
@@ -255,8 +259,7 @@ driver_get_info( InputDriverInfo *info )
                DFB_INPUT_DRIVER_INFO_NAME_LENGTH, "LIRC Driver" );
 
      snprintf( info->vendor,
-               DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH,
-               "convergence integrated media GmbH" );
+               DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH, "directfb.org" );
 
      info->version.major = 0;
      info->version.minor = 2;
@@ -274,7 +277,7 @@ driver_open_device( CoreInputDevice      *device,
 
      /* create socket */
      sa.sun_family = AF_UNIX;
-     strncpy( sa.sun_path, "/dev/lircd", sizeof(sa.sun_path) );
+     direct_snputs( sa.sun_path, "/dev/lircd", sizeof(sa.sun_path) );
 
      fd = socket( PF_UNIX, SOCK_STREAM, 0 );
      if (fd < 0) {
