@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -26,6 +27,7 @@
 */
 
 #include <config.h>
+#include <stdarg.h>
 
 #include <directfb.h>
 
@@ -38,7 +40,7 @@
 
 #include <media/idirectfbfont.h>
 
-#include <direct/tree.h>
+#include <direct/hash.h>
 
 #include <direct/interface.h>
 #include <direct/mem.h>
@@ -57,9 +59,7 @@ Probe( IDirectFBFont_ProbeContext *ctx );
 
 static DFBResult
 Construct( IDirectFBFont      *thiz,
-           CoreDFB            *core,
-           const char         *filename,
-           DFBFontDescription *desc );
+	   ... );
 
 #include <direct/interface_implementation.h>
 
@@ -78,20 +78,35 @@ Probe( IDirectFBFont_ProbeContext *ctx )
 
 static DFBResult
 Construct( IDirectFBFont      *thiz,
-           CoreDFB            *core,
-           const char         *filename,
-           DFBFontDescription *desc )
+	   ... )
 {
-     CoreFont    *font;
-     CoreSurface *surface;
-     void        *dst;
-     __u8        *pixels;
-     int          pitch;
-     int          i;
+     DFBResult         ret;
+     CoreFont         *font;
+     CoreSurface      *surface;
+     CoreFontCacheRow *row;
+     void             *dst;
+     u8               *pixels;
+     int               pitch;
+     int               i;
+
+     CoreDFB *core;
+     char *filename;
+     DFBFontDescription *desc;
+
+     va_list tag;
+     va_start(tag, thiz);
+     core = va_arg(tag, CoreDFB *);
+     filename = va_arg(tag, char *);
+     desc = va_arg(tag, DFBFontDescription *);
+     va_end( tag );
 
      D_HEAVYDEBUG( "DirectFB/FontDefault: Construct default font");
 
-     font = dfb_font_create( core );
+     ret = dfb_font_create( core, &font );
+     if (ret) {
+          DIRECT_DEALLOCATE_INTERFACE( thiz );
+          return ret;
+     }
 
      D_ASSERT( font->pixel_format == DSPF_ARGB ||
                font->pixel_format == DSPF_AiRGB ||
@@ -99,20 +114,33 @@ Construct( IDirectFBFont      *thiz,
                font->pixel_format == DSPF_ARGB2554 ||
                font->pixel_format == DSPF_ARGB1555 ||
                font->pixel_format == DSPF_A8 ||
+               font->pixel_format == DSPF_A4 ||
                font->pixel_format == DSPF_A1 );
 
      font->height    = DEFAULT_FONT_HEIGHT;
      font->ascender  = DEFAULT_FONT_ASCENDER;
      font->descender = DEFAULT_FONT_DESCENDER;
 
+     row = D_CALLOC( 1, sizeof(CoreFontCacheRow) );
+     if (!row) {
+          D_OOM();
+          dfb_font_destroy( font );
+          DIRECT_DEALLOCATE_INTERFACE( thiz );
+          return DFB_NOSYSTEMMEMORY;
+     }
+
      dfb_surface_create( core,
                          font_desc.width, font_desc.height, font->pixel_format,
                          CSP_VIDEOHIGH, DSCAPS_NONE, NULL, &surface );
 
-     font->rows = 1;
+     font->num_rows  = 1;
      font->row_width = font_desc.width;
-     font->surfaces = D_MALLOC(sizeof (void *));
-     font->surfaces[0] = surface;
+     font->rows      = D_MALLOC(sizeof (void *));
+     font->rows[0]   = row;
+
+     row->surface = surface;
+
+     D_MAGIC_SET( row, CoreFontCacheRow );
 
      pixels = font_data;
 
@@ -125,7 +153,7 @@ Construct( IDirectFBFont      *thiz,
           const char *glyphs =
           "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
           "abcdefghijklmnopqrstuvwxyz"
-          "01234567890!\"$%&/()=?^<>"
+          "01234567890!\"$%&/()=?^<>" // FIXME: '0' is repeated!
           "|,;.:-_{[]}\\`+*~#'";
 
           if (desc && (desc->flags & DFDESC_ATTRIBUTES) &&
@@ -136,31 +164,35 @@ Construct( IDirectFBFont      *thiz,
 
           for (i = 0; i < font_desc.width; i++) {
                if (pixels[i] == 0xFF) {
-                    data = D_MALLOC(sizeof (CoreGlyphData));
-                    data->surface = surface;
-                    data->start   = start;
-                    data->width   = i - start + 1;
-                    data->height  = font_desc.height - 1;
-                    data->left    = 0;
-                    data->top     = 0;
-                    data->advance = ((desc && (desc->flags &
-                                               DFDESC_FIXEDADVANCE)) ?
-                                     desc->fixed_advance :
-                                     data->width + 1);
-
-                    D_HEAVYDEBUG( "DirectFB/core/fonts: "
-                                   "glyph '%c' at %d, width %d\n",
-                                   glyphs[index], start, i-start );
-
-                    if (font->maxadvance < data->advance)
-                         font->maxadvance = data->advance;
-
                     if (use_unicode)
                          key = glyphs[index];
                     else
                          key = index;
+                         
+                    if (!direct_hash_lookup( font->glyph_hash, key )) { 
+                         data = D_CALLOC( 1, sizeof(CoreGlyphData) );
+                         data->surface = surface;
+                         data->start   = start;
+                         data->width   = i - start + 1;
+                         data->height  = font_desc.height - 1;
+                         data->left    = 0;
+                         data->top     = 0;
+                         data->advance = ((desc && (desc->flags &
+                                                    DFDESC_FIXEDADVANCE)) ?
+                                         desc->fixed_advance :
+                                         data->width + 1);
 
-                    direct_tree_insert (font->glyph_infos, (void*) key, data);
+                         D_HEAVYDEBUG( "DirectFB/core/fonts: "
+                                       "glyph '%c' at %d, width %d\n",
+                                       glyphs[index], start, i-start );
+
+                         D_MAGIC_SET( data, CoreGlyphData );
+
+                         if (font->maxadvance < data->advance)
+                              font->maxadvance = data->advance;
+                              
+                         direct_hash_insert( font->glyph_hash, key, data );
+                    }
 
                     start = i + 1;
                     index++;
@@ -170,24 +202,26 @@ Construct( IDirectFBFont      *thiz,
           }
 
           /*  space  */
-          data = D_CALLOC(1, sizeof (CoreGlyphData));
+          data = D_CALLOC( 1, sizeof(CoreGlyphData) );
           data->advance = 5;
 
+          D_MAGIC_SET( data, CoreGlyphData );
+
           if (use_unicode)
                key = 32;
           else
                key = index;
 
-          direct_tree_insert (font->glyph_infos, (void*) key, data);
+          direct_hash_insert( font->glyph_hash, key, data );
      }
 
-     dfb_surface_soft_lock( surface, DSLF_WRITE, &dst, &pitch, 0 );
+     dfb_surface_soft_lock( core, surface, DSLF_WRITE, &dst, &pitch, 0 );
 
      for (i = 1; i < font_desc.height; i++) {
-          int    i, j, n;
-          __u8  *dst8  = dst;
-          __u16 *dst16 = dst;
-          __u32 *dst32 = dst;
+          int    j, n;
+          u8    *dst8  = dst;
+          u16   *dst16 = dst;
+          u32   *dst32 = dst;
 
           pixels += font_desc.preallocated[0].pitch;
           switch (surface->format) {
@@ -214,9 +248,13 @@ Construct( IDirectFBFont      *thiz,
                case DSPF_A8:
                     direct_memcpy(dst, pixels, font_desc.width);
                     break;
+               case DSPF_A4:
+                    for (n=0, j=0; j < font_desc.width; n++, j+=2)
+                         dst8[n] = (pixels[j] & 0xF0) | (pixels[j+1] >> 4);
+                    break;
                case DSPF_A1:
                     for (i=0, j=0; i < font_desc.width; ++j) {
-                         register __u8 p = 0;
+                         register u8 p = 0;
 
                          for (n=0; n<8 && i<font_desc.width; ++i, ++n)
                               p |= (pixels[i] & 0x80) >> n;
