@@ -1,12 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
-   
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
+
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,15 +26,16 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-
+#include <stdarg.h>
 
 #include <directfb.h>
-#include <directfb_internals.h>
 
 #include <display/idirectfbsurface.h>
 
@@ -45,8 +48,9 @@
 #include <core/surfaces.h>
 
 #include <misc/gfx_util.h>
-#include <misc/mem.h>
-#include <misc/memcpy.h>
+#include <direct/interface.h>
+#include <direct/mem.h>
+#include <direct/memcpy.h>
 #include <misc/util.h>
 
 static DFBResult
@@ -54,11 +58,11 @@ Probe( IDirectFBImageProvider_ProbeContext *ctx );
 
 static DFBResult
 Construct( IDirectFBImageProvider *thiz,
-           IDirectFBDataBuffer    *buffer );
+           ... );
 
-#include <interface_implementation.h>
+#include <direct/interface_implementation.h>
 
-DFB_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, GIF )
+DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, GIF )
 
 
 #ifndef NODEBUG
@@ -90,17 +94,22 @@ typedef struct {
      int                  ref;      /* reference counter */
 
      IDirectFBDataBuffer *buffer;
-     
-     
+
+     u32                 *image;
+     int                  image_width;
+     int                  image_height;
+     bool                 image_transparency;
+     u32                  image_colorkey;
+
      unsigned int  Width;
      unsigned int  Height;
-     __u8          ColorMap[3][MAXCOLORMAPSIZE];
+     u8            ColorMap[3][MAXCOLORMAPSIZE];
      unsigned int  BitPixel;
      unsigned int  ColorResolution;
-     __u32         Background;
+     u32           Background;
      unsigned int  AspectRatio;
-     
-     
+
+
      int GrayScale;
      int transparent;
      int delayTime;
@@ -108,10 +117,10 @@ typedef struct {
      int disposal;
 
 
-     __u8 buf[280];
+     u8 buf[280];
      int curbit, lastbit, done, last_byte;
-     
-     
+
+
      int fresh;
      int code_size, set_code_size;
      int max_code, max_code_size;
@@ -119,15 +128,20 @@ typedef struct {
      int clear_code, end_code;
      int table[2][(1<< MAX_LWZ_BITS)];
      int stack[(1<<(MAX_LWZ_BITS))*2], *sp;
+
+     DIRenderCallback  render_callback;
+     void             *render_callback_ctx;
+
+     CoreDFB *core;
 } IDirectFBImageProvider_GIF_data;
 
 static bool verbose       = false;
 static bool showComment   = false;
 static bool ZeroDataBlock = false;
 
-static __u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
+static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
                        int *width, int *height, bool *transparency,
-                       __u32 *key_rgb, bool alpha, bool headeronly);
+                       u32 *key_rgb, bool alpha, bool headeronly);
 
 static bool ReadOK( IDirectFBDataBuffer *buffer, void *data, unsigned int len );
 
@@ -160,7 +174,7 @@ IDirectFBImageProvider_GIF_GetImageDescription( IDirectFBImageProvider *thiz,
 static DFBResult
 Probe( IDirectFBImageProvider_ProbeContext *ctx )
 {
-     if (strncmp (ctx->header, "GIF8", 4) == 0)
+     if (strncmp ((char*) ctx->header, "GIF8", 4) == 0)
           return DFB_OK;
 
      return DFB_UNSUPPORTED;
@@ -168,19 +182,41 @@ Probe( IDirectFBImageProvider_ProbeContext *ctx )
 
 static DFBResult
 Construct( IDirectFBImageProvider *thiz,
-           IDirectFBDataBuffer    *buffer )
+           ... )
 {
-     DFB_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_GIF)
+     IDirectFBDataBuffer *buffer;
+     CoreDFB             *core;
+     va_list              tag;
 
-     data->ref    = 1;
-     data->buffer = buffer;
+     DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_GIF)
+
+     va_start( tag, thiz );
+     buffer = va_arg( tag, IDirectFBDataBuffer * );
+     core = va_arg( tag, CoreDFB * );
+     va_end( tag );
+     
+     data->ref = 1;
 
      data->GrayScale   = -1;
      data->transparent = -1;
      data->delayTime   = -1;
-     
+
+     data->core = core;
+     data->buffer = buffer;
      buffer->AddRef( buffer );
 
+     data->image = ReadGIF( data, 1, &data->image_width, &data->image_height,
+                            &data->image_transparency, &data->image_colorkey,
+                            true, false );
+
+     buffer->Release( buffer );
+     data->buffer = NULL;
+     
+     if (!data->image) {
+          DIRECT_DEALLOCATE_INTERFACE( thiz );
+          return DFB_FAILURE;
+     }
+
      thiz->AddRef = IDirectFBImageProvider_GIF_AddRef;
      thiz->Release = IDirectFBImageProvider_GIF_Release;
      thiz->RenderTo = IDirectFBImageProvider_GIF_RenderTo;
@@ -198,15 +234,16 @@ IDirectFBImageProvider_GIF_Destruct( IDirectFBImageProvider *thiz )
      IDirectFBImageProvider_GIF_data *data =
                                    (IDirectFBImageProvider_GIF_data*)thiz->priv;
 
-     data->buffer->Release( data->buffer );
+     if (data->image)
+          D_FREE( data->image );
 
-     DFB_DEALLOCATE_INTERFACE( thiz );
+     DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
 static DFBResult
 IDirectFBImageProvider_GIF_AddRef( IDirectFBImageProvider *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBImageProvider_GIF)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_GIF)
 
      data->ref++;
 
@@ -216,7 +253,7 @@ IDirectFBImageProvider_GIF_AddRef( IDirectFBImageProvider *thiz )
 static DFBResult
 IDirectFBImageProvider_GIF_Release( IDirectFBImageProvider *thiz )
 {
-     INTERFACE_GET_DATA(IDirectFBImageProvider_GIF)
+     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_GIF)
 
      if (--data->ref == 0) {
           IDirectFBImageProvider_GIF_Destruct( thiz );
@@ -230,13 +267,14 @@ IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
                                      IDirectFBSurface       *destination,
                                      const DFBRectangle     *dest_rect )
 {
-     DFBRectangle           rect = { 0, 0, 0, 0 };
+     DFBRegion              clip;
+     DFBRectangle           rect;
      DFBSurfacePixelFormat  format;
      IDirectFBSurface_data *dst_data;
      CoreSurface           *dst_surface;
      int err;
 
-     INTERFACE_GET_DATA (IDirectFBImageProvider_GIF)
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_GIF)
 
      dst_data = (IDirectFBSurface_data*) destination->priv;
      if (!dst_data)
@@ -246,38 +284,49 @@ IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
      if (!dst_surface)
           return DFB_DESTROYED;
 
-     err = destination->GetSize( destination, &rect.w, &rect.h );
-     if (err)
-          return err;
+     dfb_region_from_rectangle( &clip, &dst_data->area.current );
+
+     if (dest_rect) {
+          if (dest_rect->w < 1 || dest_rect->h < 1)
+               return DFB_INVARG;
+          rect = *dest_rect; 
+          rect.x += dst_data->area.wanted.x;
+          rect.y += dst_data->area.wanted.y;
+     }
+     else {
+          rect = dst_data->area.wanted;
+     }
 
      err = destination->GetPixelFormat( destination, &format );
      if (err)
           return err;
 
      /* actual loading and rendering */
-     if (dest_rect == NULL || dfb_rectangle_intersect ( &rect, dest_rect )) {
-          __u32 *image_data;
-          bool  transparency;
+     if (dfb_rectangle_region_intersects( &rect, &clip )) {
           void  *dst;
-          int    pitch, src_width, src_height;
+          int    pitch;
 
-          image_data = ReadGIF( data, 1, &src_width, &src_height,
-                                &transparency, NULL,
-                                DFB_PIXELFORMAT_HAS_ALPHA (format),
-                                false );
+          err = dfb_surface_soft_lock( data->core, dst_surface, DSLF_WRITE, &dst, &pitch, 0 );
+          if (err)
+               return err;
 
-          if (image_data) {
-               err = destination->Lock( destination, DSLF_WRITE, &dst, &pitch );
-               if (err) {
-                    DFBFREE( image_data );
-                    return err;
-               }
+          dfb_scale_linear_32( data->image, data->image_width, data->image_height,
+                               dst, pitch, &rect, dst_surface, &clip );
 
-               dfb_scale_linear_32( image_data, src_width, src_height,
-                                    dst, pitch, &rect, dst_surface );
+          dfb_surface_unlock( dst_surface, 0 );
 
-               destination->Unlock( destination );
-               DFBFREE(image_data);
+          if (data->render_callback) {
+               DIRenderCallbackResult r;
+
+               rect.x = 0;
+               rect.y = 0;
+               rect.w = data->image_width;
+               rect.h = data->image_height;
+
+               r = data->render_callback( &rect, data->render_callback_ctx );
+
+               if (r != DIRCR_OK)
+                       return DFB_INTERRUPTED;
           }
      }
 
@@ -289,25 +338,23 @@ IDirectFBImageProvider_GIF_SetRenderCallback( IDirectFBImageProvider *thiz,
                                               DIRenderCallback        callback,
                                               void                   *context )
 {
-     return DFB_UNIMPLEMENTED;
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_GIF)
+
+     data->render_callback     = callback;
+     data->render_callback_ctx = context;
+     
+     return DFB_OK;
 }
 
 static DFBResult
 IDirectFBImageProvider_GIF_GetSurfaceDescription( IDirectFBImageProvider *thiz,
                                                   DFBSurfaceDescription  *dsc )
 {
-     int  width;
-     int  height;
-     bool transparency;
-     
-     INTERFACE_GET_DATA (IDirectFBImageProvider_GIF)
-          
-     ReadGIF( data, 1, &width, &height,
-              &transparency, NULL, false, true );
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_GIF)
 
      dsc->flags       = DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT;
-     dsc->width       = width;
-     dsc->height      = height;
+     dsc->width       = data->image_width;
+     dsc->height      = data->image_height;
      dsc->pixelformat = dfb_primary_layer_pixelformat();
 
      return DFB_OK;
@@ -317,22 +364,14 @@ static DFBResult
 IDirectFBImageProvider_GIF_GetImageDescription( IDirectFBImageProvider *thiz,
                                                 DFBImageDescription    *dsc )
 {
-     int   width;
-     int   height;
-     bool  transparency;
-     __u32 key_rgb;
-     
-     INTERFACE_GET_DATA (IDirectFBImageProvider_GIF)
+     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_GIF)
 
-     ReadGIF( data, 1, &width, &height,
-              &transparency, &key_rgb, false, true );
-
-     if (transparency) {
+     if (data->image_transparency) {
           dsc->caps = DICAPS_COLORKEY;
 
-          dsc->colorkey_r = (key_rgb & 0xff0000) >> 16;
-          dsc->colorkey_g = (key_rgb & 0x00ff00) >>  8;
-          dsc->colorkey_b = (key_rgb & 0x0000ff);
+          dsc->colorkey_r = (data->image_colorkey & 0xff0000) >> 16;
+          dsc->colorkey_g = (data->image_colorkey & 0x00ff00) >>  8;
+          dsc->colorkey_b = (data->image_colorkey & 0x0000ff);
      }
      else
           dsc->caps = DICAPS_NONE;
@@ -346,10 +385,10 @@ IDirectFBImageProvider_GIF_GetImageDescription( IDirectFBImageProvider *thiz,
  **********************************/
 
 static int ReadColorMap( IDirectFBDataBuffer *buffer, int number,
-                         __u8 buf[3][MAXCOLORMAPSIZE] )
+                         u8 buf[3][MAXCOLORMAPSIZE] )
 {
      int     i;
-     __u8 rgb[3];
+     u8 rgb[3];
 
      for (i = 0; i < number; ++i) {
           if (! ReadOK( buffer, rgb, sizeof(rgb) )) {
@@ -364,7 +403,7 @@ static int ReadColorMap( IDirectFBDataBuffer *buffer, int number,
      return false;
 }
 
-static int GetDataBlock(IDirectFBDataBuffer *buffer, __u8 *buf)
+static int GetDataBlock(IDirectFBDataBuffer *buffer, u8 *buf)
 {
      unsigned char count;
 
@@ -436,21 +475,21 @@ static int DoExtension( IDirectFBImageProvider_GIF_data *data, int label )
                break;
           case 0xfe:              /* Comment Extension */
                str = "Comment Extension";
-               while (GetDataBlock( data->buffer, (__u8*) buf ) != 0) {
+               while (GetDataBlock( data->buffer, (u8*) buf ) != 0) {
                     if (showComment)
                          GIFERRORMSG("gif comment: %s", buf );
                     }
                return false;
           case 0xf9:              /* Graphic Control Extension */
                str = "Graphic Control Extension";
-               (void) GetDataBlock( data->buffer, (__u8*) buf );
+               (void) GetDataBlock( data->buffer, (u8*) buf );
                data->disposal    = (buf[0] >> 2) & 0x7;
                data->inputFlag   = (buf[0] >> 1) & 0x1;
                data->delayTime   = LM_to_uint( buf[1], buf[2] );
                if ((buf[0] & 0x1) != 0) {
                     data->transparent = buf[3];
                }
-               while (GetDataBlock( data->buffer, (__u8*) buf ) != 0)
+               while (GetDataBlock( data->buffer, (u8*) buf ) != 0)
                     ;
                return false;
           default:
@@ -462,7 +501,7 @@ static int DoExtension( IDirectFBImageProvider_GIF_data *data, int label )
      if (verbose)
           GIFERRORMSG("got a '%s' extension", str );
 
-     while (GetDataBlock( data->buffer, (__u8*) buf ) != 0)
+     while (GetDataBlock( data->buffer, (u8*) buf ) != 0)
           ;
 
      return false;
@@ -528,7 +567,7 @@ static int LWZReadByte( IDirectFBImageProvider_GIF_data *data, int flag, int inp
           }
           else if (code == data->end_code) {
                int count;
-               __u8 buf[260];
+               u8 buf[260];
 
                if (ZeroDataBlock) {
                     return -2;
@@ -584,26 +623,26 @@ static int LWZReadByte( IDirectFBImageProvider_GIF_data *data, int flag, int inp
 
 static int SortColors (const void *a, const void *b)
 {
-     return (*((const __u8 *) a) - *((const __u8 *) b));
+     return (*((const u8 *) a) - *((const u8 *) b));
 }
 
 /*  looks for a color that is not in the colormap and ideally not
     even close to the colors used in the colormap  */
-static __u32 FindColorKey( int n_colors, __u8 cmap[3][MAXCOLORMAPSIZE] )
+static u32 FindColorKey( int n_colors, u8 cmap[3][MAXCOLORMAPSIZE] )
 {
-     __u32 color = 0xFF000000;
-     __u8  csort[MAXCOLORMAPSIZE];
+     u32   color = 0xFF000000;
+     u8    csort[MAXCOLORMAPSIZE];
      int   i, j, index, d;
 
      if (n_colors < 1)
           return color;
 
-     DFB_ASSERT( n_colors <= MAXCOLORMAPSIZE );
+     D_ASSERT( n_colors <= MAXCOLORMAPSIZE );
 
      for (i = 0; i < 3; i++) {
-          dfb_memcpy( csort, cmap[i], n_colors );
+          direct_memcpy( csort, cmap[i], n_colors );
           qsort( csort, n_colors, 1, SortColors );
-          
+
           for (j = 1, index = 0, d = 0; j < n_colors; j++) {
                if (csort[j] - csort[j-1] > d) {
                     d = csort[j] - csort[j-1];
@@ -617,7 +656,7 @@ static __u32 FindColorKey( int n_colors, __u8 cmap[3][MAXCOLORMAPSIZE] )
           if (0xFF - (csort[n_colors - 1]) > d) {
                index = n_colors + 1;
           }
-          
+
           if (index < n_colors)
                csort[0] = csort[index] - (d/2);
           else if (index == n_colors)
@@ -631,14 +670,14 @@ static __u32 FindColorKey( int n_colors, __u8 cmap[3][MAXCOLORMAPSIZE] )
      return color;
 }
 
-static __u32* ReadImage( IDirectFBImageProvider_GIF_data *data, int width, int height,
-                         __u8 cmap[3][MAXCOLORMAPSIZE], __u32 key_rgb,
-                         bool interlace, bool ignore )
+static u32* ReadImage( IDirectFBImageProvider_GIF_data *data, int width, int height,
+                       u8 cmap[3][MAXCOLORMAPSIZE], u32 key_rgb,
+                       bool interlace, bool ignore )
 {
-     __u8 c;
+     u8 c;
      int v;
      int xpos = 0, ypos = 0, pass = 0;
-     __u32 *image;
+     u32 *image;
 
      /*
      **  Initialize the decompression routines
@@ -660,8 +699,11 @@ static __u32* ReadImage( IDirectFBImageProvider_GIF_data *data, int width, int h
                ;
           return NULL;
      }
-
-     if ((image = DFBMALLOC(width * height * 4)) == NULL) {
+     
+     // FIXME: allocates four additional bytes because the scaling functions
+     //        in src/misc/gfx_util.c have an off-by-one bug which causes
+     //        segfaults on darwin/osx (not on linux)           
+     if ((image = D_MALLOC(width * height * 4 + 4)) == NULL) {
           GIFERRORMSG("couldn't alloc space for image" );
      }
 
@@ -671,7 +713,7 @@ static __u32* ReadImage( IDirectFBImageProvider_GIF_data *data, int width, int h
      }
 
      while ((v = LWZReadByte( data, false, c )) >= 0 ) {
-          __u32 *dst = image + (ypos * width + xpos);
+          u32 *dst = image + (ypos * width + xpos);
 
           if (v == data->transparent) {
                *dst++ = key_rgb;
@@ -735,26 +777,18 @@ fini:
 }
 
 
-static __u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
-                       int *width, int *height, bool *transparency,
-                       __u32 *key_rgb, bool alpha, bool headeronly)
+static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
+                     int *width, int *height, bool *transparency,
+                     u32 *key_rgb, bool alpha, bool headeronly)
 {
-     DFBResult ret;
-     __u8      buf[16];
-     __u8      c;
-     __u8      localColorMap[3][MAXCOLORMAPSIZE];
-     __u32     colorKey = 0;
-     bool      useGlobalColormap;
-     int       bitPixel;
-     int       imageCount = 0;
-     char      version[4];
-
-     /* FIXME: support streamed buffers */
-     ret = data->buffer->SeekTo( data->buffer, 0 );
-     if (ret) {
-          DirectFBError( "(DirectFB/ImageProvider_GIF) Unable to seek", ret );
-          return NULL;
-     }
+     u8    buf[16];
+     u8    c;
+     u8    localColorMap[3][MAXCOLORMAPSIZE];
+     u32   colorKey = 0;
+     bool  useGlobalColormap;
+     int   bitPixel;
+     int   imageCount = 0;
+     char  version[4];
 
      if (! ReadOK( data->buffer, buf, 6 )) {
           GIFERRORMSG("error reading magic number" );
@@ -764,8 +798,7 @@ static __u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
           GIFERRORMSG("not a GIF file" );
      }
 
-     strncpy( version, (char *)buf + 3, 3 );
-     version[3] = '\0';
+     direct_snputs( version, (char *)buf + 3, 4 );
 
      if ((strcmp(version, "87a") != 0) && (strcmp(version, "89a") != 0)) {
           GIFERRORMSG("bad version number, not '87a' or '89a'" );
