@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -34,6 +35,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <stdarg.h>
 
 #include <string.h>
 
@@ -44,7 +46,9 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 
+#ifdef HAVE_LINUX_COMPILER_H
 #include <linux/compiler.h>
+#endif
 #include "videodev.h"
 
 #include <directfb.h>
@@ -83,18 +87,12 @@ Probe( IDirectFBVideoProvider_ProbeContext *ctx );
 
 static DFBResult
 Construct( IDirectFBVideoProvider *thiz,
-           IDirectFBDataBuffer    *buffer );
+           ... );
 
 #include <direct/interface_implementation.h>
 
 DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBVideoProvider, V4L )
 
-#if 0
-#define DEBUG_MSG(...) fprintf(stderr,__VA_ARGS__);
-#else
-#define DEBUG_MSG  do {} while (0)
-#endif
-
 /*
  * private data struct of IDirectFBVideoProvider
  */
@@ -137,6 +135,8 @@ typedef struct {
      pthread_mutex_t          lock;
 
      Reaction                 reaction; /* for the destination listener */
+
+     CoreDFB                 *core;
 } IDirectFBVideoProvider_V4L_data;
 
 static const unsigned int zero = 0;
@@ -240,8 +240,9 @@ static DFBResult IDirectFBVideoProvider_V4L_GetCapabilities (
           /* fixme: interlaced might not be true for field capture */
           *caps |= DVCAPS_BASIC | DVCAPS_SCALE | DVCAPS_INTERLACED;
      }
-     else {
+     else
 #endif
+     {
           *caps = ( DVCAPS_BASIC      |
                     DVCAPS_BRIGHTNESS |
                     DVCAPS_CONTRAST   |
@@ -251,9 +252,8 @@ static DFBResult IDirectFBVideoProvider_V4L_GetCapabilities (
 
           if (data->vcap.type & VID_TYPE_SCALES)
                *caps |= DVCAPS_SCALE;
-#ifdef DFB_HAVE_V4L2
      }
-#endif	
+
      return DFB_OK;
 }
 
@@ -277,13 +277,12 @@ static DFBResult IDirectFBVideoProvider_V4L_GetSurfaceDescription(
           desc->width = 720;  /* fimxe: depends on the selected standard: query standard and set accordingly */
           desc->height = 576;
      }
-     else {
+     else
 #endif
+     {
           desc->width  = data->vcap.maxwidth;
           desc->height = data->vcap.maxheight;
-#ifdef DFB_HAVE_V4L2
      }
-#endif
      desc->pixelformat = dfb_primary_layer_pixelformat();
      desc->caps = DSCAPS_INTERLACED;
 
@@ -370,8 +369,9 @@ static DFBResult IDirectFBVideoProvider_V4L_PlayTo(
      if ( 0 != data->is_v4l2 ) {
           ret = v4l2_playto(surface, &rect, data);
      }
-     else {
+     else
 #endif
+     {
           data->grab_mode = 0;
           if ( getenv("DFB_V4L_GRAB") || surface->caps & DSCAPS_SYSTEMONLY
                || surface->caps & DSCAPS_FLIPPING
@@ -384,7 +384,7 @@ static DFBResult IDirectFBVideoProvider_V4L_PlayTo(
                 * Because we're constantly writing to the surface we
                 * permanently lock it.
                 */
-               ret = dfb_surface_hardware_lock( surface,
+               ret = dfb_surface_hardware_lock( data->core, surface,
                                                 DSLF_WRITE | CSLF_FORCE, false );
 
                dfb_surfacemanager_unlock( surface->manager );
@@ -399,9 +399,7 @@ static DFBResult IDirectFBVideoProvider_V4L_PlayTo(
                ret = v4l_to_surface_grab( surface, &rect, data );
           else
                ret = v4l_to_surface_overlay( surface, &rect, data );
-#ifdef DFB_HAVE_V4L2
      }
-#endif
      if (DFB_OK != ret && !data->grab_mode)
           dfb_surface_unlock( surface, false );
 
@@ -503,8 +501,9 @@ static DFBResult IDirectFBVideoProvider_V4L_GetColorAdjustment(
                }
           }
      }
-     else {
+     else
 #endif
+     {
           ioctl( data->fd, VIDIOCGPICT, &pic );
 
           adj->flags = DCAF_BRIGHTNESS | DCAF_CONTRAST | DCAF_HUE | DCAF_SATURATION;
@@ -513,9 +512,7 @@ static DFBResult IDirectFBVideoProvider_V4L_GetColorAdjustment(
           adj->contrast   = pic.contrast;
           adj->hue        = pic.hue;
           adj->saturation = pic.colour;
-#ifdef DFB_HAVE_V4L2
      }
-#endif
      return DFB_OK;
 }
 
@@ -556,9 +553,9 @@ static DFBResult IDirectFBVideoProvider_V4L_SetColorAdjustment( IDirectFBVideoPr
                ioctl(data->fd, VIDIOC_S_CTRL, &ctrl);
           }
      }
-     else {
+     else
 #endif
-
+     {
           if (ioctl( data->fd, VIDIOCGPICT, &pic ) < 0) {
                DFBResult ret = errno2result( errno );
 
@@ -579,9 +576,7 @@ static DFBResult IDirectFBVideoProvider_V4L_SetColorAdjustment( IDirectFBVideoPr
 
                return ret;
           }
-#ifdef DFB_HAVE_V4L2
      }
-#endif
 
      return DFB_OK;
 }
@@ -612,14 +607,24 @@ Probe( IDirectFBVideoProvider_ProbeContext *ctx )
 }
 
 static DFBResult
-Construct( IDirectFBVideoProvider *thiz, IDirectFBDataBuffer *buffer )
+Construct( IDirectFBVideoProvider *thiz, ... )
 {
      int fd;
+     IDirectFBDataBuffer *buffer;
      IDirectFBDataBuffer_data *buffer_data;
+     CoreDFB             *core;
+     va_list              tag;
 
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBVideoProvider_V4L)
 
-     data->ref = 1;
+     va_start( tag, thiz );
+     buffer = va_arg( tag, IDirectFBDataBuffer * );
+     core = va_arg( tag, CoreDFB * );
+     va_end( tag );
+
+
+     data->ref  = 1;
+     data->core = core;
 
      buffer_data = (IDirectFBDataBuffer_data*) buffer->priv;
 
@@ -648,9 +653,9 @@ Construct( IDirectFBVideoProvider *thiz, IDirectFBDataBuffer *buffer )
      if ( 0 != data->is_v4l2 ) {
           /* hmm, anything to do here? */
      }
-     else {
+     else
 #endif
-
+     {
           D_INFO("DirectFB/Video4Linux: This is a Video4Linux-1 device.\n");
 
           ioctl( fd, VIDIOCGCAP, &data->vcap );
@@ -660,10 +665,7 @@ Construct( IDirectFBVideoProvider *thiz, IDirectFBDataBuffer *buffer )
 
           data->buffer = mmap( NULL, data->vmbuf.size,
                                PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0 );
-
-#ifdef DFB_HAVE_V4L2
      }
-#endif
 
      data->filename = D_STRDUP( buffer_data->filename );
      data->fd       = fd;
@@ -762,34 +764,34 @@ static void* GrabThread( DirectThread *thread, void *ctx )
 
           h = surface->height;
           src = data->buffer + data->vmbuf.offsets[frame];
-          dfb_surface_soft_lock( surface, DSLF_WRITE, &dst, &dst_pitch, 0 );
+          dfb_surface_soft_lock( data->core, surface, DSLF_WRITE, &dst, &dst_pitch, 0 );
           while (h--) {
                direct_memcpy( dst, src, src_pitch );
                dst += dst_pitch;
                src += src_pitch;
           }
           if (surface->format == DSPF_I420) {
-               h = surface->height / 2;
+               h = surface->height;
                while (h--) {
-                    direct_memcpy( dst, src, src_pitch );
-                    dst += dst_pitch;
-                    src += src_pitch;
+                    direct_memcpy( dst, src, src_pitch >> 1 );
+                    dst += dst_pitch >> 1;
+                    src += src_pitch >> 1;
                }
           }
           else if (surface->format == DSPF_YV12) {
-               h = surface->height / 4;
-               src += h * src_pitch;
+               h = surface->height >> 1;
+               src += h * (src_pitch >> 1);
                while (h--) {
-                    direct_memcpy( dst, src, src_pitch );
-                    dst += dst_pitch;
-                    src += src_pitch;
+                    direct_memcpy( dst, src, src_pitch >> 1 );
+                    dst += dst_pitch >> 1;
+                    src += src_pitch >> 1;
                }
-               h = surface->height / 4;
-               src -=  2 * h * src_pitch;
+               h = surface->height >> 1;
+               src -=  2 * h * (src_pitch >> 1);
                while (h--) {
-                    direct_memcpy( dst, src, src_pitch );
-                    dst += dst_pitch;
-                    src += src_pitch;
+                    direct_memcpy( dst, src, src_pitch >> 1 );
+                    dst += dst_pitch >> 1;
+                    src += src_pitch >> 1;
                }
           }
           dfb_surface_unlock( surface, 0 );
@@ -1006,7 +1008,7 @@ static DFBResult v4l_to_surface_overlay( CoreSurface *surface, DFBRectangle *rec
 static DFBResult v4l_to_surface_grab( CoreSurface *surface, DFBRectangle *rect,
                                       IDirectFBVideoProvider_V4L_data *data )
 {
-     int bpp, palette;
+     int palette;
 
      D_DEBUG( "DirectFB/Video4Linux: %s...\n", __FUNCTION__ );
 
@@ -1015,37 +1017,27 @@ static DFBResult v4l_to_surface_grab( CoreSurface *surface, DFBRectangle *rect,
 
      switch (surface->format) {
           case DSPF_YUY2:
-               bpp = 16;
                palette = VIDEO_PALETTE_YUYV;
                break;
           case DSPF_UYVY:
-               bpp = 16;
                palette = VIDEO_PALETTE_UYVY;
                break;
           case DSPF_I420:
-               bpp = 8;
-               palette = VIDEO_PALETTE_YUV420P;
-               break;
           case DSPF_YV12:
-               bpp = 8;
                palette = VIDEO_PALETTE_YUV420P;
                break;
           case DSPF_ARGB1555:
-               bpp = 15;
                palette = VIDEO_PALETTE_RGB555;
                break;
           case DSPF_RGB16:
-               bpp = 16;
                palette = VIDEO_PALETTE_RGB565;
                break;
           case DSPF_RGB24:
-               bpp = 24;
                palette = VIDEO_PALETTE_RGB24;
                break;
           case DSPF_ARGB:
           case DSPF_AiRGB:
           case DSPF_RGB32:
-               bpp = 32;
                palette = VIDEO_PALETTE_RGB32;
                break;
           default:
@@ -1109,17 +1101,15 @@ static DFBResult v4l_stop( IDirectFBVideoProvider_V4L_data *data, bool detach )
                /* don't quit here */
           }
      }
-     else {
+     else
 #endif
+     {
           if (!data->grab_mode) {
                if (ioctl( data->fd, VIDIOCCAPTURE, &zero ) < 0)
                     D_PERROR( "DirectFB/Video4Linux: "
                               "Could not stop capturing (VIDIOCCAPTURE failed)!\n" );
           }
-
-#ifdef DFB_HAVE_V4L2
      }
-#endif
 
      destination = data->destination;
 
@@ -1135,20 +1125,19 @@ static DFBResult v4l_stop( IDirectFBVideoProvider_V4L_data *data, bool detach )
                int i;
                for (i = 0; i < data->req.count; i++) {
                     struct v4l2_buffer *vidbuf = &data->vidbuf[i];
-                    D_DEBUG("DirectFB/Video4Linux2: %d => 0x%08x, len:%d\n", i, (__u32) data->ptr[i], vidbuf->length);
+                    D_DEBUG("DirectFB/Video4Linux2: %d => 0x%08x, len:%d\n", i, (u32) data->ptr[i], vidbuf->length);
                     if (0 != munmap(data->ptr[i], vidbuf->length)) {
                          D_PERROR("DirectFB/Video4Linux2: munmap().\n");
                     }
                }
           }
      }
-     else {
+     else
 #endif
+     {
           if (!data->grab_mode)
                dfb_surface_unlock( destination, false );
-#ifdef DFB_HAVE_V4L2
      }
-#endif
 
      data->destination = NULL;
 
@@ -1187,7 +1176,7 @@ static void v4l_cleanup( void *ctx, int emergency )
 
 /* v4l2 specific stuff */
 #ifdef DFB_HAVE_V4L2
-int wait_for_buffer(int vid, struct v4l2_buffer *cur)
+static int wait_for_buffer(int vid, struct v4l2_buffer *cur)
 {
      fd_set rdset;
      struct timeval timeout;
@@ -1272,30 +1261,39 @@ static void *V4L2_Thread(DirectThread * thread, void *ctx)
 
                h = surface->height;
                src = data->ptr[cur.index];
-               dfb_surface_soft_lock(surface, DSLF_WRITE, &dst, &dst_pitch, 0);
+               dfb_surface_soft_lock(data->core, surface, DSLF_WRITE, &dst, &dst_pitch, 0);
                while (h--) {
                     direct_memcpy(dst, src, src_pitch);
                     dst += dst_pitch;
                     src += src_pitch;
                }
                if (surface->format == DSPF_I420) {
-                    h = surface->height / 2;
+                    h = surface->height;
                     while (h--) {
-                         direct_memcpy(dst, src, src_pitch);
-                         dst += dst_pitch;
-                         src += src_pitch;
+                         direct_memcpy(dst, src, src_pitch >> 1);
+                         dst += dst_pitch >> 1;
+                         src += src_pitch >> 1;
                     }
                }
                else if (surface->format == DSPF_YV12) {
-                    h = surface->height / 4;
-                    src += h * src_pitch;
+                    h = surface->height >> 1;
+                    src += h * (src_pitch >> 1);
                     while (h--) {
-                         direct_memcpy(dst, src, src_pitch);
-                         dst += dst_pitch;
-                         src += src_pitch;
+                         direct_memcpy(dst, src, src_pitch >> 1);
+                         dst += dst_pitch >> 1;
+                         src += src_pitch >> 1;
                     }
-                    h = surface->height / 4;
-                    src -= 2 * h * src_pitch;
+                    h = surface->height >> 1;
+                    src -= 2 * h * (src_pitch >> 1);
+                    while (h--) {
+                         direct_memcpy(dst, src, src_pitch >> 1);
+                         dst += dst_pitch >> 1;
+                         src += src_pitch >> 1;
+                    }
+               }
+               else if (surface->format == DSPF_NV12 ||
+                        surface->format == DSPF_NV21) {
+                    h = surface->height >> 1;
                     while (h--) {
                          direct_memcpy(dst, src, src_pitch);
                          dst += dst_pitch;
@@ -1323,7 +1321,7 @@ static void *V4L2_Thread(DirectThread * thread, void *ctx)
 static DFBResult v4l2_playto(CoreSurface * surface, DFBRectangle * rect, IDirectFBVideoProvider_V4L_data * data)
 {
      SurfaceBuffer *buffer = surface->back_buffer;
-     int bpp, palette;
+     int palette;
 
      int err;
      int i;
@@ -1332,39 +1330,38 @@ static DFBResult v4l2_playto(CoreSurface * surface, DFBRectangle * rect, IDirect
 
      switch (surface->format) {
           case DSPF_YUY2:
-               bpp = 16;
                palette = V4L2_PIX_FMT_YUYV;
                break;
           case DSPF_UYVY:
-               bpp = 16;
                palette = V4L2_PIX_FMT_UYVY;
                break;
-/*
-     case DSPF_I420:
-          bpp = 8;
-          palette = VIDEO_PALETTE_YUV420P;
-          break;
-     case DSPF_YV12:
-          bpp = 8;
-          palette = VIDEO_PALETTE_YUV420P;
-          break;
-     case DSPF_ARGB1555:
-          bpp = 15;
-          palette = VIDEO_PALETTE_RGB555;
-          break;
-*/
+          case DSPF_I420:
+               palette = V4L2_PIX_FMT_YUV420;
+               break;
+          case DSPF_YV12:
+               palette = V4L2_PIX_FMT_YVU420;
+               break;
+          case DSPF_NV12:
+               palette = V4L2_PIX_FMT_NV12;
+               break;
+          case DSPF_NV21:
+               palette = V4L2_PIX_FMT_NV21;
+               break;
+          case DSPF_RGB332:
+               palette = V4L2_PIX_FMT_RGB332;
+               break;
+          case DSPF_ARGB1555:
+               palette = V4L2_PIX_FMT_RGB555;
+               break;
           case DSPF_RGB16:
-               bpp = 16;
                palette = V4L2_PIX_FMT_RGB565;
                break;
           case DSPF_RGB24:
-               bpp = 24;
                palette = V4L2_PIX_FMT_BGR24;
                break;
           case DSPF_ARGB:
           case DSPF_AiRGB:
           case DSPF_RGB32:
-               bpp = 32;
                palette = V4L2_PIX_FMT_BGR32;
                break;
           default:
@@ -1419,7 +1416,7 @@ static DFBResult v4l2_playto(CoreSurface * surface, DFBRectangle * rect, IDirect
           fb.fmt.height = surface->height;
           fb.fmt.pixelformat = palette;
 
-          D_DEBUG("w:%d, h:%d, bpp:%d, bpl:%d, base:0x%08lx\n",fb.fmt.width, fb.fmt.height,bpp,fb.fmt.bytesperline, (unsigned long)fb.base);
+          D_DEBUG("w:%d, h:%d, bpl:%d, base:0x%08lx\n",fb.fmt.width, fb.fmt.height, fb.fmt.bytesperline, (unsigned long)fb.base);
 
           if (ioctl(data->fd, VIDIOC_S_FBUF, &fb) < 0) {
                DFBResult ret = errno2result(errno);
@@ -1465,7 +1462,7 @@ static DFBResult v4l2_playto(CoreSurface * surface, DFBRectangle * rect, IDirect
                     return err;
                }
           }
-          D_DEBUG("DirectFB/Video4Linux2: len:0x%08x, %d => 0x%08x\n", vidbuf->length, i, (__u32) data->ptr[i]);
+          D_DEBUG("DirectFB/Video4Linux2: len:0x%08x, %d => 0x%08x\n", vidbuf->length, i, (u32) data->ptr[i]);
      }
 
      if (!data->cleanup)
