@@ -29,9 +29,12 @@
 #define __DIRECT__BUILD_H__
 
 #define DIRECT_BUILD_DEBUG   (1)
+#define DIRECT_BUILD_DEBUGS  (1)
 #define DIRECT_BUILD_TRACE   (1)
 #define DIRECT_BUILD_TEXT    (1)
 #define DIRECT_BUILD_GETTID  (1)
+#define DIRECT_BUILD_NETWORK (1)
+#define DIRECT_BUILD_STDBOOL (1)
 
 #endif
 
