@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -40,7 +41,7 @@ struct __D_DirectLink {
                           to the last element of the list, for fast appending ;-) */
 };
 
-static inline void
+static __inline__ void
 direct_list_prepend( DirectLink **list, DirectLink *link )
 {
      DirectLink *first = *list;
@@ -62,7 +63,7 @@ direct_list_prepend( DirectLink **list, DirectLink *link )
      D_MAGIC_SET( link, DirectLink );
 }
 
-static inline void
+static __inline__ void
 direct_list_append( DirectLink **list, DirectLink *link )
 {
      DirectLink *first = *list;
@@ -85,7 +86,7 @@ direct_list_append( DirectLink **list, DirectLink *link )
      D_MAGIC_SET( link, DirectLink );
 }
 
-static inline bool
+static __inline__ bool
 direct_list_contains_element_EXPENSIVE( DirectLink *list, DirectLink *link )
 {
      D_MAGIC_ASSERT_IF( list, DirectLink );
@@ -105,7 +106,7 @@ direct_list_contains_element_EXPENSIVE( DirectLink *list, DirectLink *link )
      return false;
 }
 
-static inline int
+static __inline__ int
 direct_list_count_elements_EXPENSIVE( DirectLink *list )
 {
      int count = 0;
@@ -121,12 +122,14 @@ direct_list_count_elements_EXPENSIVE( DirectLink *list )
      return count;
 }
 
-static inline void
+static __inline__ void
 direct_list_remove( DirectLink **list, DirectLink *link )
 {
      DirectLink *next;
      DirectLink *prev;
 
+     D_ASSERT( list != NULL );
+
      D_ASSERT( direct_list_contains_element_EXPENSIVE( *list, link ) );
 
      D_MAGIC_ASSERT( *list, DirectLink );
@@ -156,7 +159,49 @@ direct_list_remove( DirectLink **list, DirectLink *link )
      D_MAGIC_CLEAR( link );
 }
 
-static inline bool
+static __inline__ void
+direct_list_move_to_front( DirectLink **list, DirectLink *link )
+{
+     DirectLink *next;
+     DirectLink *prev;
+     DirectLink *first;
+
+     D_ASSERT( list != NULL );
+
+     first = *list;
+
+     D_ASSERT( direct_list_contains_element_EXPENSIVE( first, link ) );
+
+     D_MAGIC_ASSERT( first, DirectLink );
+     D_MAGIC_ASSERT( link, DirectLink );
+
+     if (first == link)
+          return;
+
+     next = link->next;
+     prev = link->prev;
+
+     D_MAGIC_ASSERT_IF( next, DirectLink );
+     D_MAGIC_ASSERT( prev, DirectLink );
+
+     if (next) {
+          next->prev = prev;
+
+          link->prev = first->prev;
+     }
+     else
+          link->prev = prev;
+
+     prev->next = next;
+
+     link->next = first;
+
+     first->prev = link;
+
+     *list = link;
+}
+
+static __inline__ bool
 direct_list_check_link( const DirectLink *link )
 {
      D_MAGIC_ASSERT_IF( link, DirectLink );
@@ -166,14 +211,19 @@ direct_list_check_link( const DirectLink *link )
 
 
 #define direct_list_foreach(elem, list)                     \
-     for (elem = (typeof(elem))(list);                      \
+     for (elem = (__typeof__(elem))(list);                  \
           direct_list_check_link( (DirectLink*)(elem) );    \
-          elem = (typeof(elem))(((DirectLink*)(elem))->next))
+          elem = (__typeof__(elem))(((DirectLink*)(elem))->next))
+
+#define direct_list_foreach_reverse(elem, list)                    \
+     for (elem = (__typeof__(elem))((list) ? (list)->prev : NULL); \
+          direct_list_check_link( (DirectLink*)(elem) );           \
+          elem = (__typeof__(elem))((((DirectLink*)(elem))->prev->next) ? ((DirectLink*)(elem))->prev : NULL))
 
 #define direct_list_foreach_safe(elem, temp, list)                                             \
-     for (elem = (typeof(elem))(list), temp = ((typeof(temp))(elem) ? (typeof(temp))(((DirectLink*)(elem))->next) : NULL); \
+     for (elem = (__typeof__(elem))(list), temp = ((__typeof__(temp))(elem) ? (__typeof__(temp))(((DirectLink*)(elem))->next) : NULL); \
           direct_list_check_link( (DirectLink*)(elem) );                                       \
-          elem = (typeof(elem))(temp), temp = ((typeof(temp))(elem) ? (typeof(temp))(((DirectLink*)(elem))->next) : NULL))
+          elem = (__typeof__(elem))(temp), temp = ((__typeof__(temp))(elem) ? (__typeof__(temp))(((DirectLink*)(elem))->next) : NULL))
 
 #endif
 
