@@ -1,13 +1,13 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
               Sven Neumann <neo@directfb.org>,
-              Ville Syrjl <syrjala@sci.fi> and
+              Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
@@ -148,6 +148,7 @@ direct_log_printf( DirectLog  *log,
      /* Write to stderr as a fallback if default is invalid, too. */
      if (!log || log->magic != D_MAGIC("DirectLog")) {
           vfprintf( stderr, format, args );
+          fflush( stderr );
      }
      else {
           int  len;
@@ -190,7 +191,7 @@ init_file( DirectLog  *log,
      DirectResult ret;
      int          fd;
 
-     fd = open( filename, O_WRONLY | O_CREAT | O_APPEND, 0644 );
+     fd = open( filename, O_WRONLY | O_CREAT | O_APPEND, 0664 );
      if (fd < 0) {
           ret = errno2result( errno );
           D_PERROR( "Direct/Log: Could not open '%s' for writing!\n", filename );
