@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -72,12 +73,12 @@ static int sigs_to_handle[] = { /*SIGALRM,*/ SIGHUP, SIGINT, /*SIGPIPE,*/ /*SIGP
                                 SIGSEGV, SIGTRAP, /*SIGSYS, SIGEMT,*/ SIGBUS,
                                 SIGXCPU, SIGXFSZ };
 
-#define NUM_SIGS_TO_HANDLE ((int)(sizeof(sigs_to_handle)/sizeof(sigs_to_handle[0])))
+#define NUM_SIGS_TO_HANDLE ((int)D_ARRAY_SIZE( sigs_to_handle ))
 
 static SigHandled sigs_handled[NUM_SIGS_TO_HANDLE];
 
-static DirectLink      *handlers      = NULL;
-static pthread_mutex_t  handlers_lock = DIRECT_UTIL_RECURSIVE_PTHREAD_MUTEX_INITIALIZER;
+static DirectLink      *handlers = NULL;
+static pthread_mutex_t  handlers_lock;
 
 /**************************************************************************************************/
 
@@ -91,6 +92,8 @@ direct_signals_initialize()
 {
      D_DEBUG_AT( Direct_Signals, "Initializing...\n" );
 
+     direct_util_recursive_pthread_mutex_init( &handlers_lock );
+
      install_handlers();
 
      return DFB_OK;
@@ -103,6 +106,8 @@ direct_signals_shutdown()
 
      remove_handlers();
 
+     pthread_mutex_destroy( &handlers_lock );
+
      return DFB_OK;
 }
 
@@ -313,6 +318,9 @@ signal_handler( int num, siginfo_t *info, void *foo )
      int         pid    = direct_gettid();
      long long   millis = direct_clock_get_millis();
 
+     fflush(stdout);
+     fflush(stderr);
+
      direct_log_printf( NULL, "(!) [%5d: %4lld.%03lld] --> Caught signal %d",
                         pid, millis/1000, millis%1000, num );
 
@@ -363,7 +371,7 @@ signal_handler( int num, siginfo_t *info, void *foo )
      direct_list_foreach_safe (l, n, handlers) {
           DirectSignalHandler *handler = (DirectSignalHandler*) l;
 
-          if (handler->num != num && handler->num != -1)
+          if (handler->num != num && handler->num != DIRECT_SIGNAL_ANY)
                continue;
 
           switch (handler->func( num, addr, handler->ctx )) {
@@ -392,10 +400,13 @@ signal_handler( int num, siginfo_t *info, void *foo )
 
      pthread_mutex_unlock( &handlers_lock );
 
-     /* Propagate signal. */
-     killpg( 0, SIGKILL );
+     remove_handlers();
 
-     _exit( -1 );
+     raise( num );
+     
+     abort();
+
+     exit( -num );
 }
 
 /**************************************************************************************************/
@@ -415,12 +426,12 @@ install_handlers()
                int              signum = sigs_to_handle[i];
 
                action.sa_sigaction = signal_handler;
-               action.sa_flags     = SA_RESTART | SA_SIGINFO;
-
+               action.sa_flags     = SA_SIGINFO;
+                
                if (signum != SIGSEGV)
-                    action.sa_flags |= SA_RESETHAND;
+                    action.sa_flags |= SA_NODEFER;
 
-               sigfillset( &action.sa_mask );
+               sigemptyset( &action.sa_mask );
 
                if (sigaction( signum, &action, &sigs_handled[i].old_action )) {
                     D_PERROR( "Direct/Signals: "
