@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -31,9 +32,9 @@
 #include <direct/types.h>
 
 void direct_trace_print_stack( DirectTraceBuffer *buffer );
-void direct_trace_print_stacks();
+void direct_trace_print_stacks( void );
 
-int  direct_trace_debug_indent();
+int  direct_trace_debug_indent( void );
 
 DirectTraceBuffer *direct_trace_copy_buffer( DirectTraceBuffer *buffer );
 void               direct_trace_free_buffer( DirectTraceBuffer *buffer );
