@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -39,6 +40,7 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
+#include <sys/utsname.h>
 
 #include <direct/clock.h>
 #include <direct/debug.h>
@@ -48,8 +50,10 @@
 #include <direct/signals.h>
 #include <direct/thread.h>
 #include <direct/trace.h>
+#include <direct/util.h>
 
 #include <fusion/build.h>
+#include <fusion/conf.h>
 #include <fusion/types.h>
 
 #include "fusion_internal.h"
@@ -64,7 +68,8 @@
 #include <linux/fusion.h>
 
 
-D_DEBUG_DOMAIN( Fusion_Main, "Fusion/Main", "Fusion - High level IPC" );
+D_DEBUG_DOMAIN( Fusion_Main,          "Fusion/Main",          "Fusion - High level IPC" );
+D_DEBUG_DOMAIN( Fusion_Main_Dispatch, "Fusion/Main/Dispatch", "Fusion - High level IPC Dispatch" );
 
 /**********************************************************************************************************************/
 
@@ -75,6 +80,7 @@ static void                      *fusion_dispatch_loop ( DirectThread *thread,
 
 static FusionWorld     *fusion_worlds[FUSION_MAX_WORLDS];
 static pthread_mutex_t  fusion_worlds_lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_once_t   fusion_init_once   = PTHREAD_ONCE_INIT;
 
 /**********************************************************************************************************************/
 
@@ -140,6 +146,175 @@ _fusion_world( const FusionWorldShared *shared )
 
 /**********************************************************************************************************************/
 
+static void
+fusion_world_fork( FusionWorld *world )
+{
+     int                fd;
+     char               buf1[20];
+     char               buf2[20];
+     FusionEnter        enter;
+     FusionFork         fork;
+     FusionWorldShared *shared;
+
+     D_MAGIC_ASSERT( world, FusionWorld );
+
+     shared = world->shared;
+
+     D_MAGIC_ASSERT( shared, FusionWorldShared );
+
+     snprintf( buf1, sizeof(buf1), "/dev/fusion%d", shared->world_index );
+     snprintf( buf2, sizeof(buf2), "/dev/fusion/%d", shared->world_index );
+
+     /* Open Fusion Kernel Device. */
+     fd = direct_try_open( buf1, buf2, O_RDWR | O_NONBLOCK, true );
+     if (fd < 0) {
+          D_PERROR( "Fusion/Main: Reopening fusion device (world %d) failed!\n", shared->world_index );
+          raise(5);
+     }
+
+     /* Drop "identity" when running another program. */
+     if (fcntl( fd, F_SETFD, FD_CLOEXEC ) < 0)
+          D_PERROR( "Fusion/Init: Setting FD_CLOEXEC flag failed!\n" );
+
+     /* Fill enter information. */
+     enter.api.major = FUSION_API_MAJOR;
+     enter.api.minor = FUSION_API_MINOR;
+     enter.fusion_id = 0;     /* Clear for check below. */
+
+     /* Enter the fusion world. */
+     while (ioctl( fd, FUSION_ENTER, &enter )) {
+          if (errno != EINTR) {
+               D_PERROR( "Fusion/Init: Could not reenter world '%d'!\n", shared->world_index );
+               raise(5);
+          }
+     }
+
+     /* Check for valid Fusion ID. */
+     if (!enter.fusion_id) {
+          D_ERROR( "Fusion/Init: Got no ID from FUSION_ENTER! Kernel module might be too old.\n" );
+          raise(5);
+     }
+
+     D_DEBUG_AT( Fusion_Main, "  -> Fusion ID 0x%08lx\n", enter.fusion_id );
+
+
+     /* Fill fork information. */
+     fork.fusion_id = world->fusion_id;
+
+     /* Fork within the fusion world. */
+     while (ioctl( fd, FUSION_FORK, &fork )) {
+          if (errno != EINTR) {
+               D_PERROR( "Fusion/Main: Could not fork in world '%d'!\n", shared->world_index );
+               raise(5);
+          }
+     }
+
+     D_DEBUG_AT( Fusion_Main, "  -> Fusion ID 0x%08lx\n", fork.fusion_id );
+
+     /* Get new fusion id back. */
+     world->fusion_id = fork.fusion_id;
+
+     /* Close old file descriptor. */
+     close( world->fusion_fd );
+
+     /* Write back new file descriptor. */
+     world->fusion_fd = fd;
+
+
+
+
+     D_DEBUG_AT( Fusion_Main, "  -> restarting dispatcher loop...\n" );
+
+     /* Restart the dispatcher thread. FIXME: free old struct */
+     world->dispatch_loop = direct_thread_create( DTT_MESSAGING,
+                                                  fusion_dispatch_loop,
+                                                  world, "Fusion Dispatch" );
+     if (!world->dispatch_loop)
+          raise(5);
+}
+
+static void
+fusion_fork_handler_child()
+{
+     int i;
+
+     D_DEBUG_AT( Fusion_Main, "%s()\n", __FUNCTION__ );
+
+     for (i=0; i<FUSION_MAX_WORLDS; i++) {
+          FusionWorld *world = fusion_worlds[i];
+
+          if (!world)
+               continue;
+
+          D_MAGIC_ASSERT( world, FusionWorld );
+
+          switch (world->fork_action) {
+               default:
+                    D_BUG( "unknown fork action %d", world->fork_action );
+
+               case FFA_CLOSE:
+                    D_DEBUG_AT( Fusion_Main, "  -> closing world %d\n", i );
+
+                    /* Remove world from global list. */
+                    fusion_worlds[i] = NULL;
+
+                    /* Unmap shared area. */
+                    munmap( world->shared, sizeof(FusionWorldShared) );
+
+                    /* Close Fusion Kernel Device. */
+                    close( world->fusion_fd );
+
+                    /* Free local world data. */
+                    D_MAGIC_CLEAR( world );
+                    D_FREE( world );
+
+                    break;
+
+               case FFA_FORK:
+                    D_DEBUG_AT( Fusion_Main, "  -> forking in world %d\n", i );
+
+                    fusion_world_fork( world );
+
+                    break;
+          }
+     }
+}
+
+/**********************************************************************************************************************/
+
+static void
+init_once()
+{
+     struct utsname uts;
+     int            i, j, k, l;
+
+     pthread_atfork( NULL, NULL, fusion_fork_handler_child );
+
+     if (uname( &uts ) < 0) {
+          D_PERROR( "Fusion/Init: uname() failed!\n" );
+          return;
+     }
+
+     switch (sscanf( uts.release, "%d.%d.%d.%d", &i, &j, &k, &l )) {
+          case 3:
+               l = 0;
+          case 4:
+               if (((i << 24) | (j << 16) | (k << 8) | l) >= 0x02061302)
+                    fusion_config->madv_remove = true;
+               break;
+
+          default:
+               D_WARN( "could not parse kernel version '%s'", uts.release );
+     }
+
+     if (fusion_config->madv_remove)
+          D_INFO( "Fusion/SHM: Using MADV_REMOVE (%d.%d.%d.%d >= 2.6.19.2)\n", i, j, k, l );
+     else
+          D_INFO( "Fusion/SHM: Not using MADV_REMOVE (%d.%d.%d.%d < 2.6.19.2)!\n", i, j, k, l );
+}
+
+/**********************************************************************************************************************/
+
 /*
  * Enters a fusion world by joining or creating it.
  *
@@ -169,6 +344,12 @@ fusion_enter( int               world_index,
           return DFB_INVARG;
      }
 
+     pthread_once( &fusion_init_once, init_once );
+
+
+     if (fusion_config->force_slave)
+          role = FER_SLAVE;
+
      direct_initialize();
 
      pthread_mutex_lock( &fusion_worlds_lock );
@@ -209,6 +390,8 @@ fusion_enter( int               world_index,
 
                if (role == FER_MASTER)
                     flags |= O_EXCL;
+               else if (role == FER_SLAVE)
+                    flags |= O_APPEND;
 
                /* Open Fusion Kernel Device. */
                fd = direct_try_open( buf1, buf2, flags, true );
@@ -316,7 +499,7 @@ fusion_enter( int               world_index,
           goto error;
      }
 
-     D_DEBUG_AT( Fusion_Main, "  -> shared area at %p, size %d\n", shared, sizeof(FusionWorldShared) );
+     D_DEBUG_AT( Fusion_Main, "  -> shared area at %p, size %zu\n", shared, sizeof(FusionWorldShared) );
 
      /* Initialize shared data. */
      if (enter.fusion_id == FUSION_ID_MASTER) {
@@ -380,7 +563,7 @@ fusion_enter( int               world_index,
 
           /* Create the main pool. */
           ret = fusion_shm_pool_create( world, "Fusion Main Pool", 0x100000,
-                                        direct_config->debug, &shared->main_pool );
+                                        fusion_config->debugshm, &shared->main_pool );
           if (ret)
                goto error3;
      }
@@ -558,6 +741,18 @@ fusion_exit( FusionWorld *world,
      return DFB_OK;
 }
 
+/*
+ * Sets the fork() action of the calling Fusionee within the world.
+ */
+void
+fusion_world_set_fork_action( FusionWorld      *world,
+                              FusionForkAction  action )
+{
+     D_MAGIC_ASSERT( world, FusionWorld );
+
+     world->fork_action = action;
+}
+
 /*
  * Return the index of the specified world.
  */
@@ -586,6 +781,17 @@ fusion_id( const FusionWorld *world )
      return world->fusion_id;
 }
 
+/*
+ * Return the thread ID of the Fusion Dispatcher within the specified world.
+ */
+pid_t
+fusion_dispatcher_tid( const FusionWorld *world )
+{
+     D_MAGIC_ASSERT( world, FusionWorld );
+
+     return direct_thread_get_tid( world->dispatch_loop );
+}
+
 /*
  * Return true if this process is the master.
  */
@@ -606,7 +812,7 @@ fusion_sync( const FusionWorld *world )
      int            result;
      fd_set         set;
      struct timeval tv;
-     int            loops = 100;
+     int            loops = 200;
 
      D_MAGIC_ASSERT( world, FusionWorld );
 
@@ -622,21 +828,26 @@ fusion_sync( const FusionWorld *world )
           tv.tv_usec = 20000;
 
           result = select( world->fusion_fd + 1, &set, NULL, NULL, &tv );
-
+          D_DEBUG_AT( Fusion_Main, "  -> select() returned %d...\n", result );
           switch (result) {
                case -1:
                     if (errno == EINTR)
-                         continue;
+                         return DFB_OK;
 
                     D_PERROR( "Fusion/Sync: select() failed!\n");
                     return DFB_FAILURE;
 
+               default:
+                    D_DEBUG_AT( Fusion_Main, "  -> FD_ISSET %d...\n", FD_ISSET( world->fusion_fd, &set ) );
+                    
+                    if (FD_ISSET( world->fusion_fd, &set )) {
+                         usleep( 20000 );
+                         break;
+                    }
+
                case 0:
                     D_DEBUG_AT( Fusion_Main, "  -> synced.\n");
                     return DFB_OK;
-
-               default:
-                    usleep( 20000 );
           }
      }
 
@@ -746,10 +957,12 @@ fusion_dispatch_loop( DirectThread *thread, void *arg )
 {
      int          len = 0;
      int          result;
-     char         buf[1024];
+     char         buf[FUSION_MESSAGE_SIZE];
      fd_set       set;
      FusionWorld *world = arg;
 
+     D_DEBUG_AT( Fusion_Main_Dispatch, "%s() running...\n", __FUNCTION__ );
+
      while (true) {
           char *buf_p = buf;
 
@@ -773,10 +986,12 @@ fusion_dispatch_loop( DirectThread *thread, void *arg )
           D_MAGIC_ASSERT( world, FusionWorld );
 
           if (FD_ISSET( world->fusion_fd, &set )) {
-               len = read( world->fusion_fd, buf, 1024 );
+               len = read( world->fusion_fd, buf, FUSION_MESSAGE_SIZE );
                if (len < 0)
                     break;
 
+               D_DEBUG_AT( Fusion_Main_Dispatch, "  -> got %d bytes...\n", len );
+
                while (buf_p < buf + len) {
                     FusionReadMessage *header = (FusionReadMessage*) buf_p;
                     void              *data   = buf_p + sizeof(FusionReadMessage);
@@ -785,16 +1000,22 @@ fusion_dispatch_loop( DirectThread *thread, void *arg )
 
                     switch (header->msg_type) {
                          case FMT_SEND:
-                              if (!world->refs)
+                              D_DEBUG_AT( Fusion_Main_Dispatch, "  -> FMT_SEND!\n" );
+                              if (!world->refs) {
+                                   D_DEBUG_AT( Fusion_Main_Dispatch, "  -> good bye!\n" );
                                    return NULL;
+                              }
                               break;
                          case FMT_CALL:
+                              D_DEBUG_AT( Fusion_Main_Dispatch, "  -> FMT_CALL...\n" );
                               _fusion_call_process( world, header->msg_id, data );
                               break;
                          case FMT_REACTOR:
-                              _fusion_reactor_process_message( header->msg_id, data, NULL, world );
+                              D_DEBUG_AT( Fusion_Main_Dispatch, "  -> FMT_REACTOR...\n" );
+                              _fusion_reactor_process_message( world, header->msg_id, data );
                               break;
                          case FMT_SHMPOOL:
+                              D_DEBUG_AT( Fusion_Main_Dispatch, "  -> FMT_SHMPOOL...\n" );
                               _fusion_shmpool_process( world, header->msg_id, data );
                               break;
                          default:
@@ -849,7 +1070,7 @@ DirectResult fusion_enter( int               world_index,
 
      /* Create the main pool. */
      ret = fusion_shm_pool_create( world, "Fusion Main Pool", 0x100000,
-                                   direct_config->debug, &world->shared->main_pool );
+                                   fusion_config->debugshm, &world->shared->main_pool );
      if (ret)
           goto error;
 
@@ -900,6 +1121,16 @@ DirectResult fusion_exit( FusionWorld *world,
      return DFB_OK;
 }
 
+/*
+ * Sets the fork() action of the calling Fusionee within the world.
+ */
+void
+fusion_world_set_fork_action( FusionWorld      *world,
+                              FusionForkAction  action )
+{
+     D_MAGIC_ASSERT( world, FusionWorld );
+}
+
 /*
  * Return the index of the specified world.
  */
