@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -28,6 +29,8 @@
 #ifndef __FUSION__FUSION_H__
 #define __FUSION__FUSION_H__
 
+#include <sys/types.h>
+
 #include <fusion/types.h>
 
 typedef enum {
@@ -36,6 +39,11 @@ typedef enum {
      FER_SLAVE
 } FusionEnterRole;
 
+typedef enum {
+     FFA_CLOSE,
+     FFA_FORK
+} FusionForkAction;
+
 /*
  * Enters a fusion world by joining or creating it.
  *
@@ -55,6 +63,12 @@ DirectResult fusion_enter( int               world_index,
 DirectResult fusion_exit( FusionWorld *world,
                           bool         emergency );
 
+/*
+ * Sets the fork() action of the calling Fusionee within the world.
+ */
+void fusion_world_set_fork_action( FusionWorld      *world,
+                                   FusionForkAction  action );
+
 /*
  * Return the index of the specified world.
  */
@@ -65,6 +79,11 @@ int fusion_world_index( const FusionWorld *world );
  */
 FusionID fusion_id( const FusionWorld *world );
 
+/*
+ * Return the thread ID of the Fusion Dispatcher within the specified world.
+ */
+pid_t fusion_dispatcher_tid( const FusionWorld *world );
+
 /*
  * Return true if this process is the master.
  */
