@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -79,7 +80,7 @@ fusion_skirmish_init( FusionSkirmish    *skirmish,
      info.type = FT_SKIRMISH;
      info.id   = skirmish->multi.id;
 
-     strncpy( info.name, name, sizeof(info.name) );
+     direct_snputs( info.name, name, sizeof(info.name) );
 
      ioctl( world->fusion_fd, FUSION_ENTRY_SET_INFO, &info );
 
@@ -136,6 +137,34 @@ fusion_skirmish_swoop( FusionSkirmish *skirmish )
      return DFB_OK;
 }
 
+DirectResult
+fusion_skirmish_lock_count( FusionSkirmish *skirmish, int *lock_count )
+{
+     int data[2];
+
+     D_ASSERT( skirmish != NULL );
+
+     data[0] = skirmish->multi.id;
+     data[1] = 0;
+
+     while (ioctl (_fusion_fd( skirmish->multi.shared ), FUSION_SKIRMISH_LOCK_COUNT, data)) {
+           switch (errno) {
+               case EINTR:
+                    continue;
+
+               case EINVAL:
+                    D_ERROR ("Fusion/Lock: invalid skirmish\n");
+                    return DFB_DESTROYED;
+           }
+
+          D_PERROR ("FUSION_SKIRMISH_LOCK_COUNT");
+          return DFB_FUSION;
+     }
+
+     *lock_count = data[1];
+     return DFB_OK;
+}
+
 DirectResult
 fusion_skirmish_dismiss (FusionSkirmish *skirmish)
 {
