@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -35,6 +36,7 @@
 
 #include <fusion/build.h>
 #include <fusion/object.h>
+#include <fusion/hash.h>
 #include <fusion/shmalloc.h>
 
 #include "fusion_internal.h"
@@ -93,13 +95,13 @@ object_reference_watcher( int caller, int call_arg, void *call_ptr, void *ctx )
                     fusion_skirmish_dismiss( &pool->lock );
                     return 0;
 
-               case DFB_BUSY:
-                    D_BUG( "revived object %p [%ld] in '%s'", object, object->id, pool->name );
-                    /* fall through */
 
                default:
-                    D_ERROR( "Fusion/ObjectPool: Error locking ref of %p [%ld] in '%s'",
+                    D_ERROR( "Fusion/ObjectPool: Error locking ref of %p [%ld] in '%s'\n",
                              object, object->id, pool->name );
+                    /* fall through */
+
+               case DFB_BUSY:
                     fusion_skirmish_dismiss( &pool->lock );
                     return 0;
           }
@@ -166,8 +168,10 @@ fusion_object_pool_create( const char             *name,
 
      /* Allocate shared memory for the pool. */
      pool = SHCALLOC( shared->main_pool, 1, sizeof(FusionObjectPool) );
-     if (!pool)
+     if (!pool) {
+          D_OOSHM();
           return NULL;
+     }
 
      /* Initialize the pool lock. */
      fusion_skirmish_init( &pool->lock, name, world );
@@ -210,7 +214,8 @@ fusion_object_pool_destroy( FusionObjectPool  *pool,
      D_DEBUG_AT( Fusion_Object, "  -> syncing...\n" );
 
      /* Wait for processing of pending messages. */
-     fusion_sync( world );
+     if (pool->objects)
+          fusion_sync( world );
 
      D_DEBUG_AT( Fusion_Object, "  -> locking...\n" );
 
@@ -313,6 +318,7 @@ fusion_object_create( FusionObjectPool  *pool,
      /* Allocate shared memory for the object. */
      object = SHCALLOC( shared->main_pool, 1, pool->object_size );
      if (!object) {
+          D_OOSHM();
           fusion_skirmish_dismiss( &pool->lock );
           return NULL;
      }
@@ -324,7 +330,7 @@ fusion_object_create( FusionObjectPool  *pool,
      object->id = ++pool->id_pool;
 
      /* Initialize the reference counter. */
-     if (fusion_ref_init( &object->ref, world )) {
+     if (fusion_ref_init( &object->ref, pool->name, world )) {
           SHFREE( shared->main_pool, object );
           fusion_skirmish_dismiss( &pool->lock );
           return NULL;
@@ -418,7 +424,7 @@ fusion_object_set_lock( FusionObject   *object,
 
      D_ASSUME( object->state == FOS_INIT );
 
-     return fusion_reactor_set_lock( object->reactor, lock );
+     return fusion_reactor_set_lock_only( object->reactor, lock );
 }
 
 DirectResult
@@ -481,10 +487,141 @@ fusion_object_destroy( FusionObject *object )
 
      fusion_reactor_free( object->reactor );
 
+     if ( object->properties )
+          fusion_hash_destroy(object->properties);
+
      D_MAGIC_CLEAR( object );
-
      SHFREE( shared->main_pool, object );
-
      return DFB_OK;
 }
 
+/*
+ * Sets a value for a key.
+ * If the key currently has a value the old value is returned
+ * in old_value.
+ * If old_value is null the object is freed with SHFREE.
+ * If this is not the correct semantics for your data, if for example
+ * its reference counted  you must pass in a old_value.
+ */
+DirectResult
+fusion_object_set_property( FusionObject  *object,
+                            const char    *key,
+                            void          *value,
+                            void         **old_value )
+{
+     DirectResult  ret;
+     char         *sharedkey;
+
+     D_MAGIC_ASSERT( object, FusionObject );
+     D_ASSERT( object->shared != NULL );
+     D_ASSERT( key != NULL );
+     D_ASSERT( value != NULL );
+
+     /* Create property hash on demand. */
+     if (!object->properties) {
+          ret = fusion_hash_create( object->shared->main_pool,
+                                    HASH_STRING, HASH_PTR,
+                                    FUSION_HASH_MIN_SIZE,
+                                    &object->properties );
+          if (ret)
+               return ret;
+     }
+
+     /* Create a shared copy of the key. */
+     sharedkey = SHSTRDUP( object->shared->main_pool, key );
+     if (!sharedkey)
+          return D_OOSHM();
+
+     /* Put it into the hash. */
+     ret = fusion_hash_replace( object->properties, sharedkey,
+                                value, NULL, old_value );
+     if (ret)
+          SHFREE( object->shared->main_pool, sharedkey );
+
+     return ret;
+}
+
+/*
+ * Helper function for int values
+ */
+DirectResult
+fusion_object_set_int_property( FusionObject *object,
+                                const char   *key,
+                                int           value )
+{
+     DirectResult  ret;
+     int          *iptr;
+
+     D_MAGIC_ASSERT( object, FusionObject );
+     D_ASSERT( key != NULL );
+
+     iptr = SHMALLOC( object->shared->main_pool, sizeof(int) );
+     if (!iptr)
+          return D_OOSHM();
+
+     *iptr = value;
+
+     ret = fusion_object_set_property( object, key, iptr, NULL );
+     if (ret)
+          SHFREE( object->shared->main_pool, iptr );
+
+     return ret;
+}
+
+/*
+ * Helper function for char* values use if the string 
+ * is not in shared memory
+ * Assumes that the old value was a string and frees it.
+ */
+DirectResult
+fusion_object_set_string_property( FusionObject *object,
+                                   const char   *key,
+                                   char         *value )
+{
+     DirectResult  ret;
+     char         *copy;
+
+     D_MAGIC_ASSERT( object, FusionObject );
+     D_ASSERT( key != NULL );
+     D_ASSERT( value != NULL );
+
+     copy = SHSTRDUP( object->shared->main_pool, value );
+     if (!copy)
+          return D_OOSHM();
+
+     ret = fusion_object_set_property( object, key, copy, NULL );
+     if (ret)
+          SHFREE( object->shared->main_pool, copy );
+
+     return ret;
+}
+
+void *
+fusion_object_get_property( FusionObject *object, const char *key )
+{
+     D_MAGIC_ASSERT( object, FusionObject );
+     D_ASSERT( key != NULL );
+
+     if (!object->properties)
+          return NULL;
+
+     return fusion_hash_lookup( object->properties, key );
+}
+
+void 
+fusion_object_remove_property( FusionObject  *object,
+                               const char    *key,
+                               void         **old_value)
+{
+     D_MAGIC_ASSERT( object, FusionObject );
+     D_ASSERT( key != NULL );
+
+     if (!object->properties)
+          return;
+
+     fusion_hash_remove( object->properties, key, NULL, old_value );
+
+     if (fusion_hash_should_resize( object->properties ))
+          fusion_hash_resize( object->properties );
+}
+
