@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -46,7 +47,7 @@ typedef struct {
      DirectLink    link;
      ReactionFunc  func;
      void         *ctx;
-     bool          attached;
+     void         *node_link;
 } Reaction;
 
 typedef struct {
@@ -67,6 +68,11 @@ FusionReactor *fusion_reactor_new          ( int                 msg_size,
 /*
  * Destroy the reactor.
  */
+DirectResult   fusion_reactor_destroy      ( FusionReactor      *reactor );
+
+/*
+ * Free the reactor.
+ */
 DirectResult   fusion_reactor_free         ( FusionReactor      *reactor );
 
 
@@ -82,6 +88,9 @@ DirectResult   fusion_reactor_free         ( FusionReactor      *reactor );
 DirectResult   fusion_reactor_set_lock     ( FusionReactor      *reactor,
                                              FusionSkirmish     *skirmish );
 
+DirectResult   fusion_reactor_set_lock_only( FusionReactor      *reactor,
+                                             FusionSkirmish     *lock );
+
 /*
  * Attach a local reaction to the reactor.
  */
@@ -141,5 +150,12 @@ DirectResult   fusion_reactor_sized_dispatch( FusionReactor      *reactor,
                                               int                 msg_size,
                                               bool                self,
                                               const ReactionFunc *globals );
+                                              
+/*
+ * Specify whether local message handlers (reactions) should be called directly.
+ */
+DirectResult   fusion_reactor_direct        ( FusionReactor      *reactor,
+                                              bool                direct );
+
 #endif
 
