@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -38,6 +39,7 @@ typedef union {
      struct {
           int                      id;
           const FusionWorldShared *shared;
+          FusionID                 creator;
      } multi;
 
      /* single app */
@@ -57,6 +59,7 @@ typedef union {
  * Initialize.
  */
 DirectResult fusion_ref_init         (FusionRef         *ref,
+                                      const char        *name,
                                       const FusionWorld *world);
 
 /*
