@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,28 @@
    Boston, MA 02111-1307, USA.
 */
 
+/* Heap management adapted from libc
+   Copyright 1990, 1991, 1992 Free Software Foundation, Inc.
+                  Written May 1989 by Mike Haertel.
+
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Library General Public License as
+published by the Free Software Foundation; either version 2 of the
+License, or (at your option) any later version.
+
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Library General Public License for more details.
+
+You should have received a copy of the GNU Library General Public
+License along with this library; see the file COPYING.LIB.  If
+not, write to the Free Software Foundation, Inc., 675 Mass Ave,
+Cambridge, MA 02139, USA.
+
+   The author may be reached (Email) at the address mike@ai.mit.edu,
+   or (US mail) as Mike Haertel c/o Free Software Foundation.  */
+
 #include <config.h>
 
 #include <unistd.h>
@@ -33,13 +56,16 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <fcntl.h>
+#include <grp.h>
 
 #include <direct/debug.h>
 #include <direct/list.h>
 #include <direct/mem.h>
 #include <direct/memcpy.h>
 #include <direct/messages.h>
+#include <direct/util.h>
 
+#include <fusion/conf.h>
 #include <fusion/shmalloc.h>
 #include <fusion/fusion_internal.h>
 
@@ -58,7 +84,7 @@ align( shmalloc_heap *heap, size_t size )
      void          *result;
      unsigned long  adj;
 
-     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, %d )\n", __FUNCTION__, heap, size );
+     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, %zu )\n", __FUNCTION__, heap, size );
 
      D_MAGIC_ASSERT( heap, shmalloc_heap );
 
@@ -83,7 +109,7 @@ morecore( shmalloc_heap *heap, size_t size )
      shmalloc_info *newinfo, *oldinfo;
      size_t newsize;
 
-     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, %d )\n", __FUNCTION__, heap, size );
+     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, %zu )\n", __FUNCTION__, heap, size );
 
      D_MAGIC_ASSERT( heap, shmalloc_heap );
 
@@ -138,7 +164,7 @@ _fusion_shmalloc( shmalloc_heap *heap, size_t size )
      register size_t i;
      struct list *next;
 
-     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, %d )\n", __FUNCTION__, heap, size );
+     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, %zu )\n", __FUNCTION__, heap, size );
 
      D_MAGIC_ASSERT( heap, shmalloc_heap );
 
@@ -304,7 +330,7 @@ _fusion_shrealloc( shmalloc_heap *heap, void *ptr, size_t size )
      int    type;
      size_t block, blocks, oldlimit;
 
-     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, %p, %d )\n", __FUNCTION__, heap, ptr, size );
+     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, %p, %zu )\n", __FUNCTION__, heap, ptr, size );
 
      D_MAGIC_ASSERT( heap, shmalloc_heap );
 
@@ -464,17 +490,25 @@ _fusion_shfree( shmalloc_heap *heap, void *ptr )
                     heap->chunks_free--;
                }
 
-               /* Now see if we can return stuff to the system.  */
                blocks = heap->heapinfo[block].free.size;
+
+/* FIXME: as this is used when kernel is detected as >= 2.6.19.2 only, this fallback definition should be ok for now */
+#ifndef MADV_REMOVE
+#define MADV_REMOVE 9
+#endif
+               /* Punch a hole into the tmpfs file to really free RAM. */
+               if (fusion_config->madv_remove)
+                    madvise( ADDRESS(block), blocks * BLOCKSIZE, MADV_REMOVE );
+
+               /* Now see if we can truncate the end.  */
                if (blocks >= FINAL_FREE_BLOCKS && block + blocks == heap->heaplimit
-                   && __shmalloc_brk( heap, 0 ) == ADDRESS (block + blocks)) {
+                   && __shmalloc_brk( heap, 0 ) == ADDRESS (block + blocks))
+               {
                     register size_t bytes = blocks * BLOCKSIZE;
                     heap->heaplimit -= blocks;
                     __shmalloc_brk( heap, -bytes );
-                    heap->heapinfo[heap->heapinfo[block].free.prev].free.next
-                    = heap->heapinfo[block].free.next;
-                    heap->heapinfo[heap->heapinfo[block].free.next].free.prev
-                    = heap->heapinfo[block].free.prev;
+                    heap->heapinfo[heap->heapinfo[block].free.prev].free.next = heap->heapinfo[block].free.next;
+                    heap->heapinfo[heap->heapinfo[block].free.next].free.prev = heap->heapinfo[block].free.prev;
                     block = heap->heapinfo[block].free.prev;
                     heap->chunks_free--;
                     heap->bytes_free -= bytes;
@@ -561,15 +595,17 @@ DirectResult
 __shmalloc_init_heap( FusionSHM  *shm,
                       const char *filename,
                       void       *addr_base,
+                      int         space,
                       int        *ret_fd,
                       int        *ret_size )
 {
      DirectResult     ret;
      int              size;
-     int              heapsize = HEAP / BLOCKSIZE;
      FusionSHMShared *shared;
-     int              fd   = -1;
-     shmalloc_heap   *heap = NULL;
+     int              heapsize = (space + BLOCKSIZE-1) / BLOCKSIZE;
+     int              fd       = -1;
+     shmalloc_heap   *heap     = NULL;
+     struct group    *pGroupInfo;
 
      D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, '%s', %p, %p )\n",
                  __FUNCTION__, shm, filename, addr_base, ret_fd );
@@ -581,7 +617,7 @@ __shmalloc_init_heap( FusionSHM  *shm,
      D_ASSERT( ret_size != NULL );
 
      shared = shm->shared;
-     
+
      D_MAGIC_ASSERT( shared, FusionSHMShared );
      D_ASSERT( shared->tmpfs[0] != 0 );
 
@@ -591,26 +627,41 @@ __shmalloc_init_heap( FusionSHM  *shm,
      D_DEBUG_AT( Fusion_SHMHeap, "  -> opening shared memory file '%s'...\n", filename );
 
      /* open the virtual file */
-     fd = open( filename, O_RDWR | O_CREAT, 0660 );
+     fd = open( filename, O_RDWR | O_CREAT | O_TRUNC, 0660 );
      if (fd < 0) {
           ret = errno2result(errno);
           D_PERROR( "Fusion/SHM: Could not open shared memory file '%s'!\n", filename );
           goto error;
      }
 
+
+
+     if (fusion_config->shmfile_group) {
+          // obtain the group id #
+          pGroupInfo = getgrnam(fusion_config->shmfile_group);
+
+          // chgrp the SH_FILE dev entry
+          if (fchown(fd, -1, pGroupInfo->gr_gid) != 0) {
+               D_WARN("Fusion/SHM: changing permissions on /dev/shm/fusion.# failed... continuing on.");
+          }
+     }
+
      fchmod( fd, 0660 );
      ftruncate( fd, size );
 
      D_DEBUG_AT( Fusion_SHMHeap, "  -> mmaping shared memory file... (%d bytes)\n", size );
 
      /* map it shared */
-     heap = mmap( addr_base, size, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_FIXED, fd, 0 );
+     heap = mmap( addr_base, size + space, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_FIXED, fd, 0 );
      if (heap == MAP_FAILED) {
           ret = errno2result(errno);
           D_PERROR( "Fusion/SHM: Could not mmap shared memory file '%s'!\n", filename );
           goto error;
      }
 
+     close( fd );
+     fd = -1;
+
      if (heap != addr_base) {
           D_ERROR( "Fusion/SHM: mmap() returned address (%p) differs from requested (%p)\n", heap, addr_base );
           ret = DFB_FUSION;
@@ -619,8 +670,6 @@ __shmalloc_init_heap( FusionSHM  *shm,
 
      D_DEBUG_AT( Fusion_SHMHeap, "  -> done.\n" );
 
-     memset( heap, 0, size );
-
      heap->size     = size;
      heap->heapsize = heapsize;
      heap->heapinfo = (void*) heap + BLOCKALIGN(sizeof(shmalloc_heap));
@@ -668,7 +717,7 @@ __shmalloc_join_heap( FusionSHM  *shm,
      D_ASSERT( ret_fd != NULL );
 
      shared = shm->shared;
-     
+
      D_MAGIC_ASSERT( shared, FusionSHMShared );
      D_ASSERT( shared->tmpfs[0] != 0 );
 
@@ -692,6 +741,9 @@ __shmalloc_join_heap( FusionSHM  *shm,
           goto error;
      }
 
+     close( fd );
+     fd = -1;
+
      if (heap != addr_base) {
           D_ERROR( "Fusion/SHM: mmap() returned address (%p) differs from requested (%p)\n", heap, addr_base );
           ret = DFB_FUSION;
@@ -730,81 +782,34 @@ __shmalloc_brk( shmalloc_heap *heap, int increment )
      D_MAGIC_ASSERT( heap, shmalloc_heap );
 
      shared = heap->pool;
-
      D_MAGIC_ASSERT( shared, FusionSHMPoolShared );
 
      shm = shared->shm;
-
      D_MAGIC_ASSERT( shm, FusionSHMShared );
 
      world = _fusion_world( shm->world );
-
      D_MAGIC_ASSERT( world, FusionWorld );
 
      pool = &world->shm.pools[shared->index];
-
      D_MAGIC_ASSERT( pool, FusionSHMPool );
 
-     if (pool->size != heap->size) {
-          D_WARN( "local pool size %d differs from shared heap size %d", pool->size, heap->size );
-
-          if (!increment) {
-               void *new_mem;
-
-               new_mem = mremap( shared->addr_base, pool->size, heap->size, 0 );
-               if (new_mem == MAP_FAILED) {
-                    D_PERROR ("Fusion/SHM: mremapping shared memory file failed!\n");
-                    return NULL;
-               }
-
-               D_DEBUG_AT( Fusion_SHMHeap, "  -> remapped (%d -> %d)\n", pool->size, heap->size );
-
-               pool->size = heap->size;
-
-               if (new_mem != shared->addr_base)
-                    D_BREAK ("mremap returned a different address!");
-          }
-     }
-
      if (increment) {
-          void                  *new_mem;
-          int                    new_size = heap->size + increment;
-          FusionSHMPoolDispatch  dispatch;
+          int new_size = heap->size + increment;
 
           if (new_size > shared->max_size) {
-               D_WARN ("maximum shared memory size reached!");
+               D_WARN( "maximum shared memory file size (%dk) exceeded (%dk by %dk) in '%s'!",
+                       shared->max_size >> 10, new_size >> 10, increment >> 10, pool->filename );
+               fusion_dbg_print_memleaks( shared );
                return NULL;
           }
 
-          if (ftruncate( pool->fd, new_size ) < 0) {
-               D_PERROR ("Fusion/SHM: ftruncating shared memory file failed!\n");
+          if (truncate( pool->filename, new_size ) < 0) {
+               D_PERROR( "Fusion/SHM: truncating shared memory file '%s' from %dkb to %dkb failed!\n",
+                         pool->filename, heap->size >> 10, new_size >> 10 );
                return NULL;
           }
 
-          new_mem = mremap( shared->addr_base, pool->size, new_size, 0 );
-          if (new_mem == MAP_FAILED) {
-               D_PERROR ("Fusion/SHM: mremapping shared memory file failed!\n");
-               ftruncate( pool->fd, heap->size );
-               return NULL;
-          }
-
-          D_DEBUG_AT( Fusion_SHMHeap, "  -> remapped (%d -> %d)\n", pool->size, new_size );
-
           heap->size = new_size;
-          pool->size = new_size;
-
-          if (new_mem != shared->addr_base)
-               D_BREAK ("mremap returned a different address!");
-
-          dispatch.pool_id = shared->pool_id;
-          dispatch.size    = heap->size;
-
-          while (ioctl( world->fusion_fd, FUSION_SHMPOOL_DISPATCH, &dispatch )) {
-               if (errno != EINTR) {
-                    D_PERROR( "Fusion/SHM: FUSION_SHMPOOL_DISPATCH failed!\n" );
-                    break;
-               }
-          }
      }
 
      return shared->addr_base + heap->size - increment;
