@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -28,6 +29,7 @@
 #include <config.h>
 
 #include <direct/debug.h>
+#include <direct/mem.h>
 #include <direct/memcpy.h>
 
 #include <fusion/object.h>
@@ -41,7 +43,11 @@ static inline bool ensure_capacity( FusionVector *vector )
      D_ASSERT( vector->capacity > 0 );
 
      if (!vector->elements) {
-          vector->elements = SHMALLOC( vector->pool, vector->capacity * sizeof(void*) );
+          if (vector->pool)
+               vector->elements = SHMALLOC( vector->pool, vector->capacity * sizeof(void*) );
+          else
+               vector->elements = D_MALLOC( vector->capacity * sizeof(void*) );
+
           if (!vector->elements)
                return false;
      }
@@ -50,7 +56,11 @@ static inline bool ensure_capacity( FusionVector *vector )
           void *oldelements = vector->elements;
           int   capacity    = vector->capacity << 1;
 
-          elements = SHMALLOC( vector->pool, capacity * sizeof(void*) );
+          if (vector->pool)
+               elements = SHMALLOC( vector->pool, capacity * sizeof(void*) );
+          else
+               elements = D_MALLOC( capacity * sizeof(void*) );
+
           if (!elements)
                return false;
 
@@ -60,7 +70,10 @@ static inline bool ensure_capacity( FusionVector *vector )
           vector->elements = elements;
           vector->capacity = capacity;
 
-          SHFREE( vector->pool, oldelements );
+          if (vector->pool)
+               SHFREE( vector->pool, oldelements );
+          else
+               D_FREE( oldelements );
      }
 
      return true;
@@ -89,7 +102,11 @@ fusion_vector_destroy( FusionVector *vector )
      D_ASSERT( vector->count == 0 || vector->elements != NULL );
 
      if (vector->elements) {
-          SHFREE( vector->pool, vector->elements );
+          if (vector->pool)
+               SHFREE( vector->pool, vector->elements );
+          else
+               D_FREE( vector->elements );
+
           vector->elements = NULL;
      }
 
