@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -104,7 +105,7 @@ struct __V_VoodooManager {
           DirectThread          *thread;
           pthread_mutex_t        lock;
           pthread_cond_t         wait;
-          __u8                   buffer[IN_BUF_MAX + MAX_MSG_SIZE];
+          u8                     buffer[IN_BUF_MAX + MAX_MSG_SIZE];
           int                    start;
           int                    end;
           int                    max;
@@ -114,7 +115,7 @@ struct __V_VoodooManager {
           DirectThread          *thread;
           pthread_mutex_t        lock;
           pthread_cond_t         wait;
-          __u8                   buffer[OUT_BUF_MAX];
+          u8                     buffer[OUT_BUF_MAX];
           int                    start;
           int                    end;
      } output;
@@ -282,11 +283,11 @@ voodoo_manager_quit( VoodooManager *manager )
 
 static bool
 instance_iterator( DirectHash *hash,
-                   __u32       key,
+                   u32         key,
                    void       *value,
                    void       *ctx )
 {
-     bool            super    = (bool) ctx;
+     bool            super    = (unsigned long) ctx;
      VoodooInstance *instance = value;
 
      D_ASSERT( instance != NULL );
@@ -417,7 +418,7 @@ voodoo_manager_super( VoodooManager    *manager,
      /* Append the name of the super interface to create. */
      direct_memcpy( msg + 1, name, len );
 
-     D_DEBUG( "Voodoo/Manager: Sending SUPER message %llu for '%s' (%d bytes).\n", serial, name, size );
+     D_DEBUG( "Voodoo/Manager: Sending SUPER message %zu for '%s' (%d bytes).\n", serial, name, size );
 
      /* Unlock the output buffer. */
      manager_unlock_output( manager, true );
@@ -431,7 +432,7 @@ voodoo_manager_super( VoodooManager    *manager,
           return ret;
      }
 
-     D_DEBUG( "Voodoo/Manager: Got response %llu (%s) with instance %u for request %llu "
+     D_DEBUG( "Voodoo/Manager: Got response %zu (%s) with instance %u for request %zu "
               "(%d bytes).\n", response->header.serial, DirectResultString( ret ),
               response->instance, response->request, response->header.size );
 
@@ -460,29 +461,29 @@ calc_blocks( VoodooMessageBlockType type, va_list args )
      int size = 4;  /* for the terminating VMBT_NONE */
 
      while (type != VMBT_NONE) {
-          __u32 arg;
-          __s32 in;
+          u32 arg;
+          s32 in;
           void *ptr;
           int   len = 0;
 
           switch (type) {
                case VMBT_ID:
                     len = 4;
-                    arg = va_arg( args, __u32 );
+                    arg = va_arg( args, u32 );
 
                     D_DEBUG( "Voodoo/Message: + ID %u\n", arg );
                     break;
 
                case VMBT_INT:
                     len = 4;
-                    in  = va_arg( args, __s32 );
+                    in  = va_arg( args, s32 );
 
                     D_DEBUG( "Voodoo/Message: + INT %d\n", in );
                     break;
 
                case VMBT_UINT:
                     len = 4;
-                    arg = va_arg( args, __u32 );
+                    arg = va_arg( args, u32 );
 
                     D_DEBUG( "Voodoo/Message: + UINT %u\n", arg );
                     break;
@@ -534,7 +535,7 @@ static inline void
 write_blocks( void *dst, VoodooMessageBlockType type, va_list args )
 {
      while (type != VMBT_NONE) {
-          __u32 arg = 0;
+          u32 arg = 0;
           int   len = 0;
           void *ptr = NULL;
 
@@ -543,7 +544,7 @@ write_blocks( void *dst, VoodooMessageBlockType type, va_list args )
                case VMBT_INT:
                case VMBT_UINT:
                     len = 4;
-                    arg = va_arg( args, __u32 );
+                    arg = va_arg( args, u32 );
                     break;
                case VMBT_DATA:
                     len = va_arg( args, int );
@@ -564,14 +565,14 @@ write_blocks( void *dst, VoodooMessageBlockType type, va_list args )
           }
 
           /* Write block type and length. */
-          *(__u32*) (dst + 0) = type;
-          *(__s32*) (dst + 4) = len;
+          *(u32*) (dst + 0) = type;
+          *(s32*) (dst + 4) = len;
 
           /* Write block content. */
           if (ptr)
                direct_memcpy( dst + 8, ptr, len );
           else if (len)
-               *(__u32*) (dst + 8) = arg;
+               *(u32*) (dst + 8) = arg;
 
           /* Advance message data pointer. */
           dst += len + 8;
@@ -581,7 +582,7 @@ write_blocks( void *dst, VoodooMessageBlockType type, va_list args )
      }
 
      /* Write terminator. */
-     *(__u32*) dst = VMBT_NONE;
+     *(u32*) dst = VMBT_NONE;
 }
 
 DirectResult
@@ -640,7 +641,7 @@ voodoo_manager_request( VoodooManager           *manager,
      va_end( args );
 
 
-     D_DEBUG( "Voodoo/Manager: Sending REQUEST message %llu to %u::%u %s(%d bytes).\n",
+     D_DEBUG( "Voodoo/Manager: Sending REQUEST message %zu to %u::%u %s(%d bytes).\n",
               serial, instance, method, (flags & VREQ_RESPOND) ? "[RESPONDING] " : "", size );
 
      /* Unlock the output buffer. */
@@ -658,7 +659,7 @@ voodoo_manager_request( VoodooManager           *manager,
                return ret;
           }
 
-          D_DEBUG( "Voodoo/Manager: Got response %llu (%s) with instance %u for request %llu "
+          D_DEBUG( "Voodoo/Manager: Got response %zu (%s) with instance %u for request %zu "
                    "(%d bytes).\n", response->header.serial, DirectResultString( response->result ),
                    response->instance, response->request, response->header.size );
 
@@ -696,7 +697,7 @@ voodoo_manager_respond( VoodooManager          *manager,
      D_MAGIC_ASSERT( manager, VoodooManager );
 
      D_DEBUG( "Voodoo/Response: "
-              "Request %llu, result %d, instance %u...\n", request, result, instance );
+              "Request %zu, result %d, instance %u...\n", request, result, instance );
 
      /* Calculate the total message size. */
      va_start( args, block_type );
@@ -732,7 +733,7 @@ voodoo_manager_respond( VoodooManager          *manager,
 
 
      D_DEBUG( "Voodoo/Manager: "
-              "Sending RESPONSE message %llu (%s) with instance %u for request %llu (%d bytes).\n",
+              "Sending RESPONSE message %zu (%s) with instance %u for request %zu (%d bytes).\n",
               serial, DirectResultString( result ), instance, request, size );
 
      /* Unlock the output buffer. */
@@ -924,7 +925,7 @@ handle_super( VoodooManager      *manager,
 
      name = (const char *) (super + 1);
 
-     D_DEBUG( "Voodoo/Dispatch: Handling SUPER message %llu for '%s' (%d bytes).\n",
+     D_DEBUG( "Voodoo/Dispatch: Handling SUPER message %zu for '%s' (%d bytes).\n",
               super->header.serial, name, super->header.size );
 
      if (manager->server) {
@@ -989,7 +990,7 @@ handle_request( VoodooManager        *manager,
      D_ASSERT( request->header.size >= sizeof(VoodooRequestMessage) );
      D_ASSERT( request->header.type == VMSG_REQUEST );
 
-     D_DEBUG( "Voodoo/Dispatch: Handling REQUEST message %llu to %u::%u %s%s(%d bytes).\n",
+     D_DEBUG( "Voodoo/Dispatch: Handling REQUEST message %zu to %u::%u %s%s(%d bytes).\n",
               request->header.serial, request->instance, request->method,
               (request->flags & VREQ_RESPOND) ? "[RESPONDING] " : "",
               (request->flags & VREQ_ASYNC) ? "[ASYNC] " : "",
@@ -1054,8 +1055,8 @@ handle_response( VoodooManager         *manager,
      D_ASSERT( response->header.type == VMSG_RESPONSE );
      D_ASSERT( response->request < manager->msg_serial );
 
-     D_DEBUG( "Voodoo/Dispatch: Handling RESPONSE message %llu (%s) with instance %u for request "
-              "%llu (%d bytes).\n", response->header.serial, DirectResultString( response->result ),
+     D_DEBUG( "Voodoo/Dispatch: Handling RESPONSE message %zu (%s) with instance %u for request "
+              "%zu (%d bytes).\n", response->header.serial, DirectResultString( response->result ),
               response->instance, response->request, response->header.size );
 
      pthread_mutex_lock( &manager->response.lock );
@@ -1391,7 +1392,7 @@ manager_lock_response( VoodooManager          *manager,
      D_MAGIC_ASSERT( manager, VoodooManager );
      D_ASSERT( ret_response != NULL );
 
-     D_DEBUG( "Voodoo/Manager: Locking response to request %llu...\n", request );
+     D_DEBUG( "Voodoo/Manager: Locking response to request %zu...\n", request );
 
      pthread_mutex_lock( &manager->response.lock );
 
@@ -1401,9 +1402,9 @@ manager_lock_response( VoodooManager          *manager,
                break;
 
           if (response)
-               D_DEBUG( "Voodoo/Manager: ...current response is for request %llu...\n", response->request );
+               D_DEBUG( "Voodoo/Manager: ...current response is for request %zu...\n", response->request );
 
-          D_DEBUG( "Voodoo/Manager: ...(still) waiting for response to request %llu...\n", request );
+          D_DEBUG( "Voodoo/Manager: ...(still) waiting for response to request %zu...\n", request );
 
           pthread_cond_wait( &manager->response.wait, &manager->response.lock );
      }
@@ -1413,7 +1414,7 @@ manager_lock_response( VoodooManager          *manager,
           return DFB_DESTROYED;
      }
 
-     D_DEBUG( "Voodoo/Manager: ...locked response %llu to request %llu (%d bytes).\n",
+     D_DEBUG( "Voodoo/Manager: ...locked response %zu to request %zu (%d bytes).\n",
               response->header.serial, request, response->header.size );
 
      *ret_response = response;
@@ -1429,7 +1430,7 @@ manager_unlock_response( VoodooManager         *manager,
      D_ASSERT( response != NULL );
      D_ASSERT( response == manager->response.current );
 
-     D_DEBUG( "Voodoo/Manager: Unlocking response %llu to request %llu (%d bytes)...\n",
+     D_DEBUG( "Voodoo/Manager: Unlocking response %zu to request %zu (%d bytes)...\n",
               response->header.serial, response->request, response->header.size );
 
      manager->response.current = NULL;
