@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -109,12 +110,12 @@ typedef struct {
      _vp_ptr = (parser).ptr;                                \
                                                             \
      /* Read message block type. */                         \
-     _vp_type = *(const __u32*) _vp_ptr;                    \
+     _vp_type = *(const u32*) _vp_ptr;                      \
                                                             \
      D_ASSERT( _vp_type == (req_type) );                    \
                                                             \
      /* Read data block length. */                          \
-     _vp_length = *(const __s32*) (_vp_ptr + 4)
+     _vp_length = *(const s32*) (_vp_ptr + 4)
 
 
 #define __VOODOO_PARSER_EPILOG( parser )                    \
@@ -144,7 +145,7 @@ typedef struct {
           D_ASSERT( _vp_length == 4 );                                \
                                                                       \
           /* Read the ID. */                                          \
-          (ret_id) = *(const __u32*) (_vp_ptr + 8);                   \
+          (ret_id) = *(const u32*) (_vp_ptr + 8);                     \
                                                                       \
           __VOODOO_PARSER_EPILOG( parser );                           \
      } while (0)
@@ -156,7 +157,7 @@ typedef struct {
           D_ASSERT( _vp_length == 4 );                                \
                                                                       \
           /* Read the integer. */                                     \
-          (ret_int) = *(const __s32*) (_vp_ptr + 8);                  \
+          (ret_int) = *(const s32*) (_vp_ptr + 8);                    \
                                                                       \
           __VOODOO_PARSER_EPILOG( parser );                           \
      } while (0)
@@ -168,7 +169,7 @@ typedef struct {
           D_ASSERT( _vp_length == 4 );                                \
                                                                       \
           /* Read the unsigned integer. */                            \
-          (ret_uint) = *(const __u32*) (_vp_ptr + 8);                 \
+          (ret_uint) = *(const u32*) (_vp_ptr + 8);                   \
                                                                       \
           __VOODOO_PARSER_EPILOG( parser );                           \
      } while (0)
@@ -245,7 +246,7 @@ typedef struct {
      do {                                                             \
           D_MAGIC_ASSERT( &(parser), VoodooMessageParser );           \
                                                                       \
-          D_ASSUME( *(const __u32*) ((parser).ptr) == VMBT_NONE );    \
+          D_ASSUME( *(const u32*) ((parser).ptr) == VMBT_NONE );      \
                                                                       \
           D_MAGIC_CLEAR( &(parser) );                                 \
      } while (0)
