@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -569,9 +570,9 @@ Dispatch_CreateEventBuffer( IDirectFB *thiz, IDirectFB *real,
 {
      DirectResult          ret;
      IDirectFBEventBuffer *buffer;
-     IDirectFBEventBuffer *requestor;
      VoodooInstanceID      instance;
      VoodooMessageParser   parser;
+     void                 *requestor;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
@@ -584,7 +585,7 @@ Dispatch_CreateEventBuffer( IDirectFB *thiz, IDirectFB *real,
           return ret;
 
      ret = voodoo_construct_requestor( manager, "IDirectFBEventBuffer",
-                                       instance, buffer, (void**) &requestor );
+                                       instance, buffer, &requestor );
      if (ret) {
           buffer->Release( buffer );
           return ret;
@@ -599,11 +600,11 @@ Dispatch_CreateInputEventBuffer( IDirectFB *thiz, IDirectFB *real,
 {
      DirectResult                ret;
      IDirectFBEventBuffer       *buffer;
-     IDirectFBEventBuffer       *requestor;
      VoodooInstanceID            instance;
      DFBInputDeviceCapabilities  caps;
      DFBBoolean                  global;
      VoodooMessageParser         parser;
+     void                       *requestor;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
@@ -618,7 +619,7 @@ Dispatch_CreateInputEventBuffer( IDirectFB *thiz, IDirectFB *real,
           return ret;
 
      ret = voodoo_construct_requestor( manager, "IDirectFBEventBuffer",
-                                       instance, buffer, (void**) &requestor );
+                                       instance, buffer, &requestor );
      if (ret) {
           buffer->Release( buffer );
           return ret;
@@ -936,6 +937,7 @@ Dispatch_CreateDataBuffer( IDirectFB *thiz, IDirectFB *real,
      VoodooMessageParser  parser;
      VoodooInstanceID     instance;
      DataBufferEntry     *entry;
+     void                *ptr;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
@@ -944,10 +946,12 @@ Dispatch_CreateDataBuffer( IDirectFB *thiz, IDirectFB *real,
      VOODOO_PARSER_END( parser );
 
      ret = voodoo_construct_requestor( manager, "IDirectFBDataBuffer",
-                                       instance, NULL, (void**) &requestor );
+                                       instance, NULL, &ptr );
      if (ret)
           return ret;
 
+     requestor = ptr;
+
      entry = D_CALLOC( 1, sizeof(DataBufferEntry) );
      if (!entry) {
           D_WARN( "out of memory" );
