@@ -1,12 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -113,7 +115,6 @@ IDirectFBInputDevice_Dispatcher_CreateEventBuffer( IDirectFBInputDevice  *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
-
 static DFBResult
 IDirectFBInputDevice_Dispatcher_AttachEventBuffer( IDirectFBInputDevice *thiz,
                                                    IDirectFBEventBuffer *buffer )
@@ -125,6 +126,17 @@ IDirectFBInputDevice_Dispatcher_AttachEventBuffer( IDirectFBInputDevice *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBInputDevice_Dispatcher_DetachEventBuffer( IDirectFBInputDevice *thiz,
+                                                   IDirectFBEventBuffer *buffer )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice_Dispatcher)
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBInputDevice_Dispatcher_GetDescription( IDirectFBInputDevice      *thiz,
                                                 DFBInputDeviceDescription *ret_desc )
@@ -327,9 +339,9 @@ Dispatch_CreateEventBuffer( IDirectFBInputDevice *thiz, IDirectFBInputDevice *re
 {
      DirectResult          ret;
      IDirectFBEventBuffer *buffer;
-     IDirectFBEventBuffer *requestor;
      VoodooInstanceID      instance;
      VoodooMessageParser   parser;
+     void                 *requestor;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice_Dispatcher)
 
@@ -342,7 +354,7 @@ Dispatch_CreateEventBuffer( IDirectFBInputDevice *thiz, IDirectFBInputDevice *re
           return ret;
 
      ret = voodoo_construct_requestor( manager, "IDirectFBEventBuffer",
-                                       instance, buffer, (void**) &requestor );
+                                       instance, buffer, &requestor );
      if (ret)
           buffer->Release( buffer );
 
@@ -360,6 +372,7 @@ Dispatch_AttachEventBuffer( IDirectFBInputDevice *thiz, IDirectFBInputDevice *re
      IDirectFBEventBuffer_Requestor_data *buffer_data;
      VoodooInstanceID                     instance;
      VoodooMessageParser                  parser;
+     void                                *ptr;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice_Dispatcher)
 
@@ -367,10 +380,12 @@ Dispatch_AttachEventBuffer( IDirectFBInputDevice *thiz, IDirectFBInputDevice *re
      VOODOO_PARSER_GET_ID( parser, instance );
      VOODOO_PARSER_END( parser );
 
-     ret = voodoo_manager_lookup_remote( manager, instance, (void**) &buffer );
+     ret = voodoo_manager_lookup_remote( manager, instance, &ptr );
      if (ret)
           return ret;
 
+     buffer = ptr;
+
      DIRECT_INTERFACE_GET_DATA_FROM( buffer, buffer_data, IDirectFBEventBuffer_Requestor );
 
      ret = real->AttachEventBuffer( real, buffer_data->src );
@@ -380,6 +395,38 @@ Dispatch_AttachEventBuffer( IDirectFBInputDevice *thiz, IDirectFBInputDevice *re
                                     VMBT_NONE );
 }
 
+static DirectResult
+Dispatch_DetachEventBuffer( IDirectFBInputDevice *thiz, IDirectFBInputDevice *real,
+                            VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult                         ret;
+     IDirectFBEventBuffer                *buffer;
+     IDirectFBEventBuffer_Requestor_data *buffer_data;
+     VoodooInstanceID                     instance;
+     VoodooMessageParser                  parser;
+     void                                *ptr;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_ID( parser, instance );
+     VOODOO_PARSER_END( parser );
+
+     ret = voodoo_manager_lookup_remote( manager, instance, &ptr );
+     if (ret)
+          return ret;
+
+     buffer = ptr;
+
+     DIRECT_INTERFACE_GET_DATA_FROM( buffer, buffer_data, IDirectFBEventBuffer_Requestor );
+
+     ret = real->DetachEventBuffer( real, buffer_data->src );
+
+     return voodoo_manager_respond( manager, msg->header.serial,
+                                    ret, VOODOO_INSTANCE_NONE,
+                                    VMBT_NONE );
+}
+
 static DirectResult
 Dispatch_GetKeyState( IDirectFBInputDevice *thiz, IDirectFBInputDevice *real,
                       VoodooManager *manager, VoodooRequestMessage *msg )
@@ -478,6 +525,9 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
 
           case IDIRECTFBINPUTDEVICE_METHOD_ID_AttachEventBuffer:
                return Dispatch_AttachEventBuffer( dispatcher, real, manager, msg );
+               
+          case IDIRECTFBINPUTDEVICE_METHOD_ID_DetachEventBuffer:
+               return Dispatch_DetachEventBuffer( dispatcher, real, manager, msg );
 
           case IDIRECTFBINPUTDEVICE_METHOD_ID_GetKeyState:
                return Dispatch_GetKeyState( dispatcher, real, manager, msg );
@@ -555,6 +605,7 @@ Construct( IDirectFBInputDevice *thiz,     /* Dispatcher interface */
      thiz->GetKeymapEntry     = IDirectFBInputDevice_Dispatcher_GetKeymapEntry;
      thiz->CreateEventBuffer  = IDirectFBInputDevice_Dispatcher_CreateEventBuffer;
      thiz->AttachEventBuffer  = IDirectFBInputDevice_Dispatcher_AttachEventBuffer;
+     thiz->DetachEventBuffer  = IDirectFBInputDevice_Dispatcher_DetachEventBuffer;
      thiz->GetKeyState        = IDirectFBInputDevice_Dispatcher_GetKeyState;
      thiz->GetModifiers       = IDirectFBInputDevice_Dispatcher_GetModifiers;
      thiz->GetLockState       = IDirectFBInputDevice_Dispatcher_GetLockState;
