@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -128,27 +129,26 @@ IDirectFBSurface_Dispatcher_Release( IDirectFBSurface *thiz )
      return DFB_OK;
 }
 
-
 static DFBResult
-IDirectFBSurface_Dispatcher_GetPixelFormat( IDirectFBSurface      *thiz,
-                                            DFBSurfacePixelFormat *format )
+IDirectFBSurface_Dispatcher_GetCapabilities( IDirectFBSurface       *thiz,
+                                             DFBSurfaceCapabilities *caps )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
-     if (!format)
+     if (!caps)
           return DFB_INVARG;
 
      return DFB_UNIMPLEMENTED;
 }
 
 static DFBResult
-IDirectFBSurface_Dispatcher_GetAccelerationMask( IDirectFBSurface    *thiz,
-                                                 IDirectFBSurface    *source,
-                                                 DFBAccelerationMask *mask )
+IDirectFBSurface_Dispatcher_GetPosition( IDirectFBSurface *thiz,
+                                         int              *x,
+                                         int              *y )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
-     if (!mask)
+     if (!x && !y)
           return DFB_INVARG;
 
      return DFB_UNIMPLEMENTED;
@@ -180,12 +180,25 @@ IDirectFBSurface_Dispatcher_GetVisibleRectangle( IDirectFBSurface *thiz,
 }
 
 static DFBResult
-IDirectFBSurface_Dispatcher_GetCapabilities( IDirectFBSurface       *thiz,
-                                             DFBSurfaceCapabilities *caps )
+IDirectFBSurface_Dispatcher_GetPixelFormat( IDirectFBSurface      *thiz,
+                                            DFBSurfacePixelFormat *format )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
-     if (!caps)
+     if (!format)
+          return DFB_INVARG;
+
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBSurface_Dispatcher_GetAccelerationMask( IDirectFBSurface    *thiz,
+                                                 IDirectFBSurface    *source,
+                                                 DFBAccelerationMask *mask )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     if (!mask)
           return DFB_INVARG;
 
      return DFB_UNIMPLEMENTED;
@@ -215,6 +228,16 @@ IDirectFBSurface_Dispatcher_SetPalette( IDirectFBSurface *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Dispatcher_SetAlphaRamp( IDirectFBSurface *thiz,
+                                          u8 a0, u8 a1, u8 a2, u8 a3 )
+ 
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBSurface_Dispatcher_Lock( IDirectFBSurface *thiz,
                                   DFBSurfaceLockFlags flags,
@@ -228,6 +251,18 @@ IDirectFBSurface_Dispatcher_Lock( IDirectFBSurface *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Dispatcher_GetFramebufferOffset( IDirectFBSurface *thiz,
+                                                  int              *offset )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     if (!offset)
+          return DFB_INVARG;
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBSurface_Dispatcher_Unlock( IDirectFBSurface *thiz )
 {
@@ -260,7 +295,7 @@ IDirectFBSurface_Dispatcher_SetField( IDirectFBSurface    *thiz,
 
 static DFBResult
 IDirectFBSurface_Dispatcher_Clear( IDirectFBSurface *thiz,
-                                   __u8 r, __u8 g, __u8 b, __u8 a )
+                                   u8 r, u8 g, u8 b, u8 a )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
@@ -268,16 +303,29 @@ IDirectFBSurface_Dispatcher_Clear( IDirectFBSurface *thiz,
 }
 
 static DFBResult
-IDirectFBSurface_Dispatcher_SetClip( IDirectFBSurface *thiz, const DFBRegion *clip )
+IDirectFBSurface_Dispatcher_SetClip( IDirectFBSurface *thiz, 
+                                     const DFBRegion  *clip )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Dispatcher_GetClip( IDirectFBSurface *thiz, 
+                                     DFBRegion        *clip )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+     
+     if (!clip)
+          return DFB_INVARG;
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBSurface_Dispatcher_SetColor( IDirectFBSurface *thiz,
-                                      __u8 r, __u8 g, __u8 b, __u8 a )
+                                      u8 r, u8 g, u8 b, u8 a )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
@@ -322,9 +370,9 @@ IDirectFBSurface_Dispatcher_SetPorterDuff( IDirectFBSurface         *thiz,
 
 static DFBResult
 IDirectFBSurface_Dispatcher_SetSrcColorKey( IDirectFBSurface *thiz,
-                                            __u8              r,
-                                            __u8              g,
-                                            __u8              b )
+                                            u8                r,
+                                            u8                g,
+                                            u8                b )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
@@ -342,9 +390,9 @@ IDirectFBSurface_Dispatcher_SetSrcColorKeyIndex( IDirectFBSurface *thiz,
 
 static DFBResult
 IDirectFBSurface_Dispatcher_SetDstColorKey( IDirectFBSurface *thiz,
-                                            __u8              r,
-                                            __u8              g,
-                                            __u8              b )
+                                            u8                r,
+                                            u8                g,
+                                            u8                b )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
@@ -574,6 +622,15 @@ IDirectFBSurface_Dispatcher_DrawGlyph( IDirectFBSurface *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Dispatcher_SetEncoding( IDirectFBSurface  *thiz,
+                                         DFBTextEncodingID  encoding )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBSurface_Dispatcher_GetSubSurface( IDirectFBSurface    *thiz,
                                            const DFBRectangle  *rect,
@@ -612,8 +669,54 @@ IDirectFBSurface_Dispatcher_Dump( IDirectFBSurface   *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Dispatcher_DisableAcceleration( IDirectFBSurface   *thiz,
+                                                 DFBAccelerationMask mask )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBSurface_Dispatcher_ReleaseSource( IDirectFBSurface *thiz )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+     
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBSurface_Dispatcher_SetIndexTranslation( IDirectFBSurface *thiz,
+                                                 const int        *indices,
+                                                 int               num_indices )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+     
+     return DFB_UNIMPLEMENTED;
+}
+
 /**************************************************************************************************/
 
+static DirectResult
+Dispatch_GetPosition( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                      VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DFBResult ret;
+     DFBPoint  position;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     ret = real->GetPosition( real, &position.x, &position.y );
+     if (ret)
+          return ret;
+
+     return voodoo_manager_respond( manager, msg->header.serial,
+                                    DFB_OK, VOODOO_INSTANCE_NONE,
+                                    VMBT_DATA, sizeof(DFBPoint), &position,
+                                    VMBT_NONE );
+}
+
 static DirectResult
 Dispatch_GetSize( IDirectFBSurface *thiz, IDirectFBSurface *real,
                   VoodooManager *manager, VoodooRequestMessage *msg )
@@ -1297,6 +1400,24 @@ Dispatch_DrawGlyph( IDirectFBSurface *thiz, IDirectFBSurface *real,
      return DFB_OK;
 }
 
+static DirectResult
+Dispatch_SetEncoding( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                      VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     VoodooMessageParser parser;
+     DFBTextEncodingID   encoding;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_UINT( parser, encoding );
+     VOODOO_PARSER_END( parser );
+
+     real->SetEncoding( real, encoding );
+     
+     return DFB_OK;
+}
+
 static DirectResult
 Dispatch_GetSubSurface( IDirectFBSurface *thiz, IDirectFBSurface *real,
                         VoodooManager *manager, VoodooRequestMessage *msg )
@@ -1329,6 +1450,35 @@ Dispatch_GetSubSurface( IDirectFBSurface *thiz, IDirectFBSurface *real,
                                     VMBT_NONE );
 }
 
+static DirectResult
+Dispatch_DisableAcceleration( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                              VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     VoodooMessageParser parser;
+     DFBAccelerationMask mask;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_INT( parser, mask );
+     VOODOO_PARSER_END( parser );
+
+     real->DisableAcceleration( real, mask );
+
+     return DFB_OK;
+}
+
+static DirectResult
+Dispatch_ReleaseSource( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                        VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     real->ReleaseSource( real );
+
+     return DFB_OK;
+}
+
 static DirectResult
 Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMessage *msg )
 {
@@ -1336,6 +1486,9 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
               "Handling request for instance %u with method %u...\n", msg->instance, msg->method );
 
      switch (msg->method) {
+          case IDIRECTFBSURFACE_METHOD_ID_GetPosition:
+               return Dispatch_GetPosition( dispatcher, real, manager, msg );
+               
           case IDIRECTFBSURFACE_METHOD_ID_GetSize:
                return Dispatch_GetSize( dispatcher, real, manager, msg );
 
@@ -1422,6 +1575,9 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
 
           case IDIRECTFBSURFACE_METHOD_ID_DrawGlyph:
                return Dispatch_DrawGlyph( dispatcher, real, manager, msg );
+               
+          case IDIRECTFBSURFACE_METHOD_ID_SetEncoding:
+               return Dispatch_SetEncoding( dispatcher, real, manager, msg );
 
           case IDIRECTFBSURFACE_METHOD_ID_GetSubSurface:
                return Dispatch_GetSubSurface( dispatcher, real, manager, msg );
@@ -1431,6 +1587,12 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
 
           case IDIRECTFBSURFACE_METHOD_ID_FillSpans:
                return Dispatch_FillSpans( dispatcher, real, manager, msg );
+               
+          case IDIRECTFBSURFACE_METHOD_ID_DisableAcceleration:
+               return Dispatch_DisableAcceleration( dispatcher, real, manager, msg );
+               
+          case IDIRECTFBSURFACE_METHOD_ID_ReleaseSource:
+               return Dispatch_ReleaseSource( dispatcher, real, manager, msg );
      }
 
      return DFB_NOSUCHMETHOD;
@@ -1471,6 +1633,7 @@ Construct( IDirectFBSurface *thiz,
      thiz->Release = IDirectFBSurface_Dispatcher_Release;
 
      thiz->GetCapabilities = IDirectFBSurface_Dispatcher_GetCapabilities;
+     thiz->GetPosition = IDirectFBSurface_Dispatcher_GetPosition;
      thiz->GetSize = IDirectFBSurface_Dispatcher_GetSize;
      thiz->GetVisibleRectangle = IDirectFBSurface_Dispatcher_GetVisibleRectangle;
      thiz->GetPixelFormat = IDirectFBSurface_Dispatcher_GetPixelFormat;
@@ -1478,14 +1641,17 @@ Construct( IDirectFBSurface *thiz,
 
      thiz->GetPalette = IDirectFBSurface_Dispatcher_GetPalette;
      thiz->SetPalette = IDirectFBSurface_Dispatcher_SetPalette;
+     thiz->SetAlphaRamp = IDirectFBSurface_Dispatcher_SetAlphaRamp;
 
      thiz->Lock = IDirectFBSurface_Dispatcher_Lock;
+     thiz->GetFramebufferOffset = IDirectFBSurface_Dispatcher_GetFramebufferOffset;
      thiz->Unlock = IDirectFBSurface_Dispatcher_Unlock;
      thiz->Flip = IDirectFBSurface_Dispatcher_Flip;
      thiz->SetField = IDirectFBSurface_Dispatcher_SetField;
      thiz->Clear = IDirectFBSurface_Dispatcher_Clear;
 
      thiz->SetClip = IDirectFBSurface_Dispatcher_SetClip;
+     thiz->GetClip = IDirectFBSurface_Dispatcher_GetClip;
      thiz->SetColor = IDirectFBSurface_Dispatcher_SetColor;
      thiz->SetColorIndex = IDirectFBSurface_Dispatcher_SetColorIndex;
      thiz->SetSrcBlendFunction = IDirectFBSurface_Dispatcher_SetSrcBlendFunction;
@@ -1516,12 +1682,19 @@ Construct( IDirectFBSurface *thiz,
      thiz->GetFont = IDirectFBSurface_Dispatcher_GetFont;
      thiz->DrawString = IDirectFBSurface_Dispatcher_DrawString;
      thiz->DrawGlyph = IDirectFBSurface_Dispatcher_DrawGlyph;
+     thiz->SetEncoding = IDirectFBSurface_Dispatcher_SetEncoding;
 
      thiz->GetSubSurface = IDirectFBSurface_Dispatcher_GetSubSurface;
 
      thiz->GetGL = IDirectFBSurface_Dispatcher_GetGL;
 
      thiz->Dump = IDirectFBSurface_Dispatcher_Dump;
+     
+     thiz->DisableAcceleration = IDirectFBSurface_Dispatcher_DisableAcceleration;
+     
+     thiz->ReleaseSource = IDirectFBSurface_Dispatcher_ReleaseSource;
+     
+     thiz->SetIndexTranslation = IDirectFBSurface_Dispatcher_SetIndexTranslation;
 
      return DFB_OK;
 }
