@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -73,5 +74,9 @@
 #define IDIRECTFBSURFACE_METHOD_ID_Dump                      43
 #define IDIRECTFBSURFACE_METHOD_ID_FillRectangles            44
 #define IDIRECTFBSURFACE_METHOD_ID_FillSpans                 45
+#define IDIRECTFBSURFACE_METHOD_ID_GetPosition               46
+#define IDIRECTFBSURFACE_METHOD_ID_SetEncoding               47
+#define IDIRECTFBSURFACE_METHOD_ID_DisableAcceleration       48
+#define IDIRECTFBSURFACE_METHOD_ID_ReleaseSource             49
 
 #endif
