@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -111,6 +112,17 @@ IDirectFBWindow_Dispatcher_AttachEventBuffer( IDirectFBWindow       *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBWindow_Dispatcher_DetachEventBuffer( IDirectFBWindow       *thiz,
+                                              IDirectFBEventBuffer  *buffer )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+     
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBWindow_Dispatcher_EnableEvents( IDirectFBWindow       *thiz,
                                          DFBWindowEventType     mask )
@@ -179,6 +191,43 @@ IDirectFBWindow_Dispatcher_GetSurface( IDirectFBWindow   *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBWindow_Dispatcher_SetProperty( IDirectFBWindow  *thiz,
+                                        const char       *key,
+                                        void             *value,
+                                        void            **old_value )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Dispatcher_GetProperty( IDirectFBWindow  *thiz,
+                                        const char       *key,
+                                        void            **value )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Dispatcher_RemoveProperty( IDirectFBWindow  *thiz,
+                                           const char       *key,
+                                           void            **value )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBWindow_Dispatcher_SetOptions( IDirectFBWindow  *thiz,
                                        DFBWindowOptions  options )
@@ -203,9 +252,9 @@ IDirectFBWindow_Dispatcher_GetOptions( IDirectFBWindow  *thiz,
 
 static DFBResult
 IDirectFBWindow_Dispatcher_SetColorKey( IDirectFBWindow *thiz,
-                                        __u8             r,
-                                        __u8             g,
-                                        __u8             b )
+                                        u8               r,
+                                        u8               g,
+                                        u8               b )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
 
@@ -241,7 +290,7 @@ IDirectFBWindow_Dispatcher_SetOpaqueRegion( IDirectFBWindow *thiz,
 
 static DFBResult
 IDirectFBWindow_Dispatcher_SetOpacity( IDirectFBWindow *thiz,
-                                       __u8             opacity )
+                                       u8               opacity )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
 
@@ -252,7 +301,7 @@ IDirectFBWindow_Dispatcher_SetOpacity( IDirectFBWindow *thiz,
 
 static DFBResult
 IDirectFBWindow_Dispatcher_GetOpacity( IDirectFBWindow *thiz,
-                                       __u8            *opacity )
+                                       u8              *opacity )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
 
@@ -473,6 +522,30 @@ IDirectFBWindow_Dispatcher_Destroy( IDirectFBWindow *thiz )
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBWindow_Dispatcher_SetBounds( IDirectFBWindow *thiz,
+                                      int x, int y, int w, int h )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Dispatcher_ResizeSurface( IDirectFBWindow *thiz,
+                                          int              width,
+                                          int              height )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
+
 /**************************************************************************************************/
 
 static DirectResult
@@ -481,9 +554,9 @@ Dispatch_CreateEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
 {
      DirectResult                ret;
      IDirectFBEventBuffer       *buffer;
-     IDirectFBEventBuffer       *requestor;
      VoodooInstanceID            instance;
      VoodooMessageParser         parser;
+     void                       *requestor;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
 
@@ -496,7 +569,7 @@ Dispatch_CreateEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
           return ret;
 
      ret = voodoo_construct_requestor( manager, "IDirectFBEventBuffer",
-                                       instance, buffer, (void**) &requestor );
+                                       instance, buffer, &requestor );
      if (ret)
           buffer->Release( buffer );
 
@@ -514,6 +587,7 @@ Dispatch_AttachEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
      IDirectFBEventBuffer_Requestor_data *buffer_data;
      VoodooInstanceID                     instance;
      VoodooMessageParser                  parser;
+     void                                *ptr;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
 
@@ -521,10 +595,12 @@ Dispatch_AttachEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
      VOODOO_PARSER_GET_ID( parser, instance );
      VOODOO_PARSER_END( parser );
 
-     ret = voodoo_manager_lookup_remote( manager, instance, (void**) &buffer );
+     ret = voodoo_manager_lookup_remote( manager, instance, &ptr );
      if (ret)
           return ret;
 
+     buffer = ptr;
+
      DIRECT_INTERFACE_GET_DATA_FROM( buffer, buffer_data, IDirectFBEventBuffer_Requestor );
 
      ret = real->AttachEventBuffer( real, buffer_data->src );
@@ -534,6 +610,38 @@ Dispatch_AttachEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
                                     VMBT_NONE );
 }
 
+static DirectResult
+Dispatch_DetachEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                            VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult                         ret;
+     IDirectFBEventBuffer                *buffer;
+     IDirectFBEventBuffer_Requestor_data *buffer_data;
+     VoodooInstanceID                     instance;
+     VoodooMessageParser                  parser;
+     void                                *ptr;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_ID( parser, instance );
+     VOODOO_PARSER_END( parser );
+
+     ret = voodoo_manager_lookup_remote( manager, instance, &ptr );
+     if (ret)
+          return ret;
+
+     buffer = ptr;
+
+     DIRECT_INTERFACE_GET_DATA_FROM( buffer, buffer_data, IDirectFBEventBuffer_Requestor );
+
+     ret = real->DetachEventBuffer( real, buffer_data->src );
+
+     return voodoo_manager_respond( manager, msg->header.serial,
+                                    ret, VOODOO_INSTANCE_NONE,
+                                    VMBT_NONE );
+}
+
 static DirectResult
 Dispatch_GetID( IDirectFBWindow *thiz, IDirectFBWindow *real,
                 VoodooManager *manager, VoodooRequestMessage *msg )
@@ -553,6 +661,44 @@ Dispatch_GetID( IDirectFBWindow *thiz, IDirectFBWindow *real,
                                     VMBT_NONE );
 }
 
+static DirectResult
+Dispatch_GetPosition( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                      VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult ret;
+     DFBPoint     position;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     ret = real->GetPosition( real, &position.x, &position.y );
+     if (ret)
+          return ret;
+
+     return voodoo_manager_respond( manager, msg->header.serial,
+                                    DFB_OK, VOODOO_INSTANCE_NONE,
+                                    VMBT_DATA, sizeof(DFBPoint), &position,
+                                    VMBT_NONE );
+}
+
+static DirectResult
+Dispatch_GetSize( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                  VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult ret;
+     DFBDimension size;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     ret = real->GetSize( real, &size.w, &size.h );
+     if (ret)
+          return ret;
+
+     return voodoo_manager_respond( manager, msg->header.serial,
+                                    DFB_OK, VOODOO_INSTANCE_NONE,
+                                    VMBT_DATA, sizeof(DFBDimension), &size,
+                                    VMBT_NONE );
+}
+
 static DirectResult
 Dispatch_GetSurface( IDirectFBWindow *thiz, IDirectFBWindow *real,
                      VoodooManager *manager, VoodooRequestMessage *msg )
@@ -622,7 +768,7 @@ Dispatch_SetOpacity( IDirectFBWindow *thiz, IDirectFBWindow *real,
                      VoodooManager *manager, VoodooRequestMessage *msg )
 {
      VoodooMessageParser parser;
-     __u8                opacity;
+     u8                  opacity;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
 
@@ -640,7 +786,7 @@ Dispatch_GetOpacity( IDirectFBWindow *thiz, IDirectFBWindow *real,
                      VoodooManager *manager, VoodooRequestMessage *msg )
 {
      DirectResult     ret;
-     __u8             opacity;
+     u8               opacity;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
 
@@ -873,6 +1019,49 @@ Dispatch_Destroy( IDirectFBWindow *thiz, IDirectFBWindow *real,
                                     VMBT_NONE );
 }
 
+static DirectResult
+Dispatch_SetBounds( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                    VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DFBResult           ret;
+     VoodooMessageParser parser;
+     const DFBRectangle *bounds;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_DATA( parser, bounds );
+     VOODOO_PARSER_END( parser );
+
+     ret = real->SetBounds( real, bounds->x, bounds->y, bounds->w, bounds->h );
+
+     return voodoo_manager_respond( manager, msg->header.serial,
+                                    ret, VOODOO_INSTANCE_NONE,
+                                    VMBT_NONE );
+}
+
+static DirectResult
+Dispatch_ResizeSurface( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                        VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DFBResult           ret;
+     VoodooMessageParser parser;
+     const DFBDimension *size;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_DATA( parser, size );
+     VOODOO_PARSER_END( parser );
+
+     ret = real->ResizeSurface( real, size->w, size->h );
+
+     return voodoo_manager_respond( manager, msg->header.serial,
+                                    ret, VOODOO_INSTANCE_NONE,
+                                    VMBT_NONE );
+}
+
+
 static DirectResult
 Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMessage *msg )
 {
@@ -885,9 +1074,18 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
 
           case IDIRECTFBWINDOW_METHOD_ID_AttachEventBuffer:
                return Dispatch_AttachEventBuffer( dispatcher, real, manager, msg );
+               
+          case IDIRECTFBWINDOW_METHOD_ID_DetachEventBuffer:
+               return Dispatch_DetachEventBuffer( dispatcher, real, manager, msg );
 
           case IDIRECTFBWINDOW_METHOD_ID_GetID:
                return Dispatch_GetID( dispatcher, real, manager, msg );
+               
+          case IDIRECTFBWINDOW_METHOD_ID_GetPosition:
+               return Dispatch_GetPosition( dispatcher, real, manager, msg );
+               
+          case IDIRECTFBWINDOW_METHOD_ID_GetSize:
+               return Dispatch_GetSize( dispatcher, real, manager, msg );
 
           case IDIRECTFBWINDOW_METHOD_ID_GetSurface:
                return Dispatch_GetSurface( dispatcher, real, manager, msg );
@@ -945,6 +1143,12 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
 
           case IDIRECTFBWINDOW_METHOD_ID_Destroy:
                return Dispatch_Destroy( dispatcher, real, manager, msg );
+               
+          case IDIRECTFBWINDOW_METHOD_ID_SetBounds:
+               return Dispatch_SetBounds( dispatcher, real, manager, msg );
+               
+          case IDIRECTFBWINDOW_METHOD_ID_ResizeSurface:
+               return Dispatch_ResizeSurface( dispatcher, real, manager, msg );
      }
 
      return DFB_NOSUCHMETHOD;
@@ -996,14 +1200,18 @@ Construct( IDirectFBWindow  *thiz,     /* Dispatcher interface */
      /* Initialize interface methods. */
      thiz->AddRef             = IDirectFBWindow_Dispatcher_AddRef;
      thiz->Release            = IDirectFBWindow_Dispatcher_Release;
-     thiz->CreateEventBuffer  = IDirectFBWindow_Dispatcher_CreateEventBuffer;
-     thiz->AttachEventBuffer  = IDirectFBWindow_Dispatcher_AttachEventBuffer;
-     thiz->EnableEvents       = IDirectFBWindow_Dispatcher_EnableEvents;
-     thiz->DisableEvents      = IDirectFBWindow_Dispatcher_DisableEvents;
      thiz->GetID              = IDirectFBWindow_Dispatcher_GetID;
      thiz->GetPosition        = IDirectFBWindow_Dispatcher_GetPosition;
      thiz->GetSize            = IDirectFBWindow_Dispatcher_GetSize;
+     thiz->CreateEventBuffer  = IDirectFBWindow_Dispatcher_CreateEventBuffer;
+     thiz->AttachEventBuffer  = IDirectFBWindow_Dispatcher_AttachEventBuffer;
+     thiz->DetachEventBuffer  = IDirectFBWindow_Dispatcher_DetachEventBuffer;
+     thiz->EnableEvents       = IDirectFBWindow_Dispatcher_EnableEvents;
+     thiz->DisableEvents      = IDirectFBWindow_Dispatcher_DisableEvents;
      thiz->GetSurface         = IDirectFBWindow_Dispatcher_GetSurface;
+     thiz->SetProperty        = IDirectFBWindow_Dispatcher_SetProperty;
+     thiz->GetProperty        = IDirectFBWindow_Dispatcher_GetProperty;
+     thiz->RemoveProperty     = IDirectFBWindow_Dispatcher_RemoveProperty;
      thiz->SetOptions         = IDirectFBWindow_Dispatcher_SetOptions;
      thiz->GetOptions         = IDirectFBWindow_Dispatcher_GetOptions;
      thiz->SetColorKey        = IDirectFBWindow_Dispatcher_SetColorKey;
@@ -1031,6 +1239,8 @@ Construct( IDirectFBWindow  *thiz,     /* Dispatcher interface */
      thiz->PutBelow           = IDirectFBWindow_Dispatcher_PutBelow;
      thiz->Close              = IDirectFBWindow_Dispatcher_Close;
      thiz->Destroy            = IDirectFBWindow_Dispatcher_Destroy;
+     thiz->SetBounds          = IDirectFBWindow_Dispatcher_SetBounds;
+     thiz->ResizeSurface      = IDirectFBWindow_Dispatcher_ResizeSurface;
 
      return DFB_OK;
 }
