@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -462,6 +463,7 @@ IDirectFB_Requestor_CreateEventBuffer( IDirectFB             *thiz,
      IDirectFBEventBuffer *buffer;
      IDirectFBEventBuffer *dispatcher;
      VoodooInstanceID      instance;
+     void                 *ptr;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
@@ -475,12 +477,14 @@ IDirectFB_Requestor_CreateEventBuffer( IDirectFB             *thiz,
 
      /* Create the dispatcher. */
      ret = voodoo_construct_dispatcher( data->manager, "IDirectFBEventBuffer", buffer,
-                                        data->instance, NULL, &instance, (void**) &dispatcher );
+                                        data->instance, NULL, &instance, &ptr );
      if (ret) {
           buffer->Release( buffer );
           return ret;
      }
 
+     dispatcher = ptr;
+
      /* Send the request including the instance ID of the dispatcher. */
      ret = voodoo_manager_request( data->manager, data->instance,
                                    IDIRECTFB_METHOD_ID_CreateEventBuffer, VREQ_NONE, NULL,
@@ -507,6 +511,7 @@ IDirectFB_Requestor_CreateInputEventBuffer( IDirectFB                   *thiz,
      IDirectFBEventBuffer *buffer;
      IDirectFBEventBuffer *dispatcher;
      VoodooInstanceID      instance;
+     void                 *ptr;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
@@ -520,12 +525,14 @@ IDirectFB_Requestor_CreateInputEventBuffer( IDirectFB                   *thiz,
 
      /* Create the dispatcher. */
      ret = voodoo_construct_dispatcher( data->manager, "IDirectFBEventBuffer", buffer,
-                                        data->instance, NULL, &instance, (void**) &dispatcher );
+                                        data->instance, NULL, &instance, &ptr );
      if (ret) {
           buffer->Release( buffer );
           return ret;
      }
 
+     dispatcher = ptr;
+
      /* Send the request including the instance ID of the dispatcher. */
      ret = voodoo_manager_request( data->manager, data->instance,
                                    IDIRECTFB_METHOD_ID_CreateInputEventBuffer, VREQ_NONE, NULL,
@@ -649,6 +656,7 @@ IDirectFB_Requestor_CreateDataBuffer( IDirectFB                       *thiz,
      IDirectFBDataBuffer *buffer;
      IDirectFBDataBuffer *dispatcher;
      VoodooInstanceID     instance;
+     void                *ptr;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
@@ -659,7 +667,7 @@ IDirectFB_Requestor_CreateDataBuffer( IDirectFB                       *thiz,
      if (!desc) {
           DIRECT_ALLOCATE_INTERFACE( buffer, IDirectFBDataBuffer );
 
-          ret = IDirectFBDataBuffer_Streamed_Construct( buffer );
+          ret = IDirectFBDataBuffer_Streamed_Construct( buffer, NULL );
      }
      else if (desc->flags & DBDESC_FILE) {
           if (!desc->file)
@@ -667,7 +675,7 @@ IDirectFB_Requestor_CreateDataBuffer( IDirectFB                       *thiz,
 
           DIRECT_ALLOCATE_INTERFACE( buffer, IDirectFBDataBuffer );
 
-          ret = IDirectFBDataBuffer_File_Construct( buffer, desc->file );
+          ret = IDirectFBDataBuffer_File_Construct( buffer, desc->file, NULL );
      }
      else if (desc->flags & DBDESC_MEMORY) {
           if (!desc->memory.data || !desc->memory.length)
@@ -677,7 +685,8 @@ IDirectFB_Requestor_CreateDataBuffer( IDirectFB                       *thiz,
 
           ret = IDirectFBDataBuffer_Memory_Construct( buffer,
                                                       desc->memory.data,
-                                                      desc->memory.length );
+                                                      desc->memory.length,
+                                                      NULL );
      }
      else
           return DFB_INVARG;
@@ -687,13 +696,13 @@ IDirectFB_Requestor_CreateDataBuffer( IDirectFB                       *thiz,
 
      /* Create the dispatcher. */
      ret = voodoo_construct_dispatcher( data->manager, "IDirectFBDataBuffer",
-                                        buffer, data->instance, NULL, &instance, (void**) &dispatcher );
+                                        buffer, data->instance, NULL, &instance, &ptr );
      if (ret) {
           buffer->Release( buffer );
           return ret;
      }
 
-
+     dispatcher = ptr;
 
      /* Send the request including the instance ID of the dispatcher. */
      ret = voodoo_manager_request( data->manager, data->instance,
