@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -197,6 +198,49 @@ IDirectFBSurface_Requestor_GetAccelerationMask( IDirectFBSurface    *thiz,
      return ret;
 }
 
+static DFBResult
+IDirectFBSurface_Requestor_GetPosition( IDirectFBSurface *thiz,
+                                        int              *x,
+                                        int              *y )
+{
+     DFBResult              ret;
+     VoodooResponseMessage *response;
+     VoodooMessageParser    parser;
+     const DFBPoint        *position;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+
+     if (!x && !y)
+          return DFB_INVARG;
+
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBSURFACE_METHOD_ID_GetPosition, 
+                                   VREQ_RESPOND, &response,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret) {
+          voodoo_manager_finish_request( data->manager, response );
+          return ret;
+     }
+
+     VOODOO_PARSER_BEGIN( parser, response );
+     VOODOO_PARSER_GET_DATA( parser, position );
+     VOODOO_PARSER_END( parser );
+
+     if (x)
+          *x = position->x;
+
+     if (y)
+          *y = position->y;
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return DFB_OK;
+}
+
 static DFBResult
 IDirectFBSurface_Requestor_GetSize( IDirectFBSurface *thiz,
                                     int              *width,
@@ -338,6 +382,17 @@ IDirectFBSurface_Requestor_SetPalette( IDirectFBSurface *thiz,
                                     VMBT_NONE );
 }
 
+static DFBResult
+IDirectFBSurface_Requestor_SetAlphaRamp( IDirectFBSurface *thiz,
+                                         u8 a0, u8 a1, u8 a2, u8 a3 )
+{
+     DIRECT_INTERFACE_GET_DATA( IDirectFBSurface_Requestor )
+     
+     D_UNIMPLEMENTED();
+     
+     return DFB_UNIMPLEMENTED;
+}     
+
 static DFBResult
 IDirectFBSurface_Requestor_Lock( IDirectFBSurface *thiz,
                                  DFBSurfaceLockFlags flags,
@@ -353,6 +408,20 @@ IDirectFBSurface_Requestor_Lock( IDirectFBSurface *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Requestor_GetFramebufferOffset( IDirectFBSurface *thiz,
+                                                 int              *offset )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+
+     if (!offset)
+          return DFB_INVARG;
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBSurface_Requestor_Unlock( IDirectFBSurface *thiz )
 {
@@ -415,7 +484,7 @@ IDirectFBSurface_Requestor_SetField( IDirectFBSurface    *thiz,
 
 static DFBResult
 IDirectFBSurface_Requestor_Clear( IDirectFBSurface *thiz,
-                                  __u8 r, __u8 g, __u8 b, __u8 a )
+                                  u8 r, u8 g, u8 b, u8 a )
 {
      DFBColor color = { a, r, g, b };
 
@@ -440,7 +509,7 @@ IDirectFBSurface_Requestor_SetClip( IDirectFBSurface *thiz, const DFBRegion *cli
 
 static DFBResult
 IDirectFBSurface_Requestor_SetColor( IDirectFBSurface *thiz,
-                                     __u8 r, __u8 g, __u8 b, __u8 a )
+                                     u8 r, u8 g, u8 b, u8 a )
 {
      DFBColor color = { a, r, g, b };
 
@@ -498,9 +567,9 @@ IDirectFBSurface_Requestor_SetPorterDuff( IDirectFBSurface         *thiz,
 
 static DFBResult
 IDirectFBSurface_Requestor_SetSrcColorKey( IDirectFBSurface *thiz,
-                                           __u8              r,
-                                           __u8              g,
-                                           __u8              b )
+                                           u8                r,
+                                           u8                g,
+                                           u8                b )
 {
      DFBColor color = { 0, r, g, b };
 
@@ -526,9 +595,9 @@ IDirectFBSurface_Requestor_SetSrcColorKeyIndex( IDirectFBSurface *thiz,
 
 static DFBResult
 IDirectFBSurface_Requestor_SetDstColorKey( IDirectFBSurface *thiz,
-                                           __u8              r,
-                                           __u8              g,
-                                           __u8              b )
+                                           u8                r,
+                                           u8                g,
+                                           u8                b )
 {
      DFBColor color = { 0, r, g, b };
 
@@ -620,6 +689,42 @@ IDirectFBSurface_Requestor_FillRectangle( IDirectFBSurface *thiz,
                                     VMBT_NONE );
 }
 
+static DFBResult
+IDirectFBSurface_Requestor_FillRectangles( IDirectFBSurface   *thiz,
+                                           const DFBRectangle *rects,
+                                           unsigned int        num_rects )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+
+     if (!rects || !num_rects)
+          return DFB_INVARG;
+
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBSURFACE_METHOD_ID_FillRectangles, VREQ_QUEUE, NULL,
+                                    VMBT_UINT, num_rects,
+                                    VMBT_DATA, num_rects * sizeof(DFBRectangle), rects,
+                                    VMBT_NONE );
+}
+
+static DFBResult
+IDirectFBSurface_Requestor_FillSpans( IDirectFBSurface *thiz,
+                                      int               y,
+                                      const DFBSpan    *spans,
+                                      unsigned int      num_spans )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+
+     if (!spans || !num_spans)
+          return DFB_INVARG;
+
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBSURFACE_METHOD_ID_FillSpans, VREQ_QUEUE, NULL,
+                                    VMBT_INT, y,
+                                    VMBT_UINT, num_spans,
+                                    VMBT_DATA, num_spans * sizeof(DFBSpan), spans,
+                                    VMBT_NONE );
+}
+
 static DFBResult
 IDirectFBSurface_Requestor_DrawLine( IDirectFBSurface *thiz,
                                      int x1, int y1, int x2, int y2 )
@@ -888,6 +993,18 @@ IDirectFBSurface_Requestor_DrawGlyph( IDirectFBSurface    *thiz,
                                     VMBT_NONE );
 }
 
+static DFBResult
+IDirectFBSurface_Requestor_SetEncoding( IDirectFBSurface  *thiz,
+                                        DFBTextEncodingID  encoding )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBSURFACE_METHOD_ID_SetEncoding, VREQ_QUEUE, NULL,
+                                    VMBT_UINT, encoding,
+                                    VMBT_NONE );
+}
+
 static DFBResult
 IDirectFBSurface_Requestor_GetSubSurface( IDirectFBSurface    *thiz,
                                           const DFBRectangle  *rect,
@@ -950,6 +1067,42 @@ IDirectFBSurface_Requestor_Dump( IDirectFBSurface   *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Requestor_DisableAcceleration( IDirectFBSurface   *thiz,
+                                                DFBAccelerationMask mask )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+     
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBSURFACE_METHOD_ID_DisableAcceleration,
+                                    VREQ_QUEUE, NULL,
+                                    VMBT_UINT, mask,
+                                    VMBT_NONE );
+}
+
+static DFBResult
+IDirectFBSurface_Requestor_ReleaseSource( IDirectFBSurface *thiz )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+     
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBSURFACE_METHOD_ID_ReleaseSource, VREQ_QUEUE, NULL,
+                                    VMBT_NONE );
+}
+
+static DFBResult
+IDirectFBSurface_Requestor_SetIndexTranslation( IDirectFBSurface *thiz,
+                                                const int        *indices,
+                                                int               num_indices )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+     
+     D_UNIMPLEMENTED();
+     
+     return DFB_UNIMPLEMENTED;
+}
+
+
 /**************************************************************************************************/
 
 static DFBResult
@@ -975,6 +1128,7 @@ Construct( IDirectFBSurface *thiz,
      thiz->Release = IDirectFBSurface_Requestor_Release;
 
      thiz->GetCapabilities = IDirectFBSurface_Requestor_GetCapabilities;
+     thiz->GetPosition = IDirectFBSurface_Requestor_GetPosition;
      thiz->GetSize = IDirectFBSurface_Requestor_GetSize;
      thiz->GetVisibleRectangle = IDirectFBSurface_Requestor_GetVisibleRectangle;
      thiz->GetPixelFormat = IDirectFBSurface_Requestor_GetPixelFormat;
@@ -982,8 +1136,10 @@ Construct( IDirectFBSurface *thiz,
 
      thiz->GetPalette = IDirectFBSurface_Requestor_GetPalette;
      thiz->SetPalette = IDirectFBSurface_Requestor_SetPalette;
+     thiz->SetAlphaRamp = IDirectFBSurface_Requestor_SetAlphaRamp;
 
      thiz->Lock = IDirectFBSurface_Requestor_Lock;
+     thiz->GetFramebufferOffset = IDirectFBSurface_Requestor_GetFramebufferOffset;
      thiz->Unlock = IDirectFBSurface_Requestor_Unlock;
      thiz->Flip = IDirectFBSurface_Requestor_Flip;
      thiz->SetField = IDirectFBSurface_Requestor_SetField;
@@ -1009,6 +1165,8 @@ Construct( IDirectFBSurface *thiz,
 
      thiz->SetDrawingFlags = IDirectFBSurface_Requestor_SetDrawingFlags;
      thiz->FillRectangle = IDirectFBSurface_Requestor_FillRectangle;
+     thiz->FillRectangles = IDirectFBSurface_Requestor_FillRectangles;
+     thiz->FillSpans = IDirectFBSurface_Requestor_FillSpans;
      thiz->DrawLine = IDirectFBSurface_Requestor_DrawLine;
      thiz->DrawLines = IDirectFBSurface_Requestor_DrawLines;
      thiz->DrawRectangle = IDirectFBSurface_Requestor_DrawRectangle;
@@ -1018,12 +1176,19 @@ Construct( IDirectFBSurface *thiz,
      thiz->GetFont = IDirectFBSurface_Requestor_GetFont;
      thiz->DrawString = IDirectFBSurface_Requestor_DrawString;
      thiz->DrawGlyph = IDirectFBSurface_Requestor_DrawGlyph;
+     thiz->SetEncoding = IDirectFBSurface_Requestor_SetEncoding;
 
      thiz->GetSubSurface = IDirectFBSurface_Requestor_GetSubSurface;
 
      thiz->GetGL = IDirectFBSurface_Requestor_GetGL;
 
      thiz->Dump = IDirectFBSurface_Requestor_Dump;
+     
+     thiz->DisableAcceleration = IDirectFBSurface_Requestor_DisableAcceleration;
+     
+     thiz->ReleaseSource = IDirectFBSurface_Requestor_ReleaseSource;
+     
+     thiz->SetIndexTranslation = IDirectFBSurface_Requestor_SetIndexTranslation;
 
      return DFB_OK;
 }
