@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -101,6 +102,7 @@ IDirectFBWindow_Requestor_CreateEventBuffer( IDirectFBWindow       *thiz,
      IDirectFBEventBuffer  *dispatcher;
      VoodooInstanceID       instance;
      VoodooResponseMessage *response;
+     void                  *ptr;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
@@ -114,12 +116,14 @@ IDirectFBWindow_Requestor_CreateEventBuffer( IDirectFBWindow       *thiz,
 
      /* Create the dispatcher. */
      ret = voodoo_construct_dispatcher( data->manager, "IDirectFBEventBuffer",
-                                        buffer, data->instance, NULL, &instance, (void**) &dispatcher );
+                                        buffer, data->instance, NULL, &instance, &ptr );
      if (ret) {
           buffer->Release( buffer );
           return ret;
      }
 
+     dispatcher = ptr;
+
      /* Send the request including the instance ID of the dispatcher. */
      ret = voodoo_manager_request( data->manager, data->instance,
                                    IDIRECTFBWINDOW_METHOD_ID_CreateEventBuffer, VREQ_RESPOND, &response,
@@ -174,6 +178,36 @@ IDirectFBWindow_Requestor_AttachEventBuffer( IDirectFBWindow      *thiz,
      return ret;
 }
 
+static DFBResult
+IDirectFBWindow_Requestor_DetachEventBuffer( IDirectFBWindow      *thiz,
+                                             IDirectFBEventBuffer *buffer )
+{
+     DFBResult                             ret;
+     IDirectFBEventBuffer_Dispatcher_data *buffer_data;
+     VoodooResponseMessage                *response;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
+
+     if (!buffer)
+          return DFB_INVARG;
+
+     DIRECT_INTERFACE_GET_DATA_FROM( buffer, buffer_data, IDirectFBEventBuffer_Dispatcher );
+
+     /* Send the request including the instance ID of the dispatcher. */
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBWINDOW_METHOD_ID_DetachEventBuffer, VREQ_RESPOND, &response,
+                                   VMBT_ID, buffer_data->self,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return ret;
+}
+
 static DFBResult
 IDirectFBWindow_Requestor_EnableEvents( IDirectFBWindow       *thiz,
                                         DFBWindowEventType     mask )
@@ -238,11 +272,41 @@ IDirectFBWindow_Requestor_GetPosition( IDirectFBWindow *thiz,
                                        int             *x,
                                        int             *y )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     VoodooMessageParser    parser;
+     const DFBPoint        *position;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!x && !y)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBWINDOW_METHOD_ID_GetPosition, VREQ_RESPOND, &response,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret) {
+          voodoo_manager_finish_request( data->manager, response );
+          return ret;
+     }
+
+     VOODOO_PARSER_BEGIN( parser, response );
+     VOODOO_PARSER_GET_DATA( parser, position );
+     VOODOO_PARSER_END( parser );
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     if (x)
+          *x = position->x;
+
+     if (y)
+          *y = position->y;
+
+     return ret;
 }
 
 static DFBResult
@@ -250,11 +314,41 @@ IDirectFBWindow_Requestor_GetSize( IDirectFBWindow *thiz,
                                    int             *width,
                                    int             *height )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     VoodooMessageParser    parser;
+     const DFBDimension    *size;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!width && !height)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBWINDOW_METHOD_ID_GetSize, VREQ_RESPOND, &response,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret) {
+          voodoo_manager_finish_request( data->manager, response );
+          return ret;
+     }
+
+     VOODOO_PARSER_BEGIN( parser, response );
+     VOODOO_PARSER_GET_DATA( parser, size );
+     VOODOO_PARSER_END( parser );
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     if (width)
+          *width = size->w;
+
+     if (height)
+          *height = size->h;
+
+     return ret;
 }
 
 static DFBResult
@@ -288,6 +382,44 @@ IDirectFBWindow_Requestor_GetSurface( IDirectFBWindow   *thiz,
      return ret;
 }
 
+static DFBResult
+IDirectFBWindow_Requestor_SetProperty( IDirectFBWindow  *thiz,
+                                       const char       *key,
+                                       void             *value,
+                                       void            **old_value )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_GetProperty( IDirectFBWindow  *thiz,
+                                       const char       *key,
+                                       void            **value )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_RemoveProperty( IDirectFBWindow  *thiz,
+                                          const char       *key,
+                                          void            **value )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
+
 static DFBResult
 IDirectFBWindow_Requestor_SetOptions( IDirectFBWindow  *thiz,
                                       DFBWindowOptions  options )
@@ -342,9 +474,9 @@ IDirectFBWindow_Requestor_GetOptions( IDirectFBWindow  *thiz,
 
 static DFBResult
 IDirectFBWindow_Requestor_SetColorKey( IDirectFBWindow *thiz,
-                                       __u8             r,
-                                       __u8             g,
-                                       __u8             b )
+                                       u8               r,
+                                       u8               g,
+                                       u8               b )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
@@ -380,7 +512,7 @@ IDirectFBWindow_Requestor_SetOpaqueRegion( IDirectFBWindow *thiz,
 
 static DFBResult
 IDirectFBWindow_Requestor_SetOpacity( IDirectFBWindow *thiz,
-                                      __u8             opacity )
+                                      u8               opacity )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
@@ -392,12 +524,12 @@ IDirectFBWindow_Requestor_SetOpacity( IDirectFBWindow *thiz,
 
 static DFBResult
 IDirectFBWindow_Requestor_GetOpacity( IDirectFBWindow *thiz,
-                                      __u8            *ret_opacity )
+                                      u8              *ret_opacity )
 {
      DirectResult           ret;
      VoodooResponseMessage *response;
      VoodooMessageParser    parser;
-     __u8                   opacity;
+     u8                     opacity;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
@@ -723,6 +855,55 @@ IDirectFBWindow_Requestor_Destroy( IDirectFBWindow *thiz )
      return ret;
 }
 
+static DFBResult
+IDirectFBWindow_Requestor_SetBounds( IDirectFBWindow *thiz,
+                                     int x, int y, int w, int h )
+{
+     DirectResult           ret;
+     DFBRectangle           bounds = { x, y, w, h };
+     VoodooResponseMessage *response;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
+
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBWINDOW_METHOD_ID_SetBounds, VREQ_RESPOND, &response,
+                                   VMBT_DATA, sizeof(bounds), &bounds,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return ret;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_ResizeSurface( IDirectFBWindow *thiz,
+                                         int              width,
+                                         int              height )
+{
+     DirectResult           ret;
+     DFBDimension           size = { width, height };
+     VoodooResponseMessage *response;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
+
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBWINDOW_METHOD_ID_ResizeSurface, VREQ_RESPOND, &response,
+                                   VMBT_DATA, sizeof(size), &size,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return ret;
+}
+
 /**************************************************************************************************/
 
 static DFBResult
@@ -746,14 +927,18 @@ Construct( IDirectFBWindow  *thiz,
 
      thiz->AddRef             = IDirectFBWindow_Requestor_AddRef;
      thiz->Release            = IDirectFBWindow_Requestor_Release;
-     thiz->CreateEventBuffer  = IDirectFBWindow_Requestor_CreateEventBuffer;
-     thiz->AttachEventBuffer  = IDirectFBWindow_Requestor_AttachEventBuffer;
-     thiz->EnableEvents       = IDirectFBWindow_Requestor_EnableEvents;
-     thiz->DisableEvents      = IDirectFBWindow_Requestor_DisableEvents;
      thiz->GetID              = IDirectFBWindow_Requestor_GetID;
      thiz->GetPosition        = IDirectFBWindow_Requestor_GetPosition;
      thiz->GetSize            = IDirectFBWindow_Requestor_GetSize;
+     thiz->CreateEventBuffer  = IDirectFBWindow_Requestor_CreateEventBuffer;
+     thiz->AttachEventBuffer  = IDirectFBWindow_Requestor_AttachEventBuffer;
+     thiz->DetachEventBuffer  = IDirectFBWindow_Requestor_DetachEventBuffer;
+     thiz->EnableEvents       = IDirectFBWindow_Requestor_EnableEvents;
+     thiz->DisableEvents      = IDirectFBWindow_Requestor_DisableEvents;
      thiz->GetSurface         = IDirectFBWindow_Requestor_GetSurface;
+     thiz->SetProperty        = IDirectFBWindow_Requestor_SetProperty;
+     thiz->GetProperty        = IDirectFBWindow_Requestor_GetProperty;
+     thiz->RemoveProperty     = IDirectFBWindow_Requestor_RemoveProperty;
      thiz->SetOptions         = IDirectFBWindow_Requestor_SetOptions;
      thiz->GetOptions         = IDirectFBWindow_Requestor_GetOptions;
      thiz->SetColorKey        = IDirectFBWindow_Requestor_SetColorKey;
@@ -781,6 +966,8 @@ Construct( IDirectFBWindow  *thiz,
      thiz->PutBelow           = IDirectFBWindow_Requestor_PutBelow;
      thiz->Close              = IDirectFBWindow_Requestor_Close;
      thiz->Destroy            = IDirectFBWindow_Requestor_Destroy;
+     thiz->SetBounds          = IDirectFBWindow_Requestor_SetBounds;
+     thiz->ResizeSurface      = IDirectFBWindow_Requestor_ResizeSurface;
 
      return DFB_OK;
 }
