@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -35,10 +36,10 @@
 void          dfb_colorhash_attach     (CorePalette *palette);
 void          dfb_colorhash_detach     (CorePalette *palette);
 unsigned int  dfb_colorhash_lookup     (CorePalette *palette,
-                                        __u8         r,
-                                        __u8         g,
-                                        __u8         b,
-                                        __u8         a);
+                                        u8           r,
+                                        u8           g,
+                                        u8           b,
+                                        u8           a);
 void          dfb_colorhash_invalidate (CorePalette *palette);
 
 #endif
