@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -66,6 +67,7 @@
 #include <direct/util.h>
 
 #include <fusion/build.h>
+#include <fusion/conf.h>
 
 #include <misc/conf.h>
 #include <misc/util.h>
@@ -98,6 +100,9 @@ struct _CoreCleanup {
 struct __DFB_CoreDFBShared {
      int                  magic;
 
+     FusionSkirmish       lock;
+     bool                 active;
+
      FusionObjectPool    *layer_context_pool;
      FusionObjectPool    *layer_region_pool;
      FusionObjectPool    *palette_pool;
@@ -121,6 +126,7 @@ struct __DFB_CoreDFB {
      CoreDFBShared           *shared;
 
      bool                     master;
+     bool                     suspended;
 
      DirectLink              *cleanups;
 
@@ -172,7 +178,7 @@ static CorePart *core_parts[] = {
 
 #define NUM_CORE_PARTS ((int)(sizeof(core_parts)/sizeof(CorePart*)))
 
-#ifdef DFB_DYNAMIC_LINKING
+#if defined(DFB_DYNAMIC_LINKING) && defined(SOPATH)
 /*
  * the library handle for dlopen'ing ourselves
  */
@@ -193,7 +199,8 @@ dfb_core_create( CoreDFB **ret_core )
 #if FUSION_BUILD_MULTI
      char     buf[16];
 #endif
-     CoreDFB *core = NULL;
+     CoreDFB       *core   = NULL;
+     CoreDFBShared *shared = NULL;
 
      D_ASSERT( ret_core != NULL );
      D_ASSERT( dfb_config != NULL );
@@ -225,7 +232,7 @@ dfb_core_create( CoreDFB **ret_core )
              DIRECT_BUILD_TRACE ? "[ TRACE ]" : "" );
 
 
-#ifdef DFB_DYNAMIC_LINKING
+#if defined(DFB_DYNAMIC_LINKING) && defined(SOPATH)
      if (!dfb_lib_handle)
 #ifdef RTLD_GLOBAL
           dfb_lib_handle = dlopen(SOPATH, RTLD_GLOBAL|RTLD_LAZY);
@@ -257,8 +264,6 @@ dfb_core_create( CoreDFB **ret_core )
 
      direct_find_best_memcpy();
 
-     setpgid( 0, 0 );
-
      D_MAGIC_SET( core, CoreDFB );
 
      core_dfb = core;
@@ -292,6 +297,8 @@ dfb_core_create( CoreDFB **ret_core )
           goto error;
      }
 
+     shared = core->shared;
+     D_MAGIC_ASSERT( shared, CoreDFBShared );
 
      if (dfb_config->block_all_signals)
           direct_signals_block_all();
@@ -300,6 +307,21 @@ dfb_core_create( CoreDFB **ret_core )
           atexit( dfb_core_deinit_check );
 
 
+     fusion_skirmish_prevail( &shared->lock );
+
+     if (!core->master) {
+          while (!shared->active) {
+               fusion_skirmish_dismiss( &shared->lock );
+               
+               usleep( 50000 );
+               
+               fusion_skirmish_prevail( &shared->lock );
+          }
+     }
+
+     fusion_skirmish_dismiss( &shared->lock );
+
+
      *ret_core = core;
 
      pthread_mutex_unlock( &core_dfb_lock );
@@ -342,11 +364,13 @@ dfb_core_destroy( CoreDFB *core, bool emergency )
 
      D_DEBUG_AT( DirectFB_Core, "%s...\n", __FUNCTION__ );
 
-     pthread_mutex_lock( &core_dfb_lock );
+     if (!emergency) {
+          pthread_mutex_lock( &core_dfb_lock );
 
-     if (!emergency && --core->refs) {
-          pthread_mutex_unlock( &core_dfb_lock );
-          return DFB_OK;
+          if (--core->refs) {
+               pthread_mutex_unlock( &core_dfb_lock );
+               return DFB_OK;
+          }
      }
 
      direct_signal_handler_remove( core->signal_handler );
@@ -373,16 +397,19 @@ dfb_core_destroy( CoreDFB *core, bool emergency )
 
      fusion_exit( core->world, emergency );
 
-     direct_thread_remove_init_handler( core->init_handler );
+     if (!emergency)
+          direct_thread_remove_init_handler( core->init_handler );
 
      D_MAGIC_CLEAR( core );
 
      D_FREE( core );
      core_dfb = NULL;
 
-     pthread_mutex_unlock( &core_dfb_lock );
+     if (!emergency) {
+          pthread_mutex_unlock( &core_dfb_lock );
 
-     direct_shutdown();
+          direct_shutdown();
+     }
 
      return DFB_OK;
 }
@@ -558,6 +585,23 @@ dfb_core_is_master( CoreDFB *core )
      return core->master;
 }
 
+void
+dfb_core_activate( CoreDFB *core )
+{
+     CoreDFBShared *shared;
+
+     D_MAGIC_ASSERT( core, CoreDFB );
+
+     shared = core->shared;
+     D_MAGIC_ASSERT( shared, CoreDFBShared );
+
+     fusion_skirmish_prevail( &shared->lock );
+
+     shared->active = true;
+
+     fusion_skirmish_dismiss( &shared->lock );
+}
+
 FusionWorld *
 dfb_core_world( CoreDFB *core )
 {
@@ -637,23 +681,37 @@ dfb_core_suspend( CoreDFB *core )
      if (!core->master)
           return DFB_ACCESSDENIED;
 
+     if (core->suspended)
+          return DFB_BUSY;
+
      ret = dfb_core_input.Suspend( core );
      if (ret)
-          return ret;
+          goto error_input;
 
      ret = dfb_core_layers.Suspend( core );
      if (ret)
-          return ret;
+          goto error_layers;
 
      ret = dfb_core_screens.Suspend( core );
      if (ret)
-          return ret;
+          goto error_screens;
 
      ret = dfb_core_gfxcard.Suspend( core );
      if (ret)
-          return ret;
+          goto error_gfxcard;
+
+     core->suspended = true;
 
      return DFB_OK;
+
+error_gfxcard:
+     dfb_core_screens.Resume( core );
+error_screens:
+     dfb_core_layers.Resume( core );
+error_layers:
+     dfb_core_input.Resume( core );
+error_input:
+     return ret;
 }
 
 DFBResult
@@ -671,23 +729,37 @@ dfb_core_resume( CoreDFB *core )
      if (!core->master)
           return DFB_ACCESSDENIED;
 
-     ret = dfb_core_input.Resume( core );
-     if (ret)
-          return ret;
+     if (!core->suspended)
+          return DFB_BUSY;
 
      ret = dfb_core_gfxcard.Resume( core );
      if (ret)
-          return ret;
+          goto error_gfxcard;
 
      ret = dfb_core_screens.Resume( core );
      if (ret)
-          return ret;
+          goto error_screens;
 
      ret = dfb_core_layers.Resume( core );
      if (ret)
-          return ret;
+          goto error_layers;
+
+     ret = dfb_core_input.Resume( core );
+     if (ret)
+          goto error_input;
+
+     core->suspended = false;
 
      return DFB_OK;
+
+error_input:
+     dfb_core_layers.Suspend( core );
+error_layers:
+     dfb_core_screens.Suspend( core );
+error_screens:
+     dfb_core_gfxcard.Suspend( core );
+error_gfxcard:
+     return ret;
 }
 
 CoreCleanup *
@@ -789,7 +861,6 @@ dfb_core_signal_handler( int   num,
 static int
 dfb_core_shutdown( CoreDFB *core, bool emergency )
 {
-     int            i;
      CoreDFBShared *shared;
 
      D_MAGIC_ASSERT( core, CoreDFB );
@@ -798,17 +869,32 @@ dfb_core_shutdown( CoreDFB *core, bool emergency )
 
      D_MAGIC_ASSERT( shared, CoreDFBShared );
 
-     if (!emergency) {
-          fusion_object_pool_destroy( shared->window_pool, core->world );
-          fusion_object_pool_destroy( shared->layer_region_pool, core->world );
-          fusion_object_pool_destroy( shared->layer_context_pool, core->world );
-          fusion_object_pool_destroy( shared->surface_pool, core->world );
-          fusion_object_pool_destroy( shared->palette_pool, core->world );
-     }
+     /* Destroy layer context and region objects. */
+     fusion_object_pool_destroy( shared->layer_region_pool, core->world );
+     fusion_object_pool_destroy( shared->layer_context_pool, core->world );
 
-     for (i=NUM_CORE_PARTS-1; i>=0; i--)
-          dfb_core_part_shutdown( core, core_parts[i], emergency );
+     /* Shutdown WM core. */
+     dfb_core_part_shutdown( core, &dfb_core_wm, emergency );
 
+     /* Destroy window objects. */
+     fusion_object_pool_destroy( shared->window_pool, core->world );
+
+     /* Shutdown layer core. */
+     dfb_core_part_shutdown( core, &dfb_core_layers, emergency );
+     dfb_core_part_shutdown( core, &dfb_core_screens, emergency );
+
+     /* Destroy surface and palette objects. */
+     fusion_object_pool_destroy( shared->surface_pool, core->world );
+     fusion_object_pool_destroy( shared->palette_pool, core->world );
+
+     /* Destroy remaining core parts. */
+     dfb_core_part_shutdown( core, &dfb_core_gfxcard, emergency );
+     dfb_core_part_shutdown( core, &dfb_core_input, emergency );
+     dfb_core_part_shutdown( core, &dfb_core_system, emergency );
+     dfb_core_part_shutdown( core, &dfb_core_colorhash, emergency );
+     dfb_core_part_shutdown( core, &dfb_core_clipboard, emergency );
+
+     /* Destroy shared memory pool for surface data. */
      fusion_shm_pool_destroy( core->world, shared->shmpool_data );
 
      return 0;
@@ -828,7 +914,7 @@ dfb_core_initialize( CoreDFB *core )
      D_MAGIC_ASSERT( shared, CoreDFBShared );
 
      ret = fusion_shm_pool_create( core->world, "DirectFB Surface Pool", 0x8000000,
-                                   direct_config->debug, &shared->shmpool_data );
+                                   fusion_config->debugshm, &shared->shmpool_data );
      if (ret)
           return ret;
 
@@ -899,7 +985,7 @@ dfb_core_arena_initialize( FusionArena *arena,
 
      /* Create the shared memory pool first! */
      ret = fusion_shm_pool_create( core->world, "DirectFB Main Pool", 0x400000,
-                                   direct_config->debug, &pool );
+                                   fusion_config->debugshm, &pool );
      if (ret)
           return ret;
 
@@ -926,6 +1012,8 @@ dfb_core_arena_initialize( FusionArena *arena,
           return ret;
      }
 
+     fusion_skirmish_init( &shared->lock, "DirectFB Core", core->world );
+
      /* Register shared data. */
      fusion_arena_add_shared_field( arena, "Core/Shared", shared );
 
@@ -959,8 +1047,8 @@ dfb_core_arena_shutdown( FusionArena *arena,
 
      /* Shutdown. */
      ret = dfb_core_shutdown( core, emergency );
-     if (ret)
-          return ret;
+
+     fusion_skirmish_destroy( &shared->lock );
 
      D_MAGIC_CLEAR( shared );
 
@@ -968,7 +1056,7 @@ dfb_core_arena_shutdown( FusionArena *arena,
 
      fusion_shm_pool_destroy( core->world, pool );
 
-     return DFB_OK;
+     return ret;
 }
 
 static int
