@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -66,6 +67,11 @@ typedef struct {
      unsigned int            surface_max_power_of_two_pixelpitch;
      unsigned int            surface_max_power_of_two_bytepitch;
      unsigned int            surface_max_power_of_two_height;
+
+     DFBDimension            dst_min;
+     DFBDimension            dst_max;
+     DFBDimension            src_min;
+     DFBDimension            src_max;
 } CardLimitations;
 
 DECLARE_MODULE_DIRECTORY( dfb_graphics_drivers );
@@ -73,7 +79,7 @@ DECLARE_MODULE_DIRECTORY( dfb_graphics_drivers );
 /*
  * Increase this number when changes result in binary incompatibility!
  */
-#define DFB_GRAPHICS_DRIVER_ABI_VERSION          32
+#define DFB_GRAPHICS_DRIVER_ABI_VERSION          34
 
 #define DFB_GRAPHICS_DRIVER_INFO_NAME_LENGTH     40
 #define DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH   60
@@ -97,7 +103,7 @@ typedef struct {
 
      char               vendor[DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH];
                                 /* Vendor (or author) of the driver,
-                                   e.g. 'convergence' or 'Denis Oliver Kropp' */
+                                   e.g. 'directfb.org' or 'Denis Oliver Kropp' */
 
      char               url[DFB_GRAPHICS_DRIVER_INFO_URL_LENGTH];
                                 /* URL for driver updates,
@@ -152,7 +158,13 @@ typedef struct _GraphicsDeviceFuncs {
       *
       * It's also called before entering the OpenGL state (e.g. DRI driver).
       */
-     void (*EngineSync)( void *driver_data, void *device_data );
+     DFBResult (*EngineSync)( void *driver_data, void *device_data );
+     
+     /*
+      * Called during dfb_gfxcard_lock() to notify the driver that
+      * the current rendering state is no longer valid.
+      */
+     void (*InvalidateState)( void *driver_data, void *device_data );
 
      /*
       * after the video memory has been written to by the CPU (e.g. modification
@@ -169,7 +181,7 @@ typedef struct _GraphicsDeviceFuncs {
      /*
       * Called before a software access to a video surface buffer.
       */
-     void (*SurfaceEnter)( void *driver_data, void *device_data, 
+     void (*SurfaceEnter)( void *driver_data, void *device_data,
                            SurfaceBuffer *buffer, DFBSurfaceLockFlags flags );
 
      /*
@@ -188,7 +200,7 @@ typedef struct _GraphicsDeviceFuncs {
      /*
       * Makes sure that graphics hardware has finished the specified operation.
       */
-     void (*WaitSerial)( void *driver_data, void *device_data, const CoreGraphicsSerial *serial );
+     DFBResult (*WaitSerial)( void *driver_data, void *device_data, const CoreGraphicsSerial *serial );
 
      /*
       * emit any buffered commands, i.e. trigger processing
@@ -276,8 +288,8 @@ typedef enum {
 } GraphicsDeviceLockFlags;
 
 DFBResult dfb_gfxcard_lock( GraphicsDeviceLockFlags flags );
-void dfb_gfxcard_unlock();
-void dfb_gfxcard_holdup();
+void dfb_gfxcard_unlock( void );
+void dfb_gfxcard_holdup( void );
 
 bool dfb_gfxcard_state_check( CardState *state, DFBAccelerationMask accel );
 //bool dfb_gfxcard_state_acquire( CardState *state, DFBAccelerationMask accel );
@@ -312,24 +324,25 @@ void dfb_gfxcard_texture_triangles( DFBVertex *vertices, int num,
                                     DFBTriangleFormation formation,
                                     CardState *state );
 
-void dfb_gfxcard_drawstring( const __u8 *text, int bytes, int x, int y,
+void dfb_gfxcard_drawstring( const u8 *text, int bytes,
+                             DFBTextEncodingID encoding, int x, int y,
                              CoreFont *font, CardState *state );
 void dfb_gfxcard_drawstring_check_state( CoreFont *font, CardState *state );
-void dfb_gfxcard_drawglyph( unichar index, int x, int y,
+void dfb_gfxcard_drawglyph( unsigned int index, int x, int y,
                             CoreFont *font, CardState *state );
 
-void dfb_gfxcard_sync();
-void dfb_gfxcard_invalidate_state();
-void dfb_gfxcard_wait_serial( const CoreGraphicsSerial *serial );
-void dfb_gfxcard_flush_texture_cache();
-void dfb_gfxcard_flush_read_cache();
-void dfb_gfxcard_after_set_var();
+DFBResult dfb_gfxcard_sync( void );
+void dfb_gfxcard_invalidate_state( void );
+DFBResult dfb_gfxcard_wait_serial( const CoreGraphicsSerial *serial );
+void dfb_gfxcard_flush_texture_cache( void );
+void dfb_gfxcard_flush_read_cache( void );
+void dfb_gfxcard_after_set_var( void );
 void dfb_gfxcard_surface_enter( SurfaceBuffer *buffer, DFBSurfaceLockFlags flags );
 void dfb_gfxcard_surface_leave( SurfaceBuffer *buffer );
 
 DFBResult dfb_gfxcard_adjust_heap_offset( int offset );
 
-SurfaceManager *dfb_gfxcard_surface_manager   ();
+SurfaceManager *dfb_gfxcard_surface_manager   ( void );
 void            dfb_gfxcard_get_capabilities  ( CardCapabilities   *ret_caps );
 void            dfb_gfxcard_get_device_info   ( GraphicsDeviceInfo *ret_info );
 void            dfb_gfxcard_get_driver_info   ( GraphicsDriverInfo *ret_info );
@@ -339,8 +352,10 @@ int             dfb_gfxcard_reserve_memory    ( GraphicsDevice      *device,
 int             dfb_gfxcard_reserve_auxmemory ( GraphicsDevice      *device,
                                                 unsigned int         size );
 
-unsigned int    dfb_gfxcard_memory_length     ();
-unsigned int    dfb_gfxcard_auxmemory_length  ();
+unsigned int    dfb_gfxcard_memory_length     ( void );
+unsigned int    dfb_gfxcard_auxmemory_length  ( void );
+
+void           *dfb_gfxcard_get_device_data   ( void );
 
 /*
  * Graphics drivers call this function to get access to MMIO regions.
@@ -369,13 +384,19 @@ int dfb_gfxcard_get_accelerator( GraphicsDevice *device );
 
 unsigned long  dfb_gfxcard_memory_physical( GraphicsDevice *device,
                                             unsigned int    offset );
-void          *dfb_gfxcard_memory_virtual ( GraphicsDevice *device, 
+void          *dfb_gfxcard_memory_virtual ( GraphicsDevice *device,
                                             unsigned int    offset );
 
 unsigned long  dfb_gfxcard_auxmemory_physical( GraphicsDevice *device,
                                                unsigned int    offset );
-void          *dfb_gfxcard_auxmemory_virtual ( GraphicsDevice *device, 
+void          *dfb_gfxcard_auxmemory_virtual ( GraphicsDevice *device,
                                                unsigned int    offset );
 
+
+/* Hook for registering additional screen(s) and layer(s) in app or lib initializing DirectFB. */
+extern void (*__DFB_CoreRegisterHook)( CoreDFB *core, CoreGraphicsDevice *device, void *ctx );
+extern void  *__DFB_CoreRegisterHookCtx;
+
+
 #endif
 
