@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -84,6 +85,10 @@ DEFINE_MODULE_DIRECTORY( dfb_input_modules, "inputdrivers",
 
 /**********************************************************************************************************************/
 
+typedef enum {
+     CIDC_RELOAD_KEYMAP
+} CoreInputDeviceCommand;
+
 typedef struct {
      DirectLink               link;
 
@@ -122,6 +127,7 @@ typedef struct {
      DFBInputDeviceButtonMask     buttons;
 
      DFBInputDeviceKeyIdentifier  last_key;      /* last key pressed */
+     DFBInputDeviceKeySymbol      last_symbol;   /* last symbol pressed */
      bool                         first_press;   /* first press of key */
 
      FusionReactor               *reactor;       /* event dispatcher */
@@ -139,6 +145,8 @@ struct __DFB_CoreInputDevice {
 
      InputDriver        *driver;
      void               *driver_data;
+
+     CoreDFB            *core;
 };
 
 typedef struct {
@@ -157,9 +165,98 @@ DFB_CORE_PART( input, 0, sizeof(CoreInput) )
 
 /**********************************************************************************************************************/
 
-typedef enum {
-     CIDC_RELOAD_KEYMAP
-} CoreInputDeviceCommand;
+typedef struct {
+     DFBInputDeviceKeySymbol      target;
+     DFBInputDeviceKeySymbol      result;
+} DeadKeyCombo;
+
+typedef struct {
+     DFBInputDeviceKeySymbol      deadkey;
+     const DeadKeyCombo          *combos;
+} DeadKeyMap;
+
+/**********************************************************************************************************************/
+
+static const DeadKeyCombo combos_grave[] = {         
+     { DIKS_SPACE,     (unsigned char) '`' },        
+     { DIKS_SMALL_A,   (unsigned char) '' },        
+     { DIKS_SMALL_E,   (unsigned char) '' },        
+     { DIKS_SMALL_I,   (unsigned char) '' },        
+     { DIKS_SMALL_O,   (unsigned char) '' },        
+     { DIKS_SMALL_U,   (unsigned char) '' },        
+     { DIKS_CAPITAL_A, (unsigned char) '' },        
+     { DIKS_CAPITAL_E, (unsigned char) '' },        
+     { DIKS_CAPITAL_I, (unsigned char) '' },        
+     { DIKS_CAPITAL_O, (unsigned char) '' },        
+     { DIKS_CAPITAL_U, (unsigned char) '' },        
+     { 0, 0 }                                        
+};                                                   
+                                                     
+static const DeadKeyCombo combos_acute[] = {         
+     { DIKS_SPACE,     (unsigned char) '\'' },       
+     { DIKS_SMALL_A,   (unsigned char) '' },        
+     { DIKS_SMALL_E,   (unsigned char) '' },        
+     { DIKS_SMALL_I,   (unsigned char) '' },        
+     { DIKS_SMALL_O,   (unsigned char) '' },        
+     { DIKS_SMALL_U,   (unsigned char) '' },        
+     { DIKS_SMALL_Y,   (unsigned char) '' },        
+     { DIKS_CAPITAL_A, (unsigned char) '' },        
+     { DIKS_CAPITAL_E, (unsigned char) '' },        
+     { DIKS_CAPITAL_I, (unsigned char) '' },        
+     { DIKS_CAPITAL_O, (unsigned char) '' },        
+     { DIKS_CAPITAL_U, (unsigned char) '' },        
+     { DIKS_CAPITAL_Y, (unsigned char) '' },        
+     { 0, 0 }                                        
+};                                                   
+                                                     
+static const DeadKeyCombo combos_circumflex[] = {    
+     { DIKS_SPACE,     (unsigned char) '^' },        
+     { DIKS_SMALL_A,   (unsigned char) '' },        
+     { DIKS_SMALL_E,   (unsigned char) '' },        
+     { DIKS_SMALL_I,   (unsigned char) '' },        
+     { DIKS_SMALL_O,   (unsigned char) '' },        
+     { DIKS_SMALL_U,   (unsigned char) '' },        
+     { DIKS_CAPITAL_A, (unsigned char) '' },        
+     { DIKS_CAPITAL_E, (unsigned char) '' },        
+     { DIKS_CAPITAL_I, (unsigned char) '' },        
+     { DIKS_CAPITAL_O, (unsigned char) '' },        
+     { DIKS_CAPITAL_U, (unsigned char) '' },        
+     { 0, 0 }                                        
+};                                                   
+                                                     
+static const DeadKeyCombo combos_diaeresis[] = {     
+     { DIKS_SPACE,     (unsigned char) '' },        
+     { DIKS_SMALL_A,   (unsigned char) '' },        
+     { DIKS_SMALL_E,   (unsigned char) '' },        
+     { DIKS_SMALL_I,   (unsigned char) '' },        
+     { DIKS_SMALL_O,   (unsigned char) '' },        
+     { DIKS_SMALL_U,   (unsigned char) '' },        
+     { DIKS_CAPITAL_A, (unsigned char) '' },        
+     { DIKS_CAPITAL_E, (unsigned char) '' },        
+     { DIKS_CAPITAL_I, (unsigned char) '' },        
+     { DIKS_CAPITAL_O, (unsigned char) '' },        
+     { DIKS_CAPITAL_U, (unsigned char) '' },        
+     { 0, 0 }                                        
+};                                                   
+                                                     
+static const DeadKeyCombo combos_tilde[] = {         
+     { DIKS_SPACE,     (unsigned char) '~' },        
+     { DIKS_SMALL_A,   (unsigned char) '' },        
+     { DIKS_SMALL_N,   (unsigned char) '' },        
+     { DIKS_SMALL_O,   (unsigned char) '' },        
+     { DIKS_CAPITAL_A, (unsigned char) '' },        
+     { DIKS_CAPITAL_N, (unsigned char) '' },        
+     { DIKS_CAPITAL_O, (unsigned char) '' },        
+     { 0, 0 }                                        
+};                                                   
+                                                     
+static const DeadKeyMap deadkey_maps[] = {           
+     { DIKS_DEAD_GRAVE,      combos_grave },         
+     { DIKS_DEAD_ACUTE,      combos_acute },         
+     { DIKS_DEAD_CIRCUMFLEX, combos_circumflex },    
+     { DIKS_DEAD_DIAERESIS,  combos_diaeresis },     
+     { DIKS_DEAD_TILDE,      combos_tilde }          
+};                                                   
 
 /**********************************************************************************************************************/
 
@@ -862,6 +959,8 @@ init_devices( CoreDFB *core )
                device = D_CALLOC( 1, sizeof(CoreInputDevice) );
                shared = SHCALLOC( pool, 1, sizeof(InputDeviceShared) );
 
+               device->core = core;
+
                memset( &device_info, 0, sizeof(InputDeviceInfo) );
 
                device_info.desc.min_keycode = -1;
@@ -1104,6 +1203,7 @@ find_key_code_by_symbol( CoreInputDevice         *device,
 static void
 fixup_key_event( CoreInputDevice *device, DFBInputEvent *event )
 {
+     int                 i;
      DFBInputEventFlags  valid   = event->flags & FIXUP_KEY_FIELDS;
      DFBInputEventFlags  missing = valid ^ FIXUP_KEY_FIELDS;
      InputDeviceShared  *shared  = device->shared;
@@ -1319,6 +1419,29 @@ fixup_key_event( CoreInputDevice *device, DFBInputEvent *event )
           
           shared->first_press = true;
      }
+
+     /* Handle dead keys. */
+     if (DFB_KEY_TYPE(shared->last_symbol) == DIKT_DEAD) {
+          for (i=0; i<D_ARRAY_SIZE(deadkey_maps); i++) {
+               const DeadKeyMap *map = &deadkey_maps[i];
+
+               if (map->deadkey == shared->last_symbol) {
+                    for (i=0; map->combos[i].target; i++) {
+                         if (map->combos[i].target == event->key_symbol) {
+                              event->key_symbol = map->combos[i].result;
+                              break;
+                         }
+                    }
+                    break;
+               }
+          }
+
+          if (event->type == DIET_KEYRELEASE &&
+              DFB_KEY_TYPE(event->key_symbol) != DIKT_MODIFIER)
+               shared->last_symbol = event->key_symbol;
+     }
+     else
+          shared->last_symbol = event->key_symbol;
 }
 
 static void
@@ -1700,7 +1823,7 @@ flush_keys( CoreInputDevice *device )
 }
 
 static void
-dump_primary_layer_surface()
+dump_primary_layer_surface( CoreDFB *core )
 {
      CoreLayer        *layer = dfb_layer_at( DLID_PRIMARY );
      CoreLayerContext *context;
@@ -1721,7 +1844,7 @@ dump_primary_layer_surface()
                /* Get the surface of the region. */
                if (dfb_layer_region_get_surface( region, &surface ) == DFB_OK) {
                     /* Dump the surface contents. */
-                    dfb_surface_dump( surface,
+                    dfb_surface_dump( core, surface,
                                       dfb_config->screenshot_dir, "dfb" );
 
                     /* Release the surface. */
@@ -1752,7 +1875,7 @@ core_input_filter( CoreInputDevice *device, DFBInputEvent *event )
           switch (event->key_symbol) {
                case DIKS_PRINT:
                     if (!event->modifiers && dfb_config->screenshot_dir) {
-                         dump_primary_layer_surface();
+                         dump_primary_layer_surface( device->core );
                          return true;
                     }
                     break;
