@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,10 +26,19 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
+#include <unistd.h>
 #include <string.h>
 
 #include <directfb.h>
 
+#include <direct/memcpy.h>
+#include <direct/messages.h>
+#include <direct/util.h>
+
+#include <fusion/shmalloc.h>
+
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
@@ -48,15 +58,11 @@
 
 #include <fusion/shmalloc.h>
 
-#include <direct/messages.h>
-
-#include <direct/util.h>
-
 
 D_DEBUG_DOMAIN( Core_Layers, "Core/Layers", "DirectFB Display Layer Core" );
 
 
-/******************************************************************************/
+/**********************************************************************************************************************/
 
 static void      init_region_config  ( CoreLayerContext            *context,
                                        CoreLayerRegionConfig       *config );
@@ -83,7 +89,7 @@ static void      screen_rectangle    ( CoreLayerContext            *context,
                                        const DFBLocation           *location,
                                        DFBRectangle                *rect );
 
-/******************************************************************************/
+/**********************************************************************************************************************/
 
 static void
 context_destructor( FusionObject *object, bool zombie )
@@ -113,11 +119,15 @@ context_destructor( FusionObject *object, bool zombie )
      /* Deinitialize the lock. */
      fusion_skirmish_destroy( &context->lock );
 
+     /* Free clip regions. */
+     if (context->primary.config.clips)
+          SHFREE( context->shmpool, context->primary.config.clips );
+
      /* Destroy the object. */
      fusion_object_destroy( object );
 }
 
-/******************************************************************************/
+/**********************************************************************************************************************/
 
 FusionObjectPool *
 dfb_layer_context_pool_create( const FusionWorld *world )
@@ -128,26 +138,20 @@ dfb_layer_context_pool_create( const FusionWorld *world )
                                        context_destructor, world );
 }
 
-/******************************************************************************/
+/**********************************************************************************************************************/
 
 DFBResult
-dfb_layer_context_create( CoreLayer         *layer,
-                          CoreLayerContext **ret_context )
+dfb_layer_context_init( CoreLayerContext *context,
+                        CoreLayer        *layer )
 {
      CoreLayerShared  *shared;
-     CoreLayerContext *context;
 
+     D_ASSERT( context != NULL );
      D_ASSERT( layer != NULL );
      D_ASSERT( layer->shared != NULL );
-     D_ASSERT( ret_context != NULL );
 
      shared = layer->shared;
 
-     /* Create the object. */
-     context = dfb_core_create_layer_context( layer->core );
-     if (!context)
-          return DFB_FUSION;
-
      D_DEBUG_AT( Core_Layers, "%s -> %p\n", __FUNCTION__, context );
 
      context->shmpool = shared->shmpool;
@@ -199,9 +203,6 @@ dfb_layer_context_create( CoreLayer         *layer,
                              context->config.width,
                              context->config.height );
 
-     /* Return the new context. */
-     *ret_context = context;
-
      return DFB_OK;
 }
 
@@ -368,7 +369,7 @@ dfb_layer_context_get_primary_region( CoreLayerContext  *context,
                                       bool               create,
                                       CoreLayerRegion  **ret_region )
 {
-     DFBResult ret;
+     DFBResult ret = DFB_OK;
 
      D_ASSERT( context != NULL );
      D_ASSERT( ret_region != NULL );
@@ -377,53 +378,82 @@ dfb_layer_context_get_primary_region( CoreLayerContext  *context,
      if (dfb_layer_context_lock( context ))
           return DFB_FUSION;
 
-     if (context->primary.region) {
-          /* Increase the primary region's reference counter. */
-          if (dfb_layer_region_ref( context->primary.region )) {
-               dfb_layer_context_unlock( context );
-               return DFB_FUSION;
-          }
+restart:
+     while (context->primary.region) {
+         /* Increase the primary region's reference counter. */
+         ret = dfb_layer_region_ref( context->primary.region );
+         if (ret == DFB_OK)
+              break;
+
+         dfb_layer_context_unlock( context );
+
+         if (ret == DFB_LOCKED) {
+              //sched_yield();
+              usleep( 10000 );
+
+              if (dfb_layer_context_lock( context ))
+                   return DFB_FUSION;
+         }
+         else
+              return DFB_FUSION;
      }
-     else if (create) {
-          CoreLayerRegion *region;
 
-          /* Create the primary region. */
-          ret = dfb_layer_region_create( context, &region );
-          if (ret) {
-               D_ERROR( "DirectFB/core/layers: Could not create primary region!\n" );
-               dfb_layer_context_unlock( context );
-               return ret;
+     if (!context->primary.region) {
+          if (create) {
+              CoreLayerRegion *region;
+
+              /* Unlock the context. */
+              dfb_layer_context_unlock( context );
+
+              /* Create the primary region. */
+              ret = dfb_layer_region_create( context, &region );
+              if (ret) {
+                   D_ERROR( "DirectFB/core/layers: Could not create primary region!\n" );
+                   return ret;
+              }
+
+              /* Lock the context again. */
+              if (dfb_layer_context_lock( context )) {
+                   dfb_layer_region_unref( region );
+                   return DFB_FUSION;
+              }
+
+              /* Check for race. */
+              if (context->primary.region) {
+                   dfb_layer_region_unref( region );
+                   goto restart;
+              }
+
+              /* Set the region configuration. */
+              ret = dfb_layer_region_set_configuration( region,
+                                                        &context->primary.config,
+                                                        CLRCF_ALL );
+              if (ret) {
+                   D_DERROR( ret, "DirectFB/core/layers: "
+                             "Could not set primary region config!\n" );
+                   dfb_layer_region_unref( region );
+                   dfb_layer_context_unlock( context );
+                   return ret;
+              }
+
+              /* Remember the primary region. */
+              context->primary.region = region;
+
+              /* Allocate surface, enable region etc. */
+              ret = dfb_layer_context_set_configuration( context, &context->config );
+              if (ret) {
+                   D_DERROR( ret, "DirectFB/core/layers: "
+                             "Could not set layer context config!\n" );
+                   context->primary.region = NULL;
+                   dfb_layer_region_unref( region );
+                   dfb_layer_context_unlock( context );
+                   return ret;
+              }
           }
-
-          /* Set the region configuration. */
-          ret = dfb_layer_region_set_configuration( region,
-                                                    &context->primary.config,
-                                                    CLRCF_ALL );
-          if (ret) {
-               D_DERROR( ret, "DirectFB/core/layers: "
-                         "Could not set primary region config!\n" );
-               dfb_layer_region_unref( region );
+          else {
                dfb_layer_context_unlock( context );
-               return ret;
+               return DFB_TEMPUNAVAIL;
           }
-
-          /* Remember the primary region. */
-          context->primary.region = region;
-
-          /* Allocate surface, enable region etc. */
-          ret = dfb_layer_context_set_configuration( context, &context->config );
-          if (ret) {
-               D_DERROR( ret, "DirectFB/core/layers: "
-                         "Could not set layer context config!\n" );
-               context->primary.region = NULL;
-               dfb_layer_region_unref( region );
-               dfb_layer_context_unlock( context );
-               return ret;
-          }
-     }
-     else {
-          dfb_layer_context_unlock( context );
-          return DFB_TEMPUNAVAIL;
      }
 
      /* Return region. */
@@ -632,22 +662,6 @@ dfb_layer_context_set_configuration( CoreLayerContext            *context,
                }
           }
 
-          /* Update the window stack. */
-          if (context->stack) {
-               CoreWindowStack *stack = context->stack;
-
-               /* Update hardware flag. */
-               stack->hw_mode = (region_config.buffermode == DLBM_WINDOWS);
-
-               /* Tell the windowing core about the new size. */
-               dfb_windowstack_resize( stack,
-                                       region_config.width,
-                                       region_config.height );
-
-               /* FIXME: call only if really needed */
-               dfb_windowstack_repaint_all( stack );
-          }
-
           /* Unlock the region and give up the local reference. */
           dfb_layer_region_unlock( region );
           dfb_layer_region_unref( region );
@@ -680,6 +694,26 @@ dfb_layer_context_set_configuration( CoreLayerContext            *context,
      if (config->flags & DLCONF_SURFACE_CAPS)
           context->config.surface_caps = config->surface_caps;
 
+     /* Update the window stack. */
+     if (context->stack) {
+          CoreWindowStack *stack = context->stack;
+
+          /* Update hardware flag. */
+          stack->hw_mode = (region_config.buffermode == DLBM_WINDOWS);
+
+          /* Tell the windowing core about the new size. */
+          if (config->flags & (DLCONF_WIDTH | DLCONF_HEIGHT |
+                               DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_SURFACE_CAPS))
+          {
+               dfb_windowstack_resize( stack,
+                                       region_config.width,
+                                       region_config.height );
+
+               /* FIXME: call only if really needed */
+               dfb_windowstack_repaint_all( stack );
+          }
+     }
+
      /* Unlock the context. */
      dfb_layer_context_unlock( context );
 
@@ -734,7 +768,7 @@ update_primary_region_config( CoreLayerContext           *context,
 
 DFBResult
 dfb_layer_context_set_src_colorkey( CoreLayerContext *context,
-                                    __u8 r, __u8 g, __u8 b )
+                                    u8 r, u8 g, u8 b )
 {
      DFBResult             ret;
      CoreLayerRegionConfig config;
@@ -764,7 +798,7 @@ dfb_layer_context_set_src_colorkey( CoreLayerContext *context,
 
 DFBResult
 dfb_layer_context_set_dst_colorkey( CoreLayerContext *context,
-                                    __u8 r, __u8 g, __u8 b )
+                                    u8 r, u8 g, u8 b )
 {
      DFBResult             ret;
      CoreLayerRegionConfig config;
@@ -961,7 +995,7 @@ dfb_layer_context_set_screenposition( CoreLayerContext  *context,
 
 DFBResult
 dfb_layer_context_set_opacity( CoreLayerContext *context,
-                               __u8              opacity )
+                               u8                opacity )
 {
      DFBResult             ret;
      CoreLayerRegionConfig config;
@@ -1085,7 +1119,58 @@ dfb_layer_context_set_field_parity( CoreLayerContext *context,
 }
 
 DFBResult
-dfb_layer_context_create_window( CoreLayerContext        *context,
+dfb_layer_context_set_clip_regions( CoreLayerContext *context,
+                                    const DFBRegion  *regions,
+                                    int               num_regions,
+                                    DFBBoolean        positive )
+{
+     DFBResult              ret;
+     CoreLayerRegionConfig  config;
+     DFBRegion             *clips;
+     DFBRegion             *old_clips;
+
+     D_ASSERT( context != NULL );
+
+     clips = SHMALLOC( context->shmpool, sizeof(DFBRegion) * num_regions );
+     if (!clips)
+          return D_OOSHM();
+
+     direct_memcpy( clips, regions, sizeof(DFBRegion) * num_regions );
+
+     /* Lock the context. */
+     if (dfb_layer_context_lock( context )) {
+          SHFREE( context->shmpool, clips );
+          return DFB_FUSION;
+     }
+
+     /* Take the current configuration. */
+     config = context->primary.config;
+
+     /* Remember for freeing later on. */
+     old_clips = config.clips;
+
+     /* Change the clip regions. */
+     config.clips     = clips;
+     config.num_clips = num_regions;
+     config.positive  = positive;
+
+     /* Try to set the new configuration. */
+     ret = update_primary_region_config( context, &config, CLRCF_CLIPS );
+
+     /* Unlock the context. */
+     dfb_layer_context_unlock( context );
+
+     if (ret)
+          SHFREE( context->shmpool, clips );
+     else if (old_clips)
+          SHFREE( context->shmpool, old_clips );
+
+     return ret;
+}
+
+DFBResult
+dfb_layer_context_create_window( CoreDFB                 *core,
+                                 CoreLayerContext        *context,
                                  int                      x,
                                  int                      y,
                                  int                      width,
@@ -1115,7 +1200,7 @@ dfb_layer_context_create_window( CoreLayerContext        *context,
      stack = context->stack;
 
      if (!stack->cursor.set) {
-          ret = dfb_windowstack_cursor_enable( stack, true );
+          ret = dfb_windowstack_cursor_enable( core, stack, true );
           if (ret) {
                dfb_layer_context_unlock( context );
                return ret;
@@ -1380,7 +1465,6 @@ allocate_surface( CoreLayer             *layer,
                     break;
 
                case DLBM_BACKSYSTEM:
-                    D_ONCE("DLBM_BACKSYSTEM as default is unimplemented");
                     break;
 
                default:
@@ -1403,9 +1487,16 @@ allocate_surface( CoreLayer             *layer,
                                     config->format, CSP_VIDEOONLY,
                                     caps, NULL, &surface );
           if (ret) {
-               D_ERROR( "DirectFB/core/layers: Surface creation failed!\n" );
+               D_DERROR( ret, "Core/layers: Surface creation failed!\n" );
                return ret;
           }
+
+          if (config->buffermode == DLBM_BACKSYSTEM) {
+               surface->caps |= DSCAPS_DOUBLE;
+               ret = dfb_surface_reconfig( surface, CSP_VIDEOONLY, CSP_SYSTEMONLY );
+               if (ret)
+                    D_DERROR( ret, "Core/layers: Surface reconfiguration failed!\n" );
+          }
      }
 
      /* Tell the region about it's new surface (adds a global reference). */
