@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -42,21 +43,27 @@
 
 #include <misc/util.h>
 
-static const __u8 lookup3to8[] = { 0x00, 0x24, 0x49, 0x6d, 0x92, 0xb6, 0xdb, 0xff };
-static const __u8 lookup2to8[] = { 0x00, 0x55, 0xaa, 0xff };
+D_DEBUG_DOMAIN( Core_Palette, "Core/Palette", "DirectFB Palette Core" );
+
+/**********************************************************************************************************************/
+
+static const u8 lookup3to8[] = { 0x00, 0x24, 0x49, 0x6d, 0x92, 0xb6, 0xdb, 0xff };
+static const u8 lookup2to8[] = { 0x00, 0x55, 0xaa, 0xff };
 
 static const ReactionFunc dfb_palette_globals[] = {
 /* 0 */   _dfb_surface_palette_listener,
           NULL
 };
 
+/**********************************************************************************************************************/
+
 static void palette_destructor( FusionObject *object, bool zombie )
 {
      CorePaletteNotification  notification;
      CorePalette             *palette = (CorePalette*) object;
 
-     D_DEBUG("DirectFB/core/palette: destroying %p (%d)%s\n", palette,
-              palette->num_entries, zombie ? " (ZOMBIE)" : "");
+     D_DEBUG_AT( Core_Palette, "destroying %p (%d)%s\n", palette,
+                 palette->num_entries, zombie ? " (ZOMBIE)" : "");
 
      notification.flags   = CPNF_DESTROY;
      notification.palette = palette;
@@ -73,8 +80,6 @@ static void palette_destructor( FusionObject *object, bool zombie )
      fusion_object_destroy( object );
 }
 
-/** public **/
-
 FusionObjectPool *
 dfb_palette_pool_create( const FusionWorld *world )
 {
@@ -88,6 +93,8 @@ dfb_palette_pool_create( const FusionWorld *world )
      return pool;
 }
 
+/**********************************************************************************************************************/
+
 DFBResult
 dfb_palette_create( CoreDFB       *core,
                     unsigned int   size,
@@ -95,6 +102,8 @@ dfb_palette_create( CoreDFB       *core,
 {
      CorePalette *palette;
 
+     D_DEBUG_AT( Core_Palette, "%s( %d )\n", __FUNCTION__, size );
+
      D_ASSERT( ret_palette );
 
      palette = dfb_core_create_palette( core );
@@ -122,6 +131,8 @@ dfb_palette_create( CoreDFB       *core,
      /* return the new palette */
      *ret_palette = palette;
 
+     D_DEBUG_AT( Core_Palette, "  -> %p\n", palette );
+
      return DFB_OK;
 }
 
@@ -130,6 +141,8 @@ dfb_palette_generate_rgb332_map( CorePalette *palette )
 {
      unsigned int i;
 
+     D_DEBUG_AT( Core_Palette, "%s( %p )\n", __FUNCTION__, palette );
+
      D_ASSERT( palette != NULL );
 
      if (!palette->num_entries)
@@ -150,6 +163,8 @@ dfb_palette_generate_rgb121_map( CorePalette *palette )
 {
      unsigned int i;
 
+     D_DEBUG_AT( Core_Palette, "%s( %p )\n", __FUNCTION__, palette );
+
      D_ASSERT( palette != NULL );
 
      if (!palette->num_entries)
@@ -167,10 +182,10 @@ dfb_palette_generate_rgb121_map( CorePalette *palette )
 
 unsigned int
 dfb_palette_search( CorePalette *palette,
-                    __u8         r,
-                    __u8         g,
-                    __u8         b,
-                    __u8         a )
+                    u8           r,
+                    u8           g,
+                    u8           b,
+                    u8           a )
 {
      unsigned int index;
 
@@ -207,6 +222,8 @@ dfb_palette_update( CorePalette *palette, int first, int last )
 {
      CorePaletteNotification notification;
 
+     D_DEBUG_AT( Core_Palette, "%s( %p, %d, %d )\n", __FUNCTION__, palette, first, last );
+
      D_ASSERT( palette != NULL );
      D_ASSERT( first >= 0 );
      D_ASSERT( first < palette->num_entries );
@@ -232,3 +249,40 @@ dfb_palette_update( CorePalette *palette, int first, int last )
      dfb_palette_dispatch( palette, &notification, dfb_palette_globals );
 }
 
+bool
+dfb_palette_equal( CorePalette *palette1, CorePalette *palette2 )
+{
+     u32 *entries1;
+     u32 *entries2;
+     int    i;
+     
+     D_DEBUG_AT( Core_Palette, "%s( %p, %p )\n", __FUNCTION__, palette1, palette2 );
+
+     D_ASSERT( palette1 != NULL );
+     D_ASSERT( palette2 != NULL );
+
+     if (palette1 == palette2) {
+          D_DEBUG_AT( Core_Palette, "  -> SAME\n" );
+          return true;
+     }
+
+     if (palette1->num_entries != palette2->num_entries) {
+          D_DEBUG_AT( Core_Palette, "  -> NOT EQUAL (%d/%d)\n", palette1->num_entries, palette2->num_entries );
+          return false;
+     }
+
+     entries1 = (u32*)palette1->entries;
+     entries2 = (u32*)palette2->entries;
+
+     for (i = 0; i < palette1->num_entries; i++) {
+          if (entries1[i] != entries2[i]) {
+               D_DEBUG_AT( Core_Palette, "  -> NOT EQUAL (%d)\n", i );
+               return false;
+          }
+     }
+
+     D_DEBUG_AT( Core_Palette, "  -> EQUAL\n" );
+
+     return true;
+}
+
