@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -31,6 +32,7 @@
 
 #include <pthread.h>
 
+#include <fusion/fusion.h>
 #include <fusion/reactor.h>
 
 #include <directfb.h>
@@ -42,24 +44,34 @@
 #include <core/surfaces.h>
 
 #include <direct/mem.h>
+#include <direct/memcpy.h>
 #include <direct/util.h>
 
 
 static inline void
 validate_clip( CardState *state,
                int        xmax,
-               int        ymax )
+               int        ymax,
+               bool       warn )
 {
      D_MAGIC_ASSERT( state, CardState );
-
      DFB_REGION_ASSERT( &state->clip );
 
+     D_ASSERT( xmax >= 0 );
+     D_ASSERT( ymax >= 0 );
+     D_ASSERT( state->clip.x1 <= state->clip.x2 );
+     D_ASSERT( state->clip.y1 <= state->clip.y2 );
+
      if (state->clip.x1 <= xmax &&
          state->clip.y1 <= ymax &&
          state->clip.x2 <= xmax &&
          state->clip.y2 <= ymax)
           return;
 
+     if (warn)
+          D_WARN( "Clip %d,%d-%dx%d invalid, adjusting to fit %dx%d",
+                  DFB_RECTANGLE_VALS_FROM_REGION( &state->clip ), xmax+1, ymax+1 );
+
      if (state->clip.x1 > xmax)
           state->clip.x1 = xmax;
 
@@ -76,12 +88,14 @@ validate_clip( CardState *state,
 }
 
 int
-dfb_state_init( CardState *state )
+dfb_state_init( CardState *state, CoreDFB *core )
 {
      D_ASSERT( state != NULL );
 
      memset( state, 0, sizeof(CardState) );
 
+     state->core      = core;
+     state->fusion_id = fusion_id( dfb_core_world(core) );
      state->modified  = SMF_ALL;
      state->src_blend = DSBF_SRCALPHA;
      state->dst_blend = DSBF_INVSRCALPHA;
@@ -101,23 +115,31 @@ dfb_state_destroy( CardState *state )
 {
      D_MAGIC_ASSERT( state, CardState );
 
-     D_MAGIC_CLEAR( state );
-
      D_ASSUME( state->destination == NULL );
      D_ASSUME( state->source == NULL );
 
+     D_MAGIC_CLEAR( state );
+
      direct_serial_deinit( &state->dst_serial );
      direct_serial_deinit( &state->src_serial );
 
      if (state->gfxs) {
           GenefxState *gfxs = state->gfxs;
 
-          if (gfxs->Aacc)
-               D_FREE( gfxs->Aacc );
+          if (gfxs->ABstart)
+               D_FREE( gfxs->ABstart );
 
           D_FREE( gfxs );
      }
 
+     if (state->num_translation) {
+          D_ASSERT( state->index_translation != NULL );
+
+          D_FREE( state->index_translation );
+     }
+     else
+          D_ASSERT( state->index_translation == NULL );
+
      pthread_mutex_destroy( &state->lock );
 }
 
@@ -135,7 +157,7 @@ dfb_state_set_destination( CardState *state, CoreSurface *destination )
                     return;
                }
 
-               validate_clip( state, destination->width - 1, destination->height - 1 );
+               validate_clip( state, destination->width - 1, destination->height - 1, false );
           }
 
           if (state->destination) {
@@ -194,21 +216,25 @@ dfb_state_set_source( CardState *state, CoreSurface *source )
 void
 dfb_state_update( CardState *state, bool update_source )
 {
-     D_MAGIC_ASSERT( state, CardState );
+     CoreSurface *destination;
 
+     D_MAGIC_ASSERT( state, CardState );
      DFB_REGION_ASSERT( &state->clip );
 
+     destination = state->destination;
+
      if (D_FLAGS_IS_SET( state->flags, CSF_DESTINATION )) {
-          CoreSurface *destination = state->destination;
 
           D_ASSERT( destination != NULL );
 
           if (direct_serial_update( &state->dst_serial, &destination->serial )) {
-               validate_clip( state, destination->width - 1, destination->height - 1 );
+               validate_clip( state, destination->width - 1, destination->height - 1, true );
 
                state->modified |= SMF_DESTINATION;
           }
      }
+     else if (destination)
+          validate_clip( state, destination->width - 1, destination->height - 1, true );
 
      if (update_source && D_FLAGS_IS_SET( state->flags, CSF_SOURCE )) {
           CoreSurface *source = state->source;
@@ -220,3 +246,39 @@ dfb_state_update( CardState *state, bool update_source )
      }
 }
 
+DFBResult
+dfb_state_set_index_translation( CardState *state,
+                                 const int *indices,
+                                 int        num_indices )
+{
+     D_MAGIC_ASSERT( state, CardState );
+
+     D_ASSERT( indices != NULL || num_indices == 0 );
+
+     dfb_state_lock( state );
+
+     if (state->num_translation != num_indices) {
+          int *new_trans = D_REALLOC( state->index_translation,
+                                      num_indices * sizeof(int) );
+
+          D_ASSERT( num_indices || new_trans == NULL );
+
+          if (num_indices && !new_trans) {
+               dfb_state_unlock( state );
+               return D_OOM();
+          }
+
+          state->index_translation = new_trans;
+          state->num_translation   = num_indices;
+     }
+
+     if (num_indices)
+          direct_memcpy( state->index_translation, indices, num_indices * sizeof(int) );
+
+     state->modified |= SMF_INDEX_TRANSLATION;
+
+     dfb_state_unlock( state );
+
+     return DFB_OK;
+}
+
