@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -46,19 +47,21 @@
 
 
 typedef enum {
-     SMF_DRAWING_FLAGS   = 0x00000001,
-     SMF_BLITTING_FLAGS  = 0x00000002,
-     SMF_CLIP            = 0x00000004,
-     SMF_COLOR           = 0x00000008,
-     SMF_SRC_BLEND       = 0x00000010,
-     SMF_DST_BLEND       = 0x00000020,
-     SMF_SRC_COLORKEY    = 0x00000040,
-     SMF_DST_COLORKEY    = 0x00000080,
+     SMF_DRAWING_FLAGS     = 0x00000001,
+     SMF_BLITTING_FLAGS    = 0x00000002,
+     SMF_CLIP              = 0x00000004,
+     SMF_COLOR             = 0x00000008,
+     SMF_SRC_BLEND         = 0x00000010,
+     SMF_DST_BLEND         = 0x00000020,
+     SMF_SRC_COLORKEY      = 0x00000040,
+     SMF_DST_COLORKEY      = 0x00000080,
 
-     SMF_DESTINATION     = 0x00000100,
-     SMF_SOURCE          = 0x00000200,
+     SMF_DESTINATION       = 0x00000100,
+     SMF_SOURCE            = 0x00000200,
 
-     SMF_ALL             = 0x000003FF
+     SMF_INDEX_TRANSLATION = 0x00001000,
+
+     SMF_ALL               = 0x000013FF
 } StateModificationFlags;
 
 typedef enum {
@@ -75,6 +78,9 @@ typedef enum {
 struct _CardState {
      int                      magic;
 
+     CoreDFB                 *core;
+     FusionID                 fusion_id;
+
      pthread_mutex_t          lock;          /* lock for state handling */
 
      CardStateFlags           flags;
@@ -93,8 +99,8 @@ struct _CardState {
      unsigned int             color_index;   /* index to color in palette */
      DFBSurfaceBlendFunction  src_blend;     /* blend function for source */
      DFBSurfaceBlendFunction  dst_blend;     /* blend function for destination */
-     __u32                    src_colorkey;  /* colorkey for source */
-     __u32                    dst_colorkey;  /* colorkey for destination */
+     u32                      src_colorkey;  /* colorkey for source */
+     u32                      dst_colorkey;  /* colorkey for destination */
 
      CoreSurface             *destination;   /* destination surface */
      CoreSurface             *source;        /* source surface */
@@ -102,6 +108,9 @@ struct _CardState {
      DirectSerial             dst_serial;    /* last destination surface serial */
      DirectSerial             src_serial;    /* last source surface serial */
 
+     int                     *index_translation;
+     int                      num_translation;
+
 
      /* hardware abstraction and state handling helpers */
 
@@ -117,7 +126,7 @@ struct _CardState {
      GenefxState             *gfxs;
 };
 
-int dfb_state_init( CardState *state );
+int  dfb_state_init( CardState *state, CoreDFB *core );
 void dfb_state_destroy( CardState *state );
 
 void dfb_state_set_destination( CardState *state, CoreSurface *destination );
@@ -125,6 +134,10 @@ void dfb_state_set_source( CardState *state, CoreSurface *source );
 
 void dfb_state_update( CardState *state, bool update_source );
 
+DFBResult dfb_state_set_index_translation( CardState *state,
+                                           const int *indices,
+                                           int        num_indices );
+
 static inline void
 dfb_state_get_serial( const CardState *state, CoreGraphicsSerial *ret_serial )
 {
@@ -200,8 +213,8 @@ static inline void dfb_state_set_clip( CardState *state, const DFBRegion *clip )
      DFB_REGION_ASSERT( clip );
 
      if (! DFB_REGION_EQUAL( state->clip, *clip )) {
-          state->clip      = *clip;
-          state->modified |= SMF_CLIP;
+          state->clip     = *clip;
+          state->modified = (StateModificationFlags)( state->modified | SMF_CLIP );
      }
 }
 
@@ -211,8 +224,8 @@ static inline void dfb_state_set_color( CardState *state, const DFBColor *color
      D_ASSERT( color != NULL );
 
      if (! DFB_COLOR_EQUAL( state->color, *color )) {
-          state->color     = *color;
-          state->modified |= SMF_COLOR;
+          state->color    = *color;
+          state->modified = (StateModificationFlags)( state->modified | SMF_COLOR );
      }
 }
 
