@@ -1,13 +1,13 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2006  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
               Sven Neumann <neo@directfb.org>,
-              Ville Syrjl <syrjala@sci.fi> and
+              Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
@@ -50,6 +50,8 @@
 #include <core/surfacemanager.h>
 #include <core/system.h>
 
+#include <misc/conf.h>
+
 #include <direct/debug.h>
 #include <direct/mem.h>
 #include <direct/memcpy.h>
@@ -177,7 +179,7 @@ dfb_surfacemanager_add_heap( SurfaceManager     *manager,
      
      D_ASSERT( manager != NULL );
 
-     if (storage == CSS_SYSTEM)
+     if (storage != CSS_VIDEO && storage != CSS_AUXILIARY)
           return DFB_INVARG;
 
      heap = SHCALLOC( manager->shmpool, 1, sizeof(Heap) );
@@ -274,18 +276,35 @@ DFBResult dfb_surfacemanager_suspend( SurfaceManager *manager )
 
      dfb_surfacemanager_lock( manager );
 
-     manager->suspended = true;
-
      h = manager->heaps;
      while (h) {
           c = h->chunks;
           while (c) {
-               if (c->buffer &&
-                   c->buffer->policy != CSP_VIDEOONLY &&
-                   c->buffer->video.health == CSH_STORED)
-               {
-                    dfb_surfacemanager_assure_system( manager, c->buffer );
-                    c->buffer->video.health = CSH_RESTORE;
+               if (c->buffer) {
+                    SurfaceBuffer *buffer = c->buffer;
+
+                    switch (buffer->policy) {
+                         case CSP_SYSTEMONLY:
+                              break;
+
+                         case CSP_VIDEOONLY:
+                              /* FIXME */
+                              break;
+
+                         case CSP_VIDEOLOW:
+                         case CSP_VIDEOHIGH:
+                              switch (buffer->video.health) {
+                                   case CSH_STORED:
+                                        dfb_surfacemanager_assure_system( manager, buffer );
+                                        buffer->video.health = CSH_RESTORE;
+                                   case CSH_RESTORE:
+//                                        if (!buffer->video.locked)
+//                                             dfb_surfacemanager_deallocate( manager, buffer );
+                                   default:
+                                        break;
+                              }
+                              break;
+                    }
                }
 
                c = c->next;
@@ -294,6 +313,8 @@ DFBResult dfb_surfacemanager_suspend( SurfaceManager *manager )
           h = h->next;
      }
 
+     manager->suspended = true;
+
      dfb_surfacemanager_unlock( manager );
 
      return DFB_OK;
@@ -301,10 +322,40 @@ DFBResult dfb_surfacemanager_suspend( SurfaceManager *manager )
 
 DFBResult dfb_surfacemanager_resume( SurfaceManager *manager )
 {
+     Heap  *h;
+     Chunk *c;
+
      D_MAGIC_ASSERT( manager, SurfaceManager );
 
      dfb_surfacemanager_lock( manager );
 
+     h = manager->heaps;
+     while (h) {
+          c = h->chunks;
+          while (c) {
+               if (c->buffer) {
+                    SurfaceBuffer *buffer = c->buffer;
+
+                    switch (buffer->policy) {
+                         case CSP_SYSTEMONLY:
+                              break;
+
+                         case CSP_VIDEOONLY:
+                              /* FIXME */
+                              break;
+
+                         case CSP_VIDEOLOW:
+                         case CSP_VIDEOHIGH:
+                              break;
+                    }
+               }
+
+               c = c->next;
+          }
+
+          h = h->next;
+     }
+
      manager->suspended = false;
 
      dfb_surfacemanager_unlock( manager );
@@ -456,6 +507,9 @@ DFBResult dfb_surfacemanager_allocate( SurfaceManager *manager,
      length = DFB_PLANE_MULTIPLY( buffer->format,
                                   MAX( surface->height, surface->min_height ) * pitch );
 
+     /* Add extra space for optimized routines which are now allowed to overrun, e.g. prefetching. */
+     length += 16;
+
      if (manager->byteoffset_align > 1) {
           length += manager->byteoffset_align - 1;
           length -= length % manager->byteoffset_align;
@@ -554,7 +608,6 @@ DFBResult dfb_surfacemanager_deallocate( SurfaceManager *manager,
 
      buffer->video.health = CSH_INVALID;
      buffer->video.chunk = NULL;
-     buffer->storage = 0;
 
      dfb_surface_notify_listeners( buffer->surface, CSNF_VIDEO );
 
@@ -568,12 +621,73 @@ DFBResult dfb_surfacemanager_deallocate( SurfaceManager *manager,
      if (buffer->video.locked)
           D_WARN( "Freeing chunk with a non-zero lock counter" );
 
+     /* If hardware has (to) read/write... */
+     if (buffer->video.access & (VAF_HARDWARE_READ | VAF_HARDWARE_WRITE)) {
+          /* ...wait for it. */
+          dfb_gfxcard_sync(); /* TODO: wait for serial instead */
+
+          /* ...clear hardware access. */
+          buffer->video.access &= ~(VAF_HARDWARE_READ | VAF_HARDWARE_WRITE);
+     }
+
      if (chunk)
           free_chunk( manager, chunk );
 
      return DFB_OK;
 }
 
+static void
+transfer_buffer( SurfaceBuffer *buffer,
+                 void          *src,
+                 void          *dst,
+                 int            srcpitch,
+                 int            dstpitch )
+{
+     int          i;
+     CoreSurface *surface = buffer->surface;
+
+     for (i=0; i<surface->height; i++) {
+          direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format, surface->width) );
+
+          src += srcpitch;
+          dst += dstpitch;
+     }
+
+     switch (buffer->format) {
+          case DSPF_YV12:
+          case DSPF_I420:
+               for (i=0; i<surface->height; i++) {
+                    direct_memcpy( dst, src,
+                                   DFB_BYTES_PER_LINE( buffer->format, surface->width / 2 ) );
+                    src += srcpitch / 2;
+                    dst += dstpitch / 2;
+               }
+               break;
+
+          case DSPF_NV12:
+          case DSPF_NV21:
+               for (i=0; i<surface->height/2; i++) {
+                    direct_memcpy( dst, src,
+                                   DFB_BYTES_PER_LINE( buffer->format, surface->width ) );
+                    src += srcpitch;
+                    dst += dstpitch;
+               }
+               break;
+
+          case DSPF_NV16:
+               for (i=0; i<surface->height; i++) {
+                    direct_memcpy( dst, src,
+                                   DFB_BYTES_PER_LINE( buffer->format, surface->width ) );
+                    src += srcpitch;
+                    dst += dstpitch;
+               }
+               break;
+
+          default:
+               break;
+     }
+}
+
 DFBResult dfb_surfacemanager_assure_video( SurfaceManager *manager,
                                            SurfaceBuffer  *buffer )
 {
@@ -582,94 +696,76 @@ DFBResult dfb_surfacemanager_assure_video( SurfaceManager *manager,
 
      D_MAGIC_ASSERT( manager, SurfaceManager );
 
-     if (manager->suspended)
-          return DFB_NOVIDEOMEMORY;
+     if (manager->suspended || buffer->flags & SBF_SUSPENDED)
+          return DFB_SUSPENDED;
 
      switch (buffer->video.health) {
           case CSH_STORED:
-               if (buffer->video.chunk) {
+               if (buffer->video.chunk)
                     buffer->video.chunk->tolerations = 0;
-                    buffer->storage = buffer->video.chunk->heap->storage;
-               }
-               else {
-                    buffer->storage = CSS_VIDEO;
-               }
-               return DFB_OK;
+               else
+                    buffer->storage = CSS_VIDEO;  /* for chunk less fbdev surface */
+
+               break;
 
           case CSH_INVALID:
+               /* Allocate video memory. */
                ret = dfb_surfacemanager_allocate( manager, buffer );
                if (ret)
                     return ret;
 
-               /* FALL THROUGH, because after successful allocation
-                  the surface health is CSH_RESTORE */
+               /* fall through */
 
-          case CSH_RESTORE:
-               if (buffer->system.health != CSH_STORED)
-                    D_BUG( "system/video instances both not stored!" );
+          case CSH_RESTORE: {
+               Chunk *chunk = buffer->video.chunk;
 
-               if (buffer->flags & SBF_WRITTEN) {
-                    int   i;
-                    char *src   = buffer->system.addr;
-                    char *dst   = (buffer->video.chunk->heap->storage == CSS_VIDEO)
-                                  ? dfb_system_video_memory_virtual( buffer->video.offset )
-                                  : dfb_system_aux_memory_virtual( buffer->video.offset );
-                    int   flags = buffer->video.access & (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_WRITE);
-                   
-                    if (flags != (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_WRITE))
+               /* Upload? */
+               if (buffer->flags & SBF_WRITTEN && buffer->system.health == CSH_STORED) {
+                    void *video  = (chunk->heap->storage == CSS_VIDEO)
+                                   ? dfb_system_video_memory_virtual( buffer->video.offset )
+                                   : dfb_system_aux_memory_virtual( buffer->video.offset );
+                    bool  locked = D_FLAGS_ARE_SET( buffer->video.access,
+                                                    VAF_SOFTWARE_LOCK | VAF_SOFTWARE_WRITE );
+
+                    if (!locked)
                          dfb_gfxcard_surface_enter( buffer, DSLF_WRITE );
-                    
-                    for (i=0; i<surface->height; i++) {
-                         direct_memcpy( dst, src,
-                                        DFB_BYTES_PER_LINE(buffer->format, surface->width) );
-                         src += buffer->system.pitch;
-                         dst += buffer->video.pitch;
-                    }
 
-                    if (buffer->format == DSPF_YV12 || buffer->format == DSPF_I420) {
-                         for (i=0; i<surface->height; i++) {
-                              direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
-                                                                          surface->width / 2) );
-                              src += buffer->system.pitch / 2;
-                              dst += buffer->video.pitch  / 2;
-                         }
-                    }
-                    else if (buffer->format == DSPF_NV12 || buffer->format == DSPF_NV21) {
-                         for (i=0; i<surface->height/2; i++) {
-                              direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
-                                                                          surface->width) );
-                              src += buffer->system.pitch;
-                              dst += buffer->video.pitch;
-                         }
-                    }
-                    else if (buffer->format == DSPF_NV16) {
-                         for (i=0; i<surface->height; i++) {
-                              direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
-                                                                          surface->width) );
-                              src += buffer->system.pitch;
-                              dst += buffer->video.pitch;
-                         }
-                    }
+                    /* Copy the data. */
+                    transfer_buffer( buffer,
+                                     buffer->system.addr, video,
+                                     buffer->system.pitch, buffer->video.pitch );
 
-                    if (flags != (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_WRITE))
+                    if (!locked)
                          dfb_gfxcard_surface_leave( buffer );
                }
 
-               buffer->video.health             = CSH_STORED;
-               buffer->video.chunk->tolerations = 0;
-               
-               buffer->storage = buffer->video.chunk->heap->storage;
+               /* Update health. */
+               buffer->video.health = CSH_STORED;
+
+               /* Reset tolerations. */
+               chunk->tolerations = 0;
 
                dfb_surface_notify_listeners( surface, CSNF_VIDEO );
 
-               return DFB_OK;
+               /* Free system instance. */
+               if (dfb_config->thrifty_surface_buffers) {
+                    if (buffer->system.health && !(buffer->flags & SBF_FOREIGN_SYSTEM)) {
+                         buffer->system.health = CSH_INVALID;
+
+                         SHFREE( buffer->surface->shmpool_data, buffer->system.addr );
+                         buffer->system.addr = NULL;
+                    }
+               }
+
+               break;
+          }
 
           default:
-               break;
+               D_BUG( "unknown buffer health %d", buffer->video.health );
+               return DFB_BUG;
      }
 
-     D_BUG( "unknown video instance health" );
-     return DFB_BUG;
+     return DFB_OK;
 }
 
 DFBResult dfb_surfacemanager_assure_system( SurfaceManager *manager,
@@ -684,74 +780,67 @@ DFBResult dfb_surfacemanager_assure_system( SurfaceManager *manager,
           return DFB_BUG;
      }
 
-     if (buffer->system.health == CSH_STORED) {
-          buffer->storage = CSS_SYSTEM;
-          return DFB_OK;
-     }
-     else if (buffer->video.health == CSH_STORED) {
-          int   i;
-          char *src   = (buffer->video.chunk->heap->storage == CSS_VIDEO)
-                        ? dfb_system_video_memory_virtual( buffer->video.offset )
-                        : dfb_system_aux_memory_virtual( buffer->video.offset );
-          char *dst   = buffer->system.addr;
-          int   flags = buffer->video.access & (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_READ);
-           
-          /* from video_access_by_software() in surface.c */
-          if (buffer->video.access & VAF_HARDWARE_WRITE) {
-               dfb_gfxcard_wait_serial( &buffer->video.serial );
-               dfb_gfxcard_flush_read_cache();
-               buffer->video.access &= ~VAF_HARDWARE_WRITE;
-          }
-          buffer->video.access |= VAF_SOFTWARE_READ;
+     if (manager->suspended || buffer->flags & SBF_SUSPENDED)
+          return DFB_SUSPENDED;
 
-          if (flags != (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_READ))
-               dfb_gfxcard_surface_enter( buffer, DSLF_READ );
+     switch (buffer->system.health) {
+          case CSH_STORED:
+               break;
 
-          for (i=0; i<surface->height; i++) {
-               direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format, surface->width) );
-               src += buffer->video.pitch;
-               dst += buffer->system.pitch;
-          }
+          case CSH_INVALID:
+               /* Allocate shared memory. */
+               buffer->system.addr = SHMALLOC( surface->shmpool_data, buffer->system.size );
+               if (!buffer->system.addr)
+                    return D_OOSHM();
 
-          if (buffer->format == DSPF_YV12 || buffer->format == DSPF_I420) {
-               for (i=0; i<surface->height; i++) {
-                    direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
-                                                                surface->width / 2) );
-                    src += buffer->video.pitch  / 2;
-                    dst += buffer->system.pitch / 2;
-               }
-          }
-          else if (buffer->format == DSPF_NV12 || buffer->format == DSPF_NV21) {
-               for (i=0; i<surface->height/2; i++) {
-                    direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
-                                                                surface->width) );
-                    src += buffer->video.pitch;
-                    dst += buffer->system.pitch;
-               }
-          }
-          else if (buffer->format == DSPF_NV16) {
-               for (i=0; i<surface->height; i++) {
-                    direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format,
-                                                                surface->width) );
-                    src += buffer->video.pitch;
-                    dst += buffer->system.pitch;
+               /* fall through */
+
+          case CSH_RESTORE: {
+               Chunk *chunk = buffer->video.chunk;
+
+               /* Download? */
+               if (buffer->system.health == CSH_INVALID ||
+                   (buffer->flags & SBF_WRITTEN && buffer->video.health == CSH_STORED))
+               {
+                    void *video  = (chunk->heap->storage == CSS_VIDEO)
+                                   ? dfb_system_video_memory_virtual( buffer->video.offset )
+                                   : dfb_system_aux_memory_virtual( buffer->video.offset );
+                    bool  locked = D_FLAGS_ARE_SET( buffer->video.access,
+                                                    VAF_SOFTWARE_LOCK | VAF_SOFTWARE_READ );
+
+                    if (!locked)
+                         dfb_gfxcard_surface_enter( buffer, DSLF_READ );
+
+
+                    /* from video_access_by_software() in surface.c */
+                    if (buffer->video.access & VAF_HARDWARE_WRITE) {
+                         dfb_gfxcard_wait_serial( &buffer->video.serial );
+                         dfb_gfxcard_flush_read_cache();
+                         buffer->video.access &= ~VAF_HARDWARE_WRITE;
+                    }
+                    buffer->video.access |= VAF_SOFTWARE_READ;
+
+                    /* Copy the data. */
+                    transfer_buffer( buffer, video, buffer->system.addr,
+                                     buffer->video.pitch, buffer->system.pitch );
+
+                    if (!locked)
+                         dfb_gfxcard_surface_leave( buffer );
                }
+
+               /* Update health. */
+               buffer->system.health = CSH_STORED;
+
+               dfb_surface_notify_listeners( surface, CSNF_SYSTEM );
+               break;
           }
 
-          if (flags != (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_READ))
-               dfb_gfxcard_surface_leave( buffer );
-
-          buffer->system.health = CSH_STORED;
-
-          buffer->storage = CSS_SYSTEM;
-
-          dfb_surface_notify_listeners( surface, CSNF_SYSTEM );
-
-          return DFB_OK;
+          default:
+               D_BUG( "unknown buffer health %d", buffer->system.health );
+               return DFB_BUG;
      }
 
-     D_BUG( "no valid surface instance" );
-     return DFB_BUG;
+     return DFB_OK;
 }
 
 /** internal functions NOT locking the surfacemanager **/
@@ -807,6 +896,7 @@ free_chunk( SurfaceManager *manager, Chunk *chunk )
      if (chunk->buffer->policy == CSP_VIDEOONLY)
           chunk->heap->avail += chunk->length;
 
+     chunk->buffer->storage = CSS_NONE;
      chunk->buffer = NULL;
 
      chunk->heap->min_toleration--;
