@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -83,8 +84,12 @@ void dfb_surfacemanager_unlock( SurfaceManager *manager );
  * lock a surface for access by software, returns a pointer to the
  * surface data and the line pitch a.k.a. rowstride
  */
-DFBResult dfb_surface_software_lock( CoreSurface *surface, unsigned int flags,
-                                     void **data, int *pitch, bool front );
+DFBResult dfb_surface_software_lock( CoreDFB              *core,
+                                     CoreSurface          *surface,
+                                     DFBSurfaceLockFlags   flags,
+                                     void                **data,
+                                     int                  *pitch,
+                                     bool                  front );
 
 /*
  * lock a surface for access by hardware that enforces a video instance
@@ -92,8 +97,10 @@ DFBResult dfb_surface_software_lock( CoreSurface *surface, unsigned int flags,
  * video struct, however this function will fail if the surfacemanager could
  * not assure a video memory instance
  */
-DFBResult dfb_surface_hardware_lock( CoreSurface *surface,
-                                     unsigned int flags, bool front );
+DFBResult dfb_surface_hardware_lock( CoreDFB             *core,
+                                     CoreSurface         *surface,
+                                     DFBSurfaceLockFlags  flags,
+                                     bool                 front );
 
 
 /*
