@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -35,25 +36,6 @@
 
 #include <fusion/object.h>
 
-/*
- * Hidden capability for software cursor, window will be the "super topmost".
- */
-#define DWHC_TOPMOST   0x80000000
-
-
-typedef enum {
-     CWUF_NONE      = 0x00000000,
-
-     CWUF_SURFACE   = 0x00000001,
-     CWUF_POSITION  = 0x00000002,
-     CWUF_SIZE      = 0x00000004,
-     CWUF_OPTIONS   = 0x00000008,
-     CWUF_OPACITY   = 0x00000010,
-     CWUF_COLORKEY  = 0x00000020,
-     CWUF_PALETTE   = 0x00000040,
-
-     CWUF_ALL       = 0x0000003F
-} CoreWindowUpdateFlags;
 
 typedef enum {
      CWCF_NONE      = 0x00000000,
@@ -78,7 +60,7 @@ struct __DFB_CoreWindowConfig {
      DFBWindowStackingClass  stacking;       /* level boundaries */
      DFBWindowOptions        options;        /* flags for appearance/behaviour */
      DFBWindowEventType      events;         /* mask of enabled events */
-     __u32                   color_key;      /* transparent pixel */
+     u32                     color_key;      /* transparent pixel */
      DFBRegion               opaque;         /* region of the window forced to be opaque */
 };
 
@@ -129,6 +111,16 @@ dfb_window_move( CoreWindow *window,
                  int         y,
                  bool        relative );
 
+/*
+ * set position and size
+ */
+DFBResult
+dfb_window_set_bounds( CoreWindow *window,
+                       int         x,
+                       int         y,
+                       int         width,
+                       int         height );
+
 /*
  * resizes a window
  */
@@ -187,14 +179,14 @@ dfb_window_putbelow( CoreWindow *window,
  */
 DFBResult
 dfb_window_set_colorkey( CoreWindow *window,
-                         __u32       color_key );
+                         u32         color_key );
 
 /*
  * sets the global alpha factor
  */
 DFBResult
 dfb_window_set_opacity( CoreWindow *window,
-                        __u8        opacity );
+                        u8          opacity );
 
 /*
  * sets the window options
