@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -34,6 +35,7 @@
 #include <core/coretypes.h>
 
 #include <core/gfxcard.h>
+#include <core/surfaces.h>
 #include <core/windows.h>
 
 #include <direct/list.h>
@@ -100,19 +102,22 @@ struct __DFB_CoreWindowStack {
      int                 num;
 
      struct {
-          int            enabled;         /* is cursor enabled ? */
-          int            x, y;            /* cursor position */
-          CoreWindow    *window;          /* super-toplevel-window
-                                             for software cursor */
-          __u8           opacity;         /* cursor opacity */
-          DFBRegion      region;          /* cursor is clipped by this region */
+          int                enabled;     /* is cursor enabled ? */
+          int                x, y;        /* cursor position */
+          DFBDimension       size;        /* cursor shape size */
+          DFBPoint           hot;         /* hot spot */
+          CoreSurface       *surface;     /* shape */
+          u8                 opacity;     /* cursor opacity */
+          DFBRegion          region;      /* cursor is clipped by this region */
 
-          int            numerator;       /* cursor acceleration */
-          int            denominator;
-          int            threshold;
+          int                numerator;   /* cursor acceleration */
+          int                denominator;
+          int                threshold;
 
-          bool           set;             /* cursor enable/disable has
+          bool               set;         /* cursor enable/disable has
                                              been called at least one time */
+
+          CoreSurfacePolicy  policy;
      } cursor;
 
      /* stores information on handling the background on exposure */
@@ -141,9 +146,6 @@ struct __DFB_CoreWindowStack {
 
 
 /* global reactions */
-ReactionResult _dfb_window_surface_listener              ( const void *msg_data,
-                                                           void       *ctx );
-
 ReactionResult _dfb_windowstack_inputdevice_listener     ( const void *msg_data,
                                                            void       *ctx );
 
