@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -51,6 +52,7 @@
 #include <core/surfaces.h>
 #include <core/windows.h>
 #include <core/windowstack.h>
+#include <core/wm.h>
 
 #include <gfx/convert.h>
 
@@ -68,7 +70,8 @@
 
 IDirectFB *idirectfb_singleton = NULL;
 
-static DFBResult apply_configuration( IDirectFB *dfb );
+static DFBResult apply_configuration( IDirectFB        *dfb,
+                                      CoreLayerContext *context );
 
 static DFBResult CreateRemote( const char *host, int session, IDirectFB **ret_interface );
 
@@ -155,6 +158,7 @@ DFBResult
 DirectFBCreate( IDirectFB **interface )
 {
      DFBResult  ret;
+     IDirectFB *dfb;
      CoreDFB   *core_dfb;
 
      if (!dfb_config) {
@@ -176,12 +180,11 @@ DirectFBCreate( IDirectFB **interface )
      if (!direct_config->quiet && dfb_config->banner) {
           direct_log_printf( NULL,
                              "\n"
-                             "       ---------------------- DirectFB v%d.%d.%d ---------------------\n"
-                             "             (c) 2000-2002  convergence integrated media GmbH  \n"
-                             "             (c) 2002-2004  convergence GmbH                   \n"
-                             "        -----------------------------------------------------------\n"
-                             "\n",
-                             DIRECTFB_MAJOR_VERSION, DIRECTFB_MINOR_VERSION, DIRECTFB_MICRO_VERSION );
+                             "     =======================|  DirectFB " DIRECTFB_VERSION "  |=======================\n"
+                             "          (c) 2001-2007  The DirectFB Organization (directfb.org)\n"
+                             "          (c) 2000-2004  Convergence (integrated media) GmbH\n"
+                             "        ------------------------------------------------------------\n"
+                             "\n" );
      }
 
      if (dfb_config->remote.host)
@@ -191,25 +194,52 @@ DirectFBCreate( IDirectFB **interface )
      if (ret)
           return ret;
 
-     DIRECT_ALLOCATE_INTERFACE( idirectfb_singleton, IDirectFB );
+     DIRECT_ALLOCATE_INTERFACE( dfb, IDirectFB );
 
-     ret = IDirectFB_Construct( idirectfb_singleton, core_dfb );
+     ret = IDirectFB_Construct( dfb, core_dfb );
      if (ret) {
-          idirectfb_singleton = NULL;
           dfb_core_destroy( core_dfb, false );
           return ret;
      }
 
      if (dfb_core_is_master( core_dfb )) {
-          ret = apply_configuration( idirectfb_singleton );
+          CoreLayer                  *layer;
+          CoreLayerContext           *context;
+          CoreWindowStack            *stack;
+
+          /* the primary layer */
+          layer = dfb_layer_at_translated( DLID_PRIMARY );
+
+          /* get the default (shared) context */
+          ret = dfb_layer_get_primary_context( layer, false, &context );
           if (ret) {
-               idirectfb_singleton->Release( idirectfb_singleton );
-               idirectfb_singleton = NULL;
+               D_ERROR( "DirectFB/DirectFBCreate: "
+                        "Could not get default context of primary layer!\n" );
+               dfb->Release( dfb );
                return ret;
           }
+
+          ret = apply_configuration( dfb, context );
+          if (ret) {
+               dfb_layer_context_unref( context );
+               dfb->Release( dfb );
+               return ret;
+          }
+
+          stack = dfb_layer_context_windowstack( context );
+          D_ASSERT( stack != NULL );
+         
+          /* not fatal */
+          ret = dfb_wm_start_desktop( stack );
+          if (ret)
+               D_ERROR( "DirectFB/DirectFBCreate: Could not start desktop!\n" );
+
+          dfb_layer_context_unref( context );
+
+          dfb_core_activate( core_dfb );
      }
 
-     *interface = idirectfb_singleton;
+     *interface = idirectfb_singleton = dfb;
 
      return DFB_OK;
 }
@@ -247,30 +277,18 @@ DirectFBErrorFatal( const char *msg, DFBResult error )
 /**************************************************************************************************/
 
 static DFBResult
-apply_configuration( IDirectFB *dfb )
+apply_configuration( IDirectFB        *dfb,
+                     CoreLayerContext *context )
 {
-     DFBResult              ret;
-     CoreLayer             *layer;
-     CoreLayerContext      *context;
-     CoreWindowStack       *stack;
-     DFBDisplayLayerConfig  layer_config;
-
-     /* the primary layer */
-     layer = dfb_layer_at_translated( DLID_PRIMARY );
-
-     /* get the default (shared) context */
-     ret = dfb_layer_get_primary_context( layer, true, &context );
-     if (ret) {
-          D_ERROR( "DirectFB/DirectFBCreate: "
-                    "Could not get default context of primary layer!\n" );
-          return ret;
-     }
+     DFBResult                   ret;
+     CoreWindowStack            *stack;
+     DFBDisplayLayerConfig       layer_config;
+     DFBDisplayLayerConfigFlags  fail;
 
      stack = dfb_layer_context_windowstack( context );
-
      D_ASSERT( stack != NULL );
 
-     /* set buffer mode for desktop */
+     /* set default desktop configuration */
      layer_config.flags = DLCONF_BUFFERMODE;
 
      if (dfb_config->buffer_mode == -1) {
@@ -285,21 +303,66 @@ apply_configuration( IDirectFB *dfb )
      }
      else
           layer_config.buffermode = dfb_config->buffer_mode;
-
-     if (dfb_layer_context_set_configuration( context, &layer_config )) {
-          D_ERROR( "DirectFB/DirectFBCreate: "
-                    "Setting desktop buffer mode failed!\n"
-                    "     -> No virtual resolution support or not enough memory?\n"
-                    "        Falling back to system back buffer.\n" );
-
-          layer_config.buffermode = DLBM_BACKSYSTEM;
-
-          if (dfb_layer_context_set_configuration( context, &layer_config ))
-               D_ERROR( "DirectFB/DirectFBCreate: "
-                         "Setting system memory desktop back buffer failed!\n"
-                         "     -> Using front buffer only mode.\n" );
+     
+     if (dfb_config->mode.width > 0 && dfb_config->mode.height > 0) {
+          layer_config.flags |= DLCONF_WIDTH | DLCONF_HEIGHT;
+          layer_config.width  = dfb_config->mode.width;
+          layer_config.height = dfb_config->mode.height;
      }
 
+     if (dfb_config->mode.format != DSPF_UNKNOWN) {
+          layer_config.flags |= DLCONF_PIXELFORMAT;
+          layer_config.pixelformat = dfb_config->mode.format;
+     }
+     else if (dfb_config->mode.depth > 0) {
+          DFBSurfacePixelFormat format;
+
+          format = dfb_pixelformat_for_depth( dfb_config->mode.depth );
+          if (format != DSPF_UNKNOWN) {
+               layer_config.flags |= DLCONF_PIXELFORMAT;
+               layer_config.pixelformat = format;
+          }
+     }
+
+     if (dfb_layer_context_test_configuration( context, &layer_config, &fail )) {
+          if (fail & (DLCONF_WIDTH | DLCONF_HEIGHT)) {
+               D_ERROR( "DirectFB/DirectFBCreate: "
+                        "Setting desktop resolution to %dx%d failed!\n"
+                        "     -> Using default resolution.\n",
+                        layer_config.width, layer_config.height );
+               
+               layer_config.flags &= ~(DLCONF_WIDTH | DLCONF_HEIGHT);
+          }
+
+          if (fail & DLCONF_PIXELFORMAT) { 
+               D_ERROR( "DirectFB/DirectFBCreate: "
+                        "Setting desktop format failed!\n"
+                        "     -> Using default format.\n" );
+               
+               layer_config.flags &= ~DLCONF_PIXELFORMAT;
+          }
+
+          if (fail & DLCONF_BUFFERMODE) {
+               D_ERROR( "DirectFB/DirectFBCreate: "
+                        "Setting desktop buffer mode failed!\n"
+                        "     -> No virtual resolution support or not enough memory?\n"
+                        "        Falling back to system back buffer.\n" );
+
+               layer_config.buffermode = DLBM_BACKSYSTEM;
+
+               if (dfb_layer_context_test_configuration( context, &layer_config, &fail )) {
+                    D_ERROR( "DirectFB/DirectFBCreate: "
+                             "Setting system memory desktop back buffer failed!\n"
+                             "     -> Using front buffer only mode.\n" );
+                    
+                    layer_config.flags &= ~DLCONF_BUFFERMODE;
+               }
+          }
+     }
+
+     if (layer_config.flags)
+          dfb_layer_context_set_configuration( context, &layer_config );
+
      /* temporarily disable background */
      dfb_windowstack_set_background_mode( stack, DLBM_DONTCARE );
 
@@ -318,7 +381,6 @@ apply_configuration( IDirectFB *dfb )
           ret = dfb->CreateImageProvider( dfb, dfb_config->layer_bg_filename, &provider );
           if (ret) {
                DirectFBError( "Failed loading background image", ret );
-               dfb_layer_context_unref( context );
                return DFB_INIT;
           }
 
@@ -339,21 +401,15 @@ apply_configuration( IDirectFB *dfb )
           ret = dfb->CreateSurface( dfb, &desc, &image );
           if (ret) {
                DirectFBError( "Failed creating surface for background image", ret );
-
                provider->Release( provider );
-
-               dfb_layer_context_unref( context );
                return DFB_INIT;
           }
 
           ret = provider->RenderTo( provider, image, NULL );
           if (ret) {
                DirectFBError( "Failed loading background image", ret );
-
                image->Release( image );
                provider->Release( provider );
-
-               dfb_layer_context_unref( context );
                return DFB_INIT;
           }
 
@@ -369,8 +425,6 @@ apply_configuration( IDirectFB *dfb )
      /* now set the background mode */
      dfb_windowstack_set_background_mode( stack, dfb_config->layer_bg_mode );
 
-     dfb_layer_context_unref( context );
-
      return DFB_OK;
 }
 
