@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -70,14 +71,16 @@ D_DEBUG_DOMAIN( Layer, "IDirectFBDisplayLayer", "Display Layer Interface" );
  * private data struct of IDirectFB
  */
 typedef struct {
-     int                              ref;     /* reference counter */
-     DFBDisplayLayerDescription       desc;    /* description of the layer's caps */
-     DFBDisplayLayerCooperativeLevel  level;   /* current cooperative level */
-     CoreScreen                      *screen;  /* layer's screen */
-     CoreLayer                       *layer;   /* core layer data */
-     CoreLayerContext                *context; /* shared or exclusive context */
-     CoreLayerRegion                 *region;  /* primary region of the context */
-     CoreWindowStack                 *stack;   /* stack of shared context */
+     int                              ref;              /* reference counter */
+     DFBDisplayLayerDescription       desc;             /* description of the layer's caps */
+     DFBDisplayLayerCooperativeLevel  level;            /* current cooperative level */
+     CoreScreen                      *screen;           /* layer's screen */
+     CoreLayer                       *layer;            /* core layer data */
+     CoreLayerContext                *context;          /* shared or exclusive context */
+     CoreLayerRegion                 *region;           /* primary region of the context */
+     CoreWindowStack                 *stack;            /* stack of shared context */
+     DFBBoolean                       switch_exclusive; /* switch to exclusive context after creation? */
+     CoreDFB                         *core;
 } IDirectFBDisplayLayer_data;
 
 
@@ -177,7 +180,7 @@ IDirectFBDisplayLayer_GetSurface( IDirectFBDisplayLayer  *thiz,
      DIRECT_ALLOCATE_INTERFACE( surface, IDirectFBSurface );
 
      ret = IDirectFBSurface_Layer_Construct( surface, NULL, NULL,
-                                             region, DSCAPS_NONE );
+                                             region, DSCAPS_NONE, data->core );
 
      *interface = ret ? NULL : surface;
 
@@ -240,7 +243,7 @@ IDirectFBDisplayLayer_SetCooperativeLevel( IDirectFBDisplayLayer           *thiz
                     data->context = context;
                     data->region  = region;
                     data->stack   = dfb_layer_context_windowstack( data->context );
-                }
+               }
 
                break;
 
@@ -249,10 +252,12 @@ IDirectFBDisplayLayer_SetCooperativeLevel( IDirectFBDisplayLayer           *thiz
                if (ret)
                     return ret;
 
-               ret = dfb_layer_activate_context( data->layer, context );
-               if (ret) {
-                    dfb_layer_context_unref( context );
-                    return ret;
+               if (data->switch_exclusive) {
+                    ret = dfb_layer_activate_context( data->layer, context );
+                    if (ret) {
+                         dfb_layer_context_unref( context );
+                         return ret;
+                    }
                }
 
                ret = dfb_layer_context_get_primary_region( context, true, &region );
@@ -281,7 +286,7 @@ IDirectFBDisplayLayer_SetCooperativeLevel( IDirectFBDisplayLayer           *thiz
 
 static DFBResult
 IDirectFBDisplayLayer_SetOpacity( IDirectFBDisplayLayer *thiz,
-                                  __u8                   opacity )
+                                  u8                     opacity )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
 
@@ -310,6 +315,26 @@ IDirectFBDisplayLayer_SetFieldParity( IDirectFBDisplayLayer *thiz, int field )
      return dfb_layer_context_set_field_parity( data->context, field );
 }
 
+static DFBResult
+IDirectFBDisplayLayer_SetClipRegions( IDirectFBDisplayLayer *thiz,
+                                      const DFBRegion       *regions,
+                                      int                    num_regions,
+                                      DFBBoolean             positive )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
+
+     if (!regions || num_regions < 1)
+          return DFB_INVARG;
+
+     if (num_regions > data->desc.clip_regions)
+          return DFB_UNSUPPORTED;
+
+     if (data->level != DLSCL_EXCLUSIVE)
+          return DFB_ACCESSDENIED;
+
+     return dfb_layer_context_set_clip_regions( data->context, regions, num_regions, positive );
+}
+
 static DFBResult
 IDirectFBDisplayLayer_SetSourceRectangle( IDirectFBDisplayLayer *thiz,
                                           int                    x,
@@ -394,9 +419,9 @@ IDirectFBDisplayLayer_SetScreenRectangle( IDirectFBDisplayLayer *thiz,
 
 static DFBResult
 IDirectFBDisplayLayer_SetSrcColorKey( IDirectFBDisplayLayer *thiz,
-                                      __u8                   r,
-                                      __u8                   g,
-                                      __u8                   b )
+                                      u8                     r,
+                                      u8                     g,
+                                      u8                     b )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
 
@@ -408,9 +433,9 @@ IDirectFBDisplayLayer_SetSrcColorKey( IDirectFBDisplayLayer *thiz,
 
 static DFBResult
 IDirectFBDisplayLayer_SetDstColorKey( IDirectFBDisplayLayer *thiz,
-                                      __u8                   r,
-                                      __u8                   g,
-                                      __u8                   b )
+                                      u8                     r,
+                                      u8                     g,
+                                      u8                     b )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
 
@@ -561,7 +586,7 @@ IDirectFBDisplayLayer_SetBackgroundImage( IDirectFBDisplayLayer *thiz,
 
 static DFBResult
 IDirectFBDisplayLayer_SetBackgroundColor( IDirectFBDisplayLayer *thiz,
-                                          __u8 r, __u8 g, __u8 b, __u8 a )
+                                          u8 r, u8 g, u8 b, u8 a )
 {
      DFBColor color = { a: a, r: r, g: g, b: b };
 
@@ -612,7 +637,7 @@ IDirectFBDisplayLayer_CreateWindow( IDirectFBDisplayLayer       *thiz,
      if (width < 1 || width > 4096 || height < 1 || height > 4096)
           return DFB_INVARG;
 
-     ret = dfb_layer_context_create_window( data->context,
+     ret = dfb_layer_context_create_window( data->core, data->context,
                                             posx, posy, width, height, caps,
                                             surface_caps, format, &w );
      if (ret)
@@ -620,7 +645,7 @@ IDirectFBDisplayLayer_CreateWindow( IDirectFBDisplayLayer       *thiz,
 
      DIRECT_ALLOCATE_INTERFACE( *window, IDirectFBWindow );
 
-     return IDirectFBWindow_Construct( *window, w, data->layer );
+     return IDirectFBWindow_Construct( *window, w, data->layer, data->core );
 }
 
 static DFBResult
@@ -634,9 +659,10 @@ IDirectFBDisplayLayer_GetWindow( IDirectFBDisplayLayer  *thiz,
 
      if (!window)
           return DFB_INVARG;
-
-     if (data->level == DLSCL_SHARED)
-          return DFB_ACCESSDENIED;
+   
+      //remove for now
+     //if (data->level == DLSCL_SHARED)
+     //    return DFB_ACCESSDENIED;
 
      w = dfb_layer_context_find_window( data->context, id );
      if (!w)
@@ -644,7 +670,7 @@ IDirectFBDisplayLayer_GetWindow( IDirectFBDisplayLayer  *thiz,
 
      DIRECT_ALLOCATE_INTERFACE( *window, IDirectFBWindow );
 
-     return IDirectFBWindow_Construct( *window, w, data->layer );
+     return IDirectFBWindow_Construct( *window, w, data->layer, data->core );
 }
 
 static DFBResult
@@ -655,7 +681,7 @@ IDirectFBDisplayLayer_EnableCursor( IDirectFBDisplayLayer *thiz, int enable )
      if (data->level == DLSCL_SHARED)
           return DFB_ACCESSDENIED;
 
-     return dfb_windowstack_cursor_enable( data->stack, enable );
+     return dfb_windowstack_cursor_enable( data->core, data->stack, enable );
 }
 
 static DFBResult
@@ -730,7 +756,7 @@ IDirectFBDisplayLayer_SetCursorShape( IDirectFBDisplayLayer *thiz,
 
 static DFBResult
 IDirectFBDisplayLayer_SetCursorOpacity( IDirectFBDisplayLayer *thiz,
-                                        __u8                   opacity )
+                                        u8                     opacity )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
 
@@ -798,9 +824,36 @@ IDirectFBDisplayLayer_GetSourceDescriptions( IDirectFBDisplayLayer            *t
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBDisplayLayer_SwitchContext( IDirectFBDisplayLayer *thiz,
+                                     DFBBoolean             exclusive )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
+
+     if (!exclusive && data->level == DLSCL_EXCLUSIVE) {
+          DFBResult         ret;
+          CoreLayerContext *context;
+
+          ret = dfb_layer_get_primary_context( data->layer, false, &context );
+          if (ret)
+               return ret;
+
+          dfb_layer_activate_context( data->layer, context );
+
+          dfb_layer_context_unref( context );
+     }
+     else
+         dfb_layer_activate_context( data->layer, data->context );
+
+     data->switch_exclusive = exclusive;
+
+     return DFB_OK;
+}
+
 DFBResult
 IDirectFBDisplayLayer_Construct( IDirectFBDisplayLayer *thiz,
-                                 CoreLayer             *layer )
+                                 CoreLayer             *layer,
+                                 CoreDFB               *core )
 {
      DFBResult         ret;
      CoreLayerContext *context;
@@ -821,12 +874,14 @@ IDirectFBDisplayLayer_Construct( IDirectFBDisplayLayer *thiz,
           return ret;
      }
 
-     data->ref     = 1;
-     data->screen  = dfb_layer_screen( layer );
-     data->layer   = layer;
-     data->context = context;
-     data->region  = region;
-     data->stack   = dfb_layer_context_windowstack( context );
+     data->ref              = 1;
+     data->core             = core;
+     data->screen           = dfb_layer_screen( layer );
+     data->layer            = layer;
+     data->context          = context;
+     data->region           = region;
+     data->stack            = dfb_layer_context_windowstack( context );
+     data->switch_exclusive = DFB_TRUE;
 
      dfb_layer_get_description( data->layer, &data->desc );
 
@@ -862,10 +917,12 @@ IDirectFBDisplayLayer_Construct( IDirectFBDisplayLayer *thiz,
      thiz->SetCursorShape        = IDirectFBDisplayLayer_SetCursorShape;
      thiz->SetCursorOpacity      = IDirectFBDisplayLayer_SetCursorOpacity;
      thiz->SetFieldParity        = IDirectFBDisplayLayer_SetFieldParity;
+     thiz->SetClipRegions        = IDirectFBDisplayLayer_SetClipRegions;
      thiz->WaitForSync           = IDirectFBDisplayLayer_WaitForSync;
      thiz->GetSourceDescriptions = IDirectFBDisplayLayer_GetSourceDescriptions;
      thiz->SetScreenPosition     = IDirectFBDisplayLayer_SetScreenPosition;
      thiz->SetScreenRectangle    = IDirectFBDisplayLayer_SetScreenRectangle;
+     thiz->SwitchContext         = IDirectFBDisplayLayer_SwitchContext;
 
      return DFB_OK;
 }
