@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -37,6 +38,9 @@
 
 #include <core/layers.h>
 #include <core/screen.h>
+#include <core/screens.h>
+
+#include <misc/conf.h>
 
 #include <direct/interface.h>
 
@@ -134,6 +138,31 @@ IDirectFBScreen_GetDescription( IDirectFBScreen      *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBScreen_GetSize( IDirectFBScreen *thiz,
+                         int             *ret_width,
+                         int             *ret_height )
+{
+     DFBResult ret;
+     int       width  = 0;
+     int       height = 0;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBScreen)
+
+     if (!ret_width && !ret_height)
+          return DFB_INVARG;
+
+     ret = dfb_screen_get_screen_size( data->screen, &width, &height );
+
+     if (ret_width)
+          *ret_width = width;
+
+     if (ret_height)
+          *ret_height = height;
+
+     return ret;
+}
+
 static DFBResult
 IDirectFBScreen_EnumDisplayLayers( IDirectFBScreen         *thiz,
                                    DFBDisplayLayerCallback  callbackfunc,
@@ -515,12 +544,14 @@ IDirectFBScreen_Construct( IDirectFBScreen *thiz,
      data->ref    = 1;
      data->screen = screen;
 
-     dfb_screen_get_info( screen, &data->id, &data->description );
+     dfb_screen_get_info( screen, NULL, &data->description );
+     data->id = dfb_screen_id_translated( data->screen );
 
      thiz->AddRef                   = IDirectFBScreen_AddRef;
      thiz->Release                  = IDirectFBScreen_Release;
      thiz->GetID                    = IDirectFBScreen_GetID;
      thiz->GetDescription           = IDirectFBScreen_GetDescription;
+     thiz->GetSize                  = IDirectFBScreen_GetSize;
      thiz->EnumDisplayLayers        = IDirectFBScreen_EnumDisplayLayers;
      thiz->SetPowerMode             = IDirectFBScreen_SetPowerMode;
      thiz->WaitForSync              = IDirectFBScreen_WaitForSync;
@@ -591,6 +622,12 @@ PatchEncoderConfig( DFBScreenEncoderConfig       *patched,
      if (patch->flags & DSECONF_ADJUSTMENT)
           patched->adjustment = patch->adjustment;
 
+     if (patch->flags & DSECONF_CONNECTORS)
+          patched->out_connectors = patch->out_connectors;
+
+     if (patch->flags & DSECONF_SLOW_BLANKING)
+          patched->slow_blanking = patch->slow_blanking;
+
      return DFB_OK;
 }
 
@@ -611,6 +648,9 @@ PatchOutputConfig( DFBScreenOutputConfig       *patched,
      if (patch->flags & DSOCONF_CONNECTORS)
           patched->out_connectors = patch->out_connectors;
 
+     if (patch->flags & DSOCONF_SLOW_BLANKING)
+          patched->slow_blanking = patch->slow_blanking;
+
      return DFB_OK;
 }
 
@@ -618,14 +658,19 @@ static DFBEnumerationResult
 EnumDisplayLayers_Callback( CoreLayer *layer, void *ctx )
 {
      DFBDisplayLayerDescription  desc;
+     DFBDisplayLayerID           id;
      EnumDisplayLayers_Context  *context = (EnumDisplayLayers_Context*) ctx;
 
      if (dfb_layer_screen( layer ) != context->screen)
           return DFENUM_OK;
 
+     id = dfb_layer_id_translated( layer );
+
+     if (dfb_config->primary_only && id != DLID_PRIMARY)
+          return DFENUM_OK;
+
      dfb_layer_get_description( layer, &desc );
 
-     return context->callback( dfb_layer_id_translated( layer ), desc,
-                               context->callback_ctx );
+     return context->callback( id, desc, context->callback_ctx );
 }
 
