@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -202,6 +203,26 @@ IDirectFBSurface_GetAccelerationMask( IDirectFBSurface    *thiz,
 
      return DFB_OK;
 }
+static DFBResult
+IDirectFBSurface_GetPosition( IDirectFBSurface *thiz,
+                          int              *x,
+                          int              *y )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
+
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
+     if (!x && !y)
+          return DFB_INVARG;
+
+     if (x)
+          *x = data->area.wanted.x;
+
+     if (y)
+          *y = data->area.wanted.y;
+
+     return DFB_OK;
+}
 
 static DFBResult
 IDirectFBSurface_GetSize( IDirectFBSurface *thiz,
@@ -327,10 +348,10 @@ IDirectFBSurface_SetPalette( IDirectFBSurface *thiz,
 
 static DFBResult
 IDirectFBSurface_SetAlphaRamp( IDirectFBSurface *thiz,
-                               __u8              a0,
-                               __u8              a1,
-                               __u8              a2,
-                               __u8              a3 )
+                               u8                a0,
+                               u8                a1,
+                               u8                a2,
+                               u8                a3 )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
@@ -372,7 +393,7 @@ IDirectFBSurface_Lock( IDirectFBSurface *thiz,
 
      front = (flags & DSLF_WRITE) ? 0 : 1;
 
-     ret = dfb_surface_soft_lock( data->surface, flags, &ptr, &pitch, front );
+     ret = dfb_surface_soft_lock( data->core, data->surface, flags, &ptr, &pitch, front );
      if (ret)
           return ret;
 
@@ -387,6 +408,43 @@ IDirectFBSurface_Lock( IDirectFBSurface *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBSurface_GetFramebufferOffset( IDirectFBSurface *thiz,
+                                       int *offset )
+{
+     int            front;
+     SurfaceBuffer *buffer;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
+
+     if (!data->surface)
+          return DFB_DESTROYED;
+
+     if (!offset)
+          return DFB_INVARG;
+
+     if (!data->locked)
+          return DFB_ACCESSDENIED;
+
+     front = data->locked - 1; /* As set by Lock(). */
+
+     if (front)
+          buffer = data->surface->front_buffer;
+     else
+          buffer = data->surface->back_buffer;
+
+     if (!buffer)
+          return DFB_BUG;
+
+     if (!buffer->video.locked) {
+          /* The surface is probably in a system buffer if video is unlocked. */
+          return DFB_UNSUPPORTED;
+     }
+
+     *offset = buffer->video.offset;
+     return DFB_OK;
+}
+
 static DFBResult
 IDirectFBSurface_Unlock( IDirectFBSurface *thiz )
 {
@@ -475,7 +533,7 @@ IDirectFBSurface_SetField( IDirectFBSurface    *thiz,
 
 static DFBResult
 IDirectFBSurface_Clear( IDirectFBSurface *thiz,
-                        __u8 r, __u8 g, __u8 b, __u8 a )
+                        u8 r, u8 g, u8 b, u8 a )
 {
      DFBColor                old_color;
      unsigned int            old_index;
@@ -509,8 +567,8 @@ IDirectFBSurface_Clear( IDirectFBSurface *thiz,
      if (DFB_PIXELFORMAT_IS_INDEXED( surface->format ))
           dfb_state_set_color_index( &data->state,
                                      dfb_palette_search( surface->palette, r, g, b, a ) );
-     else
-          dfb_state_set_color( &data->state, &color );
+
+     dfb_state_set_color( &data->state, &color );
 
      /* fill the visible rectangle */
      dfb_gfxcard_fillrectangles( &data->area.current, 1, &data->state );
@@ -525,8 +583,8 @@ IDirectFBSurface_Clear( IDirectFBSurface *thiz,
      /* restore color */
      if (DFB_PIXELFORMAT_IS_INDEXED( surface->format ))
           dfb_state_set_color_index( &data->state, old_index );
-     else
-          dfb_state_set_color( &data->state, &old_color );
+
+     dfb_state_set_color( &data->state, &old_color );
 
      return DFB_OK;
 }
@@ -587,7 +645,7 @@ IDirectFBSurface_GetClip( IDirectFBSurface *thiz, DFBRegion *ret_clip )
 
 static DFBResult
 IDirectFBSurface_SetColor( IDirectFBSurface *thiz,
-                           __u8 r, __u8 g, __u8 b, __u8 a )
+                           u8 r, u8 g, u8 b, u8 a )
 {
      CoreSurface *surface;
      DFBColor     color = { a, r, g, b };
@@ -744,6 +802,22 @@ IDirectFBSurface_SetPorterDuff( IDirectFBSurface         *thiz,
                src = DSBF_ZERO;
                dst = DSBF_INVSRCALPHA;
                break;
+          case DSPD_SRC_ATOP:
+               src = DSBF_DESTALPHA;
+               dst = DSBF_INVSRCALPHA;
+               break;
+          case DSPD_DST_ATOP:
+               src = DSBF_INVDESTALPHA;
+               dst = DSBF_SRCALPHA;
+               break;
+          case DSPD_ADD:
+               src = DSBF_ONE;
+               dst = DSBF_ONE;
+               break;
+          case DSPD_XOR:
+               src = DSBF_INVDESTALPHA;
+               dst = DSBF_INVSRCALPHA;
+               break;
           default:
                return DFB_INVARG;
      }
@@ -756,9 +830,9 @@ IDirectFBSurface_SetPorterDuff( IDirectFBSurface         *thiz,
 
 static DFBResult
 IDirectFBSurface_SetSrcColorKey( IDirectFBSurface *thiz,
-                                 __u8              r,
-                                 __u8              g,
-                                 __u8              b )
+                                 u8                r,
+                                 u8                g,
+                                 u8                b )
 {
      CoreSurface *surface;
 
@@ -827,9 +901,9 @@ IDirectFBSurface_SetSrcColorKeyIndex( IDirectFBSurface *thiz,
 
 static DFBResult
 IDirectFBSurface_SetDstColorKey( IDirectFBSurface *thiz,
-                                 __u8              r,
-                                 __u8              g,
-                                 __u8              b )
+                                 u8                r,
+                                 u8                g,
+                                 u8                b )
 {
      CoreSurface *surface;
 
@@ -892,6 +966,31 @@ IDirectFBSurface_SetDstColorKeyIndex( IDirectFBSurface *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBSurface_SetIndexTranslation( IDirectFBSurface *thiz,
+                                      const int        *indices,
+                                      int               num_indices )
+{
+     CoreSurface *surface;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
+
+     surface = data->surface;
+     if (!surface)
+          return DFB_DESTROYED;
+
+     if (! DFB_PIXELFORMAT_IS_INDEXED( surface->format ))
+          return DFB_UNSUPPORTED;
+
+     if (!indices && num_indices > 0)
+          return DFB_INVAREA;
+
+     if (num_indices < 0 || num_indices > 256)
+          return DFB_INVARG;
+
+     return dfb_state_set_index_translation( &data->state, indices, num_indices );
+}
+
 static DFBResult
 IDirectFBSurface_SetFont( IDirectFBSurface *thiz,
                           IDirectFBFont    *font )
@@ -904,9 +1003,15 @@ IDirectFBSurface_SetFont( IDirectFBSurface *thiz,
 
      if (data->font != font) {
          if (font) {
+              IDirectFBFont_data *font_data;
+
               ret = font->AddRef( font );
               if (ret)
                    return ret;
+
+              DIRECT_INTERFACE_GET_DATA_FROM( font, font_data, IDirectFBFont );
+
+              data->encoding = font_data->encoding;
          }
 
          if (data->font) {
@@ -928,22 +1033,29 @@ IDirectFBSurface_SetFont( IDirectFBSurface *thiz,
 
 static DFBResult
 IDirectFBSurface_GetFont( IDirectFBSurface  *thiz,
-                          IDirectFBFont    **font )
+                          IDirectFBFont    **ret_font )
 {
+     DFBResult      ret;
+     IDirectFBFont *font;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
      D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
 
-     if (!font)
+     if (!ret_font)
           return DFB_INVARG;
 
-     if (!data->font) {
-	  *font = NULL;
+     font = data->font;
+     if (!font) {
+          *ret_font = NULL;
           return DFB_MISSINGFONT;
      }
 
-     data->font->AddRef (data->font);
-     *font = data->font;
+     ret = font->AddRef( font );
+     if (ret)
+          return ret;
+
+     *ret_font = font;
 
      return DFB_OK;
 }
@@ -997,8 +1109,6 @@ static DFBResult
 IDirectFBSurface_DrawLine( IDirectFBSurface *thiz,
                            int x1, int y1, int x2, int y2 )
 {
-     DFBRegion line = { x1, y1, x2, y2 };
-
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
      D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
@@ -1013,12 +1123,42 @@ IDirectFBSurface_DrawLine( IDirectFBSurface *thiz,
      if (data->locked)
           return DFB_LOCKED;
 
-     line.x1 += data->area.wanted.x;
-     line.x2 += data->area.wanted.x;
-     line.y1 += data->area.wanted.y;
-     line.y2 += data->area.wanted.y;
+     if (x1 == x2 || y1 == y2) {
+          DFBRectangle rect;
 
-     dfb_gfxcard_drawlines( &line, 1, &data->state );
+          if (x1 <= x2) {
+               rect.x = x1;
+               rect.w = x2 - x1 + 1;
+          }
+          else {
+               rect.x = x2;
+               rect.w = x1 - x2 + 1;
+          }
+
+          if (y1 <= y2) {
+               rect.y = y1;
+               rect.h = y2 - y1 + 1;
+          }
+          else {
+               rect.y = y2;
+               rect.h = y1 - y2 + 1;
+          }
+
+          rect.x += data->area.wanted.x;
+          rect.y += data->area.wanted.y;
+
+          dfb_gfxcard_fillrectangles( &rect, 1, &data->state );
+     }
+     else {
+          DFBRegion line = { x1, y1, x2, y2 };
+
+          line.x1 += data->area.wanted.x;
+          line.x2 += data->area.wanted.x;
+          line.y1 += data->area.wanted.y;
+          line.y2 += data->area.wanted.y;
+
+          dfb_gfxcard_drawlines( &line, 1, &data->state );
+     }
 
      return DFB_OK;
 }
@@ -1159,7 +1299,7 @@ IDirectFBSurface_FillRectangles( IDirectFBSurface   *thiz,
           bool          malloced = (num_rects > 256);
 
           if (malloced)
-               local_rects = malloc( sizeof(DFBRectangle) * num_rects );
+               local_rects = D_MALLOC( sizeof(DFBRectangle) * num_rects );
           else
                local_rects = alloca( sizeof(DFBRectangle) * num_rects );
 
@@ -1173,7 +1313,7 @@ IDirectFBSurface_FillRectangles( IDirectFBSurface   *thiz,
           dfb_gfxcard_fillrectangles( local_rects, num_rects, &data->state );
 
           if (malloced)
-               free( local_rects );
+               D_FREE( local_rects );
      }
      else
           dfb_gfxcard_fillrectangles( rects, num_rects, &data->state );
@@ -1267,7 +1407,6 @@ IDirectFBSurface_Blit( IDirectFBSurface   *thiz,
      if (!src_data->area.current.w || !src_data->area.current.h)
           return DFB_INVAREA;
 
-
      if (sr) {
           if (sr->w < 1  ||  sr->h < 1)
                return DFB_OK;
@@ -1669,7 +1808,10 @@ IDirectFBSurface_DrawString( IDirectFBSurface *thiz,
                              int x, int y,
                              DFBSurfaceTextFlags flags )
 {
+     DFBResult           ret;
+     IDirectFBFont      *font;
      IDirectFBFont_data *font_data;
+     CoreFont           *core_font;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
@@ -1690,64 +1832,93 @@ IDirectFBSurface_DrawString( IDirectFBSurface *thiz,
      if (!data->font)
           return DFB_MISSINGFONT;
 
-
      if (bytes < 0)
           bytes = strlen (text);
 
      if (bytes == 0)
           return DFB_OK;
 
+     font = data->font;
+
+     DIRECT_INTERFACE_GET_DATA_FROM( font, font_data, IDirectFBFont );
+
+     core_font = font_data->font;
+
      if (!(flags & DSTF_TOP)) {
-          int offset = 0;
+          y -= core_font->ascender;
 
-          data->font->GetAscender (data->font, &offset);
-          y -= offset;
-
-          if ((flags & DSTF_BOTTOM)) {
-               offset = 0;
-               data->font->GetDescender (data->font, &offset);
-               y += offset;
-          }
+          if (flags & DSTF_BOTTOM)
+               y += core_font->descender;
      }
 
      if (flags & (DSTF_RIGHT | DSTF_CENTER)) {
-          int width = 0;
+          int          i, num, kx;
+          int          width = 0;
+          unsigned int prev = 0;
+          unsigned int indices[bytes];
 
-          if (data->font->GetStringWidth (data->font,
-                                          text, bytes, &width) == DFB_OK) {
-               if (flags & DSTF_RIGHT) {
-                    x -= width;
-               }
-               else if (flags & DSTF_CENTER) {
-                    x -= width >> 1;
-               }
+          /* FIXME: Avoid double locking and decoding. */
+          dfb_font_lock( core_font );
+
+          /* Decode string to character indices. */
+          ret = dfb_font_decode_text( core_font, data->encoding, text, bytes, indices, &num );
+          if (ret) {
+               dfb_font_unlock( core_font );
+               return ret;
           }
+
+          /* Calculate string width. */
+          for (i=0; i<num; i++) {
+               unsigned int   current = indices[i];
+               CoreGlyphData *glyph;
+
+               if (dfb_font_get_glyph_data( core_font, current, &glyph ) == DFB_OK) {
+                    width += glyph->advance;
+
+                    if (prev && core_font->GetKerning &&
+                        core_font->GetKerning( core_font, prev, current, &kx, NULL ) == DFB_OK)
+                         width += kx;
+               }
+
+               prev = current;
+          }
+
+          dfb_font_unlock( core_font );
+
+          /* Justify. */
+          if (flags & DSTF_RIGHT)
+               x -= width;
+          else if (flags & DSTF_CENTER)
+               x -= width >> 1;
      }
 
-     font_data = (IDirectFBFont_data *)data->font->priv;
-
-     dfb_gfxcard_drawstring( (const unsigned char*) text, bytes,
+     dfb_gfxcard_drawstring( (const unsigned char*) text, bytes, data->encoding,
                              data->area.wanted.x + x, data->area.wanted.y + y,
-                             font_data->font, &data->state );
+                             core_font, &data->state );
 
      return DFB_OK;
 }
 
 static DFBResult
 IDirectFBSurface_DrawGlyph( IDirectFBSurface *thiz,
-                            unsigned int index, int x, int y,
+                            unsigned int character, int x, int y,
                             DFBSurfaceTextFlags flags )
 {
+     DFBResult           ret;
+     IDirectFBFont      *font;
      IDirectFBFont_data *font_data;
+     CoreFont           *core_font;
+     unsigned int        index;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
-     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+     D_DEBUG_AT( Surface, "%s( %p, 0x%x, %d,%d, 0x%x )\n",
+                 __FUNCTION__, thiz, character, x, y, flags );
 
      if (!data->surface)
           return DFB_DESTROYED;
 
-     if (!index)
+     if (!character)
           return DFB_INVARG;
 
      if (!data->area.current.w || !data->area.current.h)
@@ -1759,38 +1930,62 @@ IDirectFBSurface_DrawGlyph( IDirectFBSurface *thiz,
      if (!data->font)
           return DFB_MISSINGFONT;
 
+     font = data->font;
+
+     DIRECT_INTERFACE_GET_DATA_FROM( font, font_data, IDirectFBFont );
+
+     core_font = font_data->font;
+
+     /* FIXME: Avoid double locking. */
+     dfb_font_lock( core_font );
+
+     ret = dfb_font_decode_character( core_font, data->encoding, character, &index );
+     if (ret) {
+          dfb_font_unlock( core_font );
+          return ret;
+     }
+
      if (!(flags & DSTF_TOP)) {
-          int offset = 0;
+          y -= core_font->ascender;
 
-          data->font->GetAscender (data->font, &offset);
-          y -= offset;
-
-          if ((flags & DSTF_BOTTOM)) {
-               offset = 0;
-               data->font->GetDescender (data->font, &offset);
-               y += offset;
-          }
+          if (flags & DSTF_BOTTOM)
+               y += core_font->descender;
      }
 
      if (flags & (DSTF_RIGHT | DSTF_CENTER)) {
-          int advance;
+          CoreGlyphData *glyph;
 
-          if (data->font->GetGlyphExtents (data->font,
-                                           index, NULL, &advance) == DFB_OK) {
-               if (flags & DSTF_RIGHT) {
-                    x -= advance;
-               }
-               else if (flags & DSTF_CENTER) {
-                    x -= advance >> 1;
-               }
+          ret = dfb_font_get_glyph_data( core_font, index, &glyph );
+          if (ret) {
+               dfb_font_unlock( core_font );
+               return ret;
           }
-     }
 
-     font_data = (IDirectFBFont_data *)data->font->priv;
+          if (flags & DSTF_RIGHT)
+               x -= glyph->advance;
+          else if (flags & DSTF_CENTER)
+               x -= glyph->advance >> 1;
+     }
 
      dfb_gfxcard_drawglyph( index,
                             data->area.wanted.x + x, data->area.wanted.y + y,
-                            font_data->font, &data->state );
+                            core_font, &data->state );
+
+     dfb_font_unlock( core_font );
+
+     return DFB_OK;
+}
+
+static DFBResult
+IDirectFBSurface_SetEncoding( IDirectFBSurface  *thiz,
+                              DFBTextEncodingID  encoding )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
+
+     D_DEBUG_AT( Surface, "%s( %p, %d )\n", __FUNCTION__, thiz, encoding );
+
+     /* TODO: check for support or fail later? */
+     data->encoding = encoding;
 
      return DFB_OK;
 }
@@ -1800,7 +1995,7 @@ IDirectFBSurface_GetSubSurface( IDirectFBSurface    *thiz,
                                 const DFBRectangle  *rect,
                                 IDirectFBSurface   **surface )
 {
-     DFBRectangle wanted, granted;
+     DFBResult ret;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
@@ -1812,33 +2007,49 @@ IDirectFBSurface_GetSubSurface( IDirectFBSurface    *thiz,
 
      if (!surface)
           return DFB_INVARG;
-
-     /* Compute wanted rectangle */
-     if (rect) {
-          wanted = *rect;
-
-          wanted.x += data->area.wanted.x;
-          wanted.y += data->area.wanted.y;
-
-          if (wanted.w <= 0 || wanted.h <= 0) {
-               wanted.w = 0;
-               wanted.h = 0;
-          }
-     }
-     else
-          wanted = data->area.wanted;
-
-     /* Compute granted rectangle */
-     granted = wanted;
-
-     dfb_rectangle_intersect( &granted, &data->area.granted );
-
-     /* Allocate and construct */
+          
+     /* Allocate interface */
      DIRECT_ALLOCATE_INTERFACE( *surface, IDirectFBSurface );
 
-     return IDirectFBSurface_Construct( *surface, &wanted, &granted,
-                                        data->surface,
-                                        data->caps | DSCAPS_SUBSURFACE );
+     if (rect || data->limit_set) {
+          DFBRectangle wanted, granted;
+          
+          /* Compute wanted rectangle */
+          if (rect) {
+               wanted = *rect;
+
+               wanted.x += data->area.wanted.x;
+               wanted.y += data->area.wanted.y;
+
+               if (wanted.w <= 0 || wanted.h <= 0) {
+                    wanted.w = 0;
+                    wanted.h = 0;
+               }
+          }
+          else {
+               wanted = data->area.wanted;
+          }
+          
+          /* Compute granted rectangle */
+          granted = wanted;
+
+          dfb_rectangle_intersect( &granted, &data->area.granted );
+          
+          /* Construct */
+          ret = IDirectFBSurface_Construct( *surface, 
+                                            &wanted, &granted, &data->area.insets,
+                                            data->surface,
+                                            data->caps | DSCAPS_SUBSURFACE, data->core );
+     }
+     else {
+          /* Construct */
+          ret = IDirectFBSurface_Construct( *surface, 
+                                            NULL, NULL, &data->area.insets,
+                                            data->surface, 
+                                            data->caps | DSCAPS_SUBSURFACE, data->core );
+     }
+     
+     return ret;
 }
 
 static DFBResult
@@ -1862,7 +2073,7 @@ IDirectFBSurface_GetGL( IDirectFBSurface   *thiz,
           return DFB_INVAREA;
 
 
-     ret = DirectGetInterface( &funcs, "IDirectFBGL", NULL, NULL, NULL );
+     ret = DirectGetInterface( &funcs, "IDirectFBGL", NULL, DirectProbeInterface, thiz );
      if (ret)
           return ret;
 
@@ -1903,7 +2114,7 @@ IDirectFBSurface_Dump( IDirectFBSurface   *thiz,
      if (!surface)
           return DFB_DESTROYED;
 
-     return dfb_surface_dump( surface, directory, prefix );
+     return dfb_surface_dump( data->core, surface, directory, prefix );
 }
 
 static DFBResult
@@ -1922,13 +2133,27 @@ IDirectFBSurface_DisableAcceleration( IDirectFBSurface    *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBSurface_ReleaseSource( IDirectFBSurface *thiz )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
+
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
+     dfb_state_set_source( &data->state, NULL );
+
+     return DFB_OK;
+}
+
 /******/
 
 DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
                                       DFBRectangle           *wanted,
                                       DFBRectangle           *granted,
+                                      DFBInsets              *insets,
                                       CoreSurface            *surface,
-                                      DFBSurfaceCapabilities  caps )
+                                      DFBSurfaceCapabilities  caps,
+                                      CoreDFB                *core )
 {
      DFBRectangle rect = { 0, 0, surface->width, surface->height };
 
@@ -1938,12 +2163,19 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
 
      data->ref = 1;
      data->caps = caps | surface->caps;
+     data->core = core;
 
      if (dfb_surface_ref( surface )) {
           DIRECT_DEALLOCATE_INTERFACE(thiz);
           return DFB_FAILURE;
      }
 
+     /* The area insets */
+     if (insets) {
+          data->area.insets = *insets;
+          dfb_rectangle_subtract( &rect, insets );
+     }
+
      /* The area that was requested */
      if (wanted)
           data->area.wanted = *wanted;
@@ -1959,6 +2191,9 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
      /* The currently accessible rectangle */
      data->area.current = data->area.granted;
      dfb_rectangle_intersect( &data->area.current, &rect );
+     
+     /* Whether granted rectangle is meaningful */
+     data->limit_set = (granted != NULL);
 
      data->surface = surface;
 
@@ -1976,6 +2211,7 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
      thiz->Release = IDirectFBSurface_Release;
 
      thiz->GetCapabilities = IDirectFBSurface_GetCapabilities;
+     thiz->GetPosition = IDirectFBSurface_GetPosition;
      thiz->GetSize = IDirectFBSurface_GetSize;
      thiz->GetVisibleRectangle = IDirectFBSurface_GetVisibleRectangle;
      thiz->GetPixelFormat = IDirectFBSurface_GetPixelFormat;
@@ -1986,6 +2222,7 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
      thiz->SetAlphaRamp = IDirectFBSurface_SetAlphaRamp;
 
      thiz->Lock = IDirectFBSurface_Lock;
+     thiz->GetFramebufferOffset = IDirectFBSurface_GetFramebufferOffset;
      thiz->Unlock = IDirectFBSurface_Unlock;
      thiz->Flip = IDirectFBSurface_Flip;
      thiz->SetField = IDirectFBSurface_SetField;
@@ -2002,6 +2239,7 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
      thiz->SetSrcColorKeyIndex = IDirectFBSurface_SetSrcColorKeyIndex;
      thiz->SetDstColorKey = IDirectFBSurface_SetDstColorKey;
      thiz->SetDstColorKeyIndex = IDirectFBSurface_SetDstColorKeyIndex;
+     thiz->SetIndexTranslation = IDirectFBSurface_SetIndexTranslation;
 
      thiz->SetBlittingFlags = IDirectFBSurface_SetBlittingFlags;
      thiz->Blit = IDirectFBSurface_Blit;
@@ -2023,6 +2261,7 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
      thiz->GetFont = IDirectFBSurface_GetFont;
      thiz->DrawString = IDirectFBSurface_DrawString;
      thiz->DrawGlyph = IDirectFBSurface_DrawGlyph;
+     thiz->SetEncoding = IDirectFBSurface_SetEncoding;
 
      thiz->GetSubSurface = IDirectFBSurface_GetSubSurface;
 
@@ -2030,6 +2269,7 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
 
      thiz->Dump = IDirectFBSurface_Dump;
      thiz->DisableAcceleration = IDirectFBSurface_DisableAcceleration;
+     thiz->ReleaseSource = IDirectFBSurface_ReleaseSource;
 
 
      dfb_surface_attach( surface,
@@ -2061,8 +2301,10 @@ IDirectFBSurface_listener( const void *msg_data, void *ctx )
 
      if (notification->flags & CSNF_SIZEFORMAT) {
           DFBRectangle rect = { 0, 0, surface->width, surface->height };
+          
+          dfb_rectangle_subtract( &rect, &data->area.insets );
 
-          if (data->caps & DSCAPS_SUBSURFACE) {
+          if (data->limit_set) {
                data->area.current = data->area.granted;
 
                dfb_rectangle_intersect( &data->area.current, &rect );
