@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -50,10 +51,14 @@ typedef struct {
          DFBRectangle       wanted;          /* passed to GetSubSurface */
          DFBRectangle       granted;         /* clipped by parent on creation */
          DFBRectangle       current;         /* currently available area */
+         DFBInsets          insets;          /* actually set by the window manager */
      } area;
+     
+     bool                   limit_set;       /* greanted rectangle set?
+                                                (GetSubSurface called with rect != NULL) */
 
      bool                   clip_set;        /* fixed clip set? (SetClip called
-                                                with clip != NULL */
+                                                with clip != NULL) */
      DFBRegion              clip_wanted;     /* last region passed to SetClip
                                                 intersected by wanted area,
                                                 only valid if clip_set != 0 */
@@ -62,24 +67,27 @@ typedef struct {
      CoreSurface            *surface;        /* buffer to show */
      IDirectFBFont          *font;           /* font to use */
      CardState              state;           /* render state to use */
+     DFBTextEncodingID      encoding;        /* text encoding */
 
      struct {
-          __u8              r;               /* red component */
-          __u8              g;               /* green component */
-          __u8              b;               /* blue component */
-          __u32             value;           /* r/g/b in surface's format */
+          u8                r;               /* red component */
+          u8                g;               /* green component */
+          u8                b;               /* blue component */
+          u32               value;           /* r/g/b in surface's format */
      } src_key;                              /* src key for blitting from
                                                 this surface */
 
      struct {
-          __u8              r;               /* red component */
-          __u8              g;               /* green component */
-          __u8              b;               /* blue component */
-          __u32             value;           /* r/g/b in surface's format */
+          u8                r;               /* red component */
+          u8                g;               /* green component */
+          u8                b;               /* blue component */
+          u32               value;           /* r/g/b in surface's format */
      } dst_key;                              /* dst key for blitting to
                                                 this surface */
 
      Reaction               reaction;
+
+     CoreDFB               *core;
 } IDirectFBSurface_data;
 
 /*
@@ -88,8 +96,10 @@ typedef struct {
 DFBResult IDirectFBSurface_Construct( IDirectFBSurface *thiz,
                                       DFBRectangle *req_rect,
                                       DFBRectangle *clip_rect,
-                                      CoreSurface *surface,
-                                      DFBSurfaceCapabilities caps );
+                                      DFBInsets    *insets,
+                                      CoreSurface  *surface,
+                                      DFBSurfaceCapabilities  caps,
+                                      CoreDFB                *core );
 
 /*
  * destroys surface(s) and frees private data
