@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -242,16 +243,16 @@ dfb_clip_stretchblit( const DFBRegion *clip,
 
      if (drect->x != orig_dst.x)
           srect->x += (int)( (drect->x - orig_dst.x) *
-                             (srect->w / (float)orig_dst.w) + 0.5f );
+                             (srect->w / (float)orig_dst.w) );
 
      if (drect->y != orig_dst.y)
           srect->y += (int)( (drect->y - orig_dst.y) *
-                             (srect->h / (float)orig_dst.h) + 0.5f );
+                             (srect->h / (float)orig_dst.h) );
 
      if (drect->w != orig_dst.w)
-          srect->w = D_ICEIL(srect->w * (drect->w / (float)orig_dst.w));
+          srect->w = (int)( srect->w * (drect->w / (float)orig_dst.w) );
 
      if (drect->h != orig_dst.h)
-          srect->h = D_ICEIL(srect->h * (drect->h / (float)orig_dst.h));
+          srect->h = (int)( srect->h * (drect->h / (float)orig_dst.h) );
 }
 
