@@ -1,13 +1,13 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
               Sven Neumann <neo@directfb.org>,
-              Ville Syrjl <syrjala@sci.fi> and
+              Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
@@ -39,16 +39,16 @@
 
 typedef union {
      struct {
-          __u16 b;
-          __u16 g;
-          __u16 r;
-          __u16 a;
+          u16 b;
+          u16 g;
+          u16 r;
+          u16 a;
      } RGB;
      struct {
-          __u16 u;
-          __u16 v;
-          __u16 y;
-          __u16 a;
+          u16 u;
+          u16 v;
+          u16 y;
+          u16 a;
      } YUV;
 } GenefxAccumulator;
 
@@ -97,11 +97,11 @@ struct _GenefxState {
       */
      void *Aop[3];
      void *Bop[3];
-     __u32 Cop;
+     u32   Cop;
 
-     __u8 YCop;
-     __u8 CbCop;
-     __u8 CrCop;
+     u8 YCop;
+     u8 CbCop;
+     u8 CrCop;
 
      int Aop_field;
      int Bop_field;
@@ -112,8 +112,8 @@ struct _GenefxState {
      /*
       * color keys
       */
-     __u32 Dkey;
-     __u32 Skey;
+     u32 Dkey;
+     u32 Skey;
 
      /*
       * color lookup tables
@@ -143,7 +143,13 @@ struct _GenefxState {
 
      int Ostep; /* controls horizontal blitting direction */
 
-     int SperD; /* for scaled routines only */
+     int SperD;     /* for scaled routines only */
+     int Xphase;    /* initial value for fractional steps (zero if not clipped) */
+
+     bool need_accumulator;
+
+     int *trans;
+     int  num_trans;
 };
 
 
