@@ -1,13 +1,13 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2005  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
               Sven Neumann <neo@directfb.org>,
-              Ville Syrjl <syrjala@sci.fi> and
+              Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
@@ -30,9 +30,9 @@
 static void Cop_to_Aop_32_64( GenefxState *gfxs )
 {
      int    w, l = gfxs->length;
-     __u32 *D    = gfxs->Aop[0];
-     __u32  Cop  = gfxs->Cop;
-     __u64  DCop = ((__u64)Cop << 32) | Cop;
+     u32   *D    = gfxs->Aop[0];
+     u32    Cop  = gfxs->Cop;
+     u64    DCop = ((u64)Cop << 32) | Cop;
      
      if ((long)D & 4) {
           *D++ = Cop;
@@ -40,7 +40,7 @@ static void Cop_to_Aop_32_64( GenefxState *gfxs )
      }
 
      for (w = l >> 1; w; w--) {
-          *((__u64*)D) = DCop;
+          *((u64*)D) = DCop;
           D += 2;
      }
      
@@ -51,10 +51,10 @@ static void Cop_to_Aop_32_64( GenefxState *gfxs )
 static void Bop_rgb32_Kto_Aop_64( GenefxState *gfxs )
 {
      int    w, l  = gfxs->length;
-     __u32 *D     = gfxs->Aop[0];
-     __u32 *S     = gfxs->Bop[0];
-     __u32  Skey  = gfxs->Skey;
-     __u64  DSkey = ((__u64)Skey << 32) | Skey;
+     u32   *D     = gfxs->Aop[0];
+     u32   *S     = gfxs->Bop[0];
+     u32    Skey  = gfxs->Skey;
+     u64    DSkey = ((u64)Skey << 32) | Skey;
      
      if ((long)D & 4) {
           if (*S != Skey)
@@ -65,28 +65,28 @@ static void Bop_rgb32_Kto_Aop_64( GenefxState *gfxs )
      }               
 
      for (w = l >> 1; w; w--) {
-          __u64 s = *((__u64*)S);
+          u64 s = *((u64*)S);
 
           if (s != DSkey) {
                if ((s & 0x00ffffff00000000ull) != 
                     (DSkey & 0x00ffffff00000000ull)) {
                     if ((s & 0x0000000000ffffffull) != 
                          (DSkey & 0x0000000000ffffffull)) {
-                         *((__u64*)D) = s;
+                         *((u64*)D) = s;
                     }
                     else {
 #ifdef WORDS_BIGENDIAN
-                         D[0] = (__u32)(s >> 32);
+                         D[0] = (u32)(s >> 32);
 #else
-                         D[1] = (__u32)(s >> 32);
+                         D[1] = (u32)(s >> 32);
 #endif
                     }
                }
                else {
 #ifdef WORDS_BIGENDIAN
-                    D[1] = (__u32)s;
+                    D[1] = (u32)s;
 #else
-                    D[0] = (__u32)s;
+                    D[0] = (u32)s;
 #endif
                }
           }
@@ -103,10 +103,10 @@ static void Bop_rgb32_Kto_Aop_64( GenefxState *gfxs )
 static void Bop_rgb32_toK_Aop_64( GenefxState *gfxs )
 {
      int    w, l  = gfxs->length;
-     __u32 *D     = gfxs->Aop[0];
-     __u32 *S     = gfxs->Bop[0];
-     __u32  Dkey  = gfxs->Dkey;
-     __u64  DDkey = ((__u64)Dkey << 32) | Dkey;
+     u32   *D     = gfxs->Aop[0];
+     u32   *S     = gfxs->Bop[0];
+     u32    Dkey  = gfxs->Dkey;
+     u64    DDkey = ((u64)Dkey << 32) | Dkey;
      
      if ((long)D & 4) {
           if (*D == Dkey)
@@ -117,14 +117,14 @@ static void Bop_rgb32_toK_Aop_64( GenefxState *gfxs )
      }               
 
      for (w = l >> 1; w; w--) {
-          __u64 d = *((__u64*)D);
+          u64 d = *((u64*)D);
 
           if (d != DDkey) {
                if ((d & 0x00ffffff00000000ull) ==
                     (DDkey & 0x00ffffff00000000ull)) {
                     if ((d & 0x0000000000ffffffull) ==
                          (DDkey & 0x0000000000ffffffull)) {
-                         *((__u64*)D) = *((__u64*)S);
+                         *((u64*)D) = *((u64*)S);
                     }
                     else {
 #ifdef WORDS_BIGENDIAN
@@ -156,8 +156,8 @@ static void Bop_32_Sto_Aop_64( GenefxState *gfxs )
 {
      int    w, l   = gfxs->length;
      int    i      = 0;
-     __u32 *D      = gfxs->Aop[0];
-     __u32 *S      = gfxs->Bop[0];
+     u32   *D      = gfxs->Aop[0];
+     u32   *S      = gfxs->Bop[0];
      int    SperD  = gfxs->SperD;
      int    SperD2 = SperD << 1;
      
@@ -169,9 +169,9 @@ static void Bop_32_Sto_Aop_64( GenefxState *gfxs )
      
      for (w = l >> 1; w; w--) {
 #ifdef WORDS_BIGENDIAN
-          *((__u64*)D) = ((__u64)S[i>>16] << 32) | S[(i+SperD)>>16];
+          *((u64*)D) = ((u64)S[i>>16] << 32) | S[(i+SperD)>>16];
 #else
-          *((__u64*)D) = ((__u64)S[(i+SperD)>>16] << 32) | S[i>>16];
+          *((u64*)D) = ((u64)S[(i+SperD)>>16] << 32) | S[i>>16];
 #endif
           D += 2;
           i += SperD2;
@@ -184,19 +184,19 @@ static void Bop_32_Sto_Aop_64( GenefxState *gfxs )
 static void Dacc_xor_64( GenefxState *gfxs )
 {
      int    w     = gfxs->length;
-     __u64 *D     = (__u64*)gfxs->Dacc;
-     __u64  color;
+     u64   *D     = (u64*)gfxs->Dacc;
+     u64    color;
 
 #ifdef WORDS_BIGENDIAN
-     color = ((__u64)gfxs->color.b << 48) |
-             ((__u64)gfxs->color.g << 32) |
-             ((__u64)gfxs->color.r << 16) |
-             ((__u64)gfxs->color.a);
+     color = ((u64)gfxs->color.b << 48) |
+             ((u64)gfxs->color.g << 32) |
+             ((u64)gfxs->color.r << 16) |
+             ((u64)gfxs->color.a);
 #else
-     color = ((__u64)gfxs->color.a << 48) |
-             ((__u64)gfxs->color.r << 32) |
-             ((__u64)gfxs->color.g << 16) |
-             ((__u64)gfxs->color.b);
+     color = ((u64)gfxs->color.a << 48) |
+             ((u64)gfxs->color.r << 32) |
+             ((u64)gfxs->color.g << 16) |
+             ((u64)gfxs->color.b);
 #endif
 
      for (; w; w--) {
