@@ -1,6 +1,6 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2005  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
@@ -129,7 +129,7 @@ main( int argc, char **argv )
      
 /* YUV->RGB */
      /* luma */
-     gt.type      = "const __u16";
+     gt.type      = "const u16";
      gt.name      = "y_for_rgb";
      gt.range.max = 235;
      gt.sub       = 16;
@@ -138,7 +138,7 @@ main( int argc, char **argv )
      dump_table( &gt );
 
      /* chroma */
-     gt.type      = "const __s16";
+     gt.type      = "const s16";
      gt.range.max = 240;
      gt.sub       = 128;
 
@@ -166,7 +166,7 @@ main( int argc, char **argv )
      gt.sub       = 0;
      
      /* luma */
-     gt.type      = "const __u16";
+     gt.type      = "const u16";
      gt.name      = "y_from_ey";
      gt.clamp.max = 235;
      gt.mul       = 219.0/255.0;
@@ -181,22 +181,18 @@ main( int argc, char **argv )
      gt.add       = 128;
      
      /* cb */
-     gt.type = "static const __u16";
-     gt.name = "_cb_from_bey";
+     gt.type = "const u16";
+     gt.name = "cb_from_bey";
      gt.mul  = ECB_FACTOR * 224.0 / 255.0;
      dump_table( &gt );
 
      /* cr */
-     gt.type = "static const __u16";
-     gt.name = "_cr_from_rey";
+     gt.type = "const u16";
+     gt.name = "cr_from_rey";
      gt.mul  = ECR_FACTOR * 224.0 / 255.0;
      dump_table( &gt );
 
-     puts( "const __u16 *cb_from_bey = &_cb_from_bey[255];" );
-     puts( "const __u16 *cr_from_rey = &_cr_from_rey[255];" );
-     
      /* end headers */
-     puts( "" );
      puts( "#endif /* __YUV_TABLES_H__ */" );
 
      return 0;
