@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -148,7 +149,6 @@ IDirectFBInputDevice_CreateEventBuffer( IDirectFBInputDevice  *thiz,
      return DFB_OK;
 }
 
-
 static DFBResult
 IDirectFBInputDevice_AttachEventBuffer( IDirectFBInputDevice  *thiz,
                                         IDirectFBEventBuffer  *buffer )
@@ -158,6 +158,15 @@ IDirectFBInputDevice_AttachEventBuffer( IDirectFBInputDevice  *thiz,
      return IDirectFBEventBuffer_AttachInputDevice( buffer, data->device );
 }
 
+static DFBResult
+IDirectFBInputDevice_DetachEventBuffer( IDirectFBInputDevice  *thiz,
+                                        IDirectFBEventBuffer  *buffer )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice)
+
+     return IDirectFBEventBuffer_DetachInputDevice( buffer, data->device );
+}
+
 static DFBResult
 IDirectFBInputDevice_GetDescription( IDirectFBInputDevice      *thiz,
                                      DFBInputDeviceDescription *desc )
@@ -320,6 +329,7 @@ IDirectFBInputDevice_Construct( IDirectFBInputDevice *thiz,
      thiz->GetKeymapEntry = IDirectFBInputDevice_GetKeymapEntry;
      thiz->CreateEventBuffer = IDirectFBInputDevice_CreateEventBuffer;
      thiz->AttachEventBuffer = IDirectFBInputDevice_AttachEventBuffer;
+     thiz->DetachEventBuffer = IDirectFBInputDevice_DetachEventBuffer;
      thiz->GetKeyState = IDirectFBInputDevice_GetKeyState;
      thiz->GetModifiers = IDirectFBInputDevice_GetModifiers;
      thiz->GetLockState = IDirectFBInputDevice_GetLockState;
