@@ -1,12 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -181,7 +183,7 @@ IDirectFBDataBuffer_CreateImageProvider( IDirectFBDataBuffer     *thiz,
      if (!interface)
           return DFB_INVARG;
 
-     return IDirectFBImageProvider_CreateFromBuffer( thiz, interface );
+     return IDirectFBImageProvider_CreateFromBuffer( thiz, data->core, interface );
 }
 
 static DFBResult
@@ -194,15 +196,18 @@ IDirectFBDataBuffer_CreateVideoProvider( IDirectFBDataBuffer     *thiz,
      if (!interface)
           return DFB_INVARG;
 
-     return IDirectFBVideoProvider_CreateFromBuffer( thiz, interface );
+     return IDirectFBVideoProvider_CreateFromBuffer( thiz, data->core, interface );
 }
 
 DFBResult
-IDirectFBDataBuffer_Construct( IDirectFBDataBuffer *thiz, const char *filename )
+IDirectFBDataBuffer_Construct( IDirectFBDataBuffer *thiz,
+                               const char          *filename,
+                               CoreDFB             *core )
 {
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBDataBuffer)
 
-     data->ref = 1;
+     data->ref  = 1;
+     data->core = core;
 
      if (filename)
           data->filename = D_STRDUP( filename );
