@@ -1,12 +1,13 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2005  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <sven@convergence.de> and
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
@@ -251,16 +252,11 @@ IDirectFBDataBuffer_File_PeekData( IDirectFBDataBuffer *thiz,
 static DFBResult
 IDirectFBDataBuffer_File_HasData( IDirectFBDataBuffer *thiz )
 {
-     DFBResult      ret;
      struct timeval tv = {0,0};
      
      DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_File)
-
-     pthread_mutex_lock( &data->mutex );          
-     ret = direct_stream_wait( data->stream, 1, &tv );
-     pthread_mutex_unlock( &data->mutex );
-
-     return ret;
+        
+     return direct_stream_wait( data->stream, 1, &tv );
 }
 
 static DFBResult
@@ -273,19 +269,22 @@ IDirectFBDataBuffer_File_PutData( IDirectFBDataBuffer *thiz,
 
 DFBResult
 IDirectFBDataBuffer_File_Construct( IDirectFBDataBuffer *thiz,
-                                    const char          *filename )
+                                    const char          *filename,
+                                    CoreDFB             *core )
 {
      DFBResult ret;
 
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBDataBuffer_File)
 
-     ret = IDirectFBDataBuffer_Construct( thiz, filename );
+     ret = IDirectFBDataBuffer_Construct( thiz, filename, core );
      if (ret)
           return ret;
 
      ret = direct_stream_create( filename, &data->stream );
-     if (ret)
+     if (ret) {
+          DIRECT_DEALLOCATE_INTERFACE( thiz );
           return ret;
+     }
 
      direct_util_recursive_pthread_mutex_init( &data->mutex );
 
