@@ -1,12 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -91,6 +93,12 @@ IDirectFBDataBuffer_Memory_Flush( IDirectFBDataBuffer *thiz )
      return DFB_UNSUPPORTED;
 }
 
+static DFBResult
+IDirectFBDataBuffer_Memory_Finish( IDirectFBDataBuffer *thiz )
+{
+     return DFB_UNSUPPORTED;
+}
+
 static DFBResult
 IDirectFBDataBuffer_Memory_SeekTo( IDirectFBDataBuffer *thiz,
                                    unsigned int         offset )
@@ -140,7 +148,7 @@ IDirectFBDataBuffer_Memory_WaitForData( IDirectFBDataBuffer *thiz,
      DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Memory)
 
      if (data->pos + length > data->length)
-          return DFB_BUFFEREMPTY;
+          return DFB_EOF;
 
      return DFB_OK;
 }
@@ -154,7 +162,7 @@ IDirectFBDataBuffer_Memory_WaitForDataWithTimeout( IDirectFBDataBuffer *thiz,
      DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Memory)
 
      if (data->pos + length > data->length)
-          return DFB_BUFFEREMPTY;
+          return DFB_EOF;
 
      return DFB_OK;
 }
@@ -173,7 +181,7 @@ IDirectFBDataBuffer_Memory_GetData( IDirectFBDataBuffer *thiz,
           return DFB_INVARG;
 
      if (data->pos >= data->length)
-          return DFB_BUFFEREMPTY;
+          return DFB_EOF;
 
      size = MIN( length, data->length - data->pos );
 
@@ -202,7 +210,7 @@ IDirectFBDataBuffer_Memory_PeekData( IDirectFBDataBuffer *thiz,
           return DFB_INVARG;
 
      if (data->pos + offset >= data->length)
-          return DFB_BUFFEREMPTY;
+          return DFB_EOF;
 
      size = MIN( length, data->length - data->pos - offset );
 
@@ -220,7 +228,7 @@ IDirectFBDataBuffer_Memory_HasData( IDirectFBDataBuffer *thiz )
      DIRECT_INTERFACE_GET_DATA(IDirectFBDataBuffer_Memory)
 
      if (data->pos >= data->length)
-          return DFB_BUFFEREMPTY;
+          return DFB_EOF;
 
      return DFB_OK;
 }
@@ -236,13 +244,14 @@ IDirectFBDataBuffer_Memory_PutData( IDirectFBDataBuffer *thiz,
 DFBResult
 IDirectFBDataBuffer_Memory_Construct( IDirectFBDataBuffer *thiz,
                                       const void          *data_buffer,
-                                      unsigned int         length )
+                                      unsigned int         length,
+                                      CoreDFB             *core )
 {
      DFBResult ret;
 
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBDataBuffer_Memory)
 
-     ret = IDirectFBDataBuffer_Construct( thiz, NULL );
+     ret = IDirectFBDataBuffer_Construct( thiz, NULL, core );
      if (ret)
           return ret;
 
@@ -251,6 +260,7 @@ IDirectFBDataBuffer_Memory_Construct( IDirectFBDataBuffer *thiz,
 
      thiz->Release                = IDirectFBDataBuffer_Memory_Release;
      thiz->Flush                  = IDirectFBDataBuffer_Memory_Flush;
+     thiz->Finish                 = IDirectFBDataBuffer_Memory_Finish;
      thiz->SeekTo                 = IDirectFBDataBuffer_Memory_SeekTo;
      thiz->GetPosition            = IDirectFBDataBuffer_Memory_GetPosition;
      thiz->GetLength              = IDirectFBDataBuffer_Memory_GetLength;
