@@ -1,12 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -47,16 +49,25 @@
 
 #include "misc/util.h"
 
+
+D_DEBUG_DOMAIN( Font, "IDirectFBFont", "DirectFB Font Interface" );
+
+/**********************************************************************************************************************/
+
 void
 IDirectFBFont_Destruct( IDirectFBFont *thiz )
 {
      IDirectFBFont_data *data = (IDirectFBFont_data*)thiz->priv;
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      dfb_font_destroy (data->font);
 
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
+/**********************************************************************************************************************/
+
 /*
  * increments reference count of font
  */
@@ -65,6 +76,8 @@ IDirectFBFont_AddRef( IDirectFBFont *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      data->ref++;
 
      return DFB_OK;
@@ -78,6 +91,8 @@ IDirectFBFont_Release( IDirectFBFont *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (--data->ref == 0)
           IDirectFBFont_Destruct( thiz );
 
@@ -92,6 +107,8 @@ IDirectFBFont_GetAscender( IDirectFBFont *thiz, int *ascender )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!ascender)
           return DFB_INVARG;
 
@@ -109,6 +126,8 @@ IDirectFBFont_GetDescender( IDirectFBFont *thiz, int *descender )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!descender)
           return DFB_INVARG;
 
@@ -125,6 +144,8 @@ IDirectFBFont_GetHeight( IDirectFBFont *thiz, int *height )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!height)
           return DFB_INVARG;
 
@@ -141,6 +162,8 @@ IDirectFBFont_GetMaxAdvance( IDirectFBFont *thiz, int *maxadvance )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!maxadvance)
           return DFB_INVARG;
 
@@ -162,6 +185,8 @@ IDirectFBFont_GetKerning( IDirectFBFont *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!kern_x && !kern_y)
           return DFB_INVARG;
 
@@ -192,17 +217,15 @@ IDirectFBFont_GetStringExtents( IDirectFBFont *thiz,
                                 DFBRectangle *logical_rect,
                                 DFBRectangle *ink_rect )
 {
-     CoreFont      *font;
-     CoreGlyphData *glyph;
-     unichar        prev = 0;
-     unichar        current;
-     int            width = 0;
-     int            offset;
-     int            kern_x;
-     int            kern_y;
+     DFBResult  ret;
+     CoreFont  *font;
+     int        width = 0;
+
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
 
      if (!text)
           return DFB_INVARG;
@@ -210,42 +233,51 @@ IDirectFBFont_GetStringExtents( IDirectFBFont *thiz,
      if (!logical_rect && !ink_rect)
           return DFB_INVARG;
 
-     font = data->font;
-
-     dfb_font_lock( font );
+     if (bytes < 0)
+          bytes = strlen (text);
 
      if (ink_rect)
           memset (ink_rect, 0, sizeof (DFBRectangle));
 
-     if (bytes < 0)
-          bytes = strlen (text);
+     font = data->font;
 
-     for (offset = 0; offset < bytes; offset += DIRECT_UTF8_SKIP(text[offset])) {
-          unsigned int c = text[offset];
+     dfb_font_lock( font );
 
-          if (c < 128)
-               current = c;
-          else
-               current = DIRECT_UTF8_GET_CHAR( &text[offset] );
+     if (bytes > 0) {
+          int          i, num;
+          unsigned int prev  = 0;
+          unsigned int indices[bytes];
 
-          if (dfb_font_get_glyph_data (font, current, &glyph) == DFB_OK) {
-               kern_y = 0;
-
-               if (prev && font->GetKerning && font->GetKerning( font, prev, current,
-                                                                 &kern_x, &kern_y ) == DFB_OK)
-                    width += kern_x;
-
-               if (ink_rect) {
-                    DFBRectangle glyph_rect = { width + glyph->left,
-                                                kern_y + glyph->top,
-                                                glyph->width, glyph->height };
-                    dfb_rectangle_union (ink_rect, &glyph_rect);
-               }
-
-               width += glyph->advance;
+          /* Decode string to character indices. */
+          ret = dfb_font_decode_text( font, data->encoding, text, bytes, indices, &num );
+          if (ret) {
+               dfb_font_unlock( font );
+               return ret;
           }
 
-          prev = current;
+          for (i=0; i<num; i++) {
+               unsigned int   current = indices[i];
+               CoreGlyphData *glyph;
+
+               if (dfb_font_get_glyph_data( font, current, &glyph ) == DFB_OK) {
+                    int kx, ky = 0;
+
+                    if (prev && font->GetKerning &&
+                        font->GetKerning( font, prev, current, &kx, &ky ) == DFB_OK)
+                         width += kx;
+
+                    if (ink_rect) {
+                         DFBRectangle glyph_rect = { width + glyph->left,
+                              ky + glyph->top,
+                              glyph->width, glyph->height};
+                         dfb_rectangle_union (ink_rect, &glyph_rect);
+                    }
+
+                    width += glyph->advance;
+               }
+
+               prev = current;
+          }
      }
 
      if (logical_rect) {
@@ -277,59 +309,52 @@ IDirectFBFont_GetStringWidth( IDirectFBFont *thiz,
                               int            bytes,
                               int           *ret_width )
 {
-     CoreFont      *font;
-     DirectTree    *glyphs;
-     const __u8    *string;
-     const __u8    *end;
-     CoreGlyphData *glyph;
-     int            kern_x;
-     int            width = 0;
-     unichar        prev  = 0;
-     unichar        current;
+     DFBResult ret;
+     int       width = 0;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!text || !ret_width)
           return DFB_INVARG;
 
      if (bytes < 0)
           bytes = strlen (text);
 
-     if (!bytes) {
-          *ret_width = 0;
+     if (bytes > 0) {
+          int           i, num, kx;
+          unsigned int  prev = 0;
+          unsigned int  indices[bytes];
+          CoreFont     *font = data->font;
 
-          return DFB_OK;
-     }
+          dfb_font_lock( font );
 
-     font   = data->font;
-     glyphs = font->glyph_infos;
-     string = (const __u8*) text;
-     end    = string + bytes;
-
-     dfb_font_lock( font );
-
-     do {
-          current = DIRECT_UTF8_GET_CHAR( string );
-
-          if (current >= 32 && current < 128)
-               glyph = glyphs->fast_keys[current-32];
-          else
-               glyph = direct_tree_lookup( glyphs, (void *) current );
-
-          if (glyph || dfb_font_get_glyph_data( font, current, &glyph ) == DFB_OK) {
-               width += glyph->advance;
-
-               if (prev && font->GetKerning && font->GetKerning( font, prev,
-                                                                 current, &kern_x, NULL ) == DFB_OK)
-                    width += kern_x;
+          /* Decode string to character indices. */
+          ret = dfb_font_decode_text( font, data->encoding, text, bytes, indices, &num );
+          if (ret) {
+               dfb_font_unlock( font );
+               return ret;
           }
 
-          prev = current;
+          /* Calculate string width. */
+          for (i=0; i<num; i++) {
+               unsigned int   current = indices[i];
+               CoreGlyphData *glyph;
 
-          string += DIRECT_UTF8_SKIP( string[0] );
-     } while (string < end);
+               if (dfb_font_get_glyph_data( font, current, &glyph ) == DFB_OK) {
+                    width += glyph->advance;
 
-     dfb_font_unlock( font );
+                    if (prev && font->GetKerning &&
+                        font->GetKerning( font, prev, current, &kx, NULL ) == DFB_OK)
+                         width += kx;
+               }
+
+               prev = current;
+          }
+
+          dfb_font_unlock( font );
+     }
 
      *ret_width = width;
 
@@ -341,15 +366,19 @@ IDirectFBFont_GetStringWidth( IDirectFBFont *thiz,
  */
 static DFBResult
 IDirectFBFont_GetGlyphExtents( IDirectFBFont *thiz,
-                               unsigned int   index,
+                               unsigned int   character,
                                DFBRectangle  *rect,
                                int           *advance )
 {
+     DFBResult      ret;
      CoreFont      *font;
      CoreGlyphData *glyph;
+     unsigned int   index;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
 
+     D_DEBUG_AT( Font, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!rect && !advance)
           return DFB_INVARG;
 
@@ -357,14 +386,18 @@ IDirectFBFont_GetGlyphExtents( IDirectFBFont *thiz,
 
      dfb_font_lock( font );
 
-     if (dfb_font_get_glyph_data (font, index, &glyph) != DFB_OK) {
+     ret = dfb_font_decode_character( font, data->encoding, character, &index );
+     if (ret) {
+          dfb_font_unlock( font );
+          return ret;
+     }
 
-          if (rect) {
+     if (dfb_font_get_glyph_data (font, index, &glyph) != DFB_OK) {
+          if (rect)
                rect->x = rect->y = rect->w = rect->h = 0;
-          }
-          if (advance) {
+
+          if (advance)
                *advance = 0;
-          }
      }
      else {
           if (rect) {
@@ -373,9 +406,9 @@ IDirectFBFont_GetGlyphExtents( IDirectFBFont *thiz,
                rect->w = glyph->width;
                rect->h = glyph->height;
           }
-          if (advance) {
+
+          if (advance)
                *advance = glyph->advance;
-          }
      }
 
      dfb_font_unlock( font );
@@ -383,6 +416,179 @@ IDirectFBFont_GetGlyphExtents( IDirectFBFont *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBFont_GetStringBreak( IDirectFBFont *thiz,
+                              const char    *text,
+                              int            bytes,
+                              int            max_width,
+                              int           *ret_width, 
+                              int           *ret_str_length,
+                              const char   **ret_next_line)
+{
+     DFBResult      ret;
+     CoreFont      *font;
+     const u8      *string;
+     const u8      *end;
+     CoreGlyphData *glyph;
+     int            kern_x;
+     int            length = 0;
+     int            width = 0;
+     unichar        current;
+     unsigned int   index;
+     unsigned int   prev  = 0;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
+
+     if (!text || !ret_next_line || !ret_str_length || !ret_width)
+          return DFB_INVARG;
+
+     /* FIXME: Try to change the font module API *slightly* to support this. */
+     if (data->encoding != DTEID_UTF8)
+          return DFB_UNSUPPORTED;
+
+     if (bytes < 0)
+          bytes = strlen (text);
+
+     if (!bytes) {
+          *ret_next_line = NULL;
+          *ret_str_length = 0;
+          *ret_width = 0;
+
+          return DFB_OK;
+     }
+
+     font   = data->font;
+     string = (const u8*) text;
+     end    = string + bytes;
+
+     dfb_font_lock( font );
+
+     do {
+          *ret_width = width;
+          length ++;          
+
+          current = DIRECT_UTF8_GET_CHAR( string );
+
+          string += DIRECT_UTF8_SKIP( string[0] );
+
+          if (current == ' ' || current == 0x0a) {
+               *ret_next_line = (const char*) string;
+               *ret_str_length = length;
+               *ret_width = width;
+          }
+
+          ret = dfb_font_decode_character( font, data->encoding, current, &index );
+          if (ret)
+               continue;
+
+          ret = dfb_font_get_glyph_data( font, index, &glyph );
+          if (ret)
+               continue;
+          
+          width += glyph->advance;
+
+          if (prev && font->GetKerning && font->GetKerning( font, prev, index, &kern_x, NULL ) == DFB_OK)
+               width += kern_x;
+
+          prev = index;
+     } while (width < max_width && string < end && current != 0x0a);
+
+     dfb_font_unlock( font );
+
+     if (width<max_width && string >= end) {
+          *ret_next_line = NULL;
+          *ret_str_length = length-1;
+          *ret_width = width;
+
+          return DFB_OK;
+     }
+
+     if (*ret_next_line == NULL) {
+          *ret_next_line = (const char*) string;
+          *ret_str_length = length;
+          *ret_width = width;
+     }
+
+     return DFB_OK;
+}
+
+static DFBResult
+IDirectFBFont_SetEncoding( IDirectFBFont     *thiz,
+                           DFBTextEncodingID  encoding )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
+
+     D_DEBUG_AT( Font, "%s( %p, %d )\n", __FUNCTION__, thiz, encoding );
+
+     if (encoding > data->font->last_encoding)
+          return DFB_IDNOTFOUND;
+
+     data->encoding = encoding;
+
+     return DFB_OK;
+}
+
+static DFBResult
+IDirectFBFont_EnumEncodings( IDirectFBFont           *thiz,
+                             DFBTextEncodingCallback  callback,
+                             void                    *context )
+{
+     int       i;
+     CoreFont *font;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
+
+     if (!callback)
+          return DFB_INVARG;
+
+     D_DEBUG_AT( Font, "%s( %p, %p, %p )\n", __FUNCTION__, thiz, callback, context );
+
+     font = data->font;
+
+     if (callback( DTEID_UTF8, "UTF8", context ) == DFENUM_OK) {
+          for (i=DTEID_OTHER; i<=font->last_encoding; i++) {
+               if (callback( i, font->encodings[i]->name, context ) != DFENUM_OK)
+                    break;
+          }
+     }
+
+     return DFB_OK;
+}
+
+static DFBResult
+IDirectFBFont_FindEncoding( IDirectFBFont     *thiz,
+                            const char        *name,
+                            DFBTextEncodingID *ret_id )
+{
+     int       i;
+     CoreFont *font;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
+
+     if (!name || !ret_id)
+          return DFB_INVARG;
+
+     D_DEBUG_AT( Font, "%s( %p, '%s', %p )\n", __FUNCTION__, thiz, name, ret_id );
+
+     if (!strcasecmp( name, "UTF8" )) {
+          *ret_id = DTEID_UTF8;
+          return DFB_OK;
+     }
+
+     font = data->font;
+
+     for (i=DTEID_OTHER; i<=font->last_encoding; i++) {
+          if (!strcasecmp( name, font->encodings[i]->name )) {
+               *ret_id = i;
+               return DFB_OK;
+          }
+     }
+
+     return DFB_IDNOTFOUND;
+}
+
+/**********************************************************************************************************************/
+
 DFBResult
 IDirectFBFont_Construct( IDirectFBFont *thiz, CoreFont *font )
 {
@@ -401,6 +607,10 @@ IDirectFBFont_Construct( IDirectFBFont *thiz, CoreFont *font )
      thiz->GetStringWidth = IDirectFBFont_GetStringWidth;
      thiz->GetStringExtents = IDirectFBFont_GetStringExtents;
      thiz->GetGlyphExtents = IDirectFBFont_GetGlyphExtents;
+     thiz->GetStringBreak = IDirectFBFont_GetStringBreak;
+     thiz->SetEncoding = IDirectFBFont_SetEncoding;
+     thiz->EnumEncodings = IDirectFBFont_EnumEncodings;
+     thiz->FindEncoding = IDirectFBFont_FindEncoding;
 
      return DFB_OK;
 }
