@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -65,16 +66,22 @@ static const char *config_usage =
      "  fbdev=<device>                 Open <device> instead of /dev/fb0\n"
      "  busid=<id>                     Specify the bus location of the graphics card (default 1:0:0)\n"
      "  mode=<width>x<height>          Set the default resolution\n"
+     "  scaled=<width>x<height>        Scale the window to this size for 'force-windowed' apps\n"
      "  depth=<pixeldepth>             Set the default pixel depth\n"
      "  pixelformat=<pixelformat>      Set the default pixel format\n"
      "  session=<num>                  Select multi app world (zero based, -1 = new)\n"
+     "  force-slave                    Always enter as a slave, waiting for the master, if not there\n"
      "  remote=<host>[:<session>]      Select remote session to connect to\n"
      "  tmpfs=<directory>              Location of shared memory file\n"
+     "  shmfile-group=<groupname>      Group that owns shared memory files\n"
      "  memcpy=<method>                Skip memcpy() probing (help = show list)\n"
      "  primary-layer=<id>             Select an alternative primary layer\n"
+     "  primary-only                   Tell application only about the primary layer\n"
      "  quiet                          No text output except debugging\n"
      "  [no-]banner                    Show DirectFB Banner on startup\n"
      "  [no-]debug                     Enable debug output\n"
+     "  [no-]debugmem                  Enable memory allocation tracking\n"
+     "  [no-]debugshm                  Enable shared memory allocation tracking\n"
      "  [no-]trace                     Enable stack trace support\n"
      "  log-file=<name>                Write all messages to a file\n"
      "  log-udp=<host>:<port>          Send all messages via UDP to host:port\n"
@@ -82,18 +89,21 @@ static const char *config_usage =
      "  force-windowed                 Primary surface always is a window\n"
      "  force-desktop                  Primary surface is the desktop background\n"
      "  [no-]hardware                  Hardware acceleration\n"
+     "  [no-]software                  Software fallbacks\n"
      "  [no-]dma                       Enable DMA acceleration\n"
      "  [no-]sync                      Do `sync()' (default=no)\n"
 #ifdef USE_MMX
      "  [no-]mmx                       Enable mmx support\n"
 #endif
      "  [no-]agp[=<mode>]              Enable AGP support\n"
+     "  [no-]thrifty-surface-buffers   Free sysmem instance on xfer to video memory\n"
      "  font-format=<pixelformat>      Set the preferred font format\n"
      "  dont-catch=<num>[[,<num>]...]  Don't catch these signals\n"
      "  [no-]sighandler                Enable signal handler\n"
      "  [no-]deinit-check              Enable deinit check at exit\n"
      "  block-all-signals              Block all signals\n"
      "  [no-]vt-switch                 Allocate/switch to a new VT\n"
+     "  vt-num=<num>                   Use given VT instead of current/new one\n"
      "  [no-]vt-switching              Allow Ctrl+Alt+<F?> (EXPERIMENTAL)\n"
      "  [no-]graphics-vt               Put terminal into graphics mode\n"
      "  [no-]vt                        Use VT handling code at all?\n"
@@ -113,19 +123,26 @@ static const char *config_usage =
      "  [no-]translucent-windows       Allow translucent windows\n"
      "  [no-]decorations               Enable window decorations (if supported by wm)\n"
      "  videoram-limit=<amount>        Limit amount of Video RAM in kb\n"
-     "  agpmem-limit=<amount>          Limit amount of AGP memory in kb\n" 
+     "  agpmem-limit=<amount>          Limit amount of AGP memory in kb\n"
      "  screenshot-dir=<directory>     Dump screen content on <Print> key presses\n"
      "  disable-module=<module_name>   suppress loading this module\n"
+     "  video-phys=<hexaddress>        Physical start of video memory (devmem system)\n"
+     "  video-length=<bytes>           Length of video memory (devmem system)\n"
+     "  mmio-phys=<hexaddress>         Physical start of MMIO area (devmem system)\n"
+     "  mmio-length=<bytes>            Length of MMIO area (devmem system)\n"
+     "  accelerator=<id>               Accelerator ID selecting graphics driver (devmem system)\n"
      "\n"
      "  [no-]matrox-sgram              Use Matrox SGRAM features\n"
      "  [no-]matrox-crtc2              Experimental Matrox CRTC2 support\n"
-     "  matrox-tv-standard=(pal|ntsc)  Matrox TV standard (default=pal)\n"
+     "  matrox-tv-standard=(pal|ntsc|pal-60)\n"
+     "                                 Matrox TV standard (default=pal)\n"
      "  matrox-cable-type=(composite|scart-rgb|scart-composite)\n"
      "                                 Matrox cable type (default=composite)\n"
      "  h3600-device=<device>          Use this device for the H3600 TS driver\n"
      "  mut-device=<device>            Use this device for the MuTouch driver\n"
      "  penmount-device=<device>       Use this device for the PenMount driver\n"
      "  unichrome-revision=<rev>       Override unichrome hardware revision\n"
+     "  i8xx_overlay_pipe_b            Redirect videolayer to pixelpipe B\n"
      "\n"
      " Window surface swapping policy:\n"
      "  window-surface-policy=(auto|videohigh|videolow|systemonly|videoonly)\n"
@@ -155,41 +172,51 @@ typedef struct {
 } FormatString;
 
 static const FormatString format_strings[] = {
+     { "A1",       DSPF_A1       },
+     { "A4",       DSPF_A4       },
      { "A8",       DSPF_A8       },
      { "ALUT44",   DSPF_ALUT44   },
      { "ARGB",     DSPF_ARGB     },
      { "ARGB1555", DSPF_ARGB1555 },
+     { "ARGB1666", DSPF_ARGB1666 },
      { "ARGB2554", DSPF_ARGB2554 },
      { "ARGB4444", DSPF_ARGB4444 },
+     { "ARGB6666", DSPF_ARGB6666 },
      { "AYUV",     DSPF_AYUV     },
      { "AiRGB",    DSPF_AiRGB    },
      { "I420",     DSPF_I420     },
+     { "LUT2",     DSPF_LUT2     },
      { "LUT8",     DSPF_LUT8     },
      { "NV12",     DSPF_NV12     },
      { "NV16",     DSPF_NV16     },
      { "NV21",     DSPF_NV21     },
      { "RGB16",    DSPF_RGB16    },
+     { "RGB18",    DSPF_RGB18    },
      { "RGB24",    DSPF_RGB24    },
      { "RGB32",    DSPF_RGB32    },
      { "RGB332",   DSPF_RGB332   },
+     { "RGB444",   DSPF_RGB444   },
+     { "RGB555",   DSPF_RGB555   },
      { "UYVY",     DSPF_UYVY     },
      { "YUY2",     DSPF_YUY2     },
-     { "YV12",     DSPF_YV12     }
+     { "YV12",     DSPF_YV12     },
 };
 
-#define NUM_FORMAT_STRINGS (sizeof(format_strings) / sizeof(FormatString))
+#define NUM_FORMAT_STRINGS D_ARRAY_SIZE(format_strings)
 
 static const FormatString font_format_strings[] = {
      { "A1",       DSPF_A1       },
+     { "A4",       DSPF_A4       },
      { "A8",       DSPF_A8       },
      { "ARGB",     DSPF_ARGB     },
      { "ARGB1555", DSPF_ARGB1555 },
      { "ARGB2554", DSPF_ARGB2554 },
      { "ARGB4444", DSPF_ARGB4444 },
-     { "AiRGB",    DSPF_AiRGB    }
+     { "AiRGB",    DSPF_AiRGB    },
+     { "LUT2",     DSPF_LUT2     }
 };
 
-#define NUM_FONT_FORMAT_STRINGS (sizeof(font_format_strings) / sizeof(FormatString))
+#define NUM_FONT_FORMAT_STRINGS D_ARRAY_SIZE(font_format_strings)
 
 /* serial mouse device names */
 #define DEV_NAME     "/dev/mouse"
@@ -202,8 +229,8 @@ format_string_compare (const void *key,
      return strcmp ((const char *) key, ((const FormatString *) base)->string);
 }
 
-static DFBSurfacePixelFormat
-parse_pixelformat( const char *format )
+DFBSurfacePixelFormat
+dfb_config_parse_pixelformat( const char *format )
 {
      FormatString *format_string;
 
@@ -246,7 +273,7 @@ parse_args( const char *args )
                *next++ = '\0';
 
           if (strcmp (buf, "help") == 0) {
-               direct_log_printf( NULL, config_usage );
+               fprintf( stderr, config_usage );
                exit(1);
           }
 
@@ -315,7 +342,7 @@ static void config_allocate()
      dfb_config->layer_bg_color.g         = 0x7c;
      dfb_config->layer_bg_color.b         = 0xe8;
      dfb_config->layer_bg_mode            = DLBM_COLOR;
-     
+
      dfb_config->pci.bus                  = 1;
      dfb_config->pci.dev                  = 0;
      dfb_config->pci.func                 = 0;
@@ -325,6 +352,7 @@ static void config_allocate()
      dfb_config->mmx                      = true;
      dfb_config->vt                       = true;
      dfb_config->vt_switch                = true;
+     dfb_config->vt_num                   = -1;
      dfb_config->vt_switching             = true;
      dfb_config->kd_graphics              = true;
      dfb_config->translucent_windows      = true;
@@ -338,7 +366,9 @@ static void config_allocate()
      dfb_config->unichrome_revision       = -1;
      dfb_config->dma                      = false;
      dfb_config->agp                      = 0;
-     
+     dfb_config->matrox_tv_std            = DSETV_PAL;
+     dfb_config->i8xx_overlay_pipe_b      = false;
+
      /* default to fbdev */
      dfb_config->system = D_STRDUP( "FBDev" );
 
@@ -409,17 +439,17 @@ DFBResult dfb_config_set( const char *name, const char *value )
      if (strcmp (name, "busid") == 0 || strcmp (name, "pci-id") == 0) {
           if (value) {
                int bus, dev, func;
-               
+
                if (sscanf( value, "%d:%d:%d", &bus, &dev, &func ) != 3) {
                     D_ERROR( "DirectFB/Config 'busid': Could not parse busid!\n");
                     return DFB_INVARG;
-               }                  
-               
+               }
+
                dfb_config->pci.bus  = bus;
                dfb_config->pci.dev  = dev;
                dfb_config->pci.func = func;
           }
-     } else                    
+     } else
      if (strcmp (name, "tmpfs" ) == 0) {
           if (value) {
                if (fusion_config->tmpfs)
@@ -431,6 +461,18 @@ DFBResult dfb_config_set( const char *name, const char *value )
                return DFB_INVARG;
           }
      } else
+     if (strcmp (name, "shmfile-group" ) == 0) {
+          if (value) {
+               if (fusion_config->shmfile_group)
+                    D_FREE( fusion_config->shmfile_group );
+
+               fusion_config->shmfile_group = D_STRDUP( value );
+          }
+          else {
+               D_ERROR("DirectFB/Config 'shmfile-group': No file group name specified!\n");
+               return DFB_INVARG;
+          }
+     } else
      if (strcmp (name, "memcpy" ) == 0) {
           if (value) {
                if (direct_config->memcpy)
@@ -470,6 +512,23 @@ DFBResult dfb_config_set( const char *name, const char *value )
                return DFB_INVARG;
           }
      } else
+     if (strcmp (name, "scaled" ) == 0) {
+          if (value) {
+               int width, height;
+
+               if (sscanf( value, "%dx%d", &width, &height ) < 2) {
+                    D_ERROR("DirectFB/Config 'scaled': Could not parse size!\n");
+                    return DFB_INVARG;
+               }
+
+               dfb_config->scaled.width  = width;
+               dfb_config->scaled.height = height;
+          }
+          else {
+               D_ERROR("DirectFB/Config 'scaled': No size specified!\n");
+               return DFB_INVARG;
+          }
+     } else
      if (strcmp (name, "depth" ) == 0) {
           if (value) {
                int depth;
@@ -502,11 +561,14 @@ DFBResult dfb_config_set( const char *name, const char *value )
                return DFB_INVARG;
           }
      } else
+     if (strcmp (name, "primary-only" ) == 0) {
+          dfb_config->primary_only = true;
+     } else
      if (strcmp (name, "pixelformat" ) == 0) {
           if (value) {
                DFBSurfacePixelFormat format;
 
-               format = parse_pixelformat( value );
+               format = dfb_config_parse_pixelformat( value );
                if (format == DSPF_UNKNOWN) {
                     D_ERROR("DirectFB/Config 'pixelformat': Could not parse format!\n");
                     return DFB_INVARG;
@@ -552,6 +614,12 @@ DFBResult dfb_config_set( const char *name, const char *value )
                return DFB_INVARG;
           }
      } else
+     if (strcmp (name, "force-slave" ) == 0) {
+          fusion_config->force_slave = true;
+     } else
+     if (strcmp (name, "no-force-slave" ) == 0) {
+          fusion_config->force_slave = false;
+     } else
      if (strcmp (name, "remote" ) == 0) {
           if (value) {
                char host[128];
@@ -614,6 +682,18 @@ DFBResult dfb_config_set( const char *name, const char *value )
           else
                direct_config->debug = false;
      } else
+     if (strcmp (name, "debugmem" ) == 0) {
+          direct_config->debugmem = true;
+     } else
+     if (strcmp (name, "no-debugmem" ) == 0) {
+          direct_config->debugmem = false;
+     } else
+     if (strcmp (name, "debugshm" ) == 0) {
+          fusion_config->debugshm = true;
+     } else
+     if (strcmp (name, "no-debugshm" ) == 0) {
+          fusion_config->debugshm = false;
+     } else
      if (strcmp (name, "trace" ) == 0) {
           direct_config->trace = true;
      } else
@@ -671,6 +751,12 @@ DFBResult dfb_config_set( const char *name, const char *value )
      if (strcmp (name, "no-hardware" ) == 0) {
           dfb_config->software_only = true;
      } else
+     if (strcmp (name, "software" ) == 0) {
+          dfb_config->hardware_only = false;
+     } else
+     if (strcmp (name, "no-software" ) == 0) {
+          dfb_config->hardware_only = true;
+     } else
      if (strcmp (name, "dma" ) == 0) {
           dfb_config->dma = true;
      } else
@@ -699,10 +785,16 @@ DFBResult dfb_config_set( const char *name, const char *value )
                dfb_config->agp = 8; /* maximum possible */
           }
      } else
+     if (strcmp (name, "thrifty-surface-buffers" ) == 0) {
+          dfb_config->thrifty_surface_buffers = true;
+     } else
+     if (strcmp (name, "no-thrifty-surface-buffers" ) == 0) {
+          dfb_config->thrifty_surface_buffers = false;
+     } else
      if (strcmp (name, "no-agp" ) == 0) {
           dfb_config->agp = 0;
      } else
-     if (strcmp (name, "agpmem-limit" ) == 0) { 
+     if (strcmp (name, "agpmem-limit" ) == 0) {
           if (value) {
                int limit;
 
@@ -769,7 +861,7 @@ DFBResult dfb_config_set( const char *name, const char *value )
      } else
      if (strcmp (name, "mouse-source" ) == 0) {
           if (value) {
-               D_FREE( dfb_config->mouse_source );	
+               D_FREE( dfb_config->mouse_source );
                dfb_config->mouse_source = D_STRDUP( value );
           }
           else {
@@ -811,6 +903,22 @@ DFBResult dfb_config_set( const char *name, const char *value )
      if (strcmp (name, "no-vt-switch" ) == 0) {
           dfb_config->vt_switch = false;
      } else
+     if (strcmp (name, "vt-num" ) == 0) {
+          if (value) {
+               int vt_num;
+
+               if (sscanf( value, "%d", &vt_num ) < 1) {
+                    D_ERROR("DirectFB/Config 'vt-num': Could not parse value!\n");
+                    return DFB_INVARG;
+               }
+
+               dfb_config->vt_num = vt_num;
+          }
+          else {
+               D_ERROR("DirectFB/Config 'vt-num': No value specified!\n");
+               return DFB_INVARG;
+          }
+     } else
      if (strcmp (name, "vt-switching" ) == 0) {
           dfb_config->vt_switching = true;
      } else
@@ -902,7 +1010,7 @@ DFBResult dfb_config_set( const char *name, const char *value )
      if (strcmp (name, "bg-color" ) == 0) {
           if (value) {
                char *error;
-               __u32 argb;
+               u32   argb;
 
                argb = strtoul( value, &error, 16 );
 
@@ -959,13 +1067,111 @@ DFBResult dfb_config_set( const char *name, const char *value )
                return DFB_INVARG;
           }
      } else
+     if (strcmp (name, "video-phys" ) == 0) {
+          if (value) {
+               char *error;
+               ulong phys;
+
+               phys = strtoul( value, &error, 16 );
+
+               if (*error) {
+                    D_ERROR( "DirectFB/Config '%s': Error in hex value '%s'!\n", name, error );
+                    return DFB_INVARG;
+               }
+
+               dfb_config->video_phys = phys;
+          }
+          else {
+               D_ERROR( "DirectFB/Config '%s': No value specified!\n", name );
+               return DFB_INVARG;
+          }
+     } else
+     if (strcmp (name, "video-length" ) == 0) {
+          if (value) {
+               char *error;
+               ulong length;
+
+               length = strtoul( value, &error, 10 );
+
+               if (*error) {
+                    D_ERROR( "DirectFB/Config '%s': Error in value '%s'!\n", name, error );
+                    return DFB_INVARG;
+               }
+
+               dfb_config->video_length = length;
+          }
+          else {
+               D_ERROR( "DirectFB/Config '%s': No value specified!\n", name );
+               return DFB_INVARG;
+          }
+     } else
+     if (strcmp (name, "mmio-phys" ) == 0) {
+          if (value) {
+               char *error;
+               ulong phys;
+
+               phys = strtoul( value, &error, 16 );
+
+               if (*error) {
+                    D_ERROR( "DirectFB/Config '%s': Error in hex value '%s'!\n", name, error );
+                    return DFB_INVARG;
+               }
+
+               dfb_config->mmio_phys = phys;
+          }
+          else {
+               D_ERROR( "DirectFB/Config '%s': No value specified!\n", name );
+               return DFB_INVARG;
+          }
+     } else
+     if (strcmp (name, "mmio-length" ) == 0) {
+          if (value) {
+               char *error;
+               ulong length;
+
+               length = strtoul( value, &error, 10 );
+
+               if (*error) {
+                    D_ERROR( "DirectFB/Config '%s': Error in value '%s'!\n", name, error );
+                    return DFB_INVARG;
+               }
+
+               dfb_config->mmio_length = length;
+          }
+          else {
+               D_ERROR( "DirectFB/Config '%s': No value specified!\n", name );
+               return DFB_INVARG;
+          }
+     } else
+     if (strcmp (name, "accelerator" ) == 0) {
+          if (value) {
+               char *error;
+               ulong accel;
+
+               accel = strtoul( value, &error, 10 );
+
+               if (*error) {
+                    D_ERROR( "DirectFB/Config '%s': Error in value '%s'!\n", name, error );
+                    return DFB_INVARG;
+               }
+
+               dfb_config->accelerator = accel;
+          }
+          else {
+               D_ERROR( "DirectFB/Config '%s': No value specified!\n", name );
+               return DFB_INVARG;
+          }
+     } else
      if (strcmp (name, "matrox-tv-standard" ) == 0) {
           if (value) {
+               if (strcmp( value, "pal-60" ) == 0) {
+                    dfb_config->matrox_tv_std = DSETV_PAL_60;
+               } else
                if (strcmp( value, "pal" ) == 0) {
-                    dfb_config->matrox_ntsc = false;
+                    dfb_config->matrox_tv_std = DSETV_PAL;
                } else
                if (strcmp( value, "ntsc" ) == 0) {
-                    dfb_config->matrox_ntsc = true;
+                    dfb_config->matrox_tv_std = DSETV_NTSC;
                } else {
                     D_ERROR( "DirectFB/Config: Unknown TV standard "
                              "'%s'!\n", value );
@@ -1078,6 +1284,9 @@ DFBResult dfb_config_set( const char *name, const char *value )
                D_ERROR("DirectFB/Config 'unichrome-revision': No revision specified!\n");
                return DFB_INVARG;
           }
+     } else
+     if (strcmp (name, "i8xx_overlay_pipe_b") == 0) {
+          dfb_config->i8xx_overlay_pipe_b = true;
      }
      else
           return DFB_UNSUPPORTED;
@@ -1099,14 +1308,6 @@ DFBResult dfb_config_init( int *argc, char *(*argv[]) )
 
      config_allocate();
 
-     /* Current session is default. */
-     session = getenv( "DIRECTFB_SESSION" );
-     if (session) {
-          ret = dfb_config_set( "session", session );
-          if (ret)
-               return ret;
-     }
-
      /* Read system settings. */
      ret = dfb_config_read( "/etc/directfbrc" );
      if (ret  &&  ret != DFB_IO)
@@ -1166,12 +1367,17 @@ DFBResult dfb_config_init( int *argc, char *(*argv[]) )
                return ret;
      }
 
+     /* Active session is used if present, only command line can override. */
+     session = getenv( "DIRECTFB_SESSION" );
+     if (session)
+          dfb_config_set( "session", session );
+
      /* Read settings from command line. */
      if (argc && argv) {
           for (i = 1; i < *argc; i++) {
 
                if (strcmp ((*argv)[i], "--dfb-help") == 0) {
-                    direct_log_printf( NULL, config_usage );
+                    fprintf( stderr, config_usage );
                     exit(1);
                }
 
@@ -1223,18 +1429,18 @@ DFBResult dfb_config_read( const char *filename )
           D_DEBUG( "DirectFB/Config: Unable to open config file `%s'!\n", filename );
           return DFB_IO;
      } else {
-          D_INFO( "DirectFB/Config: Parsing config file '%s'.\n", filename );
+          D_DEBUG( "DirectFB/Config: Parsing config file '%s'.\n", filename );
      }
 
      while (fgets( line, 400, f )) {
           char *name = line;
           char *comment = strchr( line, '#');
           char *value;
-          
+
           if (comment) {
                *comment = 0;
           }
-          
+
           value = strchr( line, '=' );
 
           if (value) {
@@ -1249,9 +1455,11 @@ DFBResult dfb_config_read( const char *filename )
 
           ret = dfb_config_set( name, value );
           if (ret) {
-               if (ret == DFB_UNSUPPORTED)
-                    D_ERROR( "DirectFB/Config: In config file `%s': "
-                             "Invalid option `%s'!\n", filename, name );
+               if (ret == DFB_UNSUPPORTED) {
+                    D_ERROR( "DirectFB/Config: *********** In config file `%s': "
+                             "Invalid option `%s'! ***********\n", filename, name );
+                    continue;
+               }
                break;
           }
      }
