@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -54,8 +55,9 @@ typedef struct
 
      bool      pollvsync_after;
      bool      pollvsync_none;
-     bool      software_only;                     /* disable hardware
-                                                     acceleration */
+
+     bool      software_only;                     /* disable hardware acceleration */
+     bool      hardware_only;                     /* disable software fallbacks */
 
      bool      mmx;                               /* mmx support */
 
@@ -66,10 +68,12 @@ typedef struct
      bool      deinit_check;
 
      bool      vt_switch;                         /* allocate a new VT */
+     int       vt_num;                            /* number of TTY to use or -1
+                                                     if the default */
      bool      kd_graphics;                       /* put terminal into graphics
                                                      mode */
 
-     bool      matrox_ntsc;                       /* Matrox TV standard */
+     DFBScreenEncoderTVStandards matrox_tv_std;   /* Matrox TV standard */
      int       matrox_cable;                      /* Matrox cable type */
      bool      matrox_sgram;                      /* Use Matrox SGRAM features */
      bool      matrox_crtc2;                      /* Experimental CRTC2 stuff */
@@ -80,7 +84,7 @@ typedef struct
 
      char     *fb_device;                         /* Used framebuffer device,
                                                      e.g. "/dev/fb0" */
-                                                     
+
      struct {
           int  bus;                               /* PCI Bus */
           int  dev;                               /* PCI Device */
@@ -100,6 +104,11 @@ typedef struct
           DFBSurfacePixelFormat format;           /* primary layer format */
      } mode;
 
+     struct {
+          int                   width;            /* scaled window width */
+          int                   height;           /* scaled window height */
+     } scaled;
+
      int       videoram_limit;                    /* limit amount of video
                                                      memory used by DirectFB */
 
@@ -143,12 +152,22 @@ typedef struct
 
      int        unichrome_revision;               /* Unichrome hardware
                                                      revision number override */
-     
+
      bool       dma;                              /* Enable DMA */
-     
+
      int        agp;                              /* AGP mode */
      int        agpmem_limit;                     /* Limit of AGP memory
                                                      used by DirectFB */
+     bool       i8xx_overlay_pipe_b;              /* video overlay output via pixel pipe B */
+     bool       primary_only;                     /* tell application only about primary layer */
+
+     bool       thrifty_surface_buffers;          /* don't keep system instance while video instance is alive */
+
+     unsigned long video_phys;                    /* Physical base address of video memory */
+     unsigned int  video_length;                  /* Size of video memory */
+     unsigned long mmio_phys;                     /* Physical base address of MMIO area */
+     unsigned int  mmio_length;                   /* Size of MMIO area */
+     int           accelerator;                   /* Accelerator ID */
 } DFBConfig;
 
 extern DFBConfig *dfb_config;
@@ -179,5 +198,7 @@ DFBResult dfb_config_set( const char *name, const char *value );
 
 const char *dfb_config_usage( void );
 
+DFBSurfacePixelFormat dfb_config_parse_pixelformat( const char *format );
+
 #endif
 
