@@ -1,13 +1,13 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2006  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
               Sven Neumann <neo@directfb.org>,
-              Ville Syrjl <syrjala@sci.fi> and
+              Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
    Scaling routines ported from gdk_pixbuf by Sven Neumann
@@ -53,6 +53,7 @@
 #include <misc/util.h>
 #include <misc/gfx_util.h>
 
+#include <gfx/clip.h>
 #include <gfx/convert.h>
 
 
@@ -73,18 +74,18 @@ struct _PixopsFilter {
 };
 
 
-static void write_argb_span (__u32 *src, __u8 *dst[], int len,
+static void write_argb_span (u32 *src, u8 *dst[], int len,
                               int dx, int dy, CoreSurface *dst_surface)
 {
      CorePalette *palette = dst_surface->palette;
-     __u8        *d       = dst[0];
-     __u8        *d1,*d2;
-     int          i;
+     u8          *d       = dst[0];
+     u8          *d1,*d2;
+     int          i, j;
 
      if (dst_surface->caps & DSCAPS_PREMULTIPLIED) {
           for (i = 0; i < len; i++) {
-               __u32 s = src[i];
-               __u32 a = (s >> 24) + 1; 
+               u32 s = src[i];
+               u32 a = (s >> 24) + 1;
                
                src[i] = ((((s & 0x00ff00ff) * a) >> 8) & 0x00ff00ff) |
                         ((((s & 0x0000ff00) * a) >> 8) & 0x0000ff00) |
@@ -102,34 +103,77 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
                }
                break;
                
+          case DSPF_A4:
+               for (i=0, j=0; i<len; i+=2, j++)
+                    d[j] = ((src[i] >> 24) & 0xF0) | (src[i+1] >> 28);
+               break;
+
           case DSPF_A8:
-               for (i = 0; i < dx+len; i++)
+               for (i = 0; i < len; i++)
                     d[i] = src[i] >> 24;
                break;
                
           case DSPF_RGB332:
-               for (i = 0; i < dx+len; i++)
+               for (i = 0; i < len; i++)
                     d[i] = RGB32_TO_RGB332( src[i] );
                break;
 
           case DSPF_ARGB1555:
                for (i = 0; i < len; i++)
-                    ((__u16*)d)[i] = ARGB_TO_ARGB1555( src[i] );
+                    ((u16*)d)[i] = ARGB_TO_ARGB1555( src[i] );
                break;
 
           case DSPF_ARGB2554:
                for (i = 0; i < len; i++)
-                    ((__u16*)d)[i] = ARGB_TO_ARGB2554( src[i] );
+                    ((u16*)d)[i] = ARGB_TO_ARGB2554( src[i] );
                break;
 
           case DSPF_ARGB4444:
                for (i = 0; i < len; i++)
-                    ((__u16*)d)[i] = ARGB_TO_ARGB4444( src[i] );
+                    ((u16*)d)[i] = ARGB_TO_ARGB4444( src[i] );
                break;
                
           case DSPF_RGB16:
                for (i = 0; i < len; i++)
-                    ((__u16*)d)[i] = RGB32_TO_RGB16( src[i] );
+                    ((u16*)d)[i] = RGB32_TO_RGB16( src[i] );
+               break;
+
+          case DSPF_ARGB1666:
+               for (i = 0; i < len; i++) {
+                    u32 pixel = PIXEL_ARGB1666( src[i] >> 24,
+                                                src[i] >> 16,
+                                                src[i] >> 8,
+                                                src[i] );
+
+                    *d++ = pixel;
+                    *d++ = pixel >> 8;
+                    *d++ = pixel >> 16;
+               }
+               break;
+
+          case DSPF_ARGB6666:
+               for (i = 0; i < len; i++) {
+                    u32 pixel = PIXEL_ARGB6666( src[i] >> 24,
+                                                src[i] >> 16,
+                                                src[i] >> 8,
+                                                src[i] );
+
+                    *d++ = pixel;
+                    *d++ = pixel >> 8;
+                    *d++ = pixel >> 16;
+               }
+               break;
+
+          case DSPF_RGB18:
+               for (i = 0; i < len; i++) {
+                    u32 pixel = PIXEL_RGB18( src[i] >> 16,
+                                             src[i] >> 8,
+                                             src[i] );
+
+                    *d++ = pixel;
+                    *d++ = pixel >> 8;
+                    *d++ = pixel >> 16;
+               }
                break;
 
           case DSPF_RGB24:
@@ -153,7 +197,7 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
                
           case DSPF_AiRGB:
                for (i = 0; i < len; i++)
-                    ((__u32*)d)[i] = src[i] ^ 0xff000000;
+                    ((u32*)d)[i] = src[i] ^ 0xff000000;
                break;
 
           case DSPF_LUT8:
@@ -182,20 +226,19 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
 
           case DSPF_YUY2:
                if (dx & 1) {
-                    __u32 y, u, v;
+                    u32 y, u, v;
                     
                     RGB_TO_YCBCR( (src[0] >> 16) & 0xff, 
                                   (src[0] >>  8) & 0xff,
                                   (src[0]      ) & 0xff, y, u, v );
-                    *((__u16*)d) = y | (v << 8);
+                    *((u16*)d) = y | (v << 8);
                     d += 2;
                     src++;
                     len--;
                }
-               
-               for (i = 0; i < (len&~1); i += 2) {
-                    __u32 y0, u, v;
-                    __u32 y1, u1, v1;
+               for (i = 0; i < (len-1); i += 2) {
+                    u32 y0, u, v;
+                    u32 y1, u1, v1;
                     
                     RGB_TO_YCBCR( (src[i+0] >> 16) & 0xff, 
                                   (src[i+0] >>  8) & 0xff,
@@ -206,13 +249,12 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
                                   
                     u = (u + u1) >> 1;
                     v = (v + v1) >> 1;
-                                  
-                    ((__u16*)d)[i+0] = y0 | (u << 8);
-                    ((__u16*)d)[i+1] = y1 | (v << 8);
+                    
+                    ((u16*)d)[i+0] = y0 | (u << 8);
+                    ((u16*)d)[i+1] = y1 | (v << 8);
                }
-               
                if (len & 1) {
-                    __u32 y, u, v;
+                    u32 y, u, v;
                     
                     src += len-1;
                     d   += (len-1)*2;
@@ -220,26 +262,25 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
                     RGB_TO_YCBCR( (*src >> 16) & 0xff, 
                                   (*src >>  8) & 0xff,
                                   (*src      ) & 0xff, y, u, v );
-                    *((__u16*)d) = y | (u << 8);
+                    *((u16*)d) = y | (u << 8);
                }
                break;
 
           case DSPF_UYVY:
                if (dx & 1) {
-                    __u32 y, u, v;
+                    u32 y, u, v;
                     
                     RGB_TO_YCBCR( (src[0] >> 16) & 0xff, 
                                   (src[0] >>  8) & 0xff,
                                   (src[0]      ) & 0xff, y, u, v );
-                    *((__u16*)d) = v | (y << 8);
+                    *((u16*)d) = v | (y << 8);
                     d += 2;
                     src++;
                     len--;
-               }
-               
-               for (i = 0; i < (len&~1); i += 2) {
-                    __u32 y0, u, v;
-                    __u32 y1, u1, v1;
+               } 
+               for (i = 0; i < (len-1); i += 2) {
+                    u32 y0, u, v;
+                    u32 y1, u1, v1;
                     
                     RGB_TO_YCBCR( (src[i+0] >> 16) & 0xff, 
                                   (src[i+0] >>  8) & 0xff,
@@ -251,12 +292,11 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
                     u = (u + u1) >> 1;
                     v = (v + v1) >> 1;
                                   
-                    ((__u16*)d)[i+0] = u | (y0 << 8);
-                    ((__u16*)d)[i+1] = v | (y1 << 8);
+                    ((u16*)d)[i+0] = u | (y0 << 8);
+                    ((u16*)d)[i+1] = v | (y1 << 8);
                }
-               
                if (len & 1) {
-                    __u32 y, u, v;
+                    u32 y, u, v;
                     
                     src += len-1;
                     d   += (len-1)*2;
@@ -264,20 +304,20 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
                     RGB_TO_YCBCR( (*src >> 16) & 0xff, 
                                   (*src >>  8) & 0xff,
                                   (*src      ) & 0xff, y, u, v );
-                    *((__u16*)d) = u | (y << 8);
+                    *((u16*)d) = u | (y << 8);
                }
                break;
 
           case DSPF_AYUV:
                for (i = 0; i < len; i++) {
-                    __u32 a, y, u, v;
+                    u32 a, y, u, v;
 
                     RGB_TO_YCBCR( (src[i] >> 16) & 0xff,
                                   (src[i] >>  8) & 0xff,
                                   (src[i]      ) & 0xff, y, u, v );
                     a = (src[i] >> 24) & 0xff;
 
-                    ((__u32*)d)[i] = PIXEL_AYUV( a, y, u, v );
+                    ((u32*)d)[i] = PIXEL_AYUV( a, y, u, v );
                }
                break;
 
@@ -285,9 +325,9 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
           case DSPF_I420:
                d1 = dst[1];
                d2 = dst[2];
-               for (i = 0; i < (len&~1); i += 2) {
-                    __u32 y0, u0, v0;
-                    __u32 y1, u1, v1;
+               for (i = 0; i < (len-1); i += 2) {
+                    u32 y0, u0, v0;
+                    u32 y1, u1, v1;
                     
                     RGB_TO_YCBCR( (src[i+0] >> 16) & 0xff, 
                                   (src[i+0] >>  8) & 0xff,
@@ -304,14 +344,29 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
                          d2[i>>1] = (v0 + v1) >> 1;
                     }
                }
+               if (len & 1) {
+                    u32 y, u, v;
+                    
+                    i = len-1;
+                    
+                    RGB_TO_YCBCR( (src[i] >> 16) & 0xff, 
+                                  (src[i] >>  8) & 0xff,
+                                  (src[i]      ) & 0xff, y, u, v );
+                    
+                    d[i] = y;
+                    if (dy & 1) {
+                         d1[i>>1] = u;
+                         d2[i>>1] = v;
+                    }
+               }
                break;
                     
           case DSPF_NV12:
           case DSPF_NV16:
                d1 = dst[1];
-               for (i = 0; i < (len&~1); i += 2) {
-                    __u32 y0, u0, v0;
-                    __u32 y1, u1, v1;
+               for (i = 0; i < (len-1); i += 2) {
+                    u32 y0, u0, v0;
+                    u32 y1, u1, v1;
                     
                     RGB_TO_YCBCR( (src[i+0] >> 16) & 0xff, 
                                   (src[i+0] >>  8) & 0xff,
@@ -324,17 +379,30 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
                     d[i+1] = y1;
                                   
                     if (dst_surface->format == DSPF_NV16 || dy & 1) {
-                         ((__u16*)d1)[i>>1] =  ((u0 + u1) >> 1)     |
+                         ((u16*)d1)[i>>1] =    ((u0 + u1) >> 1)     |
                                               (((v0 + v1) >> 1) << 8);
                     }
                }
+               if (len & 1) {
+                    u32 y, u, v;
+                    
+                    i = len-1;
+                    
+                    RGB_TO_YCBCR( (src[i] >> 16) & 0xff, 
+                                  (src[i] >>  8) & 0xff,
+                                  (src[i]      ) & 0xff, y, u, v );
+                                  
+                    d[i] = y;
+                    if (dst_surface->format == DSPF_NV16 || dy & 1)
+                         ((u16*)d1)[i>>1] = u | (v << 8);
+               }
                break;
                
           case DSPF_NV21:
                d1 = dst[1];
-               for (i = 0; i < (len&~1); i += 2) {
-                    __u32 y0, u0, v0;
-                    __u32 y1, u1, v1;
+               for (i = 0; i < (len-1); i += 2) {
+                    u32 y0, u0, v0;
+                    u32 y1, u1, v1;
                     
                     RGB_TO_YCBCR( (src[i+0] >> 16) & 0xff, 
                                   (src[i+0] >>  8) & 0xff,
@@ -347,10 +415,33 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
                     d[i+1] = y1;
                                   
                     if (dy & 1) {
-                         ((__u16*)d1)[i>>1] =  ((v0 + v1) >> 1)     |
+                         ((u16*)d1)[i>>1] =    ((v0 + v1) >> 1)     |
                                               (((u0 + u1) >> 1) << 8);
                     }
                }
+               if (len & 1) {
+                    u32 y, u, v;
+                    
+                    i = len-1;
+                    
+                    RGB_TO_YCBCR( (src[i] >> 16) & 0xff, 
+                                  (src[i] >>  8) & 0xff,
+                                  (src[i]      ) & 0xff, y, u, v );
+                                  
+                    d[i] = y;
+                    if (dy & 1)
+                         ((u16*)d1)[i>>1] = v | (u << 8);
+               }  
+               break;
+
+          case DSPF_RGB555:
+               for (i = 0; i < len; i++)
+                    ((u16*)d)[i] = ARGB_TO_RGB555( src[i] );
+               break;
+
+          case DSPF_RGB444:
+               for (i = 0; i < len; i++)
+                    ((u16*)d)[i] = ARGB_TO_RGB444( src[i] );
                break;
 
           default:
@@ -361,15 +452,43 @@ static void write_argb_span (__u32 *src, __u8 *dst[], int len,
 
 #define LINE_PTR(dst,caps,y,h,pitch) \
      ((caps & DSCAPS_SEPARATED) \
-          ? (((__u8*)(dst)) + (y)/2 * (pitch) + (((y)%2) ? (h)/2 * (pitch) : 0)) \
-          : (((__u8*)(dst)) + (y) * (pitch)))
+          ? (((u8*)(dst)) + (y)/2 * (pitch) + (((y)%2) ? (h)/2 * (pitch) : 0)) \
+          : (((u8*)(dst)) + (y) * (pitch)))
 
-void dfb_copy_buffer_32( __u32 *src,
+void dfb_copy_buffer_32( u32 *src,
                          void  *dst, int dpitch, DFBRectangle *drect,
-                         CoreSurface *dst_surface )
+                         CoreSurface *dst_surface, const DFBRegion *dst_clip )
 {
      void *dst1, *dst2;
-     int   y, x = drect->x;
+     int   sw = drect->w;
+     int   y, x;
+
+     if (dst_clip) {
+          int sx = 0, sy = 0;
+          
+          if (drect->x < dst_clip->x1) {
+               sx = dst_clip->x1-drect->x;
+               drect->w -= sx;
+               drect->x += sx;
+          }
+          if (drect->y < dst_clip->y1) {
+               sy = dst_clip->y1-drect->y;
+               drect->h -= sy;
+               drect->y += sy;
+          }
+          if ((drect->x+drect->w-1) > dst_clip->x2) {
+               drect->w -= drect->x+drect->w-1-dst_clip->x2;
+          }
+          if ((drect->y+drect->h-1) > dst_clip->y2) {
+               drect->h -= drect->y+drect->h-1-dst_clip->y2;
+          }
+
+          src += sy * sw + sx;
+     }
+     
+     if (drect->w < 1 || drect->h < 1)
+          return;
+     x = drect->x;
      
      switch (dst_surface->format) {
           case DSPF_YV12:
@@ -383,7 +502,7 @@ void dfb_copy_buffer_32( __u32 *src,
                }
                
                for (y = drect->y; y < drect->y + drect->h; y++) {
-                    __u8 *d[3];
+                    u8 *d[3];
                     
                     d[0] = LINE_PTR( dst, dst_surface->caps, y,
                                      dst_surface->height, dpitch ) + x;
@@ -394,7 +513,7 @@ void dfb_copy_buffer_32( __u32 *src,
                                      
                     write_argb_span( src, d, drect->w, x, y, dst_surface );
                     
-                    src += drect->w;
+                    src += sw;
                }
                break;
                
@@ -403,7 +522,7 @@ void dfb_copy_buffer_32( __u32 *src,
                dst1 = dst + dpitch * dst_surface->height;
                
                for (y = drect->y; y < drect->y + drect->h; y++) {
-                    __u8 *d[2];
+                    u8 *d[2];
                     
                     d[0] = LINE_PTR( dst, dst_surface->caps, y,
                                      dst_surface->height, dpitch ) + x;
@@ -412,7 +531,7 @@ void dfb_copy_buffer_32( __u32 *src,
  
                     write_argb_span( src, d, drect->w, x, y, dst_surface );
                     
-                    src += drect->w;
+                    src += sw;
                }
                break;
           
@@ -420,7 +539,7 @@ void dfb_copy_buffer_32( __u32 *src,
                dst1 = dst + dpitch * dst_surface->height;
                
                for (y = drect->y; y < drect->y + drect->h; y++) {
-                    __u8 *d[2];
+                    u8 *d[2];
                     
                     d[0] = LINE_PTR( dst, dst_surface->caps, y,
                                      dst_surface->height, dpitch ) + x;
@@ -429,13 +548,13 @@ void dfb_copy_buffer_32( __u32 *src,
  
                     write_argb_span( src, d, drect->w, x, y, dst_surface );
                     
-                    src += drect->w;
+                    src += sw;
                }         
                break;
 
           default:
                for (y = drect->y; y < drect->y + drect->h; y++) {
-                    __u8 *d[1];
+                    u8 *d[1];
                     
                     d[0] = LINE_PTR( dst, dst_surface->caps,
                                      y, dst_surface->height, dpitch ) +
@@ -443,7 +562,7 @@ void dfb_copy_buffer_32( __u32 *src,
 
                     write_argb_span( src, d, drect->w, x, y, dst_surface );
                     
-                    src += drect->w;
+                    src += sw;
                }
                break;
      }
@@ -565,17 +684,17 @@ static int bilinear_make_fast_weights( PixopsFilter *filter, float x_scale, floa
 }
 
 static void scale_pixel( int *weights, int n_x, int n_y,
-                         __u32 *dst, __u32 **src, int x, int sw )
+                         u32 *dst, u32 **src, int x, int sw )
 {
-     __u32 r = 0, g = 0, b = 0, a = 0;
+     u32 r = 0, g = 0, b = 0, a = 0;
      int   i, j;
 
      for (i = 0; i < n_y; i++) {
           int *pixel_weights = weights + n_x * i;
 
           for (j = 0; j < n_x; j++) {
-               __u32  ta;
-               __u32 *q;
+               u32  ta;
+               u32 *q;
 
                if (x + j < 0)
                     q = src[i];
@@ -601,14 +720,14 @@ static void scale_pixel( int *weights, int n_x, int n_y,
      *dst = (a << 24) | (r << 16) | (g << 8) | b;
 }
 
-static __u32* scale_line( int *weights, int n_x, int n_y, 
-                          __u32 *dst, __u32 *dst_end,
-                          __u32 **src, int x, int x_step, int sw )
+static u32* scale_line( int *weights, int n_x, int n_y,
+                        u32 *dst, u32 *dst_end,
+                        u32 **src, int x, int x_step, int sw )
 {
      int    i, j;
      int   *pixel_weights;
-     __u32 *q;
-     __u32  r, g, b, a;
+     u32   *q;
+     u32    r, g, b, a;
      int    x_scaled;
      int   *line_weights;
 
@@ -625,7 +744,7 @@ static __u32* scale_line( int *weights, int n_x, int n_y,
                q = src[i] + x_scaled;
 
                for (j = 0; j < n_x; j++) {
-                    __u32 ta;
+                    u32 ta;
 
                     ta = ((*q & 0xFF000000) >> 24) * line_weights[j];
 
@@ -651,10 +770,11 @@ static __u32* scale_line( int *weights, int n_x, int n_y,
      return dst;
 }
 
-void dfb_scale_linear_32( __u32 *src, int sw, int sh,
+void dfb_scale_linear_32( u32 *src, int sw, int sh,
                           void  *dst, int dpitch, DFBRectangle *drect,
-                          CoreSurface *dst_surface )
+                          CoreSurface *dst_surface, const DFBRegion *dst_clip )
 {
+     DFBRectangle srect = { 0, 0, sw, sh };
      float scale_x, scale_y;
      int i, j;
      int sx, sy;
@@ -662,18 +782,23 @@ void dfb_scale_linear_32( __u32 *src, int sw, int sh,
      int scaled_x_offset;
      PixopsFilter filter;
      void  *dst1 = NULL, *dst2 = NULL;
-     __u32 *buf;
-
-     if (sw < 1 || sh < 1 || drect->w < 1 || drect->h < 1)
-          return;
+     u32 *buf;
 
      if (drect->w == sw && drect->h == sh) {
-          dfb_copy_buffer_32( src, dst, dpitch, drect, dst_surface );
+          dfb_copy_buffer_32( src, dst, dpitch, drect, dst_surface, dst_clip );
           return;
      }
+     
+     if (dst_clip)
+          dfb_clip_stretchblit( dst_clip, &srect, drect );
 
-     scale_x = (float)drect->w / sw;
-     scale_y = (float)drect->h / sh;
+     if (srect.w < 1 || srect.h < 1 || drect->w < 1 || drect->h < 1)
+          return;
+
+     src += srect.y * sw + srect.x;
+
+     scale_x = (float)drect->w / srect.w;
+     scale_y = (float)drect->h / srect.h;
 
      x_step = (1 << SCALE_SHIFT) / scale_x;
      y_step = (1 << SCALE_SHIFT) / scale_y;
@@ -702,24 +827,24 @@ void dfb_scale_linear_32( __u32 *src, int sw, int sh,
                break;
      }
      
-     buf = (__u32*) alloca( drect->w*4 );
+     buf = (u32*) alloca( drect->w*4 );
      
      for (i = drect->y; i < drect->y + drect->h; i++) {
           int     x_start;
           int     y_start;
           int    *run_weights;
-          __u32  *outbuf     = buf;
-          __u32  *outbuf_end = buf+drect->w;
-          __u32  *new_outbuf;
-          __u32 **line_bufs;
-          __u8   *d[3];
+          u32    *outbuf     = buf;
+          u32    *outbuf_end = buf+drect->w;
+          u32    *new_outbuf;
+          u32   **line_bufs;
+          u8     *d[3];
 
           y_start = sy >> SCALE_SHIFT;
 
           run_weights = filter.weights + ((sy >> (SCALE_SHIFT - SUBSAMPLE_BITS))
                                           & SUBSAMPLE_MASK) * filter.n_x * filter.n_y * SUBSAMPLE;
 
-          line_bufs = (__u32 **) alloca( filter.n_y * sizeof (void *) );
+          line_bufs = (u32 **) alloca( filter.n_y * sizeof (void *) );
 
           for (j = 0; j < filter.n_y; j++) {
                if (y_start <  0)
