@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -47,6 +48,8 @@
 # include <sysfs/libsysfs.h>
 #endif
 
+#include <asm/types.h>    /* Needs to be included before dfb_types.h */
+
 #include <fusion/arena.h>
 #include <fusion/fusion.h>
 #include <fusion/reactor.h>
@@ -222,7 +225,7 @@ static DFBResult dfb_fbdev_read_modes();
 static DFBResult dfb_fbdev_set_gamma_ramp( DFBSurfacePixelFormat format );
 static DFBResult dfb_fbdev_set_palette( CorePalette *palette );
 static DFBResult dfb_fbdev_set_rgb332_palette();
-static DFBResult dfb_fbdev_pan( int offset, bool onsync );
+static DFBResult dfb_fbdev_pan( int xoffset, int yoffset, bool onsync );
 static DFBResult dfb_fbdev_blank( int level );
 static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                                      VideoMode             *mode,
@@ -260,13 +263,16 @@ void waitretrace (void)
 
 static DFBResult dfb_fbdev_open()
 {
+     DFBResult error_result = DFB_FAILURE;
+
      if (dfb_config->fb_device) {
           dfb_fbdev->fd = open( dfb_config->fb_device, O_RDWR );
           if (dfb_fbdev->fd < 0) {
                D_PERROR( "DirectFB/FBDev: Error opening '%s'!\n",
                          dfb_config->fb_device);
 
-               return errno2result( errno );
+               error_result = errno2result( errno );
+               goto error;
           }
      }
      else if (getenv( "FRAMEBUFFER" ) && *getenv( "FRAMEBUFFER" ) != '\0') {
@@ -275,7 +281,8 @@ static DFBResult dfb_fbdev_open()
                D_PERROR( "DirectFB/FBDev: Error opening '%s'!\n",
                           getenv ("FRAMEBUFFER"));
 
-               return errno2result( errno );
+               error_result = errno2result( errno );
+               goto error;
           }
      }
      else {
@@ -283,11 +290,21 @@ static DFBResult dfb_fbdev_open()
           if (dfb_fbdev->fd < 0) {
                D_ERROR( "DirectFB/FBDev: Error opening framebuffer device!\n" );
                D_ERROR( "DirectFB/FBDev: Use 'fbdev' option or set FRAMEBUFFER environment variable.\n" );
-               return DFB_INIT;
+               error_result = DFB_INIT;
+               goto error;
           }
      }
 
+     /* should be closed automatically in children upon exec(...) */
+     if (fcntl( dfb_fbdev->fd, F_SETFD, FD_CLOEXEC ) < 0)
+     {
+          D_PERROR( "Fusion/Init: Setting FD_CLOEXEC flag failed!\n" );
+          goto error;
+     }
+
      return DFB_OK;
+error:
+     return error_result; 
 }
 
 /******************************************************************************/
@@ -406,7 +423,7 @@ system_initialize( CoreDFB *core, void **data )
      DFBResult            ret;
      CoreScreen          *screen;
      long                 page_size;
-     FBDevShared         *shared;
+     FBDevShared         *shared = NULL;
      FusionSHMPoolShared *pool;
      FusionSHMPoolShared *pool_data;
 
@@ -416,8 +433,16 @@ system_initialize( CoreDFB *core, void **data )
      pool_data = dfb_core_shmpool_data( core );
 
      dfb_fbdev = D_CALLOC( 1, sizeof(FBDev) );
+     if (!dfb_fbdev)
+          return D_OOM();
+
+     dfb_fbdev->fd = -1;
 
      shared = (FBDevShared*) SHCALLOC( pool, 1, sizeof(FBDevShared) );
+     if (!shared) {
+          ret = D_OOSHM();
+          goto error;
+     }
 
      shared->shmpool      = pool;
      shared->shmpool_data = pool_data;
@@ -432,62 +457,39 @@ system_initialize( CoreDFB *core, void **data )
      shared->page_mask = page_size < 0 ? 0 : (page_size - 1);
 
      ret = dfb_fbdev_open();
-     if (ret) {
-          SHFREE( pool, shared );
-          D_FREE( dfb_fbdev );
-          dfb_fbdev = NULL;
-
-          return ret;
-     }
+     if (ret)
+          goto error;
 
      if (dfb_config->vt) {
           ret = dfb_vt_initialize();
-          if (ret) {
-               SHFREE( pool, shared );
-               D_FREE( dfb_fbdev );
-               dfb_fbdev = NULL;
-
-               return ret;
-          }
+          if (ret)
+               goto error;
      }
 
+     ret = DFB_INIT;
+
      /* Retrieve fixed informations like video ram size */
      if (ioctl( dfb_fbdev->fd, FBIOGET_FSCREENINFO, &shared->fix ) < 0) {
           D_PERROR( "DirectFB/FBDev: "
                     "Could not get fixed screen information!\n" );
-          SHFREE( pool, shared );
-          close( dfb_fbdev->fd );
-          D_FREE( dfb_fbdev );
-          dfb_fbdev = NULL;
-
-          return DFB_INIT;
+          goto error;
      }
 
      /* Map the framebuffer */
      dfb_fbdev->framebuffer_base = mmap( NULL, shared->fix.smem_len,
                                          PROT_READ | PROT_WRITE, MAP_SHARED,
                                          dfb_fbdev->fd, 0 );
-     if ((int)(dfb_fbdev->framebuffer_base) == -1) {
+     if (dfb_fbdev->framebuffer_base == MAP_FAILED) {
           D_PERROR( "DirectFB/FBDev: "
                     "Could not mmap the framebuffer!\n");
-          SHFREE( pool, shared );
-          close( dfb_fbdev->fd );
-          D_FREE( dfb_fbdev );
-          dfb_fbdev = NULL;
-
-          return DFB_INIT;
+          dfb_fbdev->framebuffer_base = NULL;
+          goto error;
      }
 
      if (ioctl( dfb_fbdev->fd, FBIOGET_VSCREENINFO, &shared->orig_var ) < 0) {
           D_PERROR( "DirectFB/FBDev: "
                     "Could not get variable screen information!\n" );
-          SHFREE( pool, shared );
-          munmap( dfb_fbdev->framebuffer_base, shared->fix.smem_len );
-          close( dfb_fbdev->fd );
-          D_FREE( dfb_fbdev );
-          dfb_fbdev = NULL;
-
-          return DFB_INIT;
+          goto error;
      }
 
      shared->current_var = shared->orig_var;
@@ -496,45 +498,60 @@ system_initialize( CoreDFB *core, void **data )
      if (ioctl( dfb_fbdev->fd, FBIOPUT_VSCREENINFO, &shared->current_var ) < 0) {
           D_PERROR( "DirectFB/FBDev: "
                     "Could not disable console acceleration!\n" );
-          SHFREE( pool, shared );
-          munmap( dfb_fbdev->framebuffer_base, shared->fix.smem_len );
-          close( dfb_fbdev->fd );
-          D_FREE( dfb_fbdev );
-          dfb_fbdev = NULL;
-
-          return DFB_INIT;
+          goto error;
      }
 
      dfb_fbdev_var_to_mode( &shared->current_var,
                             &shared->current_mode );
 
-     shared->cmap_memory = SHMALLOC( pool_data, 256 * 2 * 12 );
+     shared->orig_cmap_memory = SHMALLOC( pool_data, 256 * 2 * 4 );
+     if (!shared->orig_cmap_memory) {
+          ret = D_OOSHM();
+          goto error;
+     }
 
      shared->orig_cmap.start  = 0;
      shared->orig_cmap.len    = 256;
-     shared->orig_cmap.red    = shared->cmap_memory + 256 * 2 * 0;
-     shared->orig_cmap.green  = shared->cmap_memory + 256 * 2 * 1;
-     shared->orig_cmap.blue   = shared->cmap_memory + 256 * 2 * 2;
-     shared->orig_cmap.transp = shared->cmap_memory + 256 * 2 * 3;
+     shared->orig_cmap.red    = shared->orig_cmap_memory + 256 * 2 * 0;
+     shared->orig_cmap.green  = shared->orig_cmap_memory + 256 * 2 * 1;
+     shared->orig_cmap.blue   = shared->orig_cmap_memory + 256 * 2 * 2;
+     shared->orig_cmap.transp = shared->orig_cmap_memory + 256 * 2 * 3;
 
      if (ioctl( dfb_fbdev->fd, FBIOGETCMAP, &shared->orig_cmap ) < 0) {
-          D_PERROR( "DirectFB/FBDev: "
-                    "Could not retrieve palette for backup!\n" );
-          SHFREE( pool_data, shared->cmap_memory );
-          shared->orig_cmap.len = 0;
+          D_DEBUG( "DirectFB/FBDev: "
+                   "Could not retrieve palette for backup!\n" );
+
+          memset( &shared->orig_cmap, 0, sizeof(shared->orig_cmap) );
+
+          SHFREE( pool_data, shared->orig_cmap_memory );
+          shared->orig_cmap_memory = NULL;
      }
 
-     shared->temp_cmap.len    = 256;
-     shared->temp_cmap.red    = shared->cmap_memory + 256 * 2 * 4;
-     shared->temp_cmap.green  = shared->cmap_memory + 256 * 2 * 5;
-     shared->temp_cmap.blue   = shared->cmap_memory + 256 * 2 * 6;
-     shared->temp_cmap.transp = shared->cmap_memory + 256 * 2 * 7;
+     shared->temp_cmap_memory = SHMALLOC( pool_data, 256 * 2 * 4 );
+     if (!shared->temp_cmap_memory) {
+          ret = D_OOSHM();
+          goto error;
+     }
 
+     shared->temp_cmap.start  = 0;
+     shared->temp_cmap.len    = 256;
+     shared->temp_cmap.red    = shared->temp_cmap_memory + 256 * 2 * 0;
+     shared->temp_cmap.green  = shared->temp_cmap_memory + 256 * 2 * 1;
+     shared->temp_cmap.blue   = shared->temp_cmap_memory + 256 * 2 * 2;
+     shared->temp_cmap.transp = shared->temp_cmap_memory + 256 * 2 * 3;
+
+     shared->current_cmap_memory = SHMALLOC( pool_data, 256 * 2 * 4 );
+     if (!shared->current_cmap_memory) {
+          ret = D_OOSHM();
+          goto error;
+     }
+
+     shared->current_cmap.start  = 0;
      shared->current_cmap.len    = 256;
-     shared->current_cmap.red    = shared->cmap_memory + 256 * 2 * 8;
-     shared->current_cmap.green  = shared->cmap_memory + 256 * 2 * 9;
-     shared->current_cmap.blue   = shared->cmap_memory + 256 * 2 * 10;
-     shared->current_cmap.transp = shared->cmap_memory + 256 * 2 * 11;
+     shared->current_cmap.red    = shared->current_cmap_memory + 256 * 2 * 0;
+     shared->current_cmap.green  = shared->current_cmap_memory + 256 * 2 * 1;
+     shared->current_cmap.blue   = shared->current_cmap_memory + 256 * 2 * 2;
+     shared->current_cmap.transp = shared->current_cmap_memory + 256 * 2 * 3;
      
      dfb_fbdev_get_pci_info( shared );
 
@@ -544,6 +561,7 @@ system_initialize( CoreDFB *core, void **data )
           if (ret) {
                D_DEBUG( "DirectFB/FBDev: dfb_agp_initialize()\n\t->%s\n",
                          DirectFBErrorString( ret ) );
+               ret = DFB_OK;
           }
      }         
 
@@ -559,6 +577,32 @@ system_initialize( CoreDFB *core, void **data )
      *data = dfb_fbdev;
 
      return DFB_OK;
+
+
+error:
+     if (shared) {
+          if (shared->orig_cmap_memory)
+               SHFREE( pool_data, shared->orig_cmap_memory );
+
+          if (shared->temp_cmap_memory)
+               SHFREE( pool_data, shared->temp_cmap_memory );
+
+          if (shared->current_cmap_memory)
+               SHFREE( pool_data, shared->current_cmap_memory );
+
+          SHFREE( pool, shared );
+     }
+
+     if (dfb_fbdev->framebuffer_base)
+          munmap( dfb_fbdev->framebuffer_base, shared->fix.smem_len );
+
+     if (dfb_fbdev->fd != -1)
+          close( dfb_fbdev->fd );
+     
+     D_FREE( dfb_fbdev );
+     dfb_fbdev = NULL;
+
+     return ret;
 }
 
 static DFBResult
@@ -566,6 +610,7 @@ system_join( CoreDFB *core, void **data )
 {
      DFBResult   ret;
      CoreScreen *screen;
+     void       *shared;
 
      D_ASSERT( dfb_fbdev == NULL );
 
@@ -578,9 +623,10 @@ system_join( CoreDFB *core, void **data )
      dfb_fbdev = D_CALLOC( 1, sizeof(FBDev) );
 
      fusion_arena_get_shared_field( dfb_core_arena( core ),
-                                    "fbdev", (void**) &dfb_fbdev->shared );
+                                    "fbdev", &shared );
 
      dfb_fbdev->core = core;
+     dfb_fbdev->shared = shared;
 
      /* Open framebuffer device */
      ret = dfb_fbdev_open();
@@ -594,7 +640,7 @@ system_join( CoreDFB *core, void **data )
      dfb_fbdev->framebuffer_base = mmap( NULL, dfb_fbdev->shared->fix.smem_len,
                                          PROT_READ | PROT_WRITE, MAP_SHARED,
                                          dfb_fbdev->fd, 0 );
-     if ((int)(dfb_fbdev->framebuffer_base) == -1) {
+     if (dfb_fbdev->framebuffer_base == MAP_FAILED) {
           D_PERROR( "DirectFB/FBDev: "
                     "Could not mmap the framebuffer!\n");
           close( dfb_fbdev->fd );
@@ -660,11 +706,18 @@ system_shutdown( bool emergency )
 
      if (shared->orig_cmap.len) {
           if (ioctl( dfb_fbdev->fd, FBIOPUTCMAP, &shared->orig_cmap ) < 0)
-               D_PERROR( "DirectFB/FBDev: "
-                         "Could not restore palette!\n" );
+               D_DEBUG( "DirectFB/FBDev: "
+                        "Could not restore palette!\n" );
      }
 
-     SHFREE( shared->shmpool_data, shared->cmap_memory );
+     if (shared->orig_cmap_memory)
+          SHFREE( shared->shmpool_data, shared->orig_cmap_memory );
+
+     if (shared->temp_cmap_memory)
+          SHFREE( shared->shmpool_data, shared->temp_cmap_memory );
+
+     if (shared->current_cmap_memory)
+          SHFREE( shared->shmpool_data, shared->current_cmap_memory );
 
      fusion_call_destroy( &shared->fbdev_ioctl );
 
@@ -738,14 +791,14 @@ system_map_mmio( unsigned int    offset,
 
      addr = mmap( NULL, length, PROT_READ | PROT_WRITE, MAP_SHARED,
                   dfb_fbdev->fd, dfb_fbdev->shared->fix.smem_len + offset );
-     if ((int)(addr) == -1) {
+     if (addr == MAP_FAILED) {
           D_PERROR( "DirectFB/FBDev: Could not mmap MMIO region "
                      "(offset %d, length %d)!\n", offset, length );
           return NULL;
      }
 
-     return(volatile void*) ((__u8*) addr + (dfb_fbdev->shared->fix.mmio_start &
-                                             dfb_fbdev->shared->page_mask));
+     return(volatile void*) ((u8*) addr + (dfb_fbdev->shared->fix.mmio_start &
+                                           dfb_fbdev->shared->page_mask));
 }
 
 static void
@@ -755,8 +808,8 @@ system_unmap_mmio( volatile void  *addr,
      if (length <= 0)
           length = dfb_fbdev->shared->fix.mmio_len;
 
-     if (munmap( (void*) ((__u8*) addr - (dfb_fbdev->shared->fix.mmio_start &
-                                          dfb_fbdev->shared->page_mask)), length ) < 0)
+     if (munmap( (void*) ((u8*) addr - (dfb_fbdev->shared->fix.mmio_start &
+                                        dfb_fbdev->shared->page_mask)), length ) < 0)
           D_PERROR( "DirectFB/FBDev: Could not unmap MMIO region "
                      "at %p (length %d)!\n", addr, length );
 }
@@ -791,9 +844,6 @@ system_thread_init()
      if (dfb_config->block_all_signals)
           direct_signals_block_all();
 
-     if (dfb_config->vt)
-          return dfb_vt_detach( false );
-
      return DFB_OK;
 }
 
@@ -834,7 +884,7 @@ system_video_memory_physical( unsigned int offset )
 static void *
 system_video_memory_virtual( unsigned int offset )
 {
-     return(void*)((__u8*)(dfb_fbdev->framebuffer_base) + offset);
+     return(void*)((u8*)(dfb_fbdev->framebuffer_base) + offset);
 }
 
 static unsigned int
@@ -855,7 +905,7 @@ static void *
 system_aux_memory_virtual( unsigned int offset )
 {
      if (dfb_fbdev->agp)
-          return (void*)(__u8*)dfb_fbdev->agp->base + offset;
+          return (void*)(u8*)dfb_fbdev->agp->base + offset;
      return NULL;
 }
 
@@ -1057,6 +1107,10 @@ primaryInitLayer( CoreLayer                  *layer,
      
      tmp.format     = DSPF_RGB16;
      tmp.buffermode = DLBM_FRONTONLY;
+     tmp.source.x   = 0;
+     tmp.source.y   = 0;
+     tmp.source.w   = config->width;
+     tmp.source.h   = config->height;
      if (dfb_fbdev_set_mode( NULL, NULL, &tmp ))
           config->pixelformat = dfb_pixelformat_for_depth( dfb_fbdev->shared->orig_var.bits_per_pixel );
      else
@@ -1205,8 +1259,8 @@ primaryTestRegion( CoreLayer                  *layer,
 
      videomode = dfb_fbdev->shared->modes;
      while (videomode) {
-          if (videomode->xres == config->width  &&
-              videomode->yres == config->height)
+          if (videomode->xres == config->source.w  &&
+              videomode->yres == config->source.h)
                break;
 
           videomode = videomode->next;
@@ -1250,11 +1304,12 @@ primarySetRegion( CoreLayer                  *layer,
      DFBResult  ret;
      VideoMode *videomode;
      VideoMode *highest = NULL;
+     FBDevShared *shared = dfb_fbdev->shared;
 
-     videomode = dfb_fbdev->shared->modes;
+     videomode = shared->modes;
      while (videomode) {
-          if (videomode->xres == config->width  &&
-              videomode->yres == config->height)
+          if (videomode->xres == config->source.w  &&
+              videomode->yres == config->source.h)
           {
                if (!highest || highest->priority < videomode->priority)
                     highest = videomode;
@@ -1266,7 +1321,21 @@ primarySetRegion( CoreLayer                  *layer,
      if (!highest)
           return DFB_UNSUPPORTED;
 
-     if (updated & (CLRCF_BUFFERMODE | CLRCF_FORMAT | CLRCF_HEIGHT | CLRCF_SURFACE | CLRCF_WIDTH)) {
+     switch (updated & (CLRCF_BUFFERMODE | CLRCF_FORMAT | CLRCF_HEIGHT |
+                        CLRCF_SURFACE | CLRCF_WIDTH |  CLRCF_SOURCE)) {
+     case CLRCF_SOURCE:
+          if (config->source.w == shared->current_mode.xres &&
+              config->source.h == shared->current_mode.yres) {
+               ret = dfb_fbdev_pan( config->source.x,
+                                    surface->front_buffer->video.offset /
+                                    surface->front_buffer->video.pitch + config->source.y,
+                                    true );
+               if (ret)
+                    return ret;
+               break;
+          }
+          /* fall through */
+     default:
           ret = dfb_fbdev_set_mode( surface, highest, config );
           if (ret)
                return ret;
@@ -1275,6 +1344,9 @@ primarySetRegion( CoreLayer                  *layer,
      if ((updated & CLRCF_PALETTE) && palette)
           dfb_fbdev_set_palette( palette );
 
+     /* remember configuration */
+     shared->config = *config;
+
      return DFB_OK;
 }
 
@@ -1296,13 +1368,15 @@ primaryFlipRegion( CoreLayer           *layer,
                    DFBSurfaceFlipFlags  flags )
 {
      DFBResult ret;
+     CoreLayerRegionConfig *config = &dfb_fbdev->shared->config;
 
      if (((flags & DSFLIP_WAITFORSYNC) == DSFLIP_WAITFORSYNC) &&
          !dfb_config->pollvsync_after)
           dfb_screen_wait_vsync( dfb_screens_at(DSCID_PRIMARY) );
 
-     ret = dfb_fbdev_pan( surface->back_buffer->video.offset /
-                          surface->back_buffer->video.pitch,
+     ret = dfb_fbdev_pan( config->source.x,
+                          surface->back_buffer->video.offset /
+                          surface->back_buffer->video.pitch + config->source.y,
                           (flags & DSFLIP_WAITFORSYNC) == DSFLIP_ONSYNC );
      if (ret)
           return ret;
@@ -1448,10 +1522,10 @@ static int dfb_fbdev_compatible_format( struct fb_var_screeninfo *var,
      vgh = var->green.length + var->green.offset - 1;
      vbh = var->blue.length + var->blue.offset - 1;
 
-     if (ah == vah && al >= (int)var->transp.length &&
-         rh == vrh && rl >= (int)var->red.length &&
-         gh == vgh && gl >= (int)var->green.length &&
-         bh == vbh && bl >= (int)var->blue.length)
+     if ((!al || (ah == vah && al >= (int)var->transp.length)) &&
+         (!rl || (rh == vrh && rl >= (int)var->red.length)) &&
+         (!gl || (gh == vgh && gl >= (int)var->green.length)) &&
+         (!bl || (bh == vbh && bl >= (int)var->blue.length)))
           return 1;
 
      return 0;
@@ -1470,28 +1544,45 @@ static DFBSurfacePixelFormat dfb_fbdev_get_pixelformat( struct fb_var_screeninfo
                if (fbdev_compatible_format( var, 0, 3, 3, 2, 0, 5, 2, 0 ))*/
 
                return DSPF_RGB332;
-
           case 15:
-               if (dfb_fbdev_compatible_format( var, 0, 5, 5, 5, 0, 10, 5, 0 ) |
-                   dfb_fbdev_compatible_format( var, 1, 5, 5, 5,15, 10, 5, 0 ) )
-                    return DSPF_ARGB1555;
+               if (dfb_fbdev_compatible_format( var, 0, 5, 5, 5, 0, 10, 5, 0 ))
+                    return DSPF_RGB555;
+
+               if(dfb_fbdev_compatible_format( var, 1, 5, 5, 5, 15, 10, 5, 0 ))
+                   return DSPF_ARGB1555;
 
                break;
 
-          case 16:
-               if (dfb_fbdev_compatible_format( var, 0, 5, 5, 5, 0, 10, 5, 0 ) |
-                   dfb_fbdev_compatible_format( var, 1, 5, 5, 5,15, 10, 5, 0 ) )
-                    return DSPF_ARGB1555;
+         case 16:
 
-               if (dfb_fbdev_compatible_format( var, 0, 4, 4, 4,  0, 8, 4, 0 ) |
-                   dfb_fbdev_compatible_format( var, 4, 4, 4, 4, 12, 8, 4, 0 ) )
+               if (dfb_fbdev_compatible_format( var, 0, 5, 5, 5, 0, 10, 5, 0 ))
+                    return DSPF_RGB555;
+
+               if(dfb_fbdev_compatible_format( var, 1, 5, 5, 5, 15, 10, 5, 0 ))
+                   return DSPF_ARGB1555;
+
+               if (dfb_fbdev_compatible_format( var, 4, 4, 4, 4, 12, 8, 4, 0 ))
                     return DSPF_ARGB4444;
 
+               if (dfb_fbdev_compatible_format( var, 0, 4, 4, 4, 0, 8, 4, 0 ))
+                    return DSPF_RGB444;
+
                if (dfb_fbdev_compatible_format( var, 0, 5, 6, 5, 0, 11, 5, 0 ))
                     return DSPF_RGB16;
 
                break;
 
+          case 18:
+               if (dfb_fbdev_compatible_format( var, 1, 6, 6, 6, 18, 12, 6, 0 ))
+                    return DSPF_ARGB1666;
+
+               if (dfb_fbdev_compatible_format( var, 6, 6, 6, 6, 18, 12, 6, 0 ))
+                    return DSPF_ARGB6666;
+
+               if (dfb_fbdev_compatible_format( var, 0, 6, 6, 6, 0, 12, 6, 0 ))
+                    return DSPF_RGB18;
+               break;
+
           case 24:
                if (dfb_fbdev_compatible_format( var, 0, 8, 8, 8, 0, 16, 8, 0 ))
                     return DSPF_RGB24;
@@ -1522,29 +1613,50 @@ static DFBSurfacePixelFormat dfb_fbdev_get_pixelformat( struct fb_var_screeninfo
 /*
  * pans display (flips buffer) using fbdev ioctl
  */
-static DFBResult dfb_fbdev_pan( int offset, bool onsync )
+static DFBResult dfb_fbdev_pan( int xoffset, int yoffset, bool onsync )
 {
      struct fb_var_screeninfo *var;
 
      var = &dfb_fbdev->shared->current_var;
 
-     if (var->yres_virtual < offset + var->yres) {
-          D_ERROR( "DirectFB/FBDev: yres %d, vyres %d, offset %d\n",
-                    var->yres, var->yres_virtual, offset );
+     if (var->xres_virtual < xoffset + var->xres) {
+          D_ERROR( "DirectFB/FBDev: xres %d, vxres %d, xoffset %d\n",
+                    var->xres, var->xres_virtual, xoffset ); 
           D_BUG( "panning buffer out of range" );
           return DFB_BUG;
      }
 
-     var->xoffset = 0;
-     var->yoffset = offset;
-     var->activate = onsync ? FB_ACTIVATE_VBL : FB_ACTIVATE_NOW;
+     if (var->yres_virtual < yoffset + var->yres) {
+          D_ERROR( "DirectFB/FBDev: yres %d, vyres %d, offset %d\n",
+                    var->yres, var->yres_virtual, yoffset );
+          D_BUG( "panning buffer out of range" );
+          return DFB_BUG;
+     }
 
-     dfb_gfxcard_sync();
+     if (dfb_fbdev->shared->fix.xpanstep)
+          var->xoffset = xoffset - (xoffset % dfb_fbdev->shared->fix.xpanstep);
+     else
+          var->xoffset = 0;
+
+     if (dfb_fbdev->shared->fix.ywrapstep) {
+          var->yoffset = yoffset - (yoffset % dfb_fbdev->shared->fix.ywrapstep);
+          var->vmode |= FB_VMODE_YWRAP;
+     } else if (dfb_fbdev->shared->fix.ypanstep) {
+          var->yoffset = yoffset - (yoffset % dfb_fbdev->shared->fix.ypanstep);
+          var->vmode &= ~FB_VMODE_YWRAP;
+     } else {
+          var->yoffset = 0;
+     }
+
+     var->activate = onsync ? FB_ACTIVATE_VBL : FB_ACTIVATE_NOW;
 
      if (FBDEV_IOCTL( FBIOPAN_DISPLAY, var ) < 0) {
           int erno = errno;
 
-          D_PERROR( "DirectFB/FBDev: Panning display failed!\n" );
+          D_PERROR( "DirectFB/FBDev: Panning display failed (x=%u y=%u ywrap=%d vbl=%d)!\n",
+                    var->xoffset, var->yoffset,
+                    (var->vmode & FB_VMODE_YWRAP) ? 1 : 0,
+                    (var->activate & FB_ACTIVATE_VBL) ? 1 : 0);
 
           return errno2result( erno );
      }
@@ -1574,7 +1686,7 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                                      VideoMode             *mode,
                                      CoreLayerRegionConfig *config )
 {
-     unsigned int              vyres;
+     unsigned int              vxres, vyres;
      struct fb_var_screeninfo  var;
      FBDevShared              *shared = dfb_fbdev->shared;
      DFBSurfacePixelFormat     format;
@@ -1593,21 +1705,34 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
 
      if (!mode)
           mode = &shared->current_mode;
-
-     vyres = mode->yres;
-
+ 
      var = shared->current_var;
 
-     var.xoffset = 0;
-     var.yoffset = 0;
-
      if (config) {
+          DFBRectangle *source = &config->source;
+
+          /* Is panning supported? */
+          if (source->w != mode->xres && shared->fix.xpanstep == 0)
+               return DFB_UNSUPPORTED;
+          if (source->h != mode->yres && shared->fix.ypanstep == 0 && shared->fix.ywrapstep == 0)
+               return DFB_UNSUPPORTED;
+
+          vxres = config->width;
+          vyres = config->height;
+
+          var.xoffset = source->x;
+          var.yoffset = source->y;
+
           switch (config->buffermode) {
                case DLBM_TRIPLE:
+                    if (shared->fix.ypanstep == 0 && shared->fix.ywrapstep == 0)
+                         return DFB_UNSUPPORTED;
                     vyres *= 3;
                     break;
 
                case DLBM_BACKVIDEO:
+                    if (shared->fix.ypanstep == 0 && shared->fix.ywrapstep == 0)
+                         return DFB_UNSUPPORTED;
                     vyres *= 2;
                     break;
 
@@ -1635,6 +1760,15 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                     var.blue.offset   = 0;
                     break;
 
+               case DSPF_RGB555:
+                    var.red.length    = 5;
+                    var.green.length  = 5;
+                    var.blue.length   = 5;
+                    var.red.offset    = 10;
+                    var.green.offset  = 5;
+                    var.blue.offset   = 0;
+                    break;
+
                case DSPF_ARGB4444:
                     var.transp.length = 4;
                     var.red.length    = 4;
@@ -1646,6 +1780,24 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                     var.blue.offset   = 0;
                     break;
 
+              case DSPF_RGB444:
+                    var.red.length = 4;
+                    var.green.length = 4;
+                    var.blue.length = 4;
+                    var.red.offset = 8;
+                    var.green.offset = 4;
+                    var.blue.offset =  0;
+                    break;
+
+              case DSPF_RGB32:
+                   var.red.length    = 8;
+                   var.green.length  = 8;
+                   var.blue.length   = 8;
+                   var.red.offset    = 16;
+                   var.green.offset  = 8;
+                   var.blue.offset   = 0;
+                   break;
+
                case DSPF_RGB16:
                     var.red.length    = 5;
                     var.green.length  = 6;
@@ -1669,22 +1821,59 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
 
                case DSPF_LUT8:
                case DSPF_RGB24:
-               case DSPF_RGB32:
                case DSPF_RGB332:
                     break;
 
+               case DSPF_ARGB1666:
+                    var.transp.length = 1;
+                    var.red.length    = 6;
+                    var.green.length  = 6;
+                    var.blue.length   = 6;
+                    var.transp.offset = 18;
+                    var.red.offset    = 12;
+                    var.green.offset  = 6;
+                    var.blue.offset   = 0;
+                    break;
+
+               case DSPF_ARGB6666:
+                    var.transp.length = 6;
+                    var.red.length    = 6;
+                    var.green.length  = 6;
+                    var.blue.length   = 6;
+                    var.transp.offset = 18;
+                    var.red.offset    = 12;
+                    var.green.offset  = 6;
+                    var.blue.offset   = 0;
+                    break;
+
+               case DSPF_RGB18:
+                    var.red.length    = 6;
+                    var.green.length  = 6;
+                    var.blue.length   = 6;
+                    var.red.offset    = 12;
+                    var.green.offset  = 6;
+                    var.blue.offset   = 0;
+                    break;
+
                default:
                     return DFB_UNSUPPORTED;
           }
      }
-     else
+     else {
+          vxres = mode->xres;
+          vyres = mode->yres;
+
+          var.xoffset = 0;
+          var.yoffset = 0;
+
           var.bits_per_pixel = mode->bpp;
+     }
 
      var.activate = surface ? FB_ACTIVATE_NOW : FB_ACTIVATE_TEST;
 
      var.xres = mode->xres;
      var.yres = mode->yres;
-     var.xres_virtual = mode->xres;
+     var.xres_virtual = vxres;
      var.yres_virtual = vyres;
 
      var.pixclock = mode->pixclock;
@@ -1737,6 +1926,7 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
      if (shared->fix.smem_len < (var.yres_virtual *
                                  var.xres_virtual *
                                  var.bits_per_pixel >> 3)
+         || (var.xres_virtual < vxres)
          || (var.yres_virtual < vyres))
      {
           if (surface) {
@@ -1758,6 +1948,19 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
 
           FBDEV_IOCTL( FBIOGET_VSCREENINFO, &var );
 
+          vxres = var.xres_virtual;
+          switch (config->buffermode) {
+          case DLBM_TRIPLE:
+               vyres = var.yres_virtual / 3;
+               break;
+          case DLBM_BACKVIDEO:
+               vyres = var.yres_virtual / 2;
+               break;
+          default:
+               vyres = var.yres_virtual;
+               break;
+          }
+
           format = dfb_fbdev_get_pixelformat( &var );
           if (format == DSPF_UNKNOWN) {
                D_WARN( "unknown format" );
@@ -1777,8 +1980,10 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
           }
 
           if (format != config->format) {
-               if (DFB_BYTES_PER_PIXEL(format) == 1 ||
-                   (format == DSPF_ARGB && config->format == DSPF_AiRGB))
+               if (DFB_BYTES_PER_PIXEL(format) == 1                      ||
+                  (format == DSPF_RGB32 && config->format == DSPF_ARGB)  ||
+                  (format == DSPF_RGB32 && config->format == DSPF_AiRGB) ||
+                  (format == DSPF_ARGB  && config->format == DSPF_AiRGB))
                     format = config->format;
           }
 
@@ -1794,8 +1999,8 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
           shared->orig_var.xoffset = 0;
           shared->orig_var.yoffset = 0;
 
-          surface->width  = mode->xres;
-          surface->height = mode->yres;
+          surface->width  = vxres;
+          surface->height = vyres;
           surface->format = format;
 
           /* To get the new pitch */
@@ -1856,7 +2061,7 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                     surface->back_buffer->video.health = CSH_STORED;
                     surface->back_buffer->video.pitch = fix.line_length;
                     surface->back_buffer->video.offset =
-                         surface->back_buffer->video.pitch * var.yres;
+                         surface->back_buffer->video.pitch * vyres;
 
                     if (surface->idle_buffer != surface->front_buffer) {
                          if (surface->idle_buffer->system.addr)
@@ -1888,7 +2093,7 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                     surface->back_buffer->video.health = CSH_STORED;
                     surface->back_buffer->video.pitch = fix.line_length;
                     surface->back_buffer->video.offset =
-                         surface->back_buffer->video.pitch * var.yres;
+                         surface->back_buffer->video.pitch * vyres;
 
                     if (surface->idle_buffer == surface->front_buffer) {
                          surface->idle_buffer = SHCALLOC( surface->shmpool, 1, sizeof(SurfaceBuffer) );
@@ -1907,7 +2112,7 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                     surface->idle_buffer->video.health = CSH_STORED;
                     surface->idle_buffer->video.pitch = fix.line_length;
                     surface->idle_buffer->video.offset =
-                         surface->idle_buffer->video.pitch * var.yres * 2;
+                         surface->idle_buffer->video.pitch * vyres * 2;
                     break;
                case DLBM_BACKSYSTEM:
                     surface->caps |= DSCAPS_DOUBLE;
@@ -1944,7 +2149,7 @@ static DFBResult dfb_fbdev_set_mode( CoreSurface           *surface,
                     break;
           }
 
-          dfb_fbdev_pan( 0, false );
+          dfb_fbdev_pan( var.xoffset, var.yoffset, false );
 
           dfb_gfxcard_after_set_var();
 
@@ -2039,7 +2244,7 @@ static DFBResult dfb_fbdev_read_modes()
  * correct colors, the gamme ramp has to be initialized.
  */
 
-static __u16 dfb_fbdev_calc_gamma(int n, int max)
+static u16 dfb_fbdev_calc_gamma(int n, int max)
 {
      int ret = 65535.0 * ((float)((float)n/(max)));
      if (ret > 65535) ret = 65535;
@@ -2068,10 +2273,13 @@ static DFBResult dfb_fbdev_set_gamma_ramp( DFBSurfacePixelFormat format )
 
      switch (format) {
           case DSPF_ARGB1555:
+          case DSPF_RGB555:
                red_size   = 32;
                green_size = 32;
                blue_size  = 32;
                break;
+          case DSPF_ARGB4444:
+          case DSPF_RGB444:
           case DSPF_RGB16:
                red_size   = 32;
                green_size = 64;
@@ -2188,10 +2396,10 @@ static DFBResult dfb_fbdev_set_rgb332_palette()
 
      cmap.start  = 0;
      cmap.len    = 256;
-     cmap.red    = (__u16*)SHMALLOC( pool, 2 * 256 );
-     cmap.green  = (__u16*)SHMALLOC( pool, 2 * 256 );
-     cmap.blue   = (__u16*)SHMALLOC( pool, 2 * 256 );
-     cmap.transp = (__u16*)SHMALLOC( pool, 2 * 256 );
+     cmap.red    = (u16*)SHMALLOC( pool, 2 * 256 );
+     cmap.green  = (u16*)SHMALLOC( pool, 2 * 256 );
+     cmap.blue   = (u16*)SHMALLOC( pool, 2 * 256 );
+     cmap.transp = (u16*)SHMALLOC( pool, 2 * 256 );
 
 
      for (red_val = 0; red_val  < 8 ; red_val++) {
@@ -2245,11 +2453,11 @@ fbdev_ioctl_call_handler( int   caller,
 
      if (dfb_config->vt) {
           if (call_arg == FBIOPUT_VSCREENINFO) {
-               if (!dfb_config->kd_graphics)
+               if (!dfb_config->kd_graphics) {
                     ioctl( dfb_fbdev->vt->fd, KDSETMODE, KD_TEXT );
-
-               write( dfb_fbdev->vt->fd, cursoroff_str, strlen(cursoroff_str) );
-               write( dfb_fbdev->vt->fd, blankoff_str, strlen(blankoff_str) );
+                    write( dfb_fbdev->vt->fd, cursoroff_str, strlen(cursoroff_str) );
+                    write( dfb_fbdev->vt->fd, blankoff_str, strlen(blankoff_str) );
+               }
           }
      }
 
