@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -32,6 +33,7 @@
 #include <directfb.h>
 
 #include <fusion/shmalloc.h>
+#include <fusion/fusion.h>
 
 #include <core/core.h>
 #include <core/coredefs.h>
@@ -313,7 +315,7 @@ primaryAllocateSurface( CoreLayer              *layer,
 
           /* reallocation just needs an allocated buffer structure */
           surface->idle_buffer = surface->back_buffer = surface->front_buffer
-                               = SHCALLOC( 1, sizeof(SurfaceBuffer) );
+                               = SHCALLOC( dfb_core_shmpool(dfb_osx_core), 1, sizeof(SurfaceBuffer) );
 
           surface->front_buffer->policy = CSP_SYSTEMONLY;
           surface->front_buffer->format = config->format;
@@ -425,7 +427,7 @@ update_screen( CoreSurface *surface, int x, int y, int w, int h )
      DFBResult    ret;
 
      D_ASSERT( surface != NULL );
-     ret = dfb_surface_soft_lock( surface, DSLF_READ, &src, &pitch, true );
+     ret = dfb_surface_soft_lock( dfb_osx_core, surface, DSLF_READ, &src, &pitch, true );
      if (ret) {
           D_ERROR( "DirectFB/OSX: Couldn't lock layer surface: %s\n",
                    DirectFBErrorString( ret ) );
@@ -557,19 +559,19 @@ dfb_osx_set_video_mode( CoreDFB *core, CoreLayerRegionConfig *config )
      if (dfb_core_is_master( core ))
           return dfb_osx_set_video_mode_handler( config );
 
-     if (!fusion_is_shared( config )) {
-          tmp = SHMALLOC( sizeof(CoreLayerRegionConfig) );
+     if (!fusion_is_shared( dfb_core_world(core), config )) {
+          tmp = SHMALLOC( dfb_core_shmpool(dfb_osx_core), sizeof(CoreLayerRegionConfig) );
           if (!tmp)
                return D_OOSHM();
 
           direct_memcpy( tmp, config, sizeof(CoreLayerRegionConfig) );
      }
 
-     fusion_call_execute( &dfb_osx->call, OSX_SET_VIDEO_MODE,
+     fusion_call_execute( &dfb_osx->call, FCEF_NONE, OSX_SET_VIDEO_MODE,
                           tmp ? tmp : config, &ret );
 
      if (tmp)
-          SHFREE( tmp );
+          SHFREE( dfb_core_shmpool(dfb_osx_core), tmp );
 
      return ret;
 }
@@ -584,8 +586,8 @@ dfb_osx_update_screen( CoreDFB *core, DFBRegion *region )
           return dfb_osx_update_screen_handler( region );
 
      if (region) {
-          if (!fusion_is_shared( region )) {
-               tmp = SHMALLOC( sizeof(DFBRegion) );
+          if (!fusion_is_shared( dfb_core_world(core), region )) {
+               tmp = SHMALLOC( dfb_core_shmpool(dfb_osx_core), sizeof(DFBRegion) );
                if (!tmp)
                     return D_OOSHM();
 
@@ -593,11 +595,11 @@ dfb_osx_update_screen( CoreDFB *core, DFBRegion *region )
           }
      }
 
-     fusion_call_execute( &dfb_osx->call, OSX_UPDATE_SCREEN,
+     fusion_call_execute( &dfb_osx->call, FCEF_ONEWAY, OSX_UPDATE_SCREEN,
                           tmp ? tmp : region, &ret );
 
      if (tmp)
-          SHFREE( tmp );
+          SHFREE( dfb_core_shmpool(dfb_osx_core), tmp );
 
      return ret;
 }
@@ -607,7 +609,7 @@ dfb_osx_set_palette( CorePalette *palette )
 {
      int ret;
 
-     fusion_call_execute( &dfb_osx->call, OSX_SET_PALETTE,
+     fusion_call_execute( &dfb_osx->call, FCEF_NONE, OSX_SET_PALETTE,
                           palette, &ret );
 
      return ret;
