@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -93,6 +94,11 @@ static void
 motion_realize( SDLInputData *data )
 {
      if (motionX.type != DIET_UNKNOWN) {
+          if (motionY.type != DIET_UNKNOWN) {
+               /* let DirectFB know two events are coming */
+               motionX.flags  |= DIEF_FOLLOW;
+          }
+
           dfb_input_dispatch( data->device, &motionX );
 
           motionX.type = DIET_UNKNOWN;
@@ -116,8 +122,8 @@ translate_key( SDLKey key, DFBInputEvent *evt )
      }
 
      /* Function keys */
-     if (key >= SDLK_F1  &&  key <= SDLK_F15) {
-          evt->key_id = DIKI_KP_F1 + key - SDLK_F1;
+     if (key >= SDLK_F1  &&  key <= SDLK_F12) {
+          evt->key_id = DIKI_F1 + key - SDLK_F1;
           return true;
      }
 
@@ -309,7 +315,7 @@ translate_key( SDLKey key, DFBInputEvent *evt )
                evt->key_symbol = DIKS_ALTGR;
                return true;
           default:
-			printf("SDL: Unknown key pressed\n");
+               break;
      }
 
      evt->flags = DIEF_NONE;
@@ -386,17 +392,34 @@ sdlEventThread( DirectThread *thread, void *driver_data )
 
                     case SDL_KEYUP:
                     case SDL_KEYDOWN:
-			             //XXX sdl does not give us the keymap
-                         evt.flags |= DIEF_KEYCODE;
-
                          if (event.type == SDL_KEYDOWN)
                               evt.type = DIET_KEYPRESS;
                          else
                               evt.type = DIET_KEYRELEASE;
-                         if (translate_key( event.key.keysym.sym, &evt )) {
-			                  evt.key_code=evt.key_id;
-                              dfb_input_dispatch( data->device, &evt );
+
+                         /* Get a key id first */
+                         translate_key( event.key.keysym.sym, &evt );
+
+                         /* If SDL provided a symbol, use it */
+                         if (event.key.keysym.unicode) {
+                              evt.flags     |= DIEF_KEYSYMBOL;
+                              evt.key_symbol = event.key.keysym.unicode;
+
+                              /**
+                               * Hack to translate the Control+[letter]
+                               * combination to
+                               * Modifier: CONTROL, Key Symbol: [letter]
+                               * A side effect here is that Control+Backspace
+                               * produces Control+h
+                               */
+                              if (evt.modifiers == DIMM_CONTROL &&
+                                  evt.key_symbol >= 1 && evt.key_symbol <= ('z'-'a'+1))
+                              {
+                                  evt.key_symbol += 'a'-1;
+                              }
                          }
+
+                         dfb_input_dispatch( data->device, &evt );
                          break;
                     case SDL_QUIT:
                          evt.type       = DIET_KEYPRESS;
@@ -501,10 +524,6 @@ driver_open_device( CoreInputDevice  *device,
 
      /* set capabilities */
      info->desc.caps   = DICAPS_ALL;
-     /* enable translation of fake raw hardware keycodes */
-     info->desc.min_keycode = DIKI_A;
-     info->desc.max_keycode = DIKI_KP_9;
- 
 
 
      /* allocate and fill private data */
@@ -522,256 +541,15 @@ driver_open_device( CoreInputDevice  *device,
      return DFB_OK;
 }
 
-static DFBInputDeviceKeySymbol
-id_to_symbol( DFBInputDeviceKeyIdentifier id,
-              DFBInputDeviceModifierMask  modifiers)
-{
-	bool shift = (modifiers & DIMM_SHIFT);
-
-     if (id >= DIKI_A && id <= DIKI_Z)
-          return (shift ? DIKS_CAPITAL_A : DIKS_SMALL_A) + id - DIKI_A;
-
-     if (id >= DIKI_0 && id <= DIKI_9) {
-		if( shift ) {
-          switch(id) {
-              case DIKI_1: return DIKS_EXCLAMATION_MARK;
-              case DIKI_2: return DIKS_AT;
-              case DIKI_3: return DIKS_NUMBER_SIGN;
-              case DIKI_4: return DIKS_DOLLAR_SIGN;
-              case DIKI_5: return DIKS_PERCENT_SIGN;
-              case DIKI_6: return DIKS_CIRCUMFLEX_ACCENT;
-              case DIKI_7: return DIKS_AMPERSAND;
-              case DIKI_8: return DIKS_ASTERISK;
-              case DIKI_9: return DIKS_PARENTHESIS_LEFT;
-              case DIKI_0: return DIKS_PARENTHESIS_RIGHT;
-              default:;
-          }
-       } 
-	   else 
-          return DIKS_0 + id - DIKI_0;
-	 }
-  
-      if (id >= DIKI_KP_0 && id <= DIKI_KP_9)
-           return DIKS_0 + id - DIKI_KP_0;
-
-     if (id >= DIKI_F1 && id <= DIKI_F12)
-          return DIKS_F1 + id - DIKI_F1;
-
-
-     switch (id) {
-          case DIKI_ESCAPE:
-               return DIKS_ESCAPE;
-
-          case DIKI_LEFT:
-               return DIKS_CURSOR_LEFT;
-
-          case DIKI_RIGHT:
-               return DIKS_CURSOR_RIGHT;
-
-          case DIKI_UP:
-               return DIKS_CURSOR_UP;
-
-          case DIKI_DOWN:
-               return DIKS_CURSOR_DOWN;
-
-          case DIKI_CONTROL_L:
-          case DIKI_CONTROL_R:
-               return DIKS_CONTROL;
-
-          case DIKI_SHIFT_L:
-          case DIKI_SHIFT_R:
-               return DIKS_SHIFT;
-
-          case DIKI_ALT_L:
-          case DIKI_ALT_R:
-               return DIKS_ALT;
-
-          case DIKI_META_L:
-          case DIKI_META_R:
-               return DIKS_META;
-
-          case DIKI_SUPER_L:
-          case DIKI_SUPER_R:
-               return DIKS_SUPER;
-
-          case DIKI_HYPER_L:
-          case DIKI_HYPER_R:
-               return DIKS_HYPER;
-
-          case DIKI_TAB:
-               return DIKS_TAB;
-
-          case DIKI_ENTER:
-               return DIKS_ENTER;
-
-          case DIKI_SPACE:
-               return DIKS_SPACE;
-
-          case DIKI_BACKSPACE:
-               return DIKS_BACKSPACE;
-
-          case DIKI_INSERT:
-               return DIKS_INSERT;
-
-          case DIKI_DELETE:
-               return DIKS_DELETE;
-
-          case DIKI_HOME:
-               return DIKS_HOME;
-
-          case DIKI_END:
-               return DIKS_END;
-
-          case DIKI_PAGE_UP:
-               return DIKS_PAGE_UP;
-
-          case DIKI_PAGE_DOWN:
-               return DIKS_PAGE_DOWN;
-
-          case DIKI_CAPS_LOCK:
-               return DIKS_CAPS_LOCK;
-
-          case DIKI_NUM_LOCK:
-               return DIKS_NUM_LOCK;
-
-          case DIKI_SCROLL_LOCK:
-               return DIKS_SCROLL_LOCK;
-
-          case DIKI_PRINT:
-               return DIKS_PRINT;
-
-          case DIKI_PAUSE:
-               return DIKS_PAUSE;
-
-          case DIKI_KP_DIV:
-               return DIKS_SLASH;
-
-          case DIKI_KP_MULT:
-               return DIKS_ASTERISK;
-
-          case DIKI_KP_MINUS:
-               return DIKS_MINUS_SIGN;
-
-          case DIKI_KP_PLUS:
-               return DIKS_PLUS_SIGN;
-
-          case DIKI_KP_ENTER:
-               return DIKS_ENTER;
-
-          case DIKI_KP_SPACE:
-               return DIKS_SPACE;
-
-          case DIKI_KP_TAB:
-               return DIKS_TAB;
-
-          case DIKI_KP_EQUAL:
-               return DIKS_EQUALS_SIGN;
-
-          case DIKI_KP_DECIMAL:
-               return DIKS_PERIOD;
-
-          case DIKI_KP_SEPARATOR:
-               return DIKS_COMMA;
-
-          case DIKI_BACKSLASH:
-                if( shift )
-                  return DIKS_VERTICAL_BAR;
-                else
-                  return DIKS_BACKSLASH;
-
-          case DIKI_EQUALS_SIGN:
-                if( shift )
-                  return DIKS_PLUS_SIGN;
-                else
-                  return DIKS_EQUALS_SIGN;
-
-          case DIKI_LESS_SIGN:
-               return DIKS_LESS_THAN_SIGN;
-
-          case DIKI_MINUS_SIGN:
-                if( shift )
-                  return DIKS_UNDERSCORE;
-                else
-                  return DIKS_MINUS_SIGN;
-
-           case DIKI_COMMA:
-                if( shift )
-                  return DIKS_LESS_THAN_SIGN;
-                else
-                  return DIKS_COMMA;
-
-          case DIKI_PERIOD:
-                if( shift )
-                  return DIKS_GREATER_THAN_SIGN;
-                else
-                  return DIKS_PERIOD;
-
-           case DIKI_BRACKET_LEFT:
-                if( shift )
-                  return DIKS_CURLY_BRACKET_LEFT;
-                else
-                  return DIKS_SQUARE_BRACKET_LEFT;
- 
-           case DIKI_BRACKET_RIGHT:
-                if( shift )
-                  return DIKS_CURLY_BRACKET_RIGHT;
-                else
-                  return DIKS_SQUARE_BRACKET_RIGHT;
-
-          case DIKI_QUOTE_LEFT:
-                if( shift )
-                  return DIKS_TILDE;
-                else
-                  return DIKS_GRAVE_ACCENT;
-          case DIKI_QUOTE_RIGHT:
-                if( shift )
-                  return DIKS_QUOTATION;
-                else
-                  return DIKS_APOSTROPHE;
-
-          case DIKI_SEMICOLON:
-                if( shift )
-                  return DIKS_COLON;
-                else
-                  return DIKS_SEMICOLON;
-
-          case DIKI_SLASH:
-                if( shift )
-                  return DIKS_QUESTION_MARK;
-                else
-                  return DIKS_SLASH;
-          default:
-               ;
-     }
-
-     return DIKS_NULL;
-}
-
 /*
  * Fetch one entry from the device's keymap if supported.
- * this does a fake mapping based on the orginal DFB code
  */
 static DFBResult
 driver_get_keymap_entry( CoreInputDevice           *device,
                          void                      *driver_data,
                          DFBInputDeviceKeymapEntry *entry )
 {
-	int  code = entry->code;
-    entry->identifier=code;
-
-     /* is CapsLock effective? */
-     if (code >= DIKI_A && code <= DIKI_Z)
-		entry->locks |= DILS_CAPS;
-
-     /* is NumLock effective? */
-     if (entry->identifier >= DIKI_KP_DECIMAL && entry->identifier <= DIKI_KP_9)
-          entry->locks |= DILS_NUM;
-
-     entry->symbols[DIKSI_BASE]=id_to_symbol(entry->identifier,0);
-     entry->symbols[DIKSI_BASE_SHIFT]=id_to_symbol(entry->identifier,DIMM_SHIFT);
-     entry->symbols[DIKSI_ALT]=entry->symbols[DIKSI_BASE];
-     entry->symbols[DIKSI_ALT_SHIFT]=entry->symbols[DIKSI_BASE_SHIFT];
-     return DFB_OK;
+     return DFB_UNSUPPORTED;
 }
 /*
  * End thread, close device and free private data.
