@@ -1,4 +1,15 @@
 /*
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
+
+   All rights reserved.
+
+   Written by Denis Oliver Kropp <dok@directfb.org>,
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
@@ -23,6 +34,7 @@
 
 #include <directfb.h>
 
+#include <fusion/fusion.h>
 #include <fusion/shmalloc.h>
 
 #include <core/core.h>
@@ -416,7 +428,7 @@ update_screen( CoreSurface *surface, int x, int y, int w, int h )
      D_ASSERT( rfb_screen != NULL );
      D_ASSERT( rfb_screen->frameBuffer != NULL );
 
-     ret = dfb_surface_soft_lock( surface, DSLF_READ, &src, &pitch, true );
+     ret = dfb_surface_soft_lock( dfb_vnc_core, surface, DSLF_READ, &src, &pitch, true );
      if (ret) {
           D_ERROR( "DirectFB/VNC: Couldn't lock layer surface: %s\n",
                    DirectFBErrorString( ret ) );
@@ -673,19 +685,19 @@ dfb_vnc_set_video_mode( CoreDFB *core, CoreLayerRegionConfig *config )
      if (dfb_core_is_master( core ))
           return dfb_vnc_set_video_mode_handler( config );
 
-     if (!fusion_is_shared( config )) {
-          tmp = SHMALLOC( sizeof(CoreLayerRegionConfig) );
+     if (!fusion_is_shared( dfb_core_world(core), config )) {
+          tmp = SHMALLOC( dfb_core_shmpool(core), sizeof(CoreLayerRegionConfig) );
           if (!tmp)
-               return DFB_NOSYSTEMMEMORY;
+               return D_OOSHM();
 
           direct_memcpy( tmp, config, sizeof(CoreLayerRegionConfig) );
      }
 
-     fusion_call_execute( &dfb_vnc->call, VNC_SET_VIDEO_MODE,
+     fusion_call_execute( &dfb_vnc->call, FCEF_NONE, VNC_SET_VIDEO_MODE,
                           tmp ? tmp : config, &ret );
 
      if (tmp)
-          SHFREE( tmp );
+          SHFREE( dfb_core_shmpool(core), tmp );
 
      return ret;
 }
@@ -700,22 +712,22 @@ dfb_vnc_update_screen( CoreDFB *core, DFBRegion *region )
           return dfb_vnc_update_screen_handler( region );
 
      if (region) {
-          if (!fusion_is_shared( region )) {
-               tmp = SHMALLOC( sizeof(DFBRegion) );
+          if (!fusion_is_shared( dfb_core_world(core), region )) {
+               tmp = SHMALLOC( dfb_core_shmpool(core), sizeof(DFBRegion) );
                if (!tmp)
-                    return DFB_NOSYSTEMMEMORY;
+                    return D_OOSHM();
 
                direct_memcpy( tmp, region, sizeof(DFBRegion) );
           }
      }
 
-     fusion_call_execute( &dfb_vnc->call, VNC_UPDATE_SCREEN,
+     fusion_call_execute( &dfb_vnc->call, FCEF_NONE, VNC_UPDATE_SCREEN,
                           tmp ? tmp : region, &ret );
 
      if (tmp)
-          SHFREE( tmp );
+          SHFREE( dfb_core_shmpool(core), tmp );
 
-     return ret;
+     return DFB_OK;
 }
 
 static DFBResult
@@ -723,7 +735,7 @@ dfb_vnc_set_palette( CorePalette *palette )
 {
      int ret;
 
-     fusion_call_execute( &dfb_vnc->call, VNC_SET_PALETTE,
+     fusion_call_execute( &dfb_vnc->call, FCEF_NONE, VNC_SET_PALETTE,
                           palette, &ret );
 
      return ret;
