@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This file is subject to the terms and conditions of the MIT License:
 
@@ -325,7 +326,7 @@ parse_int( const AnyOption *option, const char *arg )
      int   ret;
      char *end;
 
-     ret = strtoul( arg, &end, option->data ? (int) option->data : 10 );
+     ret = strtoul( arg, &end, option->data ? (unsigned long) option->data : 10 );
 
      if (*end || ret < 0) {
           fprintf( stderr, "\nInvalid argument to '%s' or '%s' specified!\n\n",
@@ -410,7 +411,7 @@ parse_enum( const AnyOption *option, const char *arg )
 static bool
 parse_ids( const AnyOption *option, const char *arg )
 {
-     __u32  val  = 0;
+     u32    val  = 0;
      int    alen = strlen( arg );
      char  *abuf = alloca( alen + 1 );
 
@@ -455,7 +456,7 @@ parse_ids( const AnyOption *option, const char *arg )
                abuf++;
      }
 
-     *((__u32*)option->value) = val;
+     *((u32*)option->value) = val;
 
      return true;
 }
