@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This file is subject to the terms and conditions of the MIT License:
 
@@ -31,6 +32,8 @@
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
+#include <config.h>
+
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -177,7 +180,7 @@ chunk_callback( SurfaceBuffer *buffer,
                 int            tolerations,
                 void          *ctx )
 {
-     __u8         r = 0, g = 0, b = 0;
+     u8           r = 0, g = 0, b = 0;
      unsigned int screen_length, screen_offset;
 
      if (buffer) {
