@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    directfb-csource is based on gdk-pixbuf-csource, a GdkPixbuf
    based image CSource generator  Copyright (C) 1999, 2001 Tim Janik
@@ -36,6 +37,9 @@
 #include <errno.h>
 #include <assert.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include <png.h>
 
 #include <directfb.h>
@@ -61,18 +65,22 @@ static struct {
      { DSPF_A8,       "A8"       },
      { DSPF_LUT8,     "LUT8"     }
 };
-static int n_pixelformats = sizeof (pixelformats) / sizeof (pixelformats[0]);
+static int n_pixelformats = D_ARRAY_SIZE( pixelformats );
 
 
 static void       print_usage     (const char             *prg_name);
 static DFBResult  load_image      (const char             *filename,
                                    DFBSurfaceDescription  *desc,
                                    DFBColor               *palette,
-                                   int                    *palette_size);
+                                   int                    *palette_size,
+                                   DFBSurfacePixelFormat   rgbformat);
 static DFBResult  merge_images    (DFBSurfaceDescription  *images,
                                    int                     num_images,
                                    DFBSurfaceDescription  *dest,
                                    DFBRectangle           *rectangles);
+static DFBResult  dump_raw_data   (const char             *name,
+                                   const unsigned char    *data,
+                                   unsigned int            len);
 static DFBResult  dump_image      (const char             *name,
                                    DFBSurfaceDescription  *desc,
                                    DFBColor               *palette,
@@ -88,8 +96,9 @@ static char *     base_name       (const char             *name);
 int main (int         argc,
           const char *argv[])
 {
-     DFBSurfaceDescription  desc   = { flags: 0 };
-     DFBSurfacePixelFormat  format = DSPF_UNKNOWN;
+     DFBSurfaceDescription desc      = { flags: 0 };
+     DFBSurfacePixelFormat format    = DSPF_UNKNOWN;
+     DFBSurfacePixelFormat rgbformat = DSPF_UNKNOWN;
      DFBColor  palette[256];
 
      const char *filename[argc];
@@ -97,6 +106,7 @@ int main (int         argc,
      int         palette_size   = 0;
      int         num_images     = 0;
      int         i, n;
+     int         rawmode        = 0;
 
      /*  parse command line  */
 
@@ -114,6 +124,10 @@ int main (int         argc,
                              DIRECTFB_VERSION);
                     return EXIT_SUCCESS;
                }
+               if (strcmp (arg, "raw") == 0) {
+                    rawmode = 1;
+                    continue;
+               }
                if (strncmp (arg, "format=", 7) == 0 && !format) {
                     for (i = 0; i < n_pixelformats && !format; i++)
                          if (!strcasecmp (pixelformats[i].name, arg + 7))
@@ -121,6 +135,13 @@ int main (int         argc,
                     if (format)
                          continue;
                }
+               if (strncmp (arg, "rgbformat=", 10) == 0 && !rgbformat) {
+                    for (i = 0; i < n_pixelformats && !rgbformat; i++)
+                         if (!strcasecmp (pixelformats[i].name, arg + 10))
+                              rgbformat = pixelformats[i].format;
+                    if (rgbformat)
+                         continue;
+               }
                if (strncmp (arg, "name=", 5) == 0 && !name) {
                     name = arg + 5;
                     if (*name)
@@ -141,6 +162,12 @@ int main (int         argc,
           return EXIT_FAILURE;
      }
 
+     if (num_images > 1 && rawmode) {
+          fprintf (stderr,
+                   "Multiple input files not allowed in raw mode.\n");
+          return EXIT_FAILURE;
+     }
+
      if (num_images > 1 && !name) {
           fprintf (stderr,
                    "You must specify a variable name when using multiple images.\n");
@@ -149,19 +176,41 @@ int main (int         argc,
 
      /*  load the first image  */
 
-     if (format) {
-          desc.flags = DSDESC_PIXELFORMAT;
-          desc.pixelformat = format;
+     if (rawmode) {
+
+          struct stat statbuf; 
+          if (0 == stat(filename[0], &statbuf))
+          {
+               unsigned char *data = alloca(statbuf.st_size);
+               memset(data, 0, statbuf.st_size);
+
+               FILE *f = fopen(filename[0], "r");
+               if (f)
+               {
+                    fread(data, statbuf.st_size, 1, f);
+                    fclose(f);
+               }
+
+               return dump_raw_data(name ? : strrchr (filename[0], '/') ? : filename[0],
+                                    data, statbuf.st_size);
+          }
+          
      }
+     else {
+          if (format) {
+               desc.flags = DSDESC_PIXELFORMAT;
+               desc.pixelformat = format;
+          }
 
-     if (load_image (filename[0], &desc, palette, &palette_size) != DFB_OK)
-          return EXIT_FAILURE;
+          if (load_image (filename[0], &desc, palette, &palette_size, rgbformat) != DFB_OK)
+               return EXIT_FAILURE;
 
-     /*  dump it and quit if this is the only image on the command line  */
-
-     if (num_images == 1)
-          return dump_image (name ? : strrchr (filename[0], '/') ? : filename[0],
-                             &desc, palette, palette_size);
+          /*  dump it and quit if this is the only image on the command line  */
+          
+          if (num_images == 1)
+               return dump_image (name ? : strrchr (filename[0], '/') ? : filename[0],
+                                  &desc, palette, palette_size);
+     }
 
      /*  merge multiple images into one surface  */
      {
@@ -177,7 +226,7 @@ int main (int         argc,
                image[i].pixelformat = desc.pixelformat;
 
                if (load_image (filename[i],
-                               image + i, foo, &foo_size) != DFB_OK)
+                               image + i, foo, &foo_size, rgbformat) != DFB_OK)
                     return EXIT_FAILURE;
           }
 
@@ -197,11 +246,13 @@ static void print_usage (const char *prg_name)
 {
      fprintf (stderr, "directfb-csource version %s\n\n", DIRECTFB_VERSION);
      fprintf (stderr, "Usage: %s [options] <imagefile>\n", prg_name);
-     fprintf (stderr, "   --name=<identifer>     specifies variable name\n");
-     fprintf (stderr, "   --format=<identifer>   specifies surface format\n");
-     fprintf (stderr, "   --multi                multiple images\n");
-     fprintf (stderr, "   --help                 show this help message\n");
-     fprintf (stderr, "   --version              print version information\n");
+     fprintf (stderr, "   --name=<identifer>      specifies variable name\n");
+     fprintf (stderr, "   --format=<identifer>    specifies surface format\n");
+     fprintf (stderr, "   --rgbformat=<identifer> specifies format for non-alpha images\n");
+     fprintf (stderr, "   --multi                 multiple images\n");
+     fprintf (stderr, "   --raw                   dump a single file directly to header\n");
+     fprintf (stderr, "   --help                  show this help message\n");
+     fprintf (stderr, "   --version               print version information\n");
      fprintf (stderr, "\n");
      fprintf (stderr, "See the directfb-csource(1) man-page for more information.\n");
      fprintf (stderr, "\n");
@@ -210,7 +261,8 @@ static void print_usage (const char *prg_name)
 static DFBResult load_image (const char            *filename,
                              DFBSurfaceDescription *desc,
                              DFBColor              *palette,
-                             int                   *palette_size)
+                             int                   *palette_size,
+                             DFBSurfacePixelFormat  rgbformat)
 {
      DFBSurfacePixelFormat dest_format;
      DFBSurfacePixelFormat src_format;
@@ -291,6 +343,8 @@ static DFBResult load_image (const char            *filename,
                /* fallthru */
           case PNG_COLOR_TYPE_GRAY_ALPHA:
                png_set_gray_to_rgb (png_ptr);
+               if (rgbformat)
+                    dest_format = rgbformat;
                break;
 
           case PNG_COLOR_TYPE_PALETTE:
@@ -301,6 +355,8 @@ static DFBResult load_image (const char            *filename,
                png_set_palette_to_rgb (png_ptr);
                /* fallthru */
           case PNG_COLOR_TYPE_RGB:
+               if (rgbformat)
+                    dest_format = rgbformat;
           case PNG_COLOR_TYPE_RGB_ALPHA:
                if (dest_format == DSPF_RGB24) {
                     png_set_strip_alpha (png_ptr);
@@ -391,27 +447,27 @@ static DFBResult load_image (const char            *filename,
           switch (dest_format) {
                case DSPF_RGB16:
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
-                         dfb_argb_to_rgb16 ((__u32 *) s, (__u16 *) d, width);
+                         dfb_argb_to_rgb16 ((u32 *) s, (u16 *) d, width);
                     break;
                case DSPF_ARGB1555:
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
-                         dfb_argb_to_argb1555 ((__u32 *) s, (__u16 *) d, width);
+                         dfb_argb_to_argb1555 ((u32 *) s, (u16 *) d, width);
                     break;
                case DSPF_ARGB2554:
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
-                         dfb_argb_to_argb2554 ((__u32 *) s, (__u16 *) d, width);
+                         dfb_argb_to_argb2554 ((u32 *) s, (u16 *) d, width);
                     break;
                case DSPF_ARGB4444:
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
-                         dfb_argb_to_argb4444 ((__u32 *) s, (__u16 *) d, width);
+                         dfb_argb_to_argb4444 ((u32 *) s, (u16 *) d, width);
                     break;
                case DSPF_RGB332:
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
-                         dfb_argb_to_rgb332 ((__u32 *) s, (__u8 *) d, width);
+                         dfb_argb_to_rgb332 ((u32 *) s, (u8 *) d, width);
                     break;
                case DSPF_A8:
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
-                         dfb_argb_to_a8 ((__u32 *) s, (__u8 *) d, width);
+                         dfb_argb_to_a8 ((u32 *) s, (u8 *) d, width);
                     break;
                default:
                     fprintf (stderr,
@@ -571,6 +627,47 @@ static inline void save_uchar (CSourceData   *csource,
      return;
 }
 
+static DFBResult dump_data(CSourceData         *csource,
+                           const char          *name,
+                           const unsigned char *data,
+                           unsigned int        len)
+{
+     fprintf (csource->fp,
+              "static const unsigned char %s_data[] =\n", name);
+     fprintf (csource->fp, "  \"");
+     
+     csource->pos = 3;
+     do
+          save_uchar (csource, *data++);
+     while (--len);
+
+     fprintf (csource->fp, "\";\n\n");
+
+     return DFB_OK;
+}
+
+static DFBResult dump_raw_data(const char            *name,
+                               const unsigned char   *data,
+                               unsigned int           len)
+{
+     CSourceData    csource = { stdout, 0, 0 };
+     char          *vname   = variable_name (name);
+
+     if (!data || !len)
+          return DFB_INVARG;
+
+     fprintf (csource.fp,
+              "/* DirectFB raw data dump created by directfb-csource %s */\n\n",
+              DIRECTFB_VERSION);
+
+     dump_data(&csource, vname, data, len);
+
+     free (vname);
+
+     return DFB_OK;
+}
+
+
 static DFBResult dump_image (const char            *name,
                              DFBSurfaceDescription *desc,
                              DFBColor              *palette,
@@ -583,7 +680,8 @@ static DFBResult dump_image (const char            *name,
      unsigned long  len;
      int            i;
 
-     if (desc->flags != (DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT |
+     if (desc &&
+         desc->flags != (DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT |
                          DSDESC_PREALLOCATED))
           return DFB_INVARG;
 
@@ -606,21 +704,12 @@ static DFBResult dump_image (const char            *name,
               DIRECTFB_VERSION);
 
      /* dump data */
-     fprintf (csource.fp,
-              "static unsigned char %s_data[] =\n", vname);
-     fprintf (csource.fp, "  \"");
-
-     csource.pos = 3;
-     do
-          save_uchar (&csource, *data++);
-     while (--len);
-
-     fprintf (csource.fp, "\";\n\n");
+     dump_data(&csource, vname, data, len);
 
      /* dump palette */
      if (palette_size > 0) {
           fprintf (csource.fp,
-                   "static DFBColor %s_palette[%d] = {\n", vname, palette_size);
+                   "static const DFBColor %s_palette[%d] = {\n", vname, palette_size);
           for (i = 0; i < palette_size; i++)
                fprintf (csource.fp,
                         "  { 0x%02x, 0x%02x, 0x%02x, 0x%02x }%c\n",
@@ -631,7 +720,7 @@ static DFBResult dump_image (const char            *name,
 
      /* dump description */
      fprintf (csource.fp,
-              "static DFBSurfaceDescription %s_desc = {\n", vname);
+              "static const DFBSurfaceDescription %s_desc = {\n", vname);
      fprintf (csource.fp,
               "  flags                   : DSDESC_WIDTH | DSDESC_HEIGHT | DSDESC_PIXELFORMAT |\n"
               "                            DSDESC_PREALLOCATED");
@@ -681,7 +770,7 @@ static DFBResult dump_rectangles (const char    *name,
               DIRECTFB_VERSION);
 
      fprintf (fp,
-              "static struct {\n"
+              "static const struct {\n"
               "  const char   *name;\n"
               "  DFBRectangle  rect;\n"
               "} %s[] = {\n", vname);
