@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This file is subject to the terms and conditions of the MIT License:
 
@@ -55,8 +56,9 @@
 #include <core/system.h>
 
 
-static long long    t1, t2;
-static unsigned int loops;
+static long long     t1, t2;
+static unsigned int  loops;
+static FusionWorld  *world;
 
 #define BENCH_START()       do { sync(); usleep(100000); sync(); t1 = direct_clock_get_millis(); loops = 0; } while (0)
 #define BENCH_STOP()        do { t2 = direct_clock_get_millis(); } while (0)
@@ -82,7 +84,7 @@ bench_reactor()
      Reaction        reaction2;
      GlobalReaction  global_reaction;
 
-     reactor = fusion_reactor_new( 16, "Benchmark" );
+     reactor = fusion_reactor_new( 16, "Benchmark", world );
      if (!reactor) {
           fprintf( stderr, "Fusion Error\n" );
           return;
@@ -166,7 +168,7 @@ bench_ref()
      DirectResult ret;
      FusionRef    ref;
 
-     ret = fusion_ref_init( &ref );
+     ret = fusion_ref_init( &ref, "Benchmark", world );
      if (ret) {
           fprintf( stderr, "Fusion Error %d\n", ret );
           return;
@@ -210,7 +212,7 @@ bench_property()
      DirectResult   ret;
      FusionProperty property;
 
-     ret = fusion_property_init( &property );
+     ret = fusion_property_init( &property, world );
      if (ret) {
           fprintf( stderr, "Fusion Error %d\n", ret );
           return;
@@ -241,7 +243,7 @@ bench_skirmish()
      DirectResult   ret;
      FusionSkirmish skirmish;
 
-     ret = fusion_skirmish_init( &skirmish, "Benchmark" );
+     ret = fusion_skirmish_init( &skirmish, "Benchmark", world );
      if (ret) {
           fprintf( stderr, "Fusion Error %d\n", ret );
           return;
@@ -286,7 +288,7 @@ bench_skirmish_threaded()
      DirectResult   ret;
      FusionSkirmish skirmish;
 
-     ret = fusion_skirmish_init( &skirmish, "Threaded Benchmark" );
+     ret = fusion_skirmish_init( &skirmish, "Threaded Benchmark", world );
      if (ret) {
           fprintf( stderr, "Fusion Error %d\n", ret );
           return;
@@ -435,10 +437,19 @@ bench_flock()
 }
 
 static void
-bench_shmalloc()
+bench_shmpool( bool debug )
 {
-     void      *mem[256];
-     const int  sizes[8] = { 12, 36, 200, 120, 39, 3082, 8, 1040 };
+     DirectResult  ret;
+     void         *mem[256];
+     const int     sizes[8] = { 12, 36, 200, 120, 39, 3082, 8, 1040 };
+
+     FusionSHMPoolShared *pool;
+
+     ret = fusion_shm_pool_create( world, "Benchmark Pool", 524288, debug, &pool );
+     if (ret) {
+          DirectFBError( "fusion_shm_pool_create() failed", ret );
+          return;
+     }
 
      BENCH_START();
 
@@ -446,29 +457,30 @@ bench_shmalloc()
           int i;
 
           for (i=0; i<128; i++)
-               mem[i] = SHMALLOC( sizes[i&7] );
+               mem[i] = SHMALLOC( pool, sizes[i&7] );
 
           for (i=0; i<64; i++)
-               SHFREE( mem[i] );
+               SHFREE( pool, mem[i] );
 
           for (i=128; i<192; i++)
-               mem[i] = SHMALLOC( sizes[i&7] );
+               mem[i] = SHMALLOC( pool, sizes[i&7] );
 
           for (i=64; i<128; i++)
-               SHFREE( mem[i] );
+               SHFREE( pool, mem[i] );
 
           for (i=192; i<256; i++)
-               mem[i] = SHMALLOC( sizes[i&7] );
+               mem[i] = SHMALLOC( pool, sizes[i&7] );
 
           for (i=128; i<256; i++)
-               SHFREE( mem[i] );
+               SHFREE( pool, mem[i] );
      }
 
      BENCH_STOP();
 
-     printf( "shm alloc/free                        -> %8.2f k/sec\n", BENCH_RESULT_BY(256) );
+     printf( "shm pool alloc/free %s           -> %8.2f k/sec\n",
+             debug ? "(debug)" : "       ", BENCH_RESULT_BY(256) );
 
-     printf( "\n" );
+     fusion_shm_pool_destroy( world, pool );
 }
 
 int
@@ -479,12 +491,13 @@ main( int argc, char *argv[] )
      /* Initialize DirectFB. */
      ret = DirectFBInit( &argc, &argv );
      if (ret)
-          return DirectFBError( "DirectFBInit", ret );
+          return DirectFBError( "DirectFBInit()", ret );
 
      dfb_system_lookup();
 
-     if (fusion_init( -1, 0, NULL ) < 0)
-          return -1;
+     ret = fusion_enter( -1, 0, FER_MASTER, &world );
+     if (ret)
+          return DirectFBError( "fusion_enter()", ret );
 
      printf( "\n" );
 
@@ -512,9 +525,12 @@ main( int argc, char *argv[] )
 
      bench_reactor();
 
-     bench_shmalloc();
+     bench_shmpool( false );
+     bench_shmpool( true );
 
-     fusion_exit( false );
+     printf( "\n" );
+
+     fusion_exit( world, false );
 
      return 0;
 }
