@@ -1,13 +1,15 @@
 #!/usr/bin/perl
 #
-#   (c) Copyright 2000-2002  convergence integrated media GmbH.
-#   (c) Copyright 2002       convergence GmbH.
+#   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+#   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 #
 #   All rights reserved.
 #
 #   Written by Denis Oliver Kropp <dok@directfb.org>,
-#              Andreas Hundt <andi@fischlustig.de> and
-#              Sven Neumann <sven@convergence.de>.
+#              Andreas Hundt <andi@fischlustig.de>,
+#              Sven Neumann <neo@directfb.org>,
+#              Ville Syrjälä <syrjala@sci.fi> and
+#              Claudio Ciccani <klan@users.sf.net>.
 #
 #   This library is free software; you can redistribute it and/or
 #   modify it under the terms of the GNU Lesser General Public
@@ -882,6 +884,18 @@ sub html_close (FILEHANDLE)
       my $FILE = shift(@_);
 
       print $FILE "\n",
+                  "<TABLE width=100% bgcolor=#eeeeee border=0 cellspacing=1 cellpadding=3>\n",
+                  "  <TR><TD width=100>\n",
+                  "    <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-sa/3.0/\">",
+                  "    <img alt=\"Creative Commons License\" style=\"border-width:0\" border=\"0\" ",
+                  "    src=\"http://i.creativecommons.org/l/by-sa/3.0/88x31.png\" />",
+                  "    </a>",
+                  "  </TD><TD>\n",
+                  "    This work is licensed under a",
+                  "    <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-sa/3.0/\">",
+                  "    Creative Commons Attribution-Share Alike 3.0 License</a>",
+                  "  </TD></TR>\n",
+                  "</TABLE>\n",
                   "</BODY>\n",
                   "</HTML>\n";
 
