@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This file is subject to the terms and conditions of the MIT License:
 
@@ -132,24 +133,19 @@ window_callback( UniqueWindow *window )
 
      printf( "%5d  ", dfb_window_id( window->window ) );
 
-     if (window->caps & DWHC_TOPMOST) {
-          printf( "*  " );
-     }
-     else {
-          switch (window->stacking) {
-               case DWSC_UPPER:
-                    printf( "^  " );
-                    break;
-               case DWSC_MIDDLE:
-                    printf( "-  " );
-                    break;
-               case DWSC_LOWER:
-                    printf( "v  " );
-                    break;
-               default:
-                    printf( "?  " );
-                    break;
-          }
+     switch (window->stacking) {
+          case DWSC_UPPER:
+               printf( "^  " );
+               break;
+          case DWSC_MIDDLE:
+               printf( "-  " );
+               break;
+          case DWSC_LOWER:
+               printf( "v  " );
+               break;
+          default:
+               printf( "?  " );
+               break;
      }
 
      if (D_FLAGS_IS_SET( window->flags, UWF_VISIBLE ))
